@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,11 +17,11 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_CHECK_STACK_OVERRUN_H
+#define DRIZZLED_CHECK_STACK_OVERRUN_H
 
-#include <drizzled/common_fwd.h>
-
-namespace drizzled { 
+namespace drizzled
+{
 
 /**
   @note
@@ -34,3 +34,4 @@ bool check_stack_overrun(Session *session, long margin, void *buff);
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_CHECK_STACK_OVERRUN_H */
