@@ -11,27 +11,123 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/internal/m_string.h>
-#include <drizzled/charset.h>
+#include "drizzled/internal/m_string.h"
+#include "drizzled/charset_info.h"
 
 #include <algorithm>
 
 using namespace std;
 
-namespace drizzled {
+namespace drizzled
+{
+
+
+size_t my_caseup_str_mb(const CHARSET_INFO * const  cs, char *str)
+{
+  register uint32_t l;
+  register unsigned char *map= cs->to_upper;
+  char *str_orig= str;
+
+  while (*str)
+  {
+    /* Pointing after the '\0' is safe here. */
+    if ((l= my_ismbchar(cs, str, str + cs->mbmaxlen)))
+      str+= l;
+    else
+    {
+      *str= (char) map[(unsigned char)*str];
+      str++;
+    }
+  }
+  return (size_t) (str - str_orig);
+}
+
+
+size_t my_casedn_str_mb(const CHARSET_INFO * const  cs, char *str)
+{
+  register uint32_t l;
+  register unsigned char *map= cs->to_lower;
+  char *str_orig= str;
+
+  while (*str)
+  {
+    /* Pointing after the '\0' is safe here. */
+    if ((l= my_ismbchar(cs, str, str + cs->mbmaxlen)))
+      str+= l;
+    else
+    {
+      *str= (char) map[(unsigned char)*str];
+      str++;
+    }
+  }
+  return (size_t) (str - str_orig);
+}
+
+
+size_t my_caseup_mb(const CHARSET_INFO * const  cs, char *src, size_t srclen,
+                    char *dst, size_t dstlen)
+{
+#ifdef NDEBUG
+  (void)dst;
+  (void)dstlen;
+#endif
+  register uint32_t l;
+  register char *srcend= src + srclen;
+  register unsigned char *map= cs->to_upper;
+
+  assert(src == dst && srclen == dstlen);
+  while (src < srcend)
+  {
+    if ((l=my_ismbchar(cs, src, srcend)))
+      src+= l;
+    else
+    {
+      *src=(char) map[(unsigned char) *src];
+      src++;
+    }
+  }
+  return srclen;
+}
+
+
+size_t my_casedn_mb(const CHARSET_INFO * const  cs, char *src, size_t srclen,
+                    char *dst, size_t dstlen)
+{
+#ifdef NDEBUG
+  (void)dst;
+  (void)dstlen;
+#endif
+  register uint32_t l;
+  register char *srcend= src + srclen;
+  register unsigned char *map=cs->to_lower;
+
+  assert(src == dst && srclen == dstlen);
+  while (src < srcend)
+  {
+    if ((l= my_ismbchar(cs, src, srcend)))
+      src+= l;
+    else
+    {
+      *src= (char) map[(unsigned char)*src];
+      src++;
+    }
+  }
+  return srclen;
+}
+
 
 /*
   my_strcasecmp_mb() returns 0 if strings are equal, non-zero otherwise.
  */
 
-int my_strcasecmp_mb(const charset_info_st * const  cs,const char *s, const char *t)
+int my_strcasecmp_mb(const CHARSET_INFO * const  cs,const char *s, const char *t)
 {
-  uint32_t l;
-  unsigned char *map=cs->to_upper;
+  register uint32_t l;
+  register unsigned char *map=cs->to_upper;
 
   while (*s && *t)
   {
@@ -51,6 +147,7 @@ int my_strcasecmp_mb(const charset_info_st * const  cs,const char *s, const char
   return (*t != *s);
 }
 
+
 /*
 ** Compare string against string with wildcard
 **	0 if matched
@@ -58,17 +155,11 @@ int my_strcasecmp_mb(const charset_info_st * const  cs,const char *s, const char
 **	 1 if matched with wildcard
 */
 
-inline static const char* inc_ptr(const charset_info_st *cs, const char *str, const char* str_end)
-{
-  return str + (my_ismbchar(cs, str, str_end) ? my_ismbchar(cs, str, str_end) : 1);
-}
+#define INC_PTR(cs,A,B) A+=(my_ismbchar(cs,A,B) ? my_ismbchar(cs,A,B) : 1)
 
-inline static int likeconv(const charset_info_st *cs, const char c) 
-{
-  return (unsigned char) cs->sort_order[(unsigned char) c];
-}
-    
-int my_wildcmp_mb(const charset_info_st * const cs,
+#define likeconv(s,A) (unsigned char) (s)->sort_order[(unsigned char) (A)]
+
+int my_wildcmp_mb(const CHARSET_INFO * const cs,
 		  const char *str,const char *str_end,
 		  const char *wildstr,const char *wildend,
 		  int escape, int w_one, int w_many)
@@ -91,7 +182,7 @@ int my_wildcmp_mb(const charset_info_st * const cs,
       }
       else
       if (str == str_end || likeconv(cs,*wildstr++) != likeconv(cs,*str++))
-	return 1;				/* No match */
+	return(1);				/* No match */
       if (wildstr == wildend)
 	return (str != str_end);		/* Match if both are at end */
       result=1;					/* Found an anchor char */
@@ -102,7 +193,7 @@ int my_wildcmp_mb(const charset_info_st * const cs,
       {
 	if (str == str_end)			/* Skip one char if possible */
 	  return (result);
-	inc_ptr(cs,str,str_end);
+	INC_PTR(cs,str,str_end);
       } while (++wildstr < wildend && *wildstr == w_one);
       if (wildstr == wildend)
 	break;
@@ -123,13 +214,13 @@ int my_wildcmp_mb(const charset_info_st * const cs,
 	{
 	  if (str == str_end)
 	    return (-1);
-	  inc_ptr(cs,str,str_end);
+	  INC_PTR(cs,str,str_end);
 	  continue;
 	}
 	break;					/* Not a wild character */
       }
       if (wildstr == wildend)
-	return 0;				/* Ok if w_many is last */
+	return(0);				/* Ok if w_many is last */
       if (str == str_end)
 	return -1;
 
@@ -138,7 +229,7 @@ int my_wildcmp_mb(const charset_info_st * const cs,
 
       mb=wildstr;
       mb_len= my_ismbchar(cs, wildstr, wildend);
-      inc_ptr(cs,wildstr,wildend);		/* This is compared trough cmp */
+      INC_PTR(cs,wildstr,wildend);		/* This is compared trough cmp */
       cmp=likeconv(cs,cmp);
       do
       {
@@ -160,7 +251,7 @@ int my_wildcmp_mb(const charset_info_st * const cs,
             str++;
             break;
           }
-          inc_ptr(cs,str, str_end);
+          INC_PTR(cs,str, str_end);
         }
 	{
 	  int tmp=my_wildcmp_mb(cs,str,str_end,wildstr,wildend,escape,w_one,
@@ -176,10 +267,10 @@ int my_wildcmp_mb(const charset_info_st * const cs,
 }
 
 
-size_t my_numchars_mb(const charset_info_st * const cs,
+size_t my_numchars_mb(const CHARSET_INFO * const cs,
 		      const char *pos, const char *end)
 {
-  size_t count= 0;
+  register size_t count= 0;
   while (pos < end)
   {
     uint32_t mb_len;
@@ -190,7 +281,8 @@ size_t my_numchars_mb(const charset_info_st * const cs,
 }
 
 
-size_t my_charpos_mb(const charset_info_st * const cs, const char *pos, const char *end, size_t length)
+size_t my_charpos_mb(const CHARSET_INFO * const cs,
+		     const char *pos, const char *end, size_t length)
 {
   const char *start= pos;
 
@@ -204,17 +296,17 @@ size_t my_charpos_mb(const charset_info_st * const cs, const char *pos, const ch
 }
 
 
-size_t my_well_formed_len_mb(const charset_info_st& cs, str_ref str, size_t pos, int *error)
+size_t my_well_formed_len_mb(const CHARSET_INFO * const cs, const char *b, const char *e,
+                             size_t pos, int *error)
 {
-  const char *b= str.begin();
-  const char *e= str.end();
   const char *b_start= b;
   *error= 0;
   while (pos)
   {
     my_wc_t wc;
-    int mb_len= cs.cset->mb_wc(&cs, &wc, (const unsigned char*) b, (const unsigned char*) e);
-    if (mb_len <= 0)
+    int mb_len;
+
+    if ((mb_len= cs->cset->mb_wc(cs, &wc, (const unsigned char*) b, (const unsigned char*) e)) <= 0)
     {
       *error= b < e ? 1 : 0;
       break;
@@ -226,12 +318,12 @@ size_t my_well_formed_len_mb(const charset_info_st& cs, str_ref str, size_t pos,
 }
 
 
-uint32_t my_instr_mb(const charset_info_st * const cs,
+uint32_t my_instr_mb(const CHARSET_INFO * const cs,
                  const char *b, size_t b_length,
                  const char *s, size_t s_length,
                  my_match_t *match, uint32_t nmatch)
 {
-  const char *end, *b0;
+  register const char *end, *b0;
   int res= 0;
 
   if (s_length <= b_length)
@@ -283,7 +375,7 @@ uint32_t my_instr_mb(const charset_info_st * const cs,
 
 /* BINARY collations handlers for MB charsets */
 
-int my_strnncoll_mb_bin(const charset_info_st * const,
+int my_strnncoll_mb_bin(const CHARSET_INFO * const,
                         const unsigned char *s, size_t slen,
                         const unsigned char *t, size_t tlen,
                         bool t_is_prefix)
@@ -319,7 +411,7 @@ int my_strnncoll_mb_bin(const charset_info_st * const,
     0 if strings are equal
 */
 
-int my_strnncollsp_mb_bin(const charset_info_st * const,
+int my_strnncollsp_mb_bin(const CHARSET_INFO * const,
                           const unsigned char *a, size_t a_length,
                           const unsigned char *b, size_t b_length,
                           bool diff_if_only_endspace_difference)
@@ -398,7 +490,7 @@ int my_strnncollsp_mb_bin(const charset_info_st * const,
   cp932, euckr, gb2312, sjis, eucjpms, ujis.
 */
 size_t
-my_strnxfrm_mb(const charset_info_st * const cs,
+my_strnxfrm_mb(const CHARSET_INFO * const cs,
                unsigned char *dst, size_t dstlen, uint32_t nweights,
                const unsigned char *src, size_t srclen, uint32_t flags)
 {
@@ -473,14 +565,14 @@ pad:
 }
 
 
-int my_strcasecmp_mb_bin(const charset_info_st * const,
+int my_strcasecmp_mb_bin(const CHARSET_INFO * const,
                          const char *s, const char *t)
 {
   return strcmp(s,t);
 }
 
 
-void my_hash_sort_mb_bin(const charset_info_st * const,
+void my_hash_sort_mb_bin(const CHARSET_INFO * const,
                          const unsigned char *key, size_t len,
                          uint32_t *nr1, uint32_t *nr2)
 {
@@ -517,7 +609,7 @@ void my_hash_sort_mb_bin(const charset_info_st * const,
         create a buffer with multibyte representation of the max_sort_char
         character, and copy it into max_str in a loop.
 */
-static void pad_max_char(const charset_info_st * const cs, char *str, char *end)
+static void pad_max_char(const CHARSET_INFO * const cs, char *str, char *end)
 {
   char buf[10];
   char buflen;
@@ -571,7 +663,7 @@ static void pad_max_char(const charset_info_st * const cs, char *str, char *end)
 ** optimized !
 */
 
-bool my_like_range_mb(const charset_info_st * const cs,
+bool my_like_range_mb(const CHARSET_INFO * const cs,
                          const char *ptr,size_t ptr_length,
                          char escape, char w_one, char w_many,
                          size_t res_length,
@@ -703,7 +795,7 @@ fill_max_and_min:
 }
 
 
-int my_wildcmp_mb_bin(const charset_info_st * const cs,
+int my_wildcmp_mb_bin(const CHARSET_INFO * const cs,
                       const char *str,const char *str_end,
                       const char *wildstr,const char *wildend,
                       int escape, int w_one, int w_many)
@@ -726,7 +818,7 @@ int my_wildcmp_mb_bin(const charset_info_st * const cs,
       }
       else
       if (str == str_end || *wildstr++ != *str++)
-	return 1;				/* No match */
+	return(1);				/* No match */
       if (wildstr == wildend)
 	return (str != str_end);		/* Match if both are at end */
       result=1;					/* Found an anchor char */
@@ -737,7 +829,7 @@ int my_wildcmp_mb_bin(const charset_info_st * const cs,
       {
 	if (str == str_end)			/* Skip one char if possible */
 	  return (result);
-	inc_ptr(cs,str,str_end);
+	INC_PTR(cs,str,str_end);
       } while (++wildstr < wildend && *wildstr == w_one);
       if (wildstr == wildend)
 	break;
@@ -758,13 +850,13 @@ int my_wildcmp_mb_bin(const charset_info_st * const cs,
 	{
 	  if (str == str_end)
 	    return (-1);
-	  inc_ptr(cs,str,str_end);
+	  INC_PTR(cs,str,str_end);
 	  continue;
 	}
 	break;					/* Not a wild character */
       }
       if (wildstr == wildend)
-	return 0;				/* Ok if w_many is last */
+	return(0);				/* Ok if w_many is last */
       if (str == str_end)
 	return -1;
 
@@ -773,7 +865,7 @@ int my_wildcmp_mb_bin(const charset_info_st * const cs,
 
       mb=wildstr;
       mb_len= my_ismbchar(cs, wildstr, wildend);
-      inc_ptr(cs,wildstr,wildend);		/* This is compared trough cmp */
+      INC_PTR(cs,wildstr,wildend);		/* This is compared trough cmp */
       do
       {
         for (;;)
@@ -793,7 +885,7 @@ int my_wildcmp_mb_bin(const charset_info_st * const cs,
             str++;
             break;
           }
-          inc_ptr(cs,str, str_end);
+          INC_PTR(cs,str, str_end);
         }
 	{
 	  int tmp=my_wildcmp_mb_bin(cs,str,str_end,wildstr,wildend,escape,w_one,w_many);
@@ -980,12 +1072,7 @@ static char pgFF[256]=
 1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 };
 
-static class {
-public:
-  int page; 
-  char *p;
-} 
-  utr11_data[256]=
+static struct {int page; char *p;} utr11_data[256]=
 {
 {0,NULL},{0,NULL},{0,NULL},{0,NULL},{0,NULL},{0,NULL},{0,NULL},{0,NULL},
 {0,NULL},{0,NULL},{0,NULL},{0,NULL},{0,NULL},{0,NULL},{0,NULL},{0,NULL},
@@ -1022,7 +1109,7 @@ public:
 };
 
 
-size_t my_numcells_mb(const charset_info_st * const cs, const char *b, const char *e)
+size_t my_numcells_mb(const CHARSET_INFO * const cs, const char *b, const char *e)
 {
   my_wc_t wc;
   size_t clen= 0;
@@ -1052,7 +1139,7 @@ size_t my_numcells_mb(const charset_info_st * const cs, const char *b, const cha
 
 
 
-int my_mb_ctype_mb(const charset_info_st * const cs, int *ctype,
+int my_mb_ctype_mb(const CHARSET_INFO * const cs, int *ctype,
                    const unsigned char *s, const unsigned char *e)
 {
   my_wc_t wc;
@@ -1060,8 +1147,26 @@ int my_mb_ctype_mb(const charset_info_st * const cs, int *ctype,
   if (res <= 0 || wc > 0xFFFF)
     *ctype= 0;
   else
-    *ctype= my_uni_ctype[wc>>8].ctype ? my_uni_ctype[wc>>8].ctype[wc&0xFF] : my_uni_ctype[wc>>8].pctype;
+    *ctype= my_uni_ctype[wc>>8].ctype ?
+            my_uni_ctype[wc>>8].ctype[wc&0xFF] :
+            my_uni_ctype[wc>>8].pctype;
   return res;
 }
 
+
+MY_COLLATION_HANDLER my_collation_mb_bin_handler =
+{
+    NULL,              /* init */
+    my_strnncoll_mb_bin,
+    my_strnncollsp_mb_bin,
+    my_strnxfrm_mb,
+    my_strnxfrmlen_simple,
+    my_like_range_mb,
+    my_wildcmp_mb_bin,
+    my_strcasecmp_mb_bin,
+    my_instr_mb,
+    my_hash_sort_mb_bin,
+    my_propagate_simple
+};
+
 } /* namespace drizzled */
