@@ -20,12 +20,15 @@
 #ifndef DRIZZLED_DATA_HOME_H
 #define DRIZZLED_DATA_HOME_H
 
+#include <string>
+
 namespace drizzled
 {
 
-extern uint32_t drizzle_data_home_len;
-extern char *drizzle_data_home,
-             drizzle_real_data_home[], drizzle_unpacked_real_data_home[];
+std::string& getDataHome();
+std::string& getDataHomeCatalog();
+char *getDatadir();
+char **getDatadirPtr();
 
 } /* namespace drizzled */
 
