@@ -35,7 +35,7 @@
 #include <drizzled/sql_base.h>
 #include <drizzled/lock.h>
 #include <drizzled/errmsg_print.h>
-#include <drizzled/replication_services.h>
+#include <drizzled/transaction_services.h>
 #include <drizzled/message/schema.pb.h>
 #include "drizzled/sql_table.h"
 #include "drizzled/plugin/storage_engine.h"
@@ -44,6 +44,10 @@
 #include "drizzled/pthread_globals.h"
 #include "drizzled/charset.h"
 
+#include <boost/thread/mutex.hpp>
+
+boost::mutex LOCK_create_db;
+
 #include "drizzled/internal/my_sys.h"
 
 #define MAX_DROP_TABLE_Q_LEN      1024
@@ -53,10 +57,11 @@ using namespace std;
 namespace drizzled
 {
 
-static long mysql_rm_known_files(Session *session,
-                                 const string &db, const char *path,
-                                 plugin::TableNameList &dropped_tables);
-static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name);
+static long drop_tables_via_filenames(Session *session,
+                                 SchemaIdentifier &schema_identifier,
+                                 TableIdentifiers &dropped_tables);
+static void mysql_change_db_impl(Session *session);
+static void mysql_change_db_impl(Session *session, SchemaIdentifier &schema_identifier);
 
 /*
   Create a database
@@ -81,7 +86,7 @@ static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name);
 
 bool mysql_create_db(Session *session, const message::Schema &schema_message, const bool is_if_not_exists)
 {
-  ReplicationServices &replication_services= ReplicationServices::singleton();
+  TransactionServices &transaction_services= TransactionServices::singleton();
   bool error= false;
 
   /*
@@ -105,36 +110,37 @@ bool mysql_create_db(Session *session, const message::Schema &schema_message, co
   assert(schema_message.has_collation());
 
   // @todo push this lock down into the engine
-  pthread_mutex_lock(&LOCK_create_db);
-
-  // Check to see if it exists already.
-  if (plugin::StorageEngine::doesSchemaExist(schema_message.name()))
   {
-    if (not is_if_not_exists)
+    boost::mutex::scoped_lock scopedLock(LOCK_create_db);
+
+    // Check to see if it exists already.  
+    SchemaIdentifier schema_identifier(schema_message.name());
+    if (plugin::StorageEngine::doesSchemaExist(schema_identifier))
     {
-      my_error(ER_DB_CREATE_EXISTS, MYF(0), schema_message.name().c_str());
+      if (not is_if_not_exists)
+      {
+        my_error(ER_DB_CREATE_EXISTS, MYF(0), schema_message.name().c_str());
+        error= true;
+      }
+      else
+      {
+        push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
+                            ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS),
+                            schema_message.name().c_str());
+        session->my_ok();
+      }
+    }
+    else if (not plugin::StorageEngine::createSchema(schema_message)) // Try to create it 
+    {
+      my_error(ER_CANT_CREATE_DB, MYF(0), schema_message.name().c_str(), errno);
       error= true;
     }
-    else
+    else // Created !
     {
-      push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
-                          ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS),
-                          schema_message.name().c_str());
-      session->my_ok();
+      transaction_services.createSchema(session, schema_message);
+      session->my_ok(1);
     }
   }
-  else if (not plugin::StorageEngine::createSchema(schema_message)) // Try to create it 
-  {
-    my_error(ER_CANT_CREATE_DB, MYF(0), schema_message.name().c_str(), errno);
-    error= true;
-  }
-  else // Created !
-  {
-    replication_services.createSchema(session, schema_message);
-    session->my_ok(1);
-  }
-
-  pthread_mutex_unlock(&LOCK_create_db);
   start_waiting_global_read_lock(session);
 
   return error;
@@ -145,7 +151,7 @@ bool mysql_create_db(Session *session, const message::Schema &schema_message, co
 
 bool mysql_alter_db(Session *session, const message::Schema &schema_message)
 {
-  ReplicationServices &replication_services= ReplicationServices::singleton();
+  TransactionServices &transaction_services= TransactionServices::singleton();
 
   /*
     Do not alter database if another thread is holding read lock.
@@ -162,28 +168,30 @@ bool mysql_alter_db(Session *session, const message::Schema &schema_message)
   if ((wait_if_global_read_lock(session, 0, 1)))
     return false;
 
-  pthread_mutex_lock(&LOCK_create_db);
-
-  if (not plugin::StorageEngine::doesSchemaExist(schema_message.name()))
+  bool success;
   {
-    my_error(ER_SCHEMA_DOES_NOT_EXIST, MYF(0), schema_message.name().c_str());
-    return false;
-  }
+    boost::mutex::scoped_lock scopedLock(LOCK_create_db);
 
-  /* Change options if current database is being altered. */
-  bool success= plugin::StorageEngine::alterSchema(schema_message);
+    SchemaIdentifier schema_idenifier(schema_message.name());
+    if (not plugin::StorageEngine::doesSchemaExist(schema_idenifier))
+    {
+      my_error(ER_SCHEMA_DOES_NOT_EXIST, MYF(0), schema_message.name().c_str());
+      return false;
+    }
 
-  if (success)
-  {
-    replication_services.rawStatement(session, session->getQueryString());
-    session->my_ok(1);
-  }
-  else
-  {
-    my_error(ER_ALTER_SCHEMA, MYF(0), schema_message.name().c_str());
-  }
+    /* Change options if current database is being altered. */
+    success= plugin::StorageEngine::alterSchema(schema_message);
 
-  pthread_mutex_unlock(&LOCK_create_db);
+    if (success)
+    {
+      transaction_services.rawStatement(session, session->getQueryString());
+      session->my_ok(1);
+    }
+    else
+    {
+      my_error(ER_ALTER_SCHEMA, MYF(0), schema_message.name().c_str());
+    }
+  }
   start_waiting_global_read_lock(session);
 
   return success;
@@ -207,13 +215,11 @@ bool mysql_alter_db(Session *session, const message::Schema &schema_message)
     ERROR Error
 */
 
-bool mysql_rm_db(Session *session, const std::string &schema_name, const bool if_exists)
+bool mysql_rm_db(Session *session, SchemaIdentifier &schema_identifier, const bool if_exists)
 {
   long deleted=0;
   int error= false;
-  char	path[FN_REFLEN+16];
-  uint32_t length;
-  plugin::TableNameList dropped_tables;
+  TableIdentifiers dropped_tables;
   message::Schema schema_proto;
 
   /*
@@ -233,105 +239,110 @@ bool mysql_rm_db(Session *session, const std::string &schema_name, const bool if
     return -1;
   }
 
-  pthread_mutex_lock(&LOCK_create_db);
+  // Lets delete the temporary tables first outside of locks.  
+  set<string> set_of_names;
+  session->doGetTableNames(schema_identifier, set_of_names);
 
-
-  length= build_table_filename(path, sizeof(path),
-                               schema_name.c_str(), "", false);
-  path[length]= '\0';				// Remove file name
-
-  /* See if the schema exists */
-  if (not plugin::StorageEngine::doesSchemaExist(schema_name))
+  for (set<string>::iterator iter= set_of_names.begin(); iter != set_of_names.end(); iter++)
   {
-    if (if_exists)
+    TableIdentifier identifier(schema_identifier, *iter, message::Table::TEMPORARY);
+    Table *table= session->find_temporary_table(identifier);
+    session->close_temporary_table(table);
+  }
+
+  {
+    boost::mutex::scoped_lock scopedLock(LOCK_create_db);
+
+    /* See if the schema exists */
+    if (not plugin::StorageEngine::doesSchemaExist(schema_identifier))
     {
-      push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
-			  ER_DB_DROP_EXISTS, ER(ER_DB_DROP_EXISTS),
-                          path);
+      if (if_exists)
+      {
+        push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
+                            ER_DB_DROP_EXISTS, ER(ER_DB_DROP_EXISTS),
+                            schema_identifier.getSQLPath().c_str());
+      }
+      else
+      {
+        error= -1;
+        my_error(ER_DB_DROP_EXISTS, MYF(0), schema_identifier.getSQLPath().c_str());
+        goto exit;
+      }
     }
     else
     {
+      LOCK_open.lock(); /* After deleting database, remove all cache entries related to schema */
+      remove_db_from_cache(schema_identifier);
+      LOCK_open.unlock();
+
+
       error= -1;
-      my_error(ER_DB_DROP_EXISTS, MYF(0), path);
-      goto exit;
+      deleted= drop_tables_via_filenames(session, schema_identifier, dropped_tables);
+      if (deleted >= 0)
+      {
+        error= 0;
+      }
     }
-  }
-  else
-  {
-    pthread_mutex_lock(&LOCK_open); /* After deleting database, remove all cache entries related to schema */
-    remove_db_from_cache(schema_name);
-    pthread_mutex_unlock(&LOCK_open);
-
-
-    error= -1;
-    deleted= mysql_rm_known_files(session, schema_name,
-                                  path, dropped_tables);
     if (deleted >= 0)
     {
-      error= 0;
+      assert(! session->query.empty());
+
+      TransactionServices &transaction_services= TransactionServices::singleton();
+      transaction_services.dropSchema(session, schema_identifier.getSchemaName());
+      session->clear_error();
+      session->server_status|= SERVER_STATUS_DB_DROPPED;
+      session->my_ok((uint32_t) deleted);
+      session->server_status&= ~SERVER_STATUS_DB_DROPPED;
     }
-  }
-  if (deleted >= 0)
-  {
-    assert(! session->query.empty());
-
-    ReplicationServices &replication_services= ReplicationServices::singleton();
-    replication_services.dropSchema(session, schema_name);
-    session->clear_error();
-    session->server_status|= SERVER_STATUS_DB_DROPPED;
-    session->my_ok((uint32_t) deleted);
-    session->server_status&= ~SERVER_STATUS_DB_DROPPED;
-  }
-  else
-  {
-    char *query, *query_pos, *query_end, *query_data_start;
-    uint32_t db_len;
-
-    if (!(query= (char*) session->alloc(MAX_DROP_TABLE_Q_LEN)))
-      goto exit; /* not much else we can do */
-    query_pos= query_data_start= strcpy(query,"drop table ")+11;
-    query_end= query + MAX_DROP_TABLE_Q_LEN;
-    db_len= schema_name.length();
-
-    ReplicationServices &replication_services= ReplicationServices::singleton();
-    for (plugin::TableNameList::iterator it= dropped_tables.begin();
-         it != dropped_tables.end();
-         it++)
+    else
     {
-      uint32_t tbl_name_len;
+      char *query, *query_pos, *query_end, *query_data_start;
 
-      /* 3 for the quotes and the comma*/
-      tbl_name_len= (*it).length() + 3;
-      if (query_pos + tbl_name_len + 1 >= query_end)
+      if (!(query= (char*) session->alloc(MAX_DROP_TABLE_Q_LEN)))
+        goto exit; /* not much else we can do */
+      query_pos= query_data_start= strcpy(query,"drop table ")+11;
+      query_end= query + MAX_DROP_TABLE_Q_LEN;
+
+      TransactionServices &transaction_services= TransactionServices::singleton();
+      for (TableIdentifiers::iterator it= dropped_tables.begin();
+           it != dropped_tables.end();
+           it++)
       {
-        /* These DDL methods and logging protected with LOCK_create_db */
-        replication_services.rawStatement(session, query);
-        query_pos= query_data_start;
+        uint32_t tbl_name_len;
+
+        /* 3 for the quotes and the comma*/
+        tbl_name_len= (*it).getTableName().length() + 3;
+        if (query_pos + tbl_name_len + 1 >= query_end)
+        {
+          /* These DDL methods and logging protected with LOCK_create_db */
+          transaction_services.rawStatement(session, query);
+          query_pos= query_data_start;
+        }
+
+        *query_pos++ = '`';
+        query_pos= strcpy(query_pos, (*it).getTableName().c_str()) + (tbl_name_len-3);
+        *query_pos++ = '`';
+        *query_pos++ = ',';
       }
 
-      *query_pos++ = '`';
-      query_pos= strcpy(query_pos, (*it).c_str()) + (tbl_name_len-3);
-      *query_pos++ = '`';
-      *query_pos++ = ',';
+      if (query_pos != query_data_start)
+      {
+        /* These DDL methods and logging protected with LOCK_create_db */
+        transaction_services.rawStatement(session, query);
+      }
     }
 
-    if (query_pos != query_data_start)
-    {
-      /* These DDL methods and logging protected with LOCK_create_db */
-      replication_services.rawStatement(session, query);
-    }
-  }
-
 exit:
-  /*
-    If this database was the client's selected database, we silently
-    change the client's selected database to nothing (to have an empty
-    SELECT DATABASE() in the future). For this we free() session->db and set
-    it to 0.
-  */
-  if (not session->db.empty() && session->db.compare(schema_name) == 0)
-    mysql_change_db_impl(session, NULL);
-  pthread_mutex_unlock(&LOCK_create_db);
+    /*
+      If this database was the client's selected database, we silently
+      change the client's selected database to nothing (to have an empty
+      SELECT DATABASE() in the future). For this we free() session->db and set
+      it to 0.
+    */
+    if (schema_identifier.compare(session->db))
+      mysql_change_db_impl(session);
+  }
+
   start_waiting_global_read_lock(session);
 
   return error;
@@ -340,12 +351,14 @@ exit:
 
 static int rm_table_part2(Session *session, TableList *tables)
 {
+  TransactionServices &transaction_services= TransactionServices::singleton();
+
   TableList *table;
   String wrong_tables;
   int error= 0;
   bool foreign_key_error= false;
 
-  pthread_mutex_lock(&LOCK_open); /* Part 2 of rm a table */
+  LOCK_open.lock(); /* Part 2 of rm a table */
 
   /*
     If we have the table in the definition cache, we don't have to check the
@@ -355,15 +368,18 @@ static int rm_table_part2(Session *session, TableList *tables)
 
   for (table= tables; table; table= table->next_local)
   {
+    TableIdentifier identifier(table->db, table->table_name);
     TableShare *share;
-    table->db_type= NULL;
-    if ((share= TableShare::getShare(table->db, table->table_name)))
-      table->db_type= share->db_type();
+    table->setDbType(NULL);
+    if ((share= TableShare::getShare(identifier)))
+    {
+      table->setDbType(share->db_type());
+    }
   }
 
   if (lock_table_names_exclusively(session, tables))
   {
-    pthread_mutex_unlock(&LOCK_open);
+    LOCK_open.unlock();
     return 1;
   }
 
@@ -383,35 +399,44 @@ static int rm_table_part2(Session *session, TableList *tables)
       continue;
     case -1:
       error= 1;
-      goto err_with_placeholders;
+      unlock_table_names(tables, NULL);
+      LOCK_open.unlock();
+      session->no_warnings_for_error= 0;
+
+      return(error);
     default:
       // temporary table not found
       error= 0;
     }
 
-    table_type= table->db_type;
+    table_type= table->getDbType();
+
+    TableIdentifier identifier(db, table->table_name);
 
     {
       Table *locked_table;
-      abort_locked_tables(session, db, table->table_name);
-      remove_table_from_cache(session, db, table->table_name,
+      abort_locked_tables(session, identifier);
+      remove_table_from_cache(session, identifier,
                               RTFC_WAIT_OTHER_THREAD_FLAG |
                               RTFC_CHECK_KILLED_FLAG);
       /*
         If the table was used in lock tables, remember it so that
         unlock_table_names can free it
       */
-      if ((locked_table= drop_locked_tables(session, db, table->table_name)))
+      if ((locked_table= drop_locked_tables(session, identifier)))
         table->table= locked_table;
 
       if (session->killed)
       {
         error= -1;
-        goto err_with_placeholders;
+        unlock_table_names(tables, NULL);
+        LOCK_open.unlock();
+        session->no_warnings_for_error= 0;
+
+        return(error);
       }
     }
-
-    TableIdentifier identifier(db, table->table_name);
+    identifier.getPath();
 
     if (table_type == NULL && not plugin::StorageEngine::doesTableExist(*session, identifier))
     {
@@ -438,7 +463,9 @@ static int rm_table_part2(Session *session, TableList *tables)
     }
 
     if (error == 0 || (foreign_key_error == false))
-        write_bin_log_drop_table(session, true, db, table->table_name);
+    {
+      transaction_services.dropTable(session, string(db), string(table->table_name), true);
+    }
 
     if (error)
     {
@@ -451,7 +478,7 @@ static int rm_table_part2(Session *session, TableList *tables)
     It's safe to unlock LOCK_open: we have an exclusive lock
     on the table name.
   */
-  pthread_mutex_unlock(&LOCK_open);
+  LOCK_open.unlock();
   error= 0;
   if (wrong_tables.length())
   {
@@ -465,10 +492,9 @@ static int rm_table_part2(Session *session, TableList *tables)
     error= 1;
   }
 
-  pthread_mutex_lock(&LOCK_open); /* final bit in rm table lock */
-err_with_placeholders:
+  LOCK_open.lock(); /* final bit in rm table lock */
   unlock_table_names(tables, NULL);
-  pthread_mutex_unlock(&LOCK_open);
+  LOCK_open.unlock();
   session->no_warnings_for_error= 0;
 
   return(error);
@@ -479,45 +505,39 @@ err_with_placeholders:
   session MUST be set when calling this function!
 */
 
-static long mysql_rm_known_files(Session *session,
-                                 const string &db,
-				 const char *org_path,
-                                 plugin::TableNameList &dropped_tables)
+static long drop_tables_via_filenames(Session *session,
+                                      SchemaIdentifier &schema_identifier,
+                                      TableIdentifiers &dropped_tables)
 {
-  CachedDirectory dirp(org_path);
-  if (dirp.fail())
-    return 0;
-
   long deleted= 0;
   TableList *tot_list= NULL, **tot_list_next;
 
   tot_list_next= &tot_list;
 
-  plugin::StorageEngine::getTableNames(db, dropped_tables);
+  plugin::StorageEngine::getIdentifiers(*session, schema_identifier, dropped_tables);
 
-  for (plugin::TableNameList::iterator it= dropped_tables.begin();
+  for (TableIdentifiers::iterator it= dropped_tables.begin();
        it != dropped_tables.end();
        it++)
   {
-    size_t db_len= db.size();
+    size_t db_len= schema_identifier.getSchemaName().size();
 
     /* Drop the table nicely */
     TableList *table_list=(TableList*)
       session->calloc(sizeof(*table_list) +
                       db_len + 1 +
-                      (*it).length() + 1);
+                      (*it).getTableName().length() + 1);
 
     if (not table_list)
       return -1;
 
     table_list->db= (char*) (table_list+1);
-    table_list->table_name= strcpy(table_list->db, db.c_str()) + db_len + 1;
-    filename_to_tablename((*it).c_str(), table_list->table_name,
-                          (*it).size() + 1);
+    table_list->table_name= strcpy(table_list->db, schema_identifier.getSchemaName().c_str()) + db_len + 1;
+    TableIdentifier::filename_to_tablename((*it).getTableName().c_str(), table_list->table_name, (*it).getTableName().size() + 1);
     table_list->alias= table_list->table_name;  // If lower_case_table_names=2
-    table_list->internal_tmp_table= (strncmp((*it).c_str(),
+    table_list->setInternalTmpTable((strncmp((*it).getTableName().c_str(),
                                              TMP_FILE_PREFIX,
-                                             strlen(TMP_FILE_PREFIX)) == 0);
+                                             strlen(TMP_FILE_PREFIX)) == 0));
     /* Link into list */
     (*tot_list_next)= table_list;
     tot_list_next= &table_list->next_local;
@@ -532,9 +552,10 @@ static long mysql_rm_known_files(Session *session,
       return -1;
   }
 
-  if (not plugin::StorageEngine::dropSchema(db))
+
+  if (not plugin::StorageEngine::dropSchema(schema_identifier))
   {
-    my_error(ER_DROP_SCHEMA, MYF(0), db.c_str());
+    my_error(ER_DROP_SCHEMA, MYF(0), schema_identifier.getSQLPath().c_str());
     return -1;
   }
 
@@ -603,66 +624,34 @@ static long mysql_rm_known_files(Session *session,
     @retval true  Error
 */
 
-bool mysql_change_db(Session *session, const std::string &new_db_name)
+bool mysql_change_db(Session *session, SchemaIdentifier &schema_identifier)
 {
 
-  assert(not new_db_name.empty());
-
-  if (not plugin::Authorization::isAuthorized(session->getSecurityContext(),
-                                              new_db_name))
+  if (not plugin::Authorization::isAuthorized(session->getSecurityContext(), schema_identifier))
   {
     /* Error message is set in isAuthorized */
     return true;
   }
 
-
-  /*
-    Now we need to make a copy because check_db_name requires a
-    non-constant argument. Actually, it takes database file name.
-
-    TODO: fix check_db_name().
-  */
-
-  LEX_STRING new_db_file_name;
-  new_db_file_name.length= new_db_name.length();
-  new_db_file_name.str= (char *)malloc(new_db_name.length() + 1);
-  if (new_db_file_name.str == NULL)
-    return true;                             /* the error is set */
-  memcpy(new_db_file_name.str, new_db_name.c_str(), new_db_name.length());
-  new_db_file_name.str[new_db_name.length()]= 0;
-
-
-  /*
-    NOTE: if check_db_name() fails, we should throw an error in any case,
-    even if we are called from sp_head::execute().
-
-    It's next to impossible however to get this error when we are called
-    from sp_head::execute(). But let's switch the current database to NULL
-    in this case to be sure.
-  */
-
-  if (check_db_name(&new_db_file_name))
+  if (not check_db_name(session, schema_identifier))
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), new_db_file_name.str);
-    free(new_db_file_name.str);
+    my_error(ER_WRONG_DB_NAME, MYF(0), schema_identifier.getSQLPath().c_str());
 
     return true;
   }
 
-  if (not plugin::StorageEngine::doesSchemaExist(new_db_file_name.str))
+  if (not plugin::StorageEngine::doesSchemaExist(schema_identifier))
   {
     /* Report an error and free new_db_file_name. */
 
-    my_error(ER_BAD_DB_ERROR, MYF(0), new_db_file_name.str);
-    free(new_db_file_name.str);
+    my_error(ER_BAD_DB_ERROR, MYF(0), schema_identifier.getSQLPath().c_str());
 
     /* The operation failed. */
 
     return true;
   }
 
-  mysql_change_db_impl(session, &new_db_file_name);
-  free(new_db_file_name.str);
+  mysql_change_db_impl(session, schema_identifier);
 
   return false;
 }
@@ -678,10 +667,11 @@ bool mysql_change_db(Session *session, const std::string &new_db_name)
   @param new_db_charset Character set of the new database.
 */
 
-static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name)
+static void mysql_change_db_impl(Session *session, SchemaIdentifier &schema_identifier)
 {
   /* 1. Change current database in Session. */
 
+#if 0
   if (new_db_name == NULL)
   {
     /*
@@ -692,6 +682,7 @@ static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name)
     session->set_db(NULL, 0);
   }
   else
+#endif
   {
     /*
       Here we already have a copy of database name to be used in Session. So,
@@ -699,8 +690,13 @@ static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name)
       the previous database name, we should do it explicitly.
     */
 
-    session->set_db(new_db_name->str, new_db_name->length);
+    session->set_db(schema_identifier.getSchemaName());
   }
 }
 
+static void mysql_change_db_impl(Session *session)
+{
+  session->set_db(string());
+}
+
 } /* namespace drizzled */
