@@ -27,8 +27,8 @@ namespace message { class Schema; }
 
 bool mysql_create_db(Session *session, const message::Schema &schema_message, const bool is_if_not_exists);
 bool mysql_alter_db(Session *session, const message::Schema &schema_message);
-bool mysql_rm_db(Session *session, const std::string &schema_name, const bool if_exists);
-bool mysql_change_db(Session *session, const std::string &new_db_name);
+bool mysql_rm_db(Session *session, SchemaIdentifier &identifier, const bool if_exists);
+bool mysql_change_db(Session *session, SchemaIdentifier &identifier);
 
 } /* namespace drizzled */
 
