@@ -13,11 +13,17 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
+/** @file
+ *
+ * @brief  SQL standard-compliant decimal number handling
+ *
+ * @note
+ * This library implements SQL standard "exact numeric" type
+ * and is not at all generic, but rather intentinally crippled to
+ * follow the standard :) 
+ */
+
 /*
-=======================================================================
-  NOTE: this library implements SQL standard "exact numeric" type
-  and is not at all generic, but rather intentinally crippled to
-  follow the standard :)
 =======================================================================
   Quoting the standard
   (SQL:2003, Part 2 Foundations, aka ISO/IEC 9075-2:2003)
@@ -112,11 +118,229 @@
 #endif
 
 #include <algorithm>
+#include <time.h>
+#include "drizzled/current_session.h"
+#include "drizzled/error.h"
+#include "drizzled/field.h"
+#include "drizzled/internal/my_sys.h"
 
 using namespace std;
 
 namespace drizzled
 {
+/**
+  report result of decimal operation.
+
+  @param result  decimal library return code (E_DEC_* see include/decimal.h)
+
+  @todo
+    Fix error messages
+
+  @return
+    result
+*/
+
+int decimal_operation_results(int result)
+{
+  switch (result) {
+  case E_DEC_OK:
+    break;
+  case E_DEC_TRUNCATED:
+    push_warning_printf(current_session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
+			ER_WARN_DATA_TRUNCATED, ER(ER_WARN_DATA_TRUNCATED),
+			"", (long)-1);
+    break;
+  case E_DEC_OVERFLOW:
+    push_warning_printf(current_session, DRIZZLE_ERROR::WARN_LEVEL_ERROR,
+                        ER_TRUNCATED_WRONG_VALUE,
+                        ER(ER_TRUNCATED_WRONG_VALUE),
+			"DECIMAL", "");
+    break;
+  case E_DEC_DIV_ZERO:
+    push_warning_printf(current_session, DRIZZLE_ERROR::WARN_LEVEL_ERROR,
+			ER_DIVISION_BY_ZERO, ER(ER_DIVISION_BY_ZERO));
+    break;
+  case E_DEC_BAD_NUM:
+    push_warning_printf(current_session, DRIZZLE_ERROR::WARN_LEVEL_ERROR,
+			ER_TRUNCATED_WRONG_VALUE_FOR_FIELD,
+			ER(ER_TRUNCATED_WRONG_VALUE_FOR_FIELD),
+			"decimal", "", "", (long)-1);
+    break;
+  case E_DEC_OOM:
+    my_error(ER_OUT_OF_RESOURCES, MYF(0));
+    break;
+  default:
+    assert(0);
+  }
+  return result;
+}
+
+
+/**
+  @brief Converting decimal to string
+
+  @details Convert given my_decimal to String; allocate buffer as needed.
+
+  @param[in]   mask        what problems to warn on (mask of E_DEC_* values)
+  @param[in]   d           the decimal to print
+  @param[in]   fixed_prec  overall number of digits if ZEROFILL, 0 otherwise
+  @param[in]   fixed_dec   number of decimal places (if fixed_prec != 0)
+  @param[in]   filler      what char to pad with (ZEROFILL et al.)
+  @param[out]  *str        where to store the resulting string
+
+  @return error code
+    @retval E_DEC_OK
+    @retval E_DEC_TRUNCATED
+    @retval E_DEC_OVERFLOW
+    @retval E_DEC_OOM
+*/
+
+int my_decimal2string(uint32_t mask, const my_decimal *d,
+                      uint32_t fixed_prec, uint32_t fixed_dec,
+                      char filler, String *str)
+{
+  /*
+    Calculate the size of the string: For DECIMAL(a,b), fixed_prec==a
+    holds true iff the type is also ZEROFILL, which in turn implies
+    UNSIGNED. Hence the buffer for a ZEROFILLed value is the length
+    the user requested, plus one for a possible decimal point, plus
+    one if the user only wanted decimal places, but we force a leading
+    zero on them. Because the type is implicitly UNSIGNED, we do not
+    need to reserve a character for the sign. For all other cases,
+    fixed_prec will be 0, and my_decimal_string_length() will be called
+    instead to calculate the required size of the buffer.
+  */
+  int length= (fixed_prec
+               ? (fixed_prec + ((fixed_prec == fixed_dec) ? 1 : 0) + 1)
+               : my_decimal_string_length(d));
+  int result;
+  if (str->alloc(length))
+    return check_result(mask, E_DEC_OOM);
+  result= decimal2string((decimal_t*) d, (char*) str->ptr(),
+                         &length, (int)fixed_prec, fixed_dec,
+                         filler);
+  str->length(length);
+  return check_result(mask, result);
+}
+
+
+/**
+  @brief  Convert from decimal to binary representation
+
+  @param[in]   mask        error processing mask
+  @param[in]   d           number for conversion
+  @param[out]  bin         pointer to buffer where to write result
+  @param[in]   prec        overall number of decimal digits
+  @param[in]   scale       number of decimal digits after decimal point
+
+  @note
+    Before conversion we round number if it need but produce truncation
+    error in this case
+
+  @return error code
+   @retval E_DEC_OK
+   @retval E_DEC_TRUNCATED
+   @retval E_DEC_OVERFLOW
+*/
+
+int my_decimal2binary(uint32_t mask, const my_decimal *d, unsigned char *bin, int prec,
+		      int scale)
+{
+  int err1= E_DEC_OK, err2;
+  my_decimal rounded;
+  my_decimal2decimal(d, &rounded);
+  rounded.frac= decimal_actual_fraction(&rounded);
+  if (scale < rounded.frac)
+  {
+    err1= E_DEC_TRUNCATED;
+    /* decimal_round can return only E_DEC_TRUNCATED */
+    decimal_round(&rounded, &rounded, scale, HALF_UP);
+  }
+  err2= decimal2bin(&rounded, bin, prec, scale);
+  if (!err2)
+    err2= err1;
+  return check_result(mask, err2);
+}
+
+
+/**
+  @brief Convert string for decimal when string can be in some multibyte charset
+
+  @param  mask            error processing mask
+  @param  from            string to process
+  @param  length          length of given string
+  @param  charset         charset of given string
+  @param  decimal_value   buffer for result storing
+
+  @return Error code
+   @retval E_DEC_OK
+   @retval E_DEC_TRUNCATED
+   @retval E_DEC_OVERFLOW
+   @retval E_DEC_BAD_NUM
+   @retval E_DEC_OOM
+*/
+
+int str2my_decimal(uint32_t mask, const char *from, uint32_t length,
+                   const CHARSET_INFO * charset, my_decimal *decimal_value)
+{
+  char *end, *from_end;
+  int err;
+  char buff[STRING_BUFFER_USUAL_SIZE];
+  String tmp(buff, sizeof(buff), &my_charset_bin);
+  if (charset->mbminlen > 1)
+  {
+    uint32_t dummy_errors;
+    tmp.copy(from, length, charset, &my_charset_utf8_general_ci, &dummy_errors);
+    from= tmp.ptr();
+    length=  tmp.length();
+    charset= &my_charset_bin;
+  }
+  from_end= end= (char*) from+length;
+  err= string2decimal((char *)from, (decimal_t*) decimal_value, &end);
+  if (end != from_end && !err)
+  {
+    /* Give warning if there is something other than end space */
+    for ( ; end < from_end; end++)
+    {
+      if (!my_isspace(&my_charset_utf8_general_ci, *end))
+      {
+        err= E_DEC_TRUNCATED;
+        break;
+      }
+    }
+  }
+  check_result_and_overflow(mask, err, decimal_value);
+  return err;
+}
+
+
+my_decimal *date2my_decimal(DRIZZLE_TIME *ltime, my_decimal *dec)
+{
+  int64_t date;
+  date = (ltime->year*100L + ltime->month)*100L + ltime->day;
+  if (ltime->time_type > DRIZZLE_TIMESTAMP_DATE)
+    date= ((date*100L + ltime->hour)*100L+ ltime->minute)*100L + ltime->second;
+  if (int2my_decimal(E_DEC_FATAL_ERROR, date, false, dec))
+    return dec;
+  if (ltime->second_part)
+  {
+    dec->buf[(dec->intg-1) / 9 + 1]= ltime->second_part * 1000;
+    dec->frac= 6;
+  }
+  return dec;
+}
+
+
+void my_decimal_trim(uint32_t *precision, uint32_t *scale)
+{
+  if (!(*precision) && !(*scale))
+  {
+    *precision= 10;
+    *scale= 0;
+    return;
+  }
+}
+
 
 /*
   Internally decimal numbers are stored base 10^9 (see DIG_BASE below)
@@ -236,13 +460,12 @@ static const dec1 frac_max[DIG_PER_DEC1-1]={
   } while (0)
 
 
-/*
-  Get maximum value for given precision and scale
 
-  SYNOPSIS
-    max_decimal()
-    precision/scale - see decimal_bin_size() below
-    to              - decimal where where the result will be stored
+/**
+  @brief  Get maximum value for given precision and scale
+
+  @param  precision/scale  see decimal_bin_size() below
+  @param  to              decimal where where the result will be stored
                       to->buf and to->len must be set.
 */
 
@@ -296,12 +519,10 @@ static dec1 *remove_leading_zeroes(const decimal_t *from, int *intg_result)
 }
 
 
-/*
-  Count actual length of fraction part (without ending zeroes)
+/**
+ @brief Count actual length of fraction part (without ending zeroes)
 
-  SYNOPSIS
-    decimal_actual_fraction()
-    from    number for processing
+ @param from    number for processing
 */
 
 int decimal_actual_fraction(decimal_t *from)
@@ -327,28 +548,27 @@ int decimal_actual_fraction(decimal_t *from)
 }
 
 
-/*
-  Convert decimal to its printable string representation
+/**
+ @brief  Convert decimal to its printable string representation
 
-  SYNOPSIS
-    decimal2string()
-      from            - value to convert
-      to              - points to buffer where string representation
-                        should be stored
-      *to_len         - in:  size of to buffer
-                        out: length of the actually written string
-      fixed_precision - 0 if representation can be variable length and
+ @param  from       value to convert
+ @param  to         points to buffer where string representation
+                    should be stored
+ @param  to_len     in:  size of to buffer
+                    out: length of the actually written string
+ @param  fixed_precision 0 if representation can be variable length and
                         fixed_decimals will not be checked in this case.
                         Put number as with fixed point position with this
                         number of digits (sign counted and decimal point is
                         counted)
-      fixed_decimals  - number digits after point.
-      filler          - character to fill gaps in case of fixed_precision > 0
+ @param  fixed_decimals  number digits after point.
+ @param  filler          character to fill gaps in case of fixed_precision > 0
 
-  RETURN VALUE
-    E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW
+ @return error code
+   @retval E_DEC_OK
+   @retval E_DEC_TRUNCATED
+   @retval E_DEC_OVERFLOW
 */
-
 int decimal2string(const decimal_t *from, char *to, int *to_len,
                    int fixed_precision, int fixed_decimals,
                    char filler)
@@ -455,18 +675,15 @@ int decimal2string(const decimal_t *from, char *to, int *to_len,
 }
 
 
-/*
-  Return bounds of decimal digits in the number
+/**
+ @brief  Return bounds of decimal digits in the number
 
-  SYNOPSIS
-    digits_bounds()
-      from         - decimal number for processing
-      start_result - index (from 0 ) of first decimal digits will
-                     be written by this address
-      end_result   - index of position just after last decimal digit
+ @param  from  decimal number for processing
+ @param  start_result  index (from 0 ) of first decimal digits will
+                       be written by this address
+ @param  end_result   index of position just after last decimal digit
                      be written by this address
 */
-
 static void digits_bounds(decimal_t *from, int *start_result, int *end_result)
 {
   int start, stop, i;
@@ -520,20 +737,21 @@ static void digits_bounds(decimal_t *from, int *start_result, int *end_result)
 }
 
 
-/*
-  Left shift for alignment of data in buffer
+/**
+ @param Left shift for alignment of data in buffer
 
-  SYNOPSIS
-    do_mini_left_shift()
-    dec     pointer to decimal number which have to be shifted
-    shift   number of decimal digits on which it should be shifted
-    beg/end bounds of decimal digits (see digits_bounds())
+ @param  dec     pointer to decimal number which have to be shifted
+ @param  shift   number of decimal digits on which it should be shifted
+ @param  beg     beginning of decimal digits (see digits_bounds())
+ @param  end     end of decimal digits (see digits_bounds())
 
-  NOTE
-    Result fitting in the buffer should be garanted.
-    'shift' have to be from 1 to DIG_PER_DEC1-1 (inclusive)
+ @note
+   Result fitting in the buffer should be garanted.
+   'shift' have to be from 1 to DIG_PER_DEC1-1 (inclusive)
+   
+ @todo  Above note is unclear - is 'garanted' a typo for 'guaranteed'
+ or 'granted'?
 */
-
 static void do_mini_left_shift(decimal_t *dec, int shift, int beg, int last)
 {
   dec1 *from= dec->buf + ROUND_UP(beg + 1) - 1;
@@ -550,20 +768,18 @@ static void do_mini_left_shift(decimal_t *dec, int shift, int beg, int last)
 }
 
 
-/*
-  Right shift for alignment of data in buffer
+/**
+  @brief Right shift for alignment of data in buffer
 
-  SYNOPSIS
-    do_mini_left_shift()
-    dec     pointer to decimal number which have to be shifted
-    shift   number of decimal digits on which it should be shifted
-    beg/end bounds of decimal digits (see digits_bounds())
+  @param  dec     pointer to decimal number which have to be shifted
+  @param  shift   number of decimal digits on which it should be shifted
+  @param  beg     beginning of decimal digits (see digits_bounds())
+  @param  end     end of decimal digits (see digits_bounds())
 
-  NOTE
+  @note
     Result fitting in the buffer should be garanted.
     'shift' have to be from 1 to DIG_PER_DEC1-1 (inclusive)
 */
-
 static void do_mini_right_shift(decimal_t *dec, int shift, int beg, int last)
 {
   dec1 *from= dec->buf + ROUND_UP(last) - 1;
@@ -580,23 +796,22 @@ static void do_mini_right_shift(decimal_t *dec, int shift, int beg, int last)
 }
 
 
-/*
-  Shift of decimal digits in given number (with rounding if it need)
+/**
+  @brief  Shift of decimal digits in given number (with rounding if it need)
 
-  SYNOPSIS
-    decimal_shift()
-    dec       number to be shifted
-    shift     number of decimal positions
+  @param  dec       number to be shifted
+  @param  shift     number of decimal positions
               shift > 0 means shift to left shift
               shift < 0 meand right shift
-  NOTE
-    In fact it is multipling on 10^shift.
-  RETURN
-    E_DEC_OK          OK
-    E_DEC_OVERFLOW    operation lead to overflow, number is untoched
-    E_DEC_TRUNCATED   number was rounded to fit into buffer
-*/
 
+  @note
+    In fact it is multipling on 10^shift.
+
+  @return  Error code
+   @retval E_DEC_OK          OK
+   @retval E_DEC_OVERFLOW    operation lead to overflow, number is untoched
+   @retval E_DEC_TRUNCATED   number was rounded to fit into buffer
+*/
 static int decimal_shift(decimal_t *dec, int shift)
 {
   /* index of first non zero digit (all indexes from 0) */
@@ -784,28 +999,25 @@ static int decimal_shift(decimal_t *dec, int shift)
 }
 
 
-/*
-  Convert string to decimal
+/**
+  @brief  Convert string to decimal
 
-  SYNOPSIS
-    internal_str2decl()
-      from    - value to convert. Doesn't have to be \0 terminated!
-      to      - decimal where where the result will be stored
+  @param  from    value to convert. Doesn't have to be \0 terminated!
+  @param  to      decimal where where the result will be stored
                 to->buf and to->len must be set.
-      end     - Pointer to pointer to end of string. Will on return be
+  @param  end     Pointer to pointer to end of string. Will on return be
 		set to the char after the last used character
-      fixed   - use to->intg, to->frac as limits for input number
+  @param  fixed   use to->intg, to->frac as limits for input number
 
-  NOTE
+  @note
     to->intg and to->frac can be modified even when fixed=1
     (but only decreased, in this case)
 
-  RETURN VALUE
+  @return
     E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW/E_DEC_BAD_NUM/E_DEC_OOM
     In case of E_DEC_FATAL_ERROR *to is set to decimal zero
     (to make error handling easier)
 */
-
 int
 internal_str2dec(char *from, decimal_t *to, char **end, bool fixed)
 {
@@ -955,15 +1167,13 @@ fatal_error:
 }
 
 
-/*
-  Convert decimal to double
+/**
+  @param Convert decimal to double
 
-  SYNOPSIS
-    decimal2double()
-      from    - value to convert
-      to      - result will be stored there
+  @param[in]   from   value to convert
+  @param[out]  to     result will be stored there
 
-  RETURN VALUE
+  @return
     E_DEC_OK/E_DEC_OVERFLOW/E_DEC_TRUNCATED
 */
 
@@ -981,15 +1191,13 @@ int decimal2double(const decimal_t *from, double *to)
   return (rc != E_DEC_OK) ? rc : (error ? E_DEC_OVERFLOW : E_DEC_OK);
 }
 
-/*
-  Convert double to decimal
+/**
+ @param  Convert double to decimal
 
-  SYNOPSIS
-    double2decimal()
-      from    - value to convert
-      to      - result will be stored there
+ @param[in]  from    value to convert
+ @param[out] to      result will be stored there
 
-  RETURN VALUE
+ @return
     E_DEC_OK/E_DEC_OVERFLOW/E_DEC_TRUNCATED
 */
 
@@ -1113,24 +1321,11 @@ int decimal2int64_t(const decimal_t *from, int64_t *to)
   return E_DEC_OK;
 }
 
-/*
-  Convert decimal to its binary fixed-length representation
-  two representations of the same length can be compared with memcmp
-  with the correct -1/0/+1 result
+/**
+ @brief
+  Convert decimal to its binary fixed-length representation (suitable for
+  comparing with memcmp)
 
-  SYNOPSIS
-    decimal2bin()
-      from    - value to convert
-      to      - points to buffer where string representation should be stored
-      precision/scale - see decimal_bin_size() below
-
-  NOTE
-    the buffer is assumed to be of the size decimal_bin_size(precision, scale)
-
-  RETURN VALUE
-    E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW
-
-  DESCRIPTION
     for storage decimal numbers are converted to the "binary" format.
 
     This format has the following properties:
@@ -1191,6 +1386,19 @@ int decimal2int64_t(const decimal_t *from, int64_t *to)
     And for -1234567890.1234 it would be
 
                 7E F2 04 37 2D FB 2D
+
+
+  @param from      value to convert
+  @param to        points to buffer where string representation should be stored
+  @param precision see decimal_bin_size() below
+  @param frac      see decimal_bin_size() below
+
+  @note
+    The buffer is assumed to be of the size decimal_bin_size(precision, scale)
+
+  @return
+    E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW
+
 */
 int decimal2bin(const decimal_t *from, unsigned char *to, int precision, int frac)
 {
@@ -1310,23 +1518,21 @@ int decimal2bin(const decimal_t *from, unsigned char *to, int precision, int fra
   return error;
 }
 
-/*
-  Restores decimal from its binary fixed-length representation
+/**
+ @brief Restores decimal from its binary fixed-length representation
 
-  SYNOPSIS
-    bin2decimal()
-      from    - value to convert
-      to      - result
-      precision/scale - see decimal_bin_size() below
+ @param  from    value to convert
+ @param  to      result
+ @param  precision see decimal_bin_size() below
+ @param  scale     see decimal_bin_size() below
 
-  NOTE
+ @note
     see decimal2bin()
     the buffer is assumed to be of the size decimal_bin_size(precision, scale)
 
-  RETURN VALUE
+ @return
     E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW
 */
-
 int bin2decimal(const unsigned char *from, decimal_t *to, int precision, int scale)
 {
   int error=E_DEC_OK, intg=precision-scale,
@@ -1429,13 +1635,11 @@ err:
   return(E_DEC_BAD_NUM);
 }
 
-/*
-  Returns the size of array to hold a binary representation of a decimal
+/**
+ @brief  Returns the size of array to hold a binary representation of a decimal
 
-  RETURN VALUE
-    size in bytes
+ @return  Size in bytes
 */
-
 int decimal_bin_size(int precision, int scale)
 {
   int intg=precision-scale,
@@ -1447,24 +1651,21 @@ int decimal_bin_size(int precision, int scale)
          frac0*sizeof(dec1)+dig2bytes[frac0x];
 }
 
-/*
-  Rounds the decimal to "scale" digits
+/**
+ @brief  Rounds the decimal to "scale" digits
 
-  SYNOPSIS
-    decimal_round()
-      from    - decimal to round,
-      to      - result buffer. from==to is allowed
-      scale   - to what position to round. can be negative!
-      mode    - round to nearest even or truncate
+ @param from    - decimal to round,
+ @param to      - result buffer. from==to is allowed
+ @param scale   - to what position to round. can be negative!
+ @param mode    - round to nearest even or truncate
 
-  NOTES
+ @note
     scale can be negative !
     one TRUNCATED error (line XXX below) isn't treated very logical :(
 
-  RETURN VALUE
+ @return
     E_DEC_OK/E_DEC_TRUNCATED
 */
-
 int
 decimal_round(const decimal_t *from, decimal_t *to, int scale,
               decimal_round_mode mode)
@@ -1574,7 +1775,7 @@ decimal_round(const decimal_t *from, decimal_t *to, int scale,
   }
   else
   {
-    /* TODO - fix this code as it won't work for CEILING mode */
+  /** @todo fix this code as it won't work for CEILING mode */
     int pos=frac0*DIG_PER_DEC1-scale-1;
     assert(frac0+intg0 > 0);
     x=*buf1 / powers10[pos];
@@ -1914,18 +2115,17 @@ int decimal_is_zero(const decimal_t *from)
   return 1;
 }
 
-/*
-  multiply two decimals
+/**
+ @brief multiply two decimals
 
-  SYNOPSIS
-    decimal_mul()
-      from1, from2 - factors
-      to      - product
+ @param[in]   from1  First factor
+ @param[in]   from2  Second factor
+ @param[out]  to     product
 
-  RETURN VALUE
+ @return
     E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW;
 
-  NOTES
+ @note
     in this implementation, with sizeof(dec1)=4 we have DIG_PER_DEC1=9,
     and 63-digit number will take only 7 dec1 words (basically a 7-digit
     "base 999999999" number).  Thus there's no need in fast multiplication
@@ -2041,12 +2241,13 @@ int decimal_mul(const decimal_t *from1, const decimal_t *from2, decimal_t *to)
   return error;
 }
 
-/*
+/**
   naive division algorithm (Knuth's Algorithm D in 4.3.1) -
   it's ok for short numbers
   also we're using alloca() to allocate a temporary buffer
 
-  XXX if this library is to be used with huge numbers of thousands of
+  @todo
+  If this library is to be used with huge numbers of thousands of
   digits, fast division must be implemented and alloca should be
   changed to malloc (or at least fallback to malloc if alloca() fails)
   but then, decimal_mul() should be rewritten too :(
@@ -2293,55 +2494,50 @@ done:
   return error;
 }
 
-/*
-  division of two decimals
+/**
+ @brief  division of two decimals
 
-  SYNOPSIS
-    decimal_div()
-      from1   - dividend
-      from2   - divisor
-      to      - quotient
+ @param[in]  from1   dividend
+ @param[in]  from2   divisor
+ @param[out] to      quotient
 
-  RETURN VALUE
+ @return
     E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW/E_DEC_DIV_ZERO;
 
-  NOTES
+ @note
     see do_div_mod()
 */
-
 int
 decimal_div(const decimal_t *from1, const decimal_t *from2, decimal_t *to, int scale_incr)
 {
   return do_div_mod(from1, from2, to, 0, scale_incr);
 }
 
-/*
-  modulus
+/**
+ @brief modulus
 
-  SYNOPSIS
-    decimal_mod()
-      from1   - dividend
-      from2   - divisor
-      to      - modulus
+ the modulus R in    R = M mod N
 
-  RETURN VALUE
+ is defined as
+
+ 0 <= |R| < |M|
+ sign R == sign M
+ R = M - k*N, where k is integer
+ 
+ thus, there's no requirement for M or N to be integers
+
+
+ @param from1   dividend
+ @param from2   divisor
+ @param to      modulus
+
+ @return
     E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW/E_DEC_DIV_ZERO;
 
-  NOTES
+ @note
     see do_div_mod()
 
-  DESCRIPTION
-    the modulus R in    R = M mod N
-
-   is defined as
-
-     0 <= |R| < |M|
-     sign R == sign M
-     R = M - k*N, where k is integer
-
-   thus, there's no requirement for M or N to be integers
 */
-
 int decimal_mod(const decimal_t *from1, const decimal_t *from2, decimal_t *to)
 {
   return do_div_mod(from1, from2, 0, to, 0);
@@ -2431,7 +2627,7 @@ void test_s2d(const char *s, const char *orig, int ex)
 {
   char s1[100], *end;
   int res;
-  sprintf(s1, "'%s'", s);
+  snprintf(s1, sizeof(s1), "'%s'", s);
   end= strend(s);
   printf("len=%2d %-30s => res=%d    ", a.len, s1,
          (res= string2decimal(s, &a, &end)));
@@ -2445,7 +2641,7 @@ void test_d2f(const char *s, int ex)
   double x;
   int res;
 
-  sprintf(s1, "'%s'", s);
+  snprintf(s1, sizeof(s1), "'%s'", s);
   end= strend(s);
   string2decimal(s, &a, &end);
   res=decimal2double(&a, &x);
@@ -2459,7 +2655,7 @@ void test_d2b2d(const char *str, int p, int s, const char *orig, int ex)
   char s1[100], buf[100], *end;
   int res, i, size=decimal_bin_size(p, s);
 
-  sprintf(s1, "'%s'", str);
+  snprintf(s1, sizeof(s1), "'%s'", str);
   end= strend(str);
   string2decimal(str, &a, &end);
   res=decimal2bin(&a, buf, p, s);
@@ -2554,7 +2750,7 @@ void test_da(const char *s1, const char *s2, const char *orig, int ex)
 {
   char s[100], *end;
   int res;
-  sprintf(s, "'%s' + '%s'", s1, s2);
+  snprintf(s, sizeof(s), "'%s' + '%s'", s1, s2);
   end= strend(s1);
   string2decimal(s1, &a, &end);
   end= strend(s2);
@@ -2569,7 +2765,7 @@ void test_ds(const char *s1, const char *s2, const char *orig, int ex)
 {
   char s[100], *end;
   int res;
-  sprintf(s, "'%s' - '%s'", s1, s2);
+  snprintf(s, sizeof(s), "'%s' - '%s'", s1, s2);
   end= strend(s1);
   string2decimal(s1, &a, &end);
   end= strend(s2);
@@ -2584,7 +2780,7 @@ void test_dc(const char *s1, const char *s2, int orig)
 {
   char s[100], *end;
   int res;
-  sprintf(s, "'%s' <=> '%s'", s1, s2);
+  snprintf(s, sizeof(s), "'%s' <=> '%s'", s1, s2);
   end= strend(s1);
   string2decimal(s1, &a, &end);
   end= strend(s2);
@@ -2602,7 +2798,7 @@ void test_dm(const char *s1, const char *s2, const char *orig, int ex)
 {
   char s[100], *end;
   int res;
-  sprintf(s, "'%s' * '%s'", s1, s2);
+  snprintf(s, sizeof(s), "'%s' * '%s'", s1, s2);
   end= strend(s1);
   string2decimal(s1, &a, &end);
   end= strend(s2);
@@ -2617,7 +2813,7 @@ void test_dv(const char *s1, const char *s2, const char *orig, int ex)
 {
   char s[100], *end;
   int res;
-  sprintf(s, "'%s' / '%s'", s1, s2);
+  snprintf(s, sizeof(s), "'%s' / '%s'", s1, s2);
   end= strend(s1);
   string2decimal(s1, &a, &end);
   end= strend(s2);
@@ -2636,7 +2832,7 @@ void test_md(const char *s1, const char *s2, const char *orig, int ex)
 {
   char s[100], *end;
   int res;
-  sprintf(s, "'%s' %% '%s'", s1, s2);
+  snprintf(s, sizeof(s), "'%s' %% '%s'", s1, s2);
   end= strend(s1);
   string2decimal(s1, &a, &end);
   end= strend(s2);
@@ -2659,7 +2855,7 @@ void test_ro(const char *s1, int n, decimal_round_mode mode, const char *orig,
 {
   char s[100], *end;
   int res;
-  sprintf(s, "'%s', %d, %s", s1, n, round_mode[mode]);
+  snprintf(s, sizeof(s), "'%s', %d, %s", s1, n, round_mode[mode]);
   end= strend(s1);
   string2decimal(s1, &a, &end);
   res=decimal_round(&a, &b, n, mode);
@@ -2672,7 +2868,7 @@ void test_ro(const char *s1, int n, decimal_round_mode mode, const char *orig,
 void test_mx(int precision, int frac, const char *orig)
 {
   char s[100];
-  sprintf(s, "%d, %d", precision, frac);
+  snprintf(s, sizeof(s), "%d, %d", precision, frac);
   max_decimal(precision, frac, &a);
   printf("%-40s =>          ", s);
   print_decimal(&a, orig, 0, 0);
@@ -2688,7 +2884,7 @@ void test_pr(const char *s1, int prec, int dec, char filler, const char *orig,
   int slen= sizeof(s2);
   int res;
 
-  sprintf(s, filler ? "'%s', %d, %d, '%c'" : "'%s', %d, %d, '\\0'",
+  snprintf(s, sizeof(s), filler ? "'%s', %d, %d, '%c'" : "'%s', %d, %d, '\\0'",
           s1, prec, dec, filler);
   end= strend(s1);
   string2decimal(s1, &a, &end);
@@ -2708,7 +2904,7 @@ void test_sh(const char *s1, int shift, const char *orig, int ex)
 {
   char s[100], *end;
   int res;
-  sprintf(s, "'%s' %s %d", s1, ((shift < 0) ? ">>" : "<<"), abs(shift));
+  snprintf(s, sizeof(s), "'%s' %s %d", s1, ((shift < 0) ? ">>" : "<<"), abs(shift));
   end= strend(s1);
   string2decimal(s1, &a, &end);
   res= decimal_shift(&a, shift);
@@ -2721,7 +2917,7 @@ void test_sh(const char *s1, int shift, const char *orig, int ex)
 void test_fr(const char *s1, const char *orig)
 {
   char s[100], *end;
-  sprintf(s, "'%s'", s1);
+  snprintf(s, sizeof(s), "'%s'", s1);
   printf("%-40s =>          ", s);
   end= strend(s1);
   string2decimal(s1, &a, &end);
