@@ -33,6 +33,9 @@
 namespace drizzled
 {
 
+/* Global value for how we extend our temporary directory */
+#define GLOBAL_TEMPORARY_EXT ".temporary"
+
 /* These paths are converted to other systems (WIN95) before use */
 
 #define LANGUAGE	"english/"
@@ -41,6 +44,9 @@ namespace drizzled
 
 #define ER(X) ::drizzled::error_message((X))
 
+/* buffer size for strerror_r() */
+#define STRERROR_MAX 256
+
 /* extra 4+4 bytes for slave tmp tables */
 #define MAX_DBKEY_LENGTH (NAME_LEN*2+1+1+4+4)
 #define MAX_ALIAS_NAME 256
@@ -182,8 +188,6 @@ namespace drizzled
 
 /** Characters shown for the command in 'show processlist'. */
 #define PROCESS_LIST_WIDTH 100
-/* Characters shown for the command in 'information_schema.processlist' */
-#define PROCESS_LIST_INFO_WIDTH 65535
 
 #define PRECISION_FOR_DOUBLE 53
 #define PRECISION_FOR_FLOAT  24
@@ -193,11 +197,6 @@ namespace drizzled
 #define FLUSH_TIME		0		/**< Don't flush tables */
 #define MAX_CONNECT_ERRORS	10		///< errors before disabling host
 
-#define INTERRUPT_PRIOR 10
-#define CONNECT_PRIOR	9
-#define WAIT_PRIOR	8
-#define QUERY_PRIOR	6
-
 /* Bits from testflag */
 enum test_flag_bit
 {
@@ -344,8 +343,6 @@ enum start_transaction_option_t
 #define HA_CREATE_USED_CHARSET          (1L << 8)
 #define HA_CREATE_USED_DEFAULT_CHARSET  (1L << 9)
 #define HA_CREATE_USED_ROW_FORMAT       (1L << 15)
-#define HA_CREATE_USED_KEY_BLOCK_SIZE   (1L << 19)
-#define HA_CREATE_USED_BLOCK_SIZE       (1L << 22)
 
 /*
   The below two are not used (and not handled) in this milestone of this WL
@@ -484,11 +481,6 @@ template <class T> void set_if_smaller(T &a, const T &b)
 static const uint32_t ONCE_ALLOC_INIT= 4096;
 /* Typical record cash */
 static const uint32_t RECORD_CACHE_SIZE= 64*1024;
-/* Typical key cash */
-static const uint32_t KEY_CACHE_SIZE= 8*1024*1024;
-
-/* Default size of a key cache block  */
-static const uint32_t KEY_CACHE_BLOCK_SIZE= 1024;
 
 
 /* Some things that this system doesn't have */
