@@ -26,7 +26,7 @@
 #define DRIZZLED_FIELD_H
 
 #include "drizzled/sql_error.h"
-#include "drizzled/my_decimal.h"
+#include "drizzled/decimal.h"
 #include "drizzled/key_map.h"
 #include "drizzled/sql_bitmap.h"
 #include "drizzled/sql_list.h"
@@ -44,8 +44,8 @@ namespace drizzled
 #define DOUBLE_TO_STRING_CONVERSION_BUFFER_SIZE FLOATING_POINT_BUFFER
 
 #ifdef DEBUG
-#define ASSERT_COLUMN_MARKED_FOR_READ assert(!table || (table->read_set == NULL || isReadSet()))
-#define ASSERT_COLUMN_MARKED_FOR_WRITE assert(!table || (table->write_set == NULL || isWriteSet()))
+#define ASSERT_COLUMN_MARKED_FOR_READ assert(!getTable() || (getTable()->read_set == NULL || isReadSet()))
+#define ASSERT_COLUMN_MARKED_FOR_WRITE assert(!getTable() || (getTable()->write_set == NULL || isWriteSet()))
 #else
 #define ASSERT_COLUMN_MARKED_FOR_READ
 #define ASSERT_COLUMN_MARKED_FOR_WRITE
@@ -59,15 +59,10 @@ class SendField;
 class CreateField;
 class TableShare;
 class Field;
-struct st_cache_field;
+struct CacheField;
 
 int field_conv(Field *to,Field *from);
 
-inline uint32_t get_enum_pack_length(int elements)
-{
-  return elements < 256 ? 1 : 2;
-}
-
 /**
  * Class representing a Field in a Table
  *
@@ -96,9 +91,27 @@ public:
    * @note You can use table->in_use as replacement for current_session member
    * only inside of val_*() and store() members (e.g. you can't use it in cons)
    */
+private:
   Table *table;
+public:
+  Table *getTable()
+  {
+    assert(table);
+    return table;
+  }
+
+  Table *getTable() const
+  {
+    assert(table);
+    return table;
+  }
+
+  void setTable(Table *table_arg)
+  {
+    table= table_arg;
+  }
+
   Table *orig_table; /**< Pointer to the original Table. @TODO What is "the original table"? */
-  const char **table_name; /**< Pointer to the name of the table. @TODO This is redundant with Table::table_name. */
   const char *field_name; /**< Name of the field */
   LEX_STRING comment; /**< A comment about the field */
 
@@ -536,12 +549,12 @@ public:
     return max_length;
   }
 
-  inline uint32_t offset(unsigned char *record)
+  inline uint32_t offset(const unsigned char *record)
   {
     return (uint32_t) (ptr - record);
   }
   void copy_from_tmp(int offset);
-  uint32_t fill_cache_field(struct st_cache_field *copy);
+  uint32_t fill_cache_field(CacheField *copy);
   virtual bool get_date(DRIZZLE_TIME *ltime,uint32_t fuzzydate);
   virtual bool get_time(DRIZZLE_TIME *ltime);
   virtual const CHARSET_INFO *charset(void) const { return &my_charset_bin; }
@@ -771,20 +784,6 @@ public:
   void (*do_copy2)(CopyField *);		// Used to handle null values
 };
 
-Field *make_field(TableShare *share,
-                  memory::Root *root,
-                  unsigned char *ptr,
-                  uint32_t field_length,
-                  bool is_nullable,
-                  unsigned char *null_pos,
-                  unsigned char null_bit,
-                  uint8_t decimals,
-                  enum_field_types field_type,
-                  const CHARSET_INFO * cs,
-                  Field::utype unireg_check,
-                  TYPELIB *interval,
-                  const char *field_name);
-
 uint32_t pack_length_to_packflag(uint32_t type);
 uint32_t calc_pack_length(enum_field_types type,uint32_t length);
 int set_field_to_null(Field *field);
