@@ -58,20 +58,20 @@ public:
   {}
   enum_field_types type() const { return DRIZZLE_TYPE_DATE;}
   enum_field_types real_type() const { return DRIZZLE_TYPE_DATE; }
-  enum ha_base_keytype key_type() const { return HA_KEYTYPE_UINT24; }
+  enum ha_base_keytype key_type() const { return HA_KEYTYPE_ULONG_INT; }
   enum Item_result cmp_type () const { return INT_RESULT; }
   int  store(const char *to,uint32_t length,
              const CHARSET_INFO * const charset);
   int  store(double nr);
   int  store(int64_t nr, bool unsigned_val);
   int store_time(DRIZZLE_TIME *ltime, enum enum_drizzle_timestamp_type type);
-  int reset(void) { ptr[0]=ptr[1]=ptr[2]=0; return 0; }
+  int reset(void) { ptr[0]=ptr[1]=ptr[2]=ptr[3]=0; return 0; }
   double val_real(void);
   int64_t val_int(void);
   String *val_str(String*,String *);
   int cmp(const unsigned char *,const unsigned char *);
   void sort_string(unsigned char *buff,uint32_t length);
-  uint32_t pack_length() const { return 3; }
+  uint32_t pack_length() const { return 4; }
   void sql_type(String &str) const;
   bool can_be_compared_as_int64_t() const { return true; }
   bool zero_pack() const { return 1; }
