@@ -38,7 +38,6 @@ namespace drizzled
 ** datetime type
 ** In string context: YYYY-MM-DD HH:MM:DD
 ** In number context: YYYYMMDDHHMMDD
-** Stored as a 8 byte unsigned int. Should sometimes be change to a 6 byte int.
 ****************************************************************************/
 
 int Field_datetime::store(const char *from,
@@ -61,7 +60,7 @@ int Field_datetime::store(const char *from,
   temporal.to_int64_t(&int_value);
 
 #ifdef WORDS_BIGENDIAN
-  if (table && table->s->db_low_byte_first)
+  if (getTable() && getTable()->s->db_low_byte_first)
   {
     int8store(ptr, int_value);
   }
@@ -116,7 +115,7 @@ int Field_datetime::store(int64_t from, bool)
   temporal.to_int64_t(&int_value);
 
 #ifdef WORDS_BIGENDIAN
-  if (table && table->s->db_low_byte_first)
+  if (getTable() && getTable()->s->db_low_byte_first)
   {
     int8store(ptr, int_value);
   }
@@ -152,7 +151,7 @@ int Field_datetime::store_time(DRIZZLE_TIME *ltime, enum enum_drizzle_timestamp_
   temporal.to_int64_t(&int_value);
 
 #ifdef WORDS_BIGENDIAN
-  if (table && table->s->db_low_byte_first)
+  if (getTable() && getTable()->s->db_low_byte_first)
   {
     int8store(ptr, int_value);
   }
@@ -174,7 +173,7 @@ int64_t Field_datetime::val_int(void)
   ASSERT_COLUMN_MARKED_FOR_READ;
 
 #ifdef WORDS_BIGENDIAN
-  if (table && table->s->db_low_byte_first)
+  if (getTable() && getTable()->s->db_low_byte_first)
     j=sint8korr(ptr);
   else
 #endif
@@ -193,7 +192,7 @@ String *Field_datetime::val_str(String *val_buffer,
   ASSERT_COLUMN_MARKED_FOR_READ;
 
 #ifdef WORDS_BIGENDIAN
-  if (table && table->s->db_low_byte_first)
+  if (getTable() && getTable()->s->db_low_byte_first)
     tmp=sint8korr(ptr);
   else
 #endif
@@ -207,10 +206,10 @@ String *Field_datetime::val_str(String *val_buffer,
    * not null without a default value.
    */
   dt.from_int64_t(tmp, false); /* NOTE: this does *NOT* attempt convertion
-				        from formats such as 20090101 as
-					the stored value has already been
-					converted.
-			       */
+                                 from formats such as 20090101 as
+                                 the stored value has already been
+                                 converted.
+                               */
 
   int rlen;
   rlen= dt.to_string((char*)val_buffer->ptr(), DateTime::MAX_STRING_LENGTH);
@@ -249,7 +248,7 @@ int Field_datetime::cmp(const unsigned char *a_ptr, const unsigned char *b_ptr)
 {
   int64_t a,b;
 #ifdef WORDS_BIGENDIAN
-  if (table && table->s->db_low_byte_first)
+  if (getTable() && getTable()->s->db_low_byte_first)
   {
     a=sint8korr(a_ptr);
     b=sint8korr(b_ptr);
@@ -267,7 +266,7 @@ int Field_datetime::cmp(const unsigned char *a_ptr, const unsigned char *b_ptr)
 void Field_datetime::sort_string(unsigned char *to,uint32_t )
 {
 #ifdef WORDS_BIGENDIAN
-  if (!table || !table->s->db_low_byte_first)
+  if (!getTable() || !getTable()->s->db_low_byte_first)
   {
     to[0] = ptr[0];
     to[1] = ptr[1];
