@@ -26,10 +26,8 @@
 namespace drizzled
 {
 
-class Field_enum :public Field_str 
+class Field_enum :public Field_str
 {
-protected:
-  uint32_t packlength;
 public:
 
   using Field::store;
@@ -37,6 +35,8 @@ public:
   using Field::val_str;
   using Field::cmp;
 
+  static const int max_supported_elements = 0x10000;
+
   /** Internal storage for the string values of the ENUM */
   TYPELIB *typelib;
   Field_enum(unsigned char *ptr_arg,
@@ -44,7 +44,6 @@ public:
              unsigned char *null_ptr_arg,
              unsigned char null_bit_arg,
              const char *field_name_arg,
-             uint32_t packlength_arg,
              TYPELIB *typelib_arg,
              const CHARSET_INFO * const charset_arg)
     :Field_str(ptr_arg,
@@ -53,13 +52,12 @@ public:
                null_bit_arg,
 	             field_name_arg,
                charset_arg),
-    packlength(packlength_arg),
     typelib(typelib_arg)
   {
     flags|= ENUM_FLAG;
   }
   Field *new_field(memory::Root *root, Table *new_table, bool keep_type);
-  enum ha_base_keytype key_type() const;
+  enum ha_base_keytype key_type() const { return HA_KEYTYPE_ULONG_INT; }
   int  store(const char *to, uint32_t length, const CHARSET_INFO * const);
   int  store(double nr);
   int  store(int64_t nr, bool unsigned_val);
@@ -83,10 +81,7 @@ public:
   {
     return INT_RESULT;
   }
-  uint32_t pack_length() const
-  {
-    return (uint32_t) packlength;
-  }
+  uint32_t pack_length() const { return 4; }
   uint32_t size_of() const
   {
     return sizeof(*this);
@@ -95,15 +90,11 @@ public:
   {
     return DRIZZLE_TYPE_ENUM;
   }
-  uint32_t pack_length_from_metadata(uint32_t field_metadata)
-  {
-    return (field_metadata & 0x00ff);
-  }
   virtual bool zero_pack() const
   {
     return false;
   }
-  bool optimize_range(uint32_t, uint32_t) 
+  bool optimize_range(uint32_t, uint32_t)
   {
     return false;
   }
