@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008, 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2008, 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,13 +18,13 @@
  */
 
 
-#include <config.h>
-#include <drizzled/file_exchange.h>
+#include "config.h"
+#include "drizzled/file_exchange.h"
 
 namespace drizzled
 {
 
-file_exchange::file_exchange(const char *name, bool flag, enum_filetype filetype_arg)
+file_exchange::file_exchange(char *name, bool flag, enum_filetype filetype_arg)
   : filetype(filetype_arg),
     file_name(name),
     field_term(&default_field_term),
