@@ -56,7 +56,8 @@ static unsigned char *read_buffpek_from_file(internal::IO_CACHE *buffer_file,
                                              uint32_t count,
                                              unsigned char *buf);
 
-static ha_rows find_all_keys(SORTPARAM *param,
+static ha_rows find_all_keys(Session *session,
+                             SORTPARAM *param,
                              optimizer::SqlSelect *select,
 			     unsigned char * *sort_keys, 
                              internal::IO_CACHE *buffer_file,
@@ -75,7 +76,7 @@ static void make_sortkey(SORTPARAM *param,
 static void register_used_fields(SORTPARAM *param);
 static int merge_index(SORTPARAM *param,
                        unsigned char *sort_buffer,
-		       BUFFPEK *buffpek,
+		       buffpek_st *buffpek,
 		       uint32_t maxbuffer,
                        internal::IO_CACHE *tempfile,
 		       internal::IO_CACHE *outfile);
@@ -85,14 +86,14 @@ static bool save_index(SORTPARAM *param,
                        filesort_info_st *table_sort);
 static uint32_t suffix_length(uint32_t string_length);
 static uint32_t sortlength(Session *session,
-                           SORT_FIELD *sortorder,
+                           SortField *sortorder,
                            uint32_t s_length,
                            bool *multi_byte_charset);
-static SORT_ADDON_FIELD *get_addon_fields(Session *session,
-                                          Field **ptabfield,
-                                          uint32_t sortlength,
-                                          uint32_t *plength);
-static void unpack_addon_fields(struct st_sort_addon_field *addon_field,
+static sort_addon_field_st *get_addon_fields(Session *session,
+                                             Field **ptabfield,
+                                             uint32_t sortlength,
+                                             uint32_t *plength);
+static void unpack_addon_fields(sort_addon_field_st *addon_field,
                                 unsigned char *buff);
 /**
   Sort a table.
@@ -130,14 +131,14 @@ static void unpack_addon_fields(struct st_sort_addon_field *addon_field,
     examined_rows	will be set to number of examined rows
 */
 
-ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t s_length,
+ha_rows filesort(Session *session, Table *table, SortField *sortorder, uint32_t s_length,
 		 optimizer::SqlSelect *select, ha_rows max_rows,
                  bool sort_positions, ha_rows *examined_rows)
 {
   int error;
   uint32_t memavl, min_sort_memory;
   uint32_t maxbuffer;
-  BUFFPEK *buffpek;
+  buffpek_st *buffpek;
   ha_rows records= HA_POS_ERROR;
   unsigned char **sort_keys= 0;
   internal::IO_CACHE tempfile, buffpek_pointers, *selected_records_file, *outfile;
@@ -148,7 +149,7 @@ ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t
   TableList *tab= table->pos_in_table_list;
   Item_subselect *subselect= tab ? tab->containing_subselect() : 0;
 
-  DRIZZLE_FILESORT_START(table->s->db.str, table->s->table_name.str);
+  DRIZZLE_FILESORT_START(table->getShare()->getSchemaName(), table->getShare()->getTableName());
 
   /*
    Release InnoDB's adaptive hash index latch (if holding) before
@@ -180,7 +181,7 @@ ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t
       Get the descriptors of all fields whose values are appended
       to sorted fields and get its total length in param.spack_length.
     */
-    param.addon_field= get_addon_fields(session, table->field,
+    param.addon_field= get_addon_fields(session, table->getFields(),
                                         param.sort_length,
                                         &param.addon_length);
   }
@@ -209,11 +210,11 @@ ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t
 
   if (select && select->quick)
   {
-    status_var_increment(session->status_var.filesort_range_count);
+    session->status_var.filesort_range_count++;
   }
   else
   {
-    status_var_increment(session->status_var.filesort_scan_count);
+    session->status_var.filesort_scan_count++;
   }
 #ifdef CAN_TRUST_RANGE
   if (select && select->quick && select->quick->records > 0L)
@@ -260,14 +261,14 @@ ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t
     my_error(ER_OUT_OF_SORTMEMORY,MYF(ME_ERROR+ME_WAITTANG));
     goto err;
   }
-  if (open_cached_file(&buffpek_pointers,drizzle_tmpdir,TEMP_PREFIX,
+  if (open_cached_file(&buffpek_pointers,drizzle_tmpdir.c_str(),TEMP_PREFIX,
 		       DISK_BUFFER_SIZE, MYF(MY_WME)))
     goto err;
 
   param.keys--;  			/* TODO: check why we do this */
   param.sort_form= table;
   param.end=(param.local_sortorder=sortorder)+s_length;
-  if ((records=find_all_keys(&param,select,sort_keys, &buffpek_pointers,
+  if ((records=find_all_keys(session, &param,select,sort_keys, &buffpek_pointers,
 			     &tempfile, selected_records_file)) ==
       HA_POS_ERROR)
     goto err;
@@ -290,12 +291,12 @@ ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t
           (unsigned char *) read_buffpek_from_file(&buffpek_pointers, maxbuffer,
                                  table_sort.buffpek)))
       goto err;
-    buffpek= (BUFFPEK *) table_sort.buffpek;
+    buffpek= (buffpek_st *) table_sort.buffpek;
     table_sort.buffpek_len= maxbuffer;
     close_cached_file(&buffpek_pointers);
 	/* Open cached file if it isn't open */
     if (! my_b_inited(outfile) &&
-	open_cached_file(outfile,drizzle_tmpdir,TEMP_PREFIX,READ_RECORD_BUFFER,
+	open_cached_file(outfile,drizzle_tmpdir.c_str(),TEMP_PREFIX,READ_RECORD_BUFFER,
 			  MYF(MY_WME)))
       goto err;
     if (reinit_io_cache(outfile,internal::WRITE_CACHE,0L,0,0))
@@ -328,11 +329,9 @@ ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t
       free(param.tmp_buffer);
   if (!subselect || !subselect->is_uncacheable())
   {
-    if ((unsigned char*) sort_keys)
-      free((unsigned char*) sort_keys);
+    free(sort_keys);
     table_sort.sort_keys= 0;
-    if ((unsigned char*) buffpek)
-      free((unsigned char*) buffpek);
+    free(buffpek);
     table_sort.buffpek= 0;
     table_sort.buffpek_len= 0;
   }
@@ -351,11 +350,14 @@ ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t
     }
   }
   if (error)
+  {
     my_message(ER_FILSORT_ABORT, ER(ER_FILSORT_ABORT),
                MYF(ME_ERROR+ME_WAITTANG));
+  }
   else
-    statistic_add(session->status_var.filesort_rows,
-		  (uint32_t) records, &LOCK_status);
+  {
+    session->status_var.filesort_rows+= (uint32_t) records;
+  }
   *examined_rows= param.examined_rows;
   memcpy(&table->sort, &table_sort, sizeof(filesort_info_st));
   DRIZZLE_FILESORT_DONE(error, records);
@@ -419,10 +421,10 @@ static char **make_char_array(char **old_pos, register uint32_t fields,
 static unsigned char *read_buffpek_from_file(internal::IO_CACHE *buffpek_pointers, uint32_t count,
                                      unsigned char *buf)
 {
-  uint32_t length= sizeof(BUFFPEK)*count;
+  uint32_t length= sizeof(buffpek_st)*count;
   unsigned char *tmp= buf;
-  if (count > UINT_MAX/sizeof(BUFFPEK))
-    return 0; /* sizeof(BUFFPEK)*count will overflow */
+  if (count > UINT_MAX/sizeof(buffpek_st))
+    return 0; /* sizeof(buffpek_st)*count will overflow */
   if (!tmp)
     tmp= (unsigned char *)malloc(length);
   if (tmp)
@@ -445,7 +447,7 @@ static unsigned char *read_buffpek_from_file(internal::IO_CACHE *buffpek_pointer
   @param param             Sorting parameter
   @param select            Use this to get source data
   @param sort_keys         Array of pointers to sort key + addon buffers.
-  @param buffpek_pointers  File to write BUFFPEKs describing sorted segments
+  @param buffpek_pointers  File to write buffpek_sts describing sorted segments
                            in tempfile.
   @param tempfile          File to write sorted sequences of sortkeys to.
   @param indexfile         If !NULL, use it for source data (contains rowids)
@@ -459,7 +461,7 @@ static unsigned char *read_buffpek_from_file(internal::IO_CACHE *buffpek_pointer
        {
          sort sort_keys buffer;
          dump sorted sequence to 'tempfile';
-         dump BUFFPEK describing sequence location into 'buffpek_pointers';
+         dump buffpek_st describing sequence location into 'buffpek_pointers';
        }
        put sort key into 'sort_keys';
      }
@@ -475,7 +477,8 @@ static unsigned char *read_buffpek_from_file(internal::IO_CACHE *buffpek_pointer
     HA_POS_ERROR on error.
 */
 
-static ha_rows find_all_keys(SORTPARAM *param, 
+static ha_rows find_all_keys(Session *session,
+                             SORTPARAM *param, 
                              optimizer::SqlSelect *select,
 			     unsigned char **sort_keys,
 			     internal::IO_CACHE *buffpek_pointers,
@@ -486,7 +489,6 @@ static ha_rows find_all_keys(SORTPARAM *param,
   unsigned char *ref_pos,*next_pos,ref_buff[MAX_REFLENGTH];
   internal::my_off_t record;
   Table *sort_form;
-  Session *session= current_session;
   volatile Session::killed_state *killed= &session->killed;
   Cursor *file;
   MyBitmap *save_read_set, *save_write_set;
@@ -507,18 +509,18 @@ static ha_rows find_all_keys(SORTPARAM *param,
   if (! indexfile && ! quick_select)
   {
     next_pos=(unsigned char*) 0;			/* Find records in sequence */
-    file->ha_rnd_init(1);
+    file->startTableScan(1);
     file->extra_opt(HA_EXTRA_CACHE,
-		    current_session->variables.read_buff_size);
+		    session->variables.read_buff_size);
   }
 
-  READ_RECORD read_record_info;
+  ReadRecord read_record_info;
   if (quick_select)
   {
     if (select->quick->reset())
       return(HA_POS_ERROR);
-    init_read_record(&read_record_info, current_session, select->quick->head,
-                     select, 1, 1);
+
+    read_record_info.init_read_record(session, select->quick->head, select, 1, 1);
   }
 
   /* Remember original bitmaps */
@@ -543,7 +545,7 @@ static ha_rows find_all_keys(SORTPARAM *param,
         error= HA_ERR_END_OF_FILE;
         break;
       }
-      file->position(sort_form->record[0]);
+      file->position(sort_form->getInsertRecord());
     }
     else					/* Not quick-select */
     {
@@ -554,19 +556,19 @@ static ha_rows find_all_keys(SORTPARAM *param,
 	  error= errno ? errno : -1;		/* Abort */
 	  break;
 	}
-	error=file->rnd_pos(sort_form->record[0],next_pos);
+	error=file->rnd_pos(sort_form->getInsertRecord(),next_pos);
       }
       else
       {
-	error=file->rnd_next(sort_form->record[0]);
+	error=file->rnd_next(sort_form->getInsertRecord());
 
 	if (!flag)
 	{
 	  internal::my_store_ptr(ref_pos,ref_length,record); // Position to row
-	  record+= sort_form->s->db_record_offset;
+	  record+= sort_form->getShare()->db_record_offset;
 	}
 	else if (!error)
-	  file->position(sort_form->record[0]);
+	  file->position(sort_form->getInsertRecord());
       }
       if (error && error != HA_ERR_RECORD_DELETED)
 	break;
@@ -577,7 +579,7 @@ static ha_rows find_all_keys(SORTPARAM *param,
       if (!indexfile && !quick_select)
       {
         (void) file->extra(HA_EXTRA_NO_CACHE);
-        file->ha_rnd_end();
+        file->endTableScan();
       }
       return(HA_POS_ERROR);
     }
@@ -606,13 +608,13 @@ static ha_rows find_all_keys(SORTPARAM *param,
       index_merge quick select uses table->sort when retrieving rows, so free
       resoures it has allocated.
     */
-    end_read_record(&read_record_info);
+    read_record_info.end_read_record();
   }
   else
   {
     (void) file->extra(HA_EXTRA_NO_CACHE);	/* End cacheing of records */
     if (!next_pos)
-      file->ha_rnd_end();
+      file->endTableScan();
   }
 
   if (session->is_error())
@@ -639,15 +641,15 @@ static ha_rows find_all_keys(SORTPARAM *param,
   @details
   Sort the buffer and write:
   -# the sorted sequence to tempfile
-  -# a BUFFPEK describing the sorted sequence position to buffpek_pointers
+  -# a buffpek_st describing the sorted sequence position to buffpek_pointers
 
     (was: Skriver en buffert med nycklar till filen)
 
   @param param             Sort parameters
   @param sort_keys         Array of pointers to keys to sort
   @param count             Number of elements in sort_keys array
-  @param buffpek_pointers  One 'BUFFPEK' struct will be written into this file.
-                           The BUFFPEK::{file_pos, count} will indicate where
+  @param buffpek_pointers  One 'buffpek_st' struct will be written into this file.
+                           The buffpek_st::{file_pos, count} will indicate where
                            the sorted data was stored.
   @param tempfile          The sorted sequence will be written into this file.
 
@@ -663,17 +665,17 @@ write_keys(SORTPARAM *param, register unsigned char **sort_keys, uint32_t count,
 {
   size_t sort_length, rec_length;
   unsigned char **end;
-  BUFFPEK buffpek;
+  buffpek_st buffpek;
 
   sort_length= param->sort_length;
   rec_length= param->rec_length;
   internal::my_string_ptr_sort((unsigned char*) sort_keys, (uint32_t) count, sort_length);
   if (!my_b_inited(tempfile) &&
-      open_cached_file(tempfile, drizzle_tmpdir, TEMP_PREFIX, DISK_BUFFER_SIZE,
+      open_cached_file(tempfile, drizzle_tmpdir.c_str(), TEMP_PREFIX, DISK_BUFFER_SIZE,
                        MYF(MY_WME)))
     goto err;
   /* check we won't have more buffpeks than we can possibly keep in memory */
-  if (my_b_tell(buffpek_pointers) + sizeof(BUFFPEK) > (uint64_t)UINT_MAX)
+  if (my_b_tell(buffpek_pointers) + sizeof(buffpek_st) > (uint64_t)UINT_MAX)
     goto err;
   buffpek.file_pos= my_b_tell(tempfile);
   if ((ha_rows) count > param->max_rows)
@@ -719,8 +721,8 @@ static inline void store_length(unsigned char *to, uint32_t length, uint32_t pac
 static void make_sortkey(register SORTPARAM *param,
 			 register unsigned char *to, unsigned char *ref_pos)
 {
-  register Field *field;
-  register SORT_FIELD *sort_field;
+  Field *field;
+  SortField *sort_field;
   size_t length;
 
   for (sort_field=param->local_sortorder ;
@@ -908,7 +910,7 @@ static void make_sortkey(register SORTPARAM *param,
       In this implementation we use fixed layout for field values -
       the same for all records.
     */
-    SORT_ADDON_FIELD *addonf= param->addon_field;
+    sort_addon_field_st *addonf= param->addon_field;
     unsigned char *nulls= to;
     assert(addonf != 0);
     memset(nulls, 0, addonf->offset);
@@ -952,7 +954,7 @@ static void make_sortkey(register SORTPARAM *param,
 
 static void register_used_fields(SORTPARAM *param)
 {
-  register SORT_FIELD *sort_field;
+  SortField *sort_field;
   Table *table=param->sort_form;
 
   for (sort_field= param->local_sortorder ;
@@ -962,7 +964,7 @@ static void register_used_fields(SORTPARAM *param)
     Field *field;
     if ((field= sort_field->field))
     {
-      if (field->table == table)
+      if (field->getTable() == table)
         table->setReadSet(field->field_index);
     }
     else
@@ -974,7 +976,7 @@ static void register_used_fields(SORTPARAM *param)
 
   if (param->addon_field)
   {
-    SORT_ADDON_FIELD *addonf= param->addon_field;
+    sort_addon_field_st *addonf= param->addon_field;
     Field *field;
     for ( ; (field= addonf->field) ; addonf++)
       table->setReadSet(field->field_index);
@@ -1013,16 +1015,16 @@ static bool save_index(SORTPARAM *param, unsigned char **sort_keys, uint32_t cou
 /** Merge buffers to make < MERGEBUFF2 buffers. */
 
 int merge_many_buff(SORTPARAM *param, unsigned char *sort_buffer,
-		    BUFFPEK *buffpek, uint32_t *maxbuffer, internal::IO_CACHE *t_file)
+		    buffpek_st *buffpek, uint32_t *maxbuffer, internal::IO_CACHE *t_file)
 {
   register uint32_t i;
   internal::IO_CACHE t_file2,*from_file,*to_file,*temp;
-  BUFFPEK *lastbuff;
+  buffpek_st *lastbuff;
 
   if (*maxbuffer < MERGEBUFF2)
     return(0);
   if (flush_io_cache(t_file) ||
-      open_cached_file(&t_file2,drizzle_tmpdir,TEMP_PREFIX,DISK_BUFFER_SIZE,
+      open_cached_file(&t_file2,drizzle_tmpdir.c_str(),TEMP_PREFIX,DISK_BUFFER_SIZE,
 			MYF(MY_WME)))
     return(1);
 
@@ -1069,7 +1071,7 @@ cleanup:
     (uint32_t)-1 if something goes wrong
 */
 
-uint32_t read_to_buffer(internal::IO_CACHE *fromfile, BUFFPEK *buffpek,
+uint32_t read_to_buffer(internal::IO_CACHE *fromfile, buffpek_st *buffpek,
                         uint32_t rec_length)
 {
   register uint32_t count;
@@ -1096,7 +1098,7 @@ class compare_functor
   public:
   compare_functor(qsort2_cmp in_key_compare, void *in_compare_arg)
     : key_compare(in_key_compare), key_compare_arg(in_compare_arg) { }
-  inline bool operator()(const BUFFPEK *i, const BUFFPEK *j) const
+  inline bool operator()(const buffpek_st *i, const buffpek_st *j) const
   {
     int val= key_compare(key_compare_arg,
                       &i->key, &j->key);
@@ -1109,12 +1111,12 @@ class compare_functor
   Merge buffers to one buffer.
 
   @param param        Sort parameter
-  @param from_file    File with source data (BUFFPEKs point to this file)
+  @param from_file    File with source data (buffpek_sts point to this file)
   @param to_file      File to write the sorted result data.
   @param sort_buffer  Buffer for data to store up to MERGEBUFF2 sort keys.
-  @param lastbuff     OUT Store here BUFFPEK describing data written to to_file
-  @param Fb           First element in source BUFFPEKs array
-  @param Tb           Last element in source BUFFPEKs array
+  @param lastbuff     OUT Store here buffpek_st describing data written to to_file
+  @param Fb           First element in source buffpek_sts array
+  @param Tb           Last element in source buffpek_sts array
   @param flag
 
   @retval
@@ -1125,7 +1127,7 @@ class compare_functor
 
 int merge_buffers(SORTPARAM *param, internal::IO_CACHE *from_file,
                   internal::IO_CACHE *to_file, unsigned char *sort_buffer,
-                  BUFFPEK *lastbuff, BUFFPEK *Fb, BUFFPEK *Tb,
+                  buffpek_st *lastbuff, buffpek_st *Fb, buffpek_st *Tb,
                   int flag)
 {
   int error;
@@ -1135,13 +1137,13 @@ int merge_buffers(SORTPARAM *param, internal::IO_CACHE *from_file,
   ha_rows max_rows,org_max_rows;
   internal::my_off_t to_start_filepos;
   unsigned char *strpos;
-  BUFFPEK *buffpek;
+  buffpek_st *buffpek;
   qsort2_cmp cmp;
   void *first_cmp_arg;
   volatile Session::killed_state *killed= &current_session->killed;
   Session::killed_state not_killable;
 
-  status_var_increment(current_session->status_var.filesort_merge_passes);
+  current_session->status_var.filesort_merge_passes++;
   if (param->not_killable)
   {
     killed= &not_killable;
@@ -1171,7 +1173,7 @@ int merge_buffers(SORTPARAM *param, internal::IO_CACHE *from_file,
     cmp= internal::get_ptr_compare(sort_length);
     first_cmp_arg= (void*) &sort_length;
   }
-  priority_queue<BUFFPEK *, vector<BUFFPEK *>, compare_functor > 
+  priority_queue<buffpek_st *, vector<buffpek_st *>, compare_functor > 
     queue(compare_functor(cmp, first_cmp_arg));
   for (buffpek= Fb ; buffpek <= Tb ; buffpek++)
   {
@@ -1331,7 +1333,7 @@ err:
 	/* Do a merge to output-file (save only positions) */
 
 static int merge_index(SORTPARAM *param, unsigned char *sort_buffer,
-		       BUFFPEK *buffpek, uint32_t maxbuffer,
+		       buffpek_st *buffpek, uint32_t maxbuffer,
 		       internal::IO_CACHE *tempfile, internal::IO_CACHE *outfile)
 {
   if (merge_buffers(param,tempfile,outfile,sort_buffer,buffpek,buffpek,
@@ -1373,7 +1375,7 @@ static uint32_t suffix_length(uint32_t string_length)
 */
 
 static uint32_t
-sortlength(Session *session, SORT_FIELD *sortorder, uint32_t s_length,
+sortlength(Session *session, SortField *sortorder, uint32_t s_length,
            bool *multi_byte_charset)
 {
   register uint32_t length;
@@ -1479,12 +1481,12 @@ sortlength(Session *session, SORT_FIELD *sortorder, uint32_t s_length,
     NULL   if we do not store field values with sort data.
 */
 
-static SORT_ADDON_FIELD *
+static sort_addon_field_st *
 get_addon_fields(Session *session, Field **ptabfield, uint32_t sortlength, uint32_t *plength)
 {
   Field **pfield;
   Field *field;
-  SORT_ADDON_FIELD *addonf;
+  sort_addon_field_st *addonf;
   uint32_t length= 0;
   uint32_t fields= 0;
   uint32_t null_fields= 0;
@@ -1516,7 +1518,7 @@ get_addon_fields(Session *session, Field **ptabfield, uint32_t sortlength, uint3
   length+= (null_fields+7)/8;
 
   if (length+sortlength > session->variables.max_length_for_sort_data ||
-      !(addonf= (SORT_ADDON_FIELD *) malloc(sizeof(SORT_ADDON_FIELD)*
+      !(addonf= (sort_addon_field_st *) malloc(sizeof(sort_addon_field_st)*
                                             (fields+1))))
     return 0;
 
@@ -1566,10 +1568,10 @@ get_addon_fields(Session *session, Field **ptabfield, uint32_t sortlength, uint3
 */
 
 static void
-unpack_addon_fields(struct st_sort_addon_field *addon_field, unsigned char *buff)
+unpack_addon_fields(struct sort_addon_field_st *addon_field, unsigned char *buff)
 {
   Field *field;
-  SORT_ADDON_FIELD *addonf= addon_field;
+  sort_addon_field_st *addonf= addon_field;
 
   for ( ; (field= addonf->field) ; addonf++)
   {
