@@ -26,6 +26,7 @@
 #include "drizzled/key_part_spec.h"
 #include "drizzled/sql_list.h"
 #include "drizzled/cursor.h" /* for default_key_create_info */
+#include "drizzled/message/table.pb.h"
 
 namespace drizzled
 {
@@ -35,36 +36,34 @@ class Table_ident;
 
 namespace memory { class Root; }
 
+void add_foreign_key_to_table_message(
+    message::Table *table_message,
+    const char* fkey_name,
+    List<Key_part_spec> &cols,
+    Table_ident *table,
+    List<Key_part_spec> &ref_cols,
+    message::Table::ForeignKeyConstraint::ForeignKeyOption delete_opt_arg,
+    message::Table::ForeignKeyConstraint::ForeignKeyOption update_opt_arg,
+    message::Table::ForeignKeyConstraint::ForeignKeyMatchOption match_opt_arg);
+
+
 class Foreign_key: public Key 
 {
 public:
-  enum fk_match_opt 
-  {
-    FK_MATCH_UNDEF, 
-    FK_MATCH_FULL, 
-    FK_MATCH_PARTIAL, 
-    FK_MATCH_SIMPLE
-  };
-  enum fk_option 
-  {
-    FK_OPTION_UNDEF, 
-    FK_OPTION_RESTRICT, 
-    FK_OPTION_CASCADE, 
-    FK_OPTION_SET_NULL, 
-    FK_OPTION_NO_ACTION, 
-    FK_OPTION_DEFAULT
-  };
-
   Table_ident *ref_table;
   List<Key_part_spec> ref_columns;
-  uint32_t delete_opt, update_opt, match_opt;
+
+  message::Table::ForeignKeyConstraint::ForeignKeyOption delete_opt;
+  message::Table::ForeignKeyConstraint::ForeignKeyOption update_opt;
+  message::Table::ForeignKeyConstraint::ForeignKeyMatchOption match_opt;
+
   Foreign_key(const LEX_STRING &name_arg,
               List<Key_part_spec> &cols,
               Table_ident *table,
               List<Key_part_spec> &ref_cols,
-              uint32_t delete_opt_arg,
-              uint32_t update_opt_arg,
-              uint32_t match_opt_arg) :
+              message::Table::ForeignKeyConstraint::ForeignKeyOption delete_opt_arg,
+              message::Table::ForeignKeyConstraint::ForeignKeyOption update_opt_arg,
+              message::Table::ForeignKeyConstraint::ForeignKeyMatchOption match_opt_arg) :
     Key(FOREIGN_KEY, name_arg, &default_key_create_info, 0, cols), ref_table(table),
     ref_columns(ref_cols),
     delete_opt(delete_opt_arg),
