@@ -25,12 +25,15 @@
 #include <drizzled/item.h>
 #include <drizzled/sql_list.h>
 #include <drizzled/item/bin_string.h>
+#include "drizzled/current_session.h"
 
 namespace drizzled
 {
 
 class Item_func :public Item_result_field
 {
+  Session &_session;
+
 protected:
   Item **args, *tmp_arg[2];
   /*
@@ -61,26 +64,37 @@ public:
   enum Type type() const { return FUNC_ITEM; }
   virtual enum Functype functype() const   { return UNKNOWN_FUNC; }
   virtual ~Item_func() {}
+
   Item_func(void):
+    _session(*current_session),
     allowed_arg_cols(1), arg_count(0)
   {
     with_sum_func= 0;
+    collation.set(DERIVATION_SYSCONST);
   }
+
   Item_func(Item *a):
+    _session(*current_session),
     allowed_arg_cols(1), arg_count(1)
   {
     args= tmp_arg;
     args[0]= a;
     with_sum_func= a->with_sum_func;
+    collation.set(DERIVATION_SYSCONST);
   }
+  
   Item_func(Item *a,Item *b):
+    _session(*current_session),
     allowed_arg_cols(1), arg_count(2)
   {
     args= tmp_arg;
     args[0]= a; args[1]= b;
     with_sum_func= a->with_sum_func || b->with_sum_func;
+    collation.set(DERIVATION_SYSCONST);
   }
+  
   Item_func(Item *a,Item *b,Item *c):
+    _session(*current_session),
     allowed_arg_cols(1)
   {
     arg_count= 0;
@@ -90,8 +104,11 @@ public:
       args[0]= a; args[1]= b; args[2]= c;
       with_sum_func= a->with_sum_func || b->with_sum_func || c->with_sum_func;
     }
+    collation.set(DERIVATION_SYSCONST);
   }
+  
   Item_func(Item *a,Item *b,Item *c,Item *d):
+    _session(*current_session),
     allowed_arg_cols(1)
   {
     arg_count= 0;
@@ -102,8 +119,11 @@ public:
       with_sum_func= a->with_sum_func || b->with_sum_func ||
         c->with_sum_func || d->with_sum_func;
     }
+    collation.set(DERIVATION_SYSCONST);
   }
+  
   Item_func(Item *a,Item *b,Item *c,Item *d,Item* e):
+    _session(*current_session),
     allowed_arg_cols(1)
   {
     arg_count= 5;
@@ -113,10 +133,14 @@ public:
       with_sum_func= a->with_sum_func || b->with_sum_func ||
         c->with_sum_func || d->with_sum_func || e->with_sum_func ;
     }
+    collation.set(DERIVATION_SYSCONST);
   }
+  
   Item_func(List<Item> &list);
+  
   // Constructor used for Item_cond_and/or (see Item comment)
   Item_func(Session *session, Item_func *item);
+  
   bool fix_fields(Session *, Item **ref);
   void fix_after_pullout(Select_Lex *new_parent, Item **ref);
   table_map used_tables() const;
@@ -188,6 +212,16 @@ public:
                      void * arg, traverse_order order);
   double fix_result(double value);
 
+  Session &getSession()
+  {
+    return _session;
+  }
+
+  Session *getSessionPtr()
+  {
+    return &_session;
+  }
+
 };
 
 } /* namespace drizzled */
