@@ -65,12 +65,11 @@ int64_t Item_func_get_user_var::val_int()
 
 void Item_func_get_user_var::fix_length_and_dec()
 {
-  Session *session=current_session;
   maybe_null=1;
   decimals=NOT_FIXED_DEC;
   max_length=MAX_BLOB_WIDTH;
 
-  var_entry= session->getVariable(name, false);
+  var_entry= session.getVariable(name, false);
 
   /*
     If the variable didn't exist it has been created as a STRING-type.
@@ -118,7 +117,7 @@ void Item_func_get_user_var::fix_length_and_dec()
 
 bool Item_func_get_user_var::const_item() const
 {
-  return (!var_entry || current_session->getQueryId() != var_entry->update_query_id);
+  return (!var_entry || session.getQueryId() != var_entry->update_query_id);
 }
 
 
