@@ -87,7 +87,7 @@ void Item_func_div::result_precision()
 
 void Item_func_div::fix_length_and_dec()
 {
-  prec_increment= current_session->variables.div_precincrement;
+  prec_increment= session->variables.div_precincrement;
   Item_num_op::fix_length_and_dec();
 
   switch(hybrid_type)
