@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_MATH_MINUS_H
+#define DRIZZLED_FUNCTION_MATH_MINUS_H
 
 #include <drizzled/function/func.h>
 #include <drizzled/function/num_op.h>
@@ -33,9 +34,10 @@ public:
   const char *func_name() const { return "-"; }
   int64_t int_op();
   double real_op();
-  type::Decimal *decimal_op(type::Decimal *);
+  my_decimal *decimal_op(my_decimal *);
   void fix_length_and_dec();
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_MATH_MINUS_H */
