@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,17 +17,15 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_MATH_REAL_H
+#define DRIZZLED_FUNCTION_MATH_REAL_H
 
 #include <drizzled/function/func.h>
 
-#include <drizzled/visibility.h>
-
 namespace drizzled
 {
 
-class DRIZZLED_API Item_real_func :
-  public Item_func
+class Item_real_func :public Item_func
 {
 public:
   Item_real_func() :Item_func() {}
@@ -35,7 +33,7 @@ public:
   Item_real_func(Item *a,Item *b) :Item_func(a,b) {}
   Item_real_func(List<Item> &list) :Item_func(list) {}
   String *val_str(String*str);
-  type::Decimal *val_decimal(type::Decimal *decimal_value);
+  my_decimal *val_decimal(my_decimal *decimal_value);
   int64_t val_int();
   enum Item_result result_type () const { return REAL_RESULT; }
   void fix_length_and_dec();
@@ -43,3 +41,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_MATH_REAL_H */
