@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_NUM_OP_H
+#define DRIZZLED_FUNCTION_NUM_OP_H
 
 #include <drizzled/function/numhybrid.h>
 
@@ -32,9 +33,9 @@ class Item_num_op :public Item_func_numhybrid
   Item_num_op(Item *a,Item *b) :Item_func_numhybrid(a, b) {}
   virtual void result_precision()= 0;
 
-  virtual inline void print(String *str)
+  virtual inline void print(String *str, enum_query_type query_type)
   {
-    print_op(str);
+    print_op(str, query_type);
   }
 
   void find_num_type();
@@ -44,3 +45,4 @@ class Item_num_op :public Item_func_numhybrid
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_NUM_OP_H */
