@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,11 +17,12 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <drizzled/function/str/alloc_buffer.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
 String* alloc_buffer(String *res,String *str,String *tmp_value,
                             ulong length)
@@ -34,7 +35,8 @@ String* alloc_buffer(String *res,String *str,String *tmp_value,
       str->length(length);
       return str;
     }
-    tmp_value->alloc(length);
+    if (tmp_value->alloc(length))
+      return 0;
     (void) tmp_value->copy(*res);
     tmp_value->length(length);
     return tmp_value;
