@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_STR_CHAR_H
+#define DRIZZLED_FUNCTION_STR_CHAR_H
 
 #include <drizzled/function/str/strfunc.h>
 
@@ -29,7 +30,7 @@ class Item_func_char :public Item_str_func
 public:
   Item_func_char(List<Item> &list) :Item_str_func(list)
   { collation.set(&my_charset_bin); }
-  Item_func_char(List<Item> &list, const charset_info_st * const cs) :Item_str_func(list)
+  Item_func_char(List<Item> &list, const CHARSET_INFO * const cs) :Item_str_func(list)
   { collation.set(cs); }
   String *val_str(String *);
   void fix_length_and_dec()
@@ -41,3 +42,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_STR_CHAR_H */
