@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_STR_EXPORT_SET_H
+#define DRIZZLED_FUNCTION_STR_EXPORT_SET_H
 
 #include <drizzled/function/str/strfunc.h>
 
@@ -37,3 +38,4 @@ class Item_func_export_set: public Item_str_func
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_STR_EXPORT_SET_H */
