@@ -57,7 +57,13 @@ int64_t Item_func_microsecond::val_int()
   char time_buff[DRIZZLE_MAX_LENGTH_DATETIME_AS_STRING];
   String tmp_time(time_buff,sizeof(time_buff), &my_charset_utf8_bin);
   String *time_res= args[0]->val_str(&tmp_time);
-  if (! temporal_time.from_string(time_res->c_ptr(), time_res->length()))
+
+  if (time_res && (time_res != &tmp_time))
+  {
+    tmp_time.copy(*time_res);
+  }
+
+  if (! temporal_time.from_string(tmp_time.c_ptr(), tmp_time.length()))
   {
     /* 
      * OK, we failed to match the first argument as a string
@@ -83,13 +89,19 @@ int64_t Item_func_microsecond::val_int()
           char buff[DRIZZLE_MAX_LENGTH_DATETIME_AS_STRING];
           String tmp(buff,sizeof(buff), &my_charset_utf8_bin);
           String *res= args[0]->val_str(&tmp);
-          if (! temporal_datetime.from_string(res->c_ptr(), res->length()))
+
+          if (res && (res != &tmp))
+          {
+            tmp.copy(*res);
+          }
+
+          if (! temporal_datetime.from_string(tmp.c_ptr(), tmp.length()))
           {
             /* 
             * Could not interpret the function argument as a temporal value, 
             * so throw an error and return 0
             */
-            my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+            my_error(ER_INVALID_DATETIME_VALUE, MYF(0), tmp.c_ptr());
             return 0;
           }
         }
@@ -111,7 +123,12 @@ int64_t Item_func_microsecond::val_int()
 
           res= args[0]->val_str(&tmp);
 
-          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+          if (res && (res != &tmp))
+          {
+            tmp.copy(*res);
+          }
+
+          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), tmp.c_ptr());
           return 0;
         }
     }
