@@ -82,13 +82,18 @@ int64_t Item_func_to_days::val_int()
           return false;
         }
 
-        if (! temporal.from_string(res->c_ptr(), res->length()))
+        if (res != &tmp)
+        {
+          tmp.copy(*res);
+        }
+
+        if (! temporal.from_string(tmp.c_ptr(), tmp.length()))
         {
           /* 
           * Could not interpret the function argument as a temporal value, 
           * so throw an error and return 0
           */
-          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), tmp.c_ptr());
           return 0;
         }
       }
@@ -120,7 +125,12 @@ int64_t Item_func_to_days::val_int()
           return false;
         }
 
-        my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+        if (res != &tmp)
+        {
+          tmp.copy(*res);
+        }
+
+        my_error(ER_INVALID_DATETIME_VALUE, MYF(0), tmp.c_ptr());
         return 0;
       }
   }
@@ -174,13 +184,18 @@ int64_t Item_func_to_days::val_int_endpoint(bool left_endp, bool *incl_endp)
           return 0;
         }
 
-        if (! temporal.from_string(res->c_ptr(), res->length()))
+        if (res != &tmp)
+        {
+          tmp.copy(*res);
+        }
+
+        if (! temporal.from_string(tmp.c_ptr(), tmp.length()))
         {
           /* 
           * Could not interpret the function argument as a temporal value, 
           * so throw an error and return 0
           */
-          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), tmp.c_ptr());
           return 0;
         }
       }
@@ -212,7 +227,12 @@ int64_t Item_func_to_days::val_int_endpoint(bool left_endp, bool *incl_endp)
           return 0;
         }
 
-        my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+        if (res != &tmp)
+        {
+          tmp.copy(*res);
+        }
+
+        my_error(ER_INVALID_DATETIME_VALUE, MYF(0), tmp.c_ptr());
         return 0;
       }
   }
