@@ -57,13 +57,19 @@ int64_t Item_func_weekday::val_int()
         char buff[DRIZZLE_MAX_LENGTH_DATETIME_AS_STRING];
         String tmp(buff,sizeof(buff), &my_charset_utf8_bin);
         String *res= args[0]->val_str(&tmp);
-        if (! temporal.from_string(res->c_ptr(), res->length()))
+
+        if (res && (res != &tmp))
+        {
+          tmp.copy(*res);
+        }
+
+        if (! temporal.from_string(tmp.c_ptr(), tmp.length()))
         {
           /* 
           * Could not interpret the function argument as a temporal value, 
           * so throw an error and return 0
           */
-          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), tmp.c_ptr());
           return 0;
         }
       }
@@ -85,7 +91,12 @@ int64_t Item_func_weekday::val_int()
 
         res= args[0]->val_str(&tmp);
 
-        my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+        if (res && (res != &tmp))
+        {
+          tmp.copy(*res);
+        }
+
+        my_error(ER_INVALID_DATETIME_VALUE, MYF(0), tmp.c_ptr());
         return 0;
       }
   }
