@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,27 +17,28 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_FUNCTION_UNITS_H
+#define DRIZZLED_FUNCTION_UNITS_H
 
 #include <drizzled/function/func.h>
 #include <drizzled/function/math/real.h>
 
-namespace drizzled {
-
-class Item_func_units : public Item_real_func
+namespace drizzled
 {
+
+class Item_func_units :public Item_real_func
+{
+  char *name;
+  double mul,add;
 public:
-  Item_func_units(const char *name_arg, Item *a, double mul_arg, double add_arg)
-    : Item_real_func(a),name(name_arg),mul(mul_arg),add(add_arg) {}
+  Item_func_units(char *name_arg,Item *a,double mul_arg,double add_arg)
+    :Item_real_func(a),name(name_arg),mul(mul_arg),add(add_arg) {}
   double val_real();
   const char *func_name() const { return name; }
   void fix_length_and_dec()
   { decimals= NOT_FIXED_DEC; max_length= float_length(decimals); }
-private:
-  const char *name;
-  double mul;
-  double add;
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_FUNCTION_UNITS_H */
