@@ -18,17 +18,15 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_GENERATOR_H
+#define DRIZZLED_GENERATOR_H
 
-// Needs to come first
-#include <drizzled/generator/schema.h>
-#include <drizzled/generator/all_tables.h>
+#include "drizzled/session.h"
 
-#include <drizzled/generator/all_fields.h>
-#include <drizzled/generator/all_foreign_keys.h>
-#include <drizzled/generator/all_indexes.h>
-#include <drizzled/generator/event_observers.h>
-#include <drizzled/generator/functions.h>
-#include <drizzled/generator/table.h>
-#include <drizzled/generator/table_definition_cache.h>
+#include "drizzled/generator/functions.h"
+#include "drizzled/generator/schema.h"
+#include "drizzled/generator/table.h"
+#include "drizzled/generator/all_tables.h"
+#include "drizzled/generator/all_fields.h"
 
+#endif /* DRIZZLED_GENERATOR_H */
