@@ -18,9 +18,9 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/generator.h>
+#include "drizzled/generator.h"
 
 using namespace std;
 
@@ -35,7 +35,6 @@ AllFields::AllFields(Session &arg) :
   all_tables_generator(arg)
 {
   ((table_ptr= all_tables_generator));
-  table_setup();
 }
 
 bool AllFields::table_setup()
@@ -49,11 +48,3 @@ bool AllFields::table_setup()
 
 } /* namespace generator */
 } /* namespace drizzled */
-
-bool operator!(const drizzled::generator::FieldPair &arg)
-{
-  if (arg.first == 0 and arg.second == 0)
-    return true;
-
-  return false;
-}
