@@ -18,29 +18,23 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/generator.h>
-#include <drizzled/function_container.h>
-#include <drizzled/plugin/function.h>
-#include <drizzled/session.h>
+#include "drizzled/generator.h"
+#include "drizzled/plugin/function.h"
+#include "drizzled/session.h"
 
 using namespace std;
 
-namespace drizzled {
-namespace generator {
-
-Functions::Functions(Session&)
+namespace drizzled
+{
+namespace generator
 {
-  functions.reserve(plugin::Function::getMap().size() + FunctionContainer::getMap().size());
 
-  BOOST_FOREACH(FunctionContainer::Map::const_reference it, FunctionContainer::getMap()) 
-    functions.push_back(it.first);
-
-  BOOST_FOREACH(plugin::Function::Map::const_reference it, plugin::Function::getMap())
-    functions.push_back(it.first);
-
-  iter= functions.begin();
+Functions::Functions(Session &arg) :
+  session(arg)
+{
+  udf_iter= plugin::Function::getMap().begin();
 }
 
 } /* namespace generator */
