@@ -18,34 +18,37 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_GENERATOR_FUNCTIONS_H
+#define DRIZZLED_GENERATOR_FUNCTIONS_H
 
-#include <drizzled/plugin/function.h>
-#include <drizzled/visibility.h>
+#include "drizzled/plugin/function.h"
 
 namespace drizzled {
 namespace generator {
 
-class DRIZZLED_API Functions
+class Functions
 {
+  Session &session;
+  std::string function_name;
+  plugin::Function::UdfMap::const_iterator udf_iter;
+
 public:
-  Functions(Session&);
+
+  Functions(Session &arg);
 
   operator std::string*()
   {
-    if (iter == functions.end())
+    if (udf_iter == plugin::Function::getMap().end())
       return NULL;
-    function_name= *iter++;
+
+    function_name= (*udf_iter).first;
+    udf_iter++;
+
     return &function_name;
   }
-  typedef std::vector<std::string> functions_t;
-  functions_t functions;
-
-  std::string function_name;
-  functions_t::iterator iter;
-
 };
 
 } /* namespace generator */
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_GENERATOR_FUNCTIONS_H */
