@@ -40,11 +40,6 @@
 namespace drizzled
 {
 
-struct st_key;
-typedef struct st_key KEY;
-typedef struct st_key_cache KEY_CACHE;
-
-
 namespace plugin
 {
 class StorageEngine;
@@ -57,20 +52,41 @@ typedef struct st_ha_create_information
   uint64_t auto_increment_value;
   uint32_t table_options;
   uint32_t used_fields;
-  enum row_type row_type;
   plugin::StorageEngine *db_type;
   bool table_existed;			/* 1 in create if table existed */
+
+  st_ha_create_information() :
+    table_charset(0),
+    default_table_charset(0),
+    alias(0),
+    auto_increment_value(0),
+    table_options(0),
+    used_fields(0),
+    db_type(0),
+    table_existed(0)
+  { }
 } HA_CREATE_INFO;
 
 typedef struct st_ha_alter_information
 {
-  KEY  *key_info_buffer;
+  KeyInfo  *key_info_buffer;
   uint32_t key_count;
   uint32_t index_drop_count;
   uint32_t *index_drop_buffer;
   uint32_t index_add_count;
   uint32_t *index_add_buffer;
   void *data;
+
+  st_ha_alter_information() :
+    key_info_buffer(0),
+    key_count(0),
+    index_drop_count(0),
+    index_drop_buffer(0),
+    index_add_count(0),
+    index_add_buffer(0),
+    data(0)
+  { }
+
 } HA_ALTER_INFO;
 
 
@@ -120,20 +136,6 @@ typedef struct st_range_seq_if
   uint32_t (*next) (range_seq_t seq, KEY_MULTI_RANGE *range);
 } RANGE_SEQ_IF;
 
-/*
-  This is a buffer area that the handler can use to store rows.
-  'end_of_used_area' should be kept updated after calls to
-  read-functions so that other parts of the code can use the
-  remaining area (until next read calls is issued).
-*/
-
-typedef struct st_handler_buffer
-{
-  unsigned char *buffer;         /* Buffer one can start using */
-  unsigned char *buffer_end;     /* End of buffer */
-  unsigned char *end_of_used_area;     /* End of area that was used by handler */
-} HANDLER_BUFFER;
-
 } /* namespace drizzled */
 
 #endif /* DRIZZLED_HANDLER_STRUCTS_H */
