@@ -18,35 +18,10 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_IDENTIFIER_H
+#define DRIZZLED_IDENTIFIER_H
 
-#include <string>
+#include "drizzled/identifier/schema.h"
+#include "drizzled/identifier/table.h"
 
-#include <drizzled/visibility.h>
-
-namespace drizzled {
-
-class DRIZZLED_API Identifier
-{
-public:
-  virtual ~Identifier()
-  { 
-	}
-
-	virtual std::string getSQLPath() const 
-	{ 
-		return "";
-	}
-};
-
-} // namespace drizzled
-
-#include <drizzled/identifier/catalog.h>
-#include <drizzled/identifier/schema.h>
-#include <drizzled/identifier/table.h>
-#include <drizzled/identifier/user.h>
-
-// Constant identifiers used internally
-#include <drizzled/identifier/constants/schema.h>
-#include <drizzled/identifier/constants/table.h>
-#include <drizzled/identifier/constants/user.h>
+#endif /* DRIZZLED_IDENTIFIER_H */
