@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
   Defines: llstr();
@@ -23,8 +23,8 @@
   variables with printf() as there is no usable printf() standard one can use.
 */
 
-#include <config.h>
-#include <drizzled/internal/m_string.h>
+#include "config.h"
+#include "drizzled/internal/m_string.h"
 
 namespace drizzled
 {
