@@ -21,8 +21,8 @@
 #define DRIZZLED_ITEM_H
 
 #include <drizzled/dtcollation.h>
-#include <drizzled/my_time.h>
-#include <drizzled/my_decimal.h>
+#include <drizzled/drizzle_time.h>
+#include <drizzled/decimal.h>
 #include <drizzled/sql_bitmap.h>
 #include <drizzled/sql_list.h>
 #include "drizzled/memory/sql_alloc.h"
@@ -732,6 +732,11 @@ public:
    * cost Item::execution_cost(),
    * where 'cost' is either 'double' or some structure of various cost
    * parameters.
+   *
+   *NOTE
+   *   This function is now used to prevent evaluation of materialized IN
+   *   subquery predicates before it is allowed. grep for
+   *   DontEvaluateMaterializedSubqueryTooEarly to see the uses.
    */
   virtual bool is_expensive();
 
