@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,20 +17,32 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ITEM_BASIC_CONSTANT_H
+#define DRIZZLED_ITEM_BASIC_CONSTANT_H
 
 #include <drizzled/item.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
-class Item_basic_constant : public Item
+class Item_basic_constant :public Item
 {
 public:
+  Item_basic_constant() : Item() {};
+
   /* to prevent drop fixed flag (no need parent cleanup call) */
   void cleanup()
   {
+    /*
+      Restore the original field name as it might not have been allocated
+      in the statement memory. If the name is auto generated, it must be
+      done again between subsequent executions of a prepared statement.
+    */
+    if (orig_name)
+      name= orig_name;
   }
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ITEM_BASIC_CONSTANT_H */
