@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,36 +17,41 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ITEM_CMPFUNC_H
+#define DRIZZLED_ITEM_CMPFUNC_H
 
 /* compare and test functions */
 
-#include <drizzled/common.h>
-#include <drizzled/comp_creator.h>
-#include <drizzled/function/math/int.h>
-#include <drizzled/function/numhybrid.h>
-#include <drizzled/item/decimal.h>
-#include <drizzled/item/float.h>
-#include <drizzled/item/function/boolean.h>
-#include <drizzled/item/int.h>
-#include <drizzled/item/row.h>
-#include <drizzled/item/string.h>
-#include <drizzled/item/sum.h>
-#include <drizzled/qsort_cmp.h>
+#include "drizzled/comp_creator.h"
+#include "drizzled/item/row.h"
+#include "drizzled/item/sum.h"
+#include "drizzled/item/int.h"
+#include "drizzled/item/float.h"
+#include "drizzled/item/decimal.h"
+#include "drizzled/function/math/int.h"
+#include "drizzled/function/numhybrid.h"
+#include "drizzled/session.h"
+#include "drizzled/common.h"
+#include "drizzled/qsort_cmp.h"
 
-namespace drizzled {
+namespace drizzled
+{
 
 extern Item_result item_cmp_type(Item_result a,Item_result b);
+class Item_bool_func2;
+class Arg_comparator;
+class Item_sum_hybrid;
+class Item_row;
 
 typedef int (Arg_comparator::*arg_cmp_func)();
 
 typedef int (*Item_field_cmpfunc)(Item_field *f1, Item_field *f2, void *arg);
 
-int64_t get_datetime_value(Session *session, 
-                           Item ***item_arg, 
-                           Item **cache_arg,
-                           Item *warn_item, 
-                           bool *is_null);
+uint64_t get_datetime_value(Session *session, 
+                            Item ***item_arg, 
+                            Item **cache_arg,
+                            Item *warn_item, 
+                            bool *is_null);
 
 class Arg_comparator: public memory::SqlAlloc
 {
@@ -62,14 +67,14 @@ class Arg_comparator: public memory::SqlAlloc
   bool is_nulls_eq;                // TRUE <=> compare for the EQUAL_FUNC
   enum enum_date_cmp_type { CMP_DATE_DFLT= 0, CMP_DATE_WITH_DATE,
                             CMP_DATE_WITH_STR, CMP_STR_WITH_DATE };
-  int64_t (*get_value_func)(Session *session, Item ***item_arg, Item **cache_arg,
-                            Item *warn_item, bool *is_null);
+  uint64_t (*get_value_func)(Session *session, Item ***item_arg, Item **cache_arg,
+                              Item *warn_item, bool *is_null);
 public:
   DTCollation cmp_collation;
 
-  Arg_comparator();
-
-  Arg_comparator(Item **a1, Item **a2);
+  Arg_comparator(): session(0), a_cache(0), b_cache(0) {};
+  Arg_comparator(Item **a1, Item **a2): a(a1), b(a2), session(0),
+    a_cache(0), b_cache(0) {};
 
   int set_compare_func(Item_bool_func2 *owner, Item_result type);
   inline int set_compare_func(Item_bool_func2 *owner_arg)
@@ -111,7 +116,7 @@ public:
   int compare_datetime();        // compare args[0] & args[1] as DATETIMEs
 
   static enum enum_date_cmp_type can_compare_as_dates(Item *a, Item *b,
-                                                      int64_t *const_val_arg);
+                                                      uint64_t *const_val_arg);
 
   void set_datetime_cmp_func(Item **a1, Item **b1);
   static arg_cmp_func comparator_matrix [5][2];
@@ -119,25 +124,39 @@ public:
   friend class Item_func;
 };
 
+class Item_bool_func :public Item_int_func
+{
+public:
+  Item_bool_func() :Item_int_func() {}
+  Item_bool_func(Item *a) :Item_int_func(a) {}
+  Item_bool_func(Item *a,Item *b) :Item_int_func(a,b) {}
+  Item_bool_func(Session *session, Item_bool_func *item) :Item_int_func(session, item) {}
+  bool is_bool_func() { return 1; }
+  void fix_length_and_dec() { decimals=0; max_length=1; }
+  uint32_t decimal_precision() const { return 1; }
+};
+
 
 /**
   Abstract Item class, to represent <code>X IS [NOT] (TRUE | FALSE)</code>
   boolean predicates.
 */
 
-class Item_func_truth : public item::function::Boolean
+class Item_func_truth : public Item_bool_func
 {
 public:
   virtual bool val_bool();
   virtual int64_t val_int();
   virtual void fix_length_and_dec();
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
 
 protected:
   Item_func_truth(Item *a, bool a_value, bool a_affirmative)
-  : item::function::Boolean(a), value(a_value), affirmative(a_affirmative)
+  : Item_bool_func(a), value(a_value), affirmative(a_affirmative)
   {}
 
+  ~Item_func_truth()
+  {}
 private:
   /**
     True for <code>X IS [NOT] TRUE</code>,
@@ -159,6 +178,7 @@ class Item_func_istrue : public Item_func_truth
 {
 public:
   Item_func_istrue(Item *a) : Item_func_truth(a, true, true) {}
+  ~Item_func_istrue() {}
   virtual const char* func_name() const { return "istrue"; }
 };
 
@@ -171,6 +191,7 @@ class Item_func_isnottrue : public Item_func_truth
 {
 public:
   Item_func_isnottrue(Item *a) : Item_func_truth(a, true, false) {}
+  ~Item_func_isnottrue() {}
   virtual const char* func_name() const { return "isnottrue"; }
 };
 
@@ -183,6 +204,7 @@ class Item_func_isfalse : public Item_func_truth
 {
 public:
   Item_func_isfalse(Item *a) : Item_func_truth(a, false, true) {}
+  ~Item_func_isfalse() {}
   virtual const char* func_name() const { return "isfalse"; }
 };
 
@@ -195,10 +217,12 @@ class Item_func_isnotfalse : public Item_func_truth
 {
 public:
   Item_func_isnotfalse(Item *a) : Item_func_truth(a, false, false) {}
+  ~Item_func_isnotfalse() {}
   virtual const char* func_name() const { return "isnotfalse"; }
 };
 
 
+class Item_cache;
 #define UNKNOWN ((bool)-1)
 
 
@@ -217,7 +241,7 @@ public:
     placed into a separate class called 'Item_in_optimizer'.
 */
 
-class Item_in_optimizer: public item::function::Boolean
+class Item_in_optimizer: public Item_bool_func
 {
 protected:
   Item_cache *cache;
@@ -231,7 +255,7 @@ protected:
   bool result_for_null_param;
 public:
   Item_in_optimizer(Item *a, Item_in_subselect *b):
-    item::function::Boolean(a, reinterpret_cast<Item *>(b)), cache(0),
+    Item_bool_func(a, reinterpret_cast<Item *>(b)), cache(0),
     save_cache(0), result_for_null_param(UNKNOWN)
   { with_subselect= true; }
   bool fix_fields(Session *, Item **);
@@ -248,6 +272,8 @@ public:
 class Eq_creator :public Comp_creator
 {
 public:
+  Eq_creator() {}                             /* Remove gcc warning */
+  virtual ~Eq_creator() {}                    /* Remove gcc warning */
   virtual Item_bool_func2* create(Item *a, Item *b) const;
   virtual const char* symbol(bool invert) const { return invert? "<>" : "="; }
   virtual bool eqne_op() const { return 1; }
@@ -258,6 +284,8 @@ public:
 class Ne_creator :public Comp_creator
 {
 public:
+  Ne_creator() {}                             /* Remove gcc warning */
+  virtual ~Ne_creator() {}                    /* Remove gcc warning */
   virtual Item_bool_func2* create(Item *a, Item *b) const;
   virtual const char* symbol(bool invert) const { return invert? "=" : "<>"; }
   virtual bool eqne_op() const { return 1; }
@@ -268,6 +296,8 @@ public:
 class Gt_creator :public Comp_creator
 {
 public:
+  Gt_creator() {}                             /* Remove gcc warning */
+  virtual ~Gt_creator() {}                    /* Remove gcc warning */
   virtual Item_bool_func2* create(Item *a, Item *b) const;
   virtual const char* symbol(bool invert) const { return invert? "<=" : ">"; }
   virtual bool eqne_op() const { return 0; }
@@ -278,6 +308,8 @@ public:
 class Lt_creator :public Comp_creator
 {
 public:
+  Lt_creator() {}                             /* Remove gcc warning */
+  virtual ~Lt_creator() {}                    /* Remove gcc warning */
   virtual Item_bool_func2* create(Item *a, Item *b) const;
   virtual const char* symbol(bool invert) const { return invert? ">=" : "<"; }
   virtual bool eqne_op() const { return 0; }
@@ -288,6 +320,8 @@ public:
 class Ge_creator :public Comp_creator
 {
 public:
+  Ge_creator() {}                             /* Remove gcc warning */
+  virtual ~Ge_creator() {}                    /* Remove gcc warning */
   virtual Item_bool_func2* create(Item *a, Item *b) const;
   virtual const char* symbol(bool invert) const { return invert? "<" : ">="; }
   virtual bool eqne_op() const { return 0; }
@@ -298,6 +332,8 @@ public:
 class Le_creator :public Comp_creator
 {
 public:
+  Le_creator() {}                             /* Remove gcc warning */
+  virtual ~Le_creator() {}                    /* Remove gcc warning */
   virtual Item_bool_func2* create(Item *a, Item *b) const;
   virtual const char* symbol(bool invert) const { return invert? ">" : "<="; }
   virtual bool eqne_op() const { return 0; }
@@ -324,14 +360,14 @@ public:
   virtual enum Functype rev_functype() const { return UNKNOWN_FUNC; }
   bool have_rev_func() const { return rev_functype() != UNKNOWN_FUNC; }
 
-  virtual inline void print(String *str)
+  virtual inline void print(String *str, enum_query_type query_type)
   {
-    Item_func::print_op(str);
+    Item_func::print_op(str, query_type);
   }
 
   bool is_null() { return test(args[0]->is_null() || args[1]->is_null()); }
   bool is_bool_func() { return 1; }
-  const charset_info_st *compare_collation() { return cmp.cmp_collation.collation; }
+  const CHARSET_INFO *compare_collation() { return cmp.cmp_collation.collation; }
   uint32_t decimal_precision() const { return 1; }
   void top_level_item() { abort_on_null= true; }
 
@@ -351,17 +387,19 @@ public:
   { return true; }
 };
 
-class Item_func_not :public item::function::Boolean
+class Item_func_not :public Item_bool_func
 {
 public:
-  Item_func_not(Item *a) :item::function::Boolean(a) {}
+  Item_func_not(Item *a) :Item_bool_func(a) {}
   int64_t val_int();
   enum Functype functype() const { return NOT_FUNC; }
   const char *func_name() const { return "not"; }
   Item *neg_transformer(Session *session);
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
 };
 
+class Item_maxmin_subselect;
+
 /*
   trigcond<param>(arg) ::= param? arg : TRUE
 
@@ -391,11 +429,11 @@ public:
    - To wrap condition that is pushed down into subquery
 */
 
-class Item_func_trig_cond: public item::function::Boolean
+class Item_func_trig_cond: public Item_bool_func
 {
   bool *trig_var;
 public:
-  Item_func_trig_cond(Item *a, bool *f) : item::function::Boolean(a) { trig_var= f; }
+  Item_func_trig_cond(Item *a, bool *f) : Item_bool_func(a) { trig_var= f; }
   int64_t val_int() { return *trig_var ? args[0]->val_int() : 1; }
   enum Functype functype() const { return TRIG_COND_FUNC; };
   const char *func_name() const { return "trigcond"; };
@@ -424,7 +462,7 @@ public:
   int64_t val_int();
   enum Functype functype() const { return NOT_ALL_FUNC; }
   const char *func_name() const { return "<not>"; }
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   void set_sum_test(Item_sum_hybrid *item) { test_sum_item= item; };
   void set_sub_test(Item_maxmin_subselect *item) { test_sub_item= item; };
   bool empty_underlying_subquery();
@@ -586,9 +624,9 @@ public:
   const char *func_name() const { return "between"; }
   bool fix_fields(Session *, Item **);
   void fix_length_and_dec();
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   bool is_bool_func() { return 1; }
-  const charset_info_st *compare_collation() { return cmp_collation.collation; }
+  const CHARSET_INFO *compare_collation() { return cmp_collation.collation; }
   uint32_t decimal_precision() const { return 1; }
 };
 
@@ -601,9 +639,9 @@ public:
   optimize_type select_optimize() const { return OPTIMIZE_NONE; }
   const char *func_name() const { return "strcmp"; }
 
-  virtual inline void print(String *str)
+  virtual inline void print(String *str, enum_query_type query_type)
   {
-    Item_func::print(str);
+    Item_func::print(str, query_type);
   }
 };
 
@@ -612,7 +650,7 @@ struct interval_range
 {
   Item_result type;
   double dbl;
-  type::Decimal dec;
+  my_decimal dec;
 };
 
 class Item_func_interval :public Item_int_func
@@ -643,7 +681,7 @@ public:
   double real_op();
   int64_t int_op();
   String *str_op(String *);
-  type::Decimal *decimal_op(type::Decimal *);
+  my_decimal *decimal_op(my_decimal *);
   void fix_length_and_dec();
   void find_num_type() {}
   enum Item_result result_type () const { return hybrid_type; }
@@ -662,7 +700,7 @@ public:
   double real_op();
   int64_t int_op();
   String *str_op(String *str);
-  type::Decimal *decimal_op(type::Decimal *);
+  my_decimal *decimal_op(my_decimal *);
   enum_field_types field_type() const;
   void fix_length_and_dec();
   const char *func_name() const { return "ifnull"; }
@@ -677,20 +715,17 @@ public:
 
 class Item_func_if :public Item_func
 {
-  Item_result cached_result_type;
+  enum Item_result cached_result_type;
   enum_field_types cached_field_type;
-
 public:
-  Item_func_if(Item *a, Item *b, Item *c) :
-    Item_func(a,b,c),
-    cached_result_type(INT_RESULT)
+  Item_func_if(Item *a,Item *b,Item *c)
+    :Item_func(a,b,c), cached_result_type(INT_RESULT)
   {}
-
   double val_real();
   int64_t val_int();
   String *val_str(String *str);
-  type::Decimal *val_decimal(type::Decimal *);
-  Item_result result_type () const { return cached_result_type; }
+  my_decimal *val_decimal(my_decimal *);
+  enum Item_result result_type () const { return cached_result_type; }
   enum_field_types field_type() const { return cached_field_type; }
   bool fix_fields(Session *, Item **);
   void fix_length_and_dec();
@@ -709,15 +744,15 @@ public:
   double val_real();
   int64_t val_int();
   String *val_str(String *str);
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   enum Item_result result_type () const { return cached_result_type; }
   void fix_length_and_dec();
   uint32_t decimal_precision() const { return args[0]->decimal_precision(); }
   const char *func_name() const { return "nullif"; }
 
-  virtual inline void print(String *str)
+  virtual inline void print(String *str, enum_query_type query_type)
   {
-    Item_func::print(str);
+    Item_func::print(str, query_type);
   }
 
   table_map not_null_tables() const { return 0; }
@@ -736,12 +771,12 @@ public:
   char *base;
   uint32_t size;
   qsort2_cmp compare;
-  const charset_info_st *collation;
+  const CHARSET_INFO *collation;
   uint32_t count;
   uint32_t used_count;
   in_vector() {}
   in_vector(uint32_t elements,uint32_t element_length,qsort2_cmp cmp_func,
-  	    const charset_info_st * const cmp_coll)
+  	    const CHARSET_INFO * const cmp_coll)
     :base((char*) memory::sql_calloc(elements*element_length)),
      size(element_length), compare(cmp_func), collation(cmp_coll),
      count(elements), used_count(elements) {}
@@ -784,7 +819,7 @@ class in_string :public in_vector
   char buff[STRING_BUFFER_USUAL_SIZE];
   String tmp;
 public:
-  in_string(uint32_t elements,qsort2_cmp cmp_func, const charset_info_st * const cs);
+  in_string(uint32_t elements,qsort2_cmp cmp_func, const CHARSET_INFO * const cs);
   ~in_string();
   void set(uint32_t pos,Item *item);
   unsigned char *get_value(Item *item);
@@ -854,8 +889,9 @@ public:
   /* Cache for the left item. */
   Item *lval_cache;
 
-  in_datetime(Item *warn_item_arg, uint32_t elements);
-
+  in_datetime(Item *warn_item_arg, uint32_t elements)
+    :in_int64_t(elements), session(current_session), warn_item(warn_item_arg),
+     lval_cache(0) {};
   void set(uint32_t pos,Item *item);
   unsigned char *get_value(Item *item);
   friend int cmp_int64_t(void *cmp_arg, packed_int64_t *a,packed_int64_t *b);
@@ -883,7 +919,7 @@ public:
 
 class in_decimal :public in_vector
 {
-  type::Decimal val;
+  my_decimal val;
 public:
   in_decimal(uint32_t elements);
   void set(uint32_t pos, Item *item);
@@ -894,7 +930,7 @@ public:
   }
   void value_to_item(uint32_t pos, Item *item)
   {
-    type::Decimal *dec= ((type::Decimal *)base) + pos;
+    my_decimal *dec= ((my_decimal *)base) + pos;
     Item_decimal *item_dec= (Item_decimal*)item;
     item_dec->set_decimal_value(dec);
   }
@@ -910,19 +946,14 @@ public:
 class cmp_item :public memory::SqlAlloc
 {
 public:
-  const charset_info_st *cmp_charset;
-
-  cmp_item()
-  {
-    cmp_charset= &my_charset_bin;
-  }
-
+  const CHARSET_INFO *cmp_charset;
+  cmp_item() { cmp_charset= &my_charset_bin; }
   virtual ~cmp_item() {}
   virtual void store_value(Item *item)= 0;
   virtual int cmp(Item *item)= 0;
   // for optimized IN with row
   virtual int compare(cmp_item *item)= 0;
-  static cmp_item* get_comparator(Item_result type, const charset_info_st * const cs);
+  static cmp_item* get_comparator(Item_result type, const CHARSET_INFO * const cs);
   virtual cmp_item *make_same()= 0;
   virtual void store_value_by_template(cmp_item *, Item *item)
   {
@@ -936,8 +967,8 @@ protected:
   String *value_res;
 public:
   cmp_item_string () {}
-  cmp_item_string (const charset_info_st * const cs) { cmp_charset= cs; }
-  void set_charset(const charset_info_st * const cs) { cmp_charset= cs; }
+  cmp_item_string (const CHARSET_INFO * const cs) { cmp_charset= cs; }
+  void set_charset(const CHARSET_INFO * const cs) { cmp_charset= cs; }
   friend class cmp_item_sort_string;
   friend class cmp_item_sort_string_in_static;
 };
@@ -950,7 +981,7 @@ protected:
 public:
   cmp_item_sort_string():
     cmp_item_string() {}
-  cmp_item_sort_string(const charset_info_st * const cs):
+  cmp_item_sort_string(const CHARSET_INFO * const cs):
     cmp_item_string(cs),
     value(value_buff, sizeof(value_buff), cs) {}
   void store_value(Item *item)
@@ -971,7 +1002,7 @@ public:
     return sortcmp(value_res, l_cmp->value_res, cmp_charset);
   }
   cmp_item *make_same();
-  void set_charset(const charset_info_st * const cs)
+  void set_charset(const CHARSET_INFO * const cs)
   {
     cmp_charset= cs;
     value.set_quick(value_buff, sizeof(value_buff), cs);
@@ -982,6 +1013,7 @@ class cmp_item_int :public cmp_item
 {
   int64_t value;
 public:
+  cmp_item_int() {}                           /* Remove gcc warning */
   void store_value(Item *item)
   {
     value= item->val_int();
@@ -1006,8 +1038,7 @@ public:
 */
 class cmp_item_datetime :public cmp_item
 {
-  int64_t value;
-
+  uint64_t value;
 public:
   Session *session;
   /* Item used for issuing warnings. */
@@ -1015,8 +1046,8 @@ public:
   /* Cache for the left item. */
   Item *lval_cache;
 
-  cmp_item_datetime(Item *warn_item_arg);
-
+  cmp_item_datetime(Item *warn_item_arg)
+    :session(current_session), warn_item(warn_item_arg), lval_cache(0) {}
   void store_value(Item *item);
   int cmp(Item *arg);
   int compare(cmp_item *ci);
@@ -1027,6 +1058,7 @@ class cmp_item_real :public cmp_item
 {
   double value;
 public:
+  cmp_item_real() {}                          /* Remove gcc warning */
   void store_value(Item *item)
   {
     value= item->val_real();
@@ -1046,8 +1078,9 @@ public:
 
 class cmp_item_decimal :public cmp_item
 {
-  type::Decimal value;
+  my_decimal value;
 public:
+  cmp_item_decimal() {}                       /* Remove gcc warning */
   void store_value(Item *item);
   int cmp(Item *arg);
   int compare(cmp_item *c);
@@ -1065,7 +1098,7 @@ class cmp_item_sort_string_in_static :public cmp_item_string
  protected:
   String value;
 public:
-  cmp_item_sort_string_in_static(const charset_info_st * const cs):
+  cmp_item_sort_string_in_static(const CHARSET_INFO * const cs):
     cmp_item_string(cs) {}
   void store_value(Item *item)
   {
@@ -1116,22 +1149,22 @@ class Item_func_case :public Item_func
   Item_result cmp_type;
   DTCollation cmp_collation;
   enum_field_types cached_field_type;
-  cmp_item *cmp_items[DECIMAL_RESULT+1]; /* For all result types */
+  cmp_item *cmp_items[5]; /* For all result types */
   cmp_item *case_item;
 public:
   Item_func_case(List<Item> &list, Item *first_expr_arg, Item *else_expr_arg)
     :Item_func(), first_expr_num(-1), else_expr_num(-1),
     cached_result_type(INT_RESULT), left_result_type(INT_RESULT), case_item(0)
   {
-    ncases= list.size();
+    ncases= list.elements;
     if (first_expr_arg)
     {
-      first_expr_num= list.size();
+      first_expr_num= list.elements;
       list.push_back(first_expr_arg);
     }
     if (else_expr_arg)
     {
-      else_expr_num= list.size();
+      else_expr_num= list.elements;
       list.push_back(else_expr_arg);
     }
     set_arguments(list);
@@ -1140,7 +1173,7 @@ public:
   double val_real();
   int64_t val_int();
   String *val_str(String *);
-  type::Decimal *val_decimal(type::Decimal *);
+  my_decimal *val_decimal(my_decimal *);
   bool fix_fields(Session *session, Item **ref);
   void fix_length_and_dec();
   uint32_t decimal_precision() const;
@@ -1148,9 +1181,9 @@ public:
   enum Item_result result_type () const { return cached_result_type; }
   enum_field_types field_type() const { return cached_field_type; }
   const char *func_name() const { return "case"; }
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   Item *find_item(String *str);
-  const charset_info_st *compare_collation() { return cmp_collation.collation; }
+  const CHARSET_INFO *compare_collation() { return cmp_collation.collation; }
   void cleanup();
   void agg_str_lengths(Item *arg);
   void agg_num_lengths(Item *arg);
@@ -1201,10 +1234,11 @@ public:
   uint32_t decimal_precision() const { return 1; }
   void cleanup()
   {
+    uint32_t i;
     Item_int_func::cleanup();
     delete array;
     array= 0;
-    for (int i= STRING_RESULT; i <= DECIMAL_RESULT; i++)
+    for (i= 0; i <= (uint32_t)DECIMAL_RESULT + 1; i++)
     {
       delete cmp_items[i];
       cmp_items[i]= 0;
@@ -1213,12 +1247,12 @@ public:
   }
   optimize_type select_optimize() const
     { return OPTIMIZE_KEY; }
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   enum Functype functype() const { return IN_FUNC; }
   const char *func_name() const { return " IN "; }
   bool nulls_in_row();
   bool is_bool_func() { return 1; }
-  const charset_info_st *compare_collation() { return cmp_collation.collation; }
+  const CHARSET_INFO *compare_collation() { return cmp_collation.collation; }
 };
 
 class cmp_item_row :public cmp_item
@@ -1252,12 +1286,12 @@ public:
 
 /* Functions used by where clause */
 
-class Item_func_isnull :public item::function::Boolean
+class Item_func_isnull :public Item_bool_func
 {
 protected:
   int64_t cached_value;
 public:
-  Item_func_isnull(Item *a) :item::function::Boolean(a) {}
+  Item_func_isnull(Item *a) :Item_bool_func(a) {}
   int64_t val_int();
   enum Functype functype() const { return ISNULL_FUNC; }
   void fix_length_and_dec()
@@ -1289,11 +1323,13 @@ public:
   table_map not_null_tables() const { return 0; }
   optimize_type select_optimize() const { return OPTIMIZE_NULL; }
   Item *neg_transformer(Session *session);
-  const charset_info_st *compare_collation() { return args[0]->collation.collation; }
+  const CHARSET_INFO *compare_collation() { return args[0]->collation.collation; }
 };
 
 /* Functions used by HAVING for rewriting IN subquery */
 
+class Item_in_subselect;
+
 /*
   This is like IS NOT NULL but it also remembers if it ever has
   encountered a NULL.
@@ -1317,11 +1353,11 @@ public:
 };
 
 
-class Item_func_isnotnull :public item::function::Boolean
+class Item_func_isnotnull :public Item_bool_func
 {
   bool abort_on_null;
 public:
-  Item_func_isnotnull(Item *a) :item::function::Boolean(a), abort_on_null(0) {}
+  Item_func_isnotnull(Item *a) :Item_bool_func(a), abort_on_null(0) {}
   int64_t val_int();
   enum Functype functype() const { return ISNOTNULL_FUNC; }
   void fix_length_and_dec()
@@ -1333,8 +1369,8 @@ public:
   table_map not_null_tables() const
   { return abort_on_null ? not_null_tables_cache : 0; }
   Item *neg_transformer(Session *session);
-  virtual void print(String *str);
-  const charset_info_st *compare_collation() { return args[0]->collation.collation; }
+  virtual void print(String *str, enum_query_type query_type);
+  const CHARSET_INFO *compare_collation() { return args[0]->collation.collation; }
   void top_level_item() { abort_on_null=1; }
 };
 
@@ -1379,7 +1415,9 @@ public:
 };
 
 
-class Item_cond :public item::function::Boolean
+typedef class Item COND;
+
+class Item_cond :public Item_bool_func
 {
 protected:
   List<Item> list;
@@ -1391,19 +1429,19 @@ public:
   using Item::split_sum_func;
 
   /* Item_cond() is only used to create top level items */
-  Item_cond(): item::function::Boolean(), abort_on_null(1)
+  Item_cond(): Item_bool_func(), abort_on_null(1)
   { const_item_cache=0; }
   Item_cond(Item *i1,Item *i2)
-    :item::function::Boolean(), abort_on_null(0)
+    :Item_bool_func(), abort_on_null(0)
   {
     list.push_back(i1);
     list.push_back(i2);
   }
   Item_cond(Session *session, Item_cond *item);
   Item_cond(List<Item> &nlist)
-    :item::function::Boolean(), list(nlist), abort_on_null(0) {}
-  void add(Item *item) { list.push_back(item); }
-  void add_at_head(Item *item) { list.push_front(item); }
+    :Item_bool_func(), list(nlist), abort_on_null(0) {}
+  bool add(Item *item) { return list.push_back(item); }
+  bool add_at_head(Item *item) { return list.push_front(item); }
   void add_at_head(List<Item> *nlist) { list.prepand(nlist); }
   bool fix_fields(Session *, Item **ref);
   void fix_after_pullout(Select_Lex *new_parent, Item **ref);
@@ -1412,7 +1450,7 @@ public:
   List<Item>* argument_list() { return &list; }
   table_map used_tables() const;
   void update_used_tables();
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   void split_sum_func(Session *session, Item **ref_pointer_array, List<Item> &fields);
   friend int setup_conds(Session *session, TableList *tables, TableList *leaves,
                          COND **conds);
@@ -1505,24 +1543,16 @@ public:
   object represents f1=f2= ...=fn to the projection of known fields fi1=...=fik.
 */
 
-class Item_equal: public item::function::Boolean
+class Item_equal: public Item_bool_func
 {
+  List<Item_field> fields; /* list of equal field items                    */
+  Item *const_item;        /* optional constant item equal to fields items */
+  cmp_item *eval_item;
+  bool cond_false;
 public:
-  typedef List<Item_field> fields_t;
-
-  Item_equal() :
-    const_item(0),
-    eval_item(0),
-    cond_false(0)
-  {
-    const_item_cache=0;
-  }
-
-  fields_t::iterator begin()
-  {
-    return fields.begin();
-  }
-
+  inline Item_equal()
+    : Item_bool_func(), const_item(0), eval_item(0), cond_false(0)
+  { const_item_cache=0 ;}
   Item_equal(Item_field *f1, Item_field *f2);
   Item_equal(Item *c, Item_field *f);
   Item_equal(Item_equal *item_equal);
@@ -1531,7 +1561,7 @@ public:
   void add(Item_field *f);
   uint32_t members();
   bool contains(Field *field);
-  Item_field* get_first() { return &fields.front(); }
+  Item_field* get_first() { return fields.head(); }
   void merge(Item_equal *item);
   void update_const();
   enum Functype functype() const { return MULT_EQUAL_FUNC; }
@@ -1539,20 +1569,15 @@ public:
   const char *func_name() const { return "multiple equal"; }
   optimize_type select_optimize() const { return OPTIMIZE_EQUAL; }
   void sort(Item_field_cmpfunc cmp, void *arg);
+  friend class Item_equal_iterator;
   void fix_length_and_dec();
   bool fix_fields(Session *session, Item **ref);
   void update_used_tables();
   bool walk(Item_processor processor, bool walk_subquery, unsigned char *arg);
   Item *transform(Item_transformer transformer, unsigned char *arg);
-  virtual void print(String *str);
-  const charset_info_st *compare_collation()
-  { return fields.front().collation.collation; }
-private:
-  fields_t fields; /* list of equal field items                    */
-  Item *const_item;        /* optional constant item equal to fields items */
-  cmp_item *eval_item;
-  bool cond_false;
-
+  virtual void print(String *str, enum_query_type query_type);
+  const CHARSET_INFO *compare_collation()
+  { return fields.head()->collation.collation; }
 };
 
 class COND_EQUAL: public memory::SqlAlloc
@@ -1569,7 +1594,23 @@ public:
   }
 };
 
-typedef List<Item_field>::iterator Item_equal_iterator;
+
+class Item_equal_iterator : public List_iterator_fast<Item_field>
+{
+public:
+  inline Item_equal_iterator(Item_equal &item_equal)
+    :List_iterator_fast<Item_field> (item_equal.fields)
+  {}
+  inline Item_field* operator++(int)
+  {
+    Item_field *item= (*(List_iterator_fast<Item_field> *) this)++;
+    return  item;
+  }
+  inline void rewind(void)
+  {
+    List_iterator_fast<Item_field>::rewind();
+  }
+};
 
 class Item_cond_and :public Item_cond
 {
@@ -1589,8 +1630,8 @@ public:
   Item* copy_andor_structure(Session *session)
   {
     Item_cond_and *item;
-    item= new Item_cond_and(session, this);
-    item->copy_andor_arguments(session, this);
+    if ((item= new Item_cond_and(session, this)))
+       item->copy_andor_arguments(session, this);
     return item;
   }
   Item *neg_transformer(Session *session);
@@ -1619,8 +1660,8 @@ public:
   Item* copy_andor_structure(Session *session)
   {
     Item_cond_or *item;
-    item= new Item_cond_or(session, this);
-    item->copy_andor_arguments(session, this);
+    if ((item= new Item_cond_or(session, this)))
+      item->copy_andor_arguments(session, this);
     return item;
   }
   Item *neg_transformer(Session *session);
@@ -1669,3 +1710,4 @@ Item *and_expressions(Item *a, Item *b, Item **org_item);
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ITEM_CMPFUNC_H */
