@@ -25,12 +25,12 @@
 #include <drizzled/item/func.h>
 #include <drizzled/error.h>
 
+#include "drizzled/function_container.h"
+
 #include <drizzled/function/str/binary.h>
 #include <drizzled/function/str/concat.h>
 #include <drizzled/function/str/conv.h>
-#include <drizzled/function/str/elt.h>
 #include <drizzled/function/str/export_set.h>
-#include <drizzled/function/str/format.h>
 #include <drizzled/function/str/load_file.h>
 #include <drizzled/function/str/make_set.h>
 #include <drizzled/function/str/pad.h>
@@ -61,47 +61,28 @@
 /* Function declarations */
 
 #include <drizzled/function/func.h>
-#include <drizzled/function/math/abs.h>
-#include <drizzled/function/math/acos.h>
 #include <drizzled/function/additive_op.h>
-#include <drizzled/function/math/asin.h>
-#include <drizzled/function/math/atan.h>
-#include <drizzled/function/math/ceiling.h>
-#include <drizzled/function/coercibility.h>
-#include <drizzled/function/math/cos.h>
 #include <drizzled/function/math/dec.h>
 #include <drizzled/function/math/decimal_typecast.h>
-#include <drizzled/function/math/exp.h>
 #include <drizzled/function/field.h>
 #include <drizzled/function/find_in_set.h>
-#include <drizzled/function/math/floor.h>
 #include <drizzled/function/found_rows.h>
 #include <drizzled/function/get_system_var.h>
 #include <drizzled/function/math/int_val.h>
 #include <drizzled/function/math/integer.h>
 #include <drizzled/function/last_insert.h>
-#include <drizzled/function/math/ln.h>
 #include <drizzled/function/locate.h>
-#include <drizzled/function/math/log.h>
 #include <drizzled/function/min_max.h>
 #include <drizzled/function/num1.h>
 #include <drizzled/function/num_op.h>
 #include <drizzled/function/numhybrid.h>
-#include <drizzled/function/math/ord.h>
-#include <drizzled/function/math/pow.h>
-#include <drizzled/function/math/rand.h>
 #include <drizzled/function/math/real.h>
 #include <drizzled/function/row_count.h>
 #include <drizzled/function/set_user_var.h>
 #include <drizzled/function/sign.h>
-#include <drizzled/function/math/sin.h>
-#include <drizzled/function/math/sqrt.h>
-#include <drizzled/function/str/quote.h>
 #include <drizzled/function/math/tan.h>
 #include <drizzled/function/units.h>
 
-#include <map>
-
 using namespace std;
 
 namespace drizzled
@@ -259,61 +240,6 @@ protected:
   it helps to compare code between versions, and helps with merges conflicts.
 */
 
-class Create_func_abs : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_abs s_singleton;
-
-protected:
-  Create_func_abs() {}
-  virtual ~Create_func_abs() {}
-};
-
-
-class Create_func_acos : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_acos s_singleton;
-
-protected:
-  Create_func_acos() {}
-  virtual ~Create_func_acos() {}
-};
-
-class Create_func_asin : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_asin s_singleton;
-
-protected:
-  Create_func_asin() {}
-  virtual ~Create_func_asin() {}
-};
-
-
-class Create_func_atan : public Create_native_func
-{
-public:
-  virtual Item *create_native(Session *session, LEX_STRING name, List<Item> *item_list);
-
-  static Create_func_atan s_singleton;
-
-protected:
-  Create_func_atan() {}
-  virtual ~Create_func_atan() {}
-};
 
 class Create_func_bin : public Create_func_arg1
 {
@@ -329,37 +255,6 @@ protected:
   virtual ~Create_func_bin() {}
 };
 
-
-class Create_func_ceiling : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_ceiling s_singleton;
-
-protected:
-  Create_func_ceiling() {}
-  virtual ~Create_func_ceiling() {}
-};
-
-
-class Create_func_coercibility : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_coercibility s_singleton;
-
-protected:
-  Create_func_coercibility() {}
-  virtual ~Create_func_coercibility() {}
-};
-
-
 class Create_func_concat : public Create_native_func
 {
 public:
@@ -400,22 +295,6 @@ protected:
   virtual ~Create_func_conv() {}
 };
 
-
-class Create_func_cos : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_cos s_singleton;
-
-protected:
-  Create_func_cos() {}
-  virtual ~Create_func_cos() {}
-};
-
-
 class Create_func_cot : public Create_func_arg1
 {
 public:
@@ -549,37 +428,9 @@ protected:
   virtual ~Create_func_degrees() {}
 };
 
-
-class Create_func_elt : public Create_native_func
-{
-public:
-  virtual Item *create_native(Session *session, LEX_STRING name, List<Item> *item_list);
-
-  static Create_func_elt s_singleton;
-
-protected:
-  Create_func_elt() {}
-  virtual ~Create_func_elt() {}
-};
-
-
-class Create_func_exp : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_exp s_singleton;
-
-protected:
-  Create_func_exp() {}
-  virtual ~Create_func_exp() {}
-};
-
-
 class Create_func_export_set : public Create_native_func
 {
+
 public:
   virtual Item *create_native(Session *session, LEX_STRING name, List<Item> *item_list);
 
@@ -618,37 +469,6 @@ protected:
   virtual ~Create_func_find_in_set() {}
 };
 
-
-class Create_func_floor : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_floor s_singleton;
-
-protected:
-  Create_func_floor() {}
-  virtual ~Create_func_floor() {}
-};
-
-
-class Create_func_format : public Create_func_arg2
-{
-public:
-  using Create_func_arg2::create;
-
-  virtual Item *create(Session *session, Item *arg1, Item *arg2);
-
-  static Create_func_format s_singleton;
-
-protected:
-  Create_func_format() {}
-  virtual ~Create_func_format() {}
-};
-
-
 class Create_func_found_rows : public Create_func_arg0
 {
 public:
@@ -805,22 +625,6 @@ protected:
   virtual ~Create_func_least() {}
 };
 
-
-class Create_func_ln : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_ln s_singleton;
-
-protected:
-  Create_func_ln() {}
-  virtual ~Create_func_ln() {}
-};
-
-
 class Create_func_load_file : public Create_func_arg1
 {
 public:
@@ -849,49 +653,6 @@ protected:
 };
 
 
-class Create_func_log : public Create_native_func
-{
-public:
-  virtual Item *create_native(Session *session, LEX_STRING name, List<Item> *item_list);
-
-  static Create_func_log s_singleton;
-
-protected:
-  Create_func_log() {}
-  virtual ~Create_func_log() {}
-};
-
-
-class Create_func_log10 : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_log10 s_singleton;
-
-protected:
-  Create_func_log10() {}
-  virtual ~Create_func_log10() {}
-};
-
-
-class Create_func_log2 : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_log2 s_singleton;
-
-protected:
-  Create_func_log2() {}
-  virtual ~Create_func_log2() {}
-};
-
-
 class Create_func_lpad : public Create_func_arg3
 {
 public:
@@ -1008,22 +769,6 @@ protected:
   virtual ~Create_func_oct() {}
 };
 
-
-class Create_func_ord : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_ord s_singleton;
-
-protected:
-  Create_func_ord() {}
-  virtual ~Create_func_ord() {}
-};
-
-
 class Create_func_period_add : public Create_func_arg2
 {
 public:
@@ -1068,37 +813,6 @@ protected:
   virtual ~Create_func_pi() {}
 };
 
-
-class Create_func_pow : public Create_func_arg2
-{
-public:
-  using Create_func_arg2::create;
-
-  virtual Item *create(Session *session, Item *arg1, Item *arg2);
-
-  static Create_func_pow s_singleton;
-
-protected:
-  Create_func_pow() {}
-  virtual ~Create_func_pow() {}
-};
-
-
-class Create_func_quote : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_quote s_singleton;
-
-protected:
-  Create_func_quote() {}
-  virtual ~Create_func_quote() {}
-};
-
-
 class Create_func_radians : public Create_func_arg1
 {
 public:
@@ -1114,19 +828,6 @@ protected:
 };
 
 
-class Create_func_rand : public Create_native_func
-{
-public:
-  virtual Item *create_native(Session *session, LEX_STRING name, List<Item> *item_list);
-
-  static Create_func_rand s_singleton;
-
-protected:
-  Create_func_rand() {}
-  virtual ~Create_func_rand() {}
-};
-
-
 class Create_func_round : public Create_native_func
 {
 public:
@@ -1198,22 +899,6 @@ protected:
   virtual ~Create_func_sign() {}
 };
 
-
-class Create_func_sin : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_sin s_singleton;
-
-protected:
-  Create_func_sin() {}
-  virtual ~Create_func_sin() {}
-};
-
-
 class Create_func_space : public Create_func_arg1
 {
 public:
@@ -1228,22 +913,6 @@ protected:
   virtual ~Create_func_space() {}
 };
 
-
-class Create_func_sqrt : public Create_func_arg1
-{
-public:
-  using Create_func_arg1::create;
-
-  virtual Item *create(Session *session, Item *arg1);
-
-  static Create_func_sqrt s_singleton;
-
-protected:
-  Create_func_sqrt() {}
-  virtual ~Create_func_sqrt() {}
-};
-
-
 class Create_func_strcmp : public Create_func_arg2
 {
 public:
@@ -1541,69 +1210,6 @@ Create_func_arg3::create(Session *session, LEX_STRING name, List<Item> *item_lis
   return create(session, param_1, param_2, param_3);
 }
 
-
-Create_func_abs Create_func_abs::s_singleton;
-
-Item*
-Create_func_abs::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_abs(arg1);
-}
-
-
-Create_func_acos Create_func_acos::s_singleton;
-
-Item*
-Create_func_acos::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_acos(arg1);
-}
-
-Create_func_asin Create_func_asin::s_singleton;
-
-Item*
-Create_func_asin::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_asin(arg1);
-}
-
-
-Create_func_atan Create_func_atan::s_singleton;
-
-Item*
-Create_func_atan::create_native(Session *session, LEX_STRING name,
-                                List<Item> *item_list)
-{
-  Item* func= NULL;
-  int arg_count= 0;
-
-  if (item_list != NULL)
-    arg_count= item_list->elements;
-
-  switch (arg_count) {
-  case 1:
-  {
-    Item *param_1= item_list->pop();
-    func= new (session->mem_root) Item_func_atan(param_1);
-    break;
-  }
-  case 2:
-  {
-    Item *param_1= item_list->pop();
-    Item *param_2= item_list->pop();
-    func= new (session->mem_root) Item_func_atan(param_1, param_2);
-    break;
-  }
-  default:
-  {
-    my_error(ER_WRONG_PARAMCOUNT_TO_FUNCTION, MYF(0), name.str);
-    break;
-  }
-  }
-
-  return func;
-}
-
 Create_func_bin Create_func_bin::s_singleton;
 
 Item*
@@ -1614,25 +1220,6 @@ Create_func_bin::create(Session *session, Item *arg1)
   return new (session->mem_root) Item_func_conv(arg1, i10, i2);
 }
 
-
-Create_func_ceiling Create_func_ceiling::s_singleton;
-
-Item*
-Create_func_ceiling::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_ceiling(arg1);
-}
-
-
-Create_func_coercibility Create_func_coercibility::s_singleton;
-
-Item*
-Create_func_coercibility::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_coercibility(arg1);
-}
-
-
 Create_func_concat Create_func_concat::s_singleton;
 
 Item*
@@ -1650,7 +1237,7 @@ Create_func_concat::create_native(Session *session, LEX_STRING name,
     return NULL;
   }
 
-  return new (session->mem_root) Item_func_concat(*item_list);
+  return new (session->mem_root) Item_func_concat(*session, *item_list);
 }
 
 
@@ -1672,7 +1259,7 @@ Create_func_concat_ws::create_native(Session *session, LEX_STRING name,
     return NULL;
   }
 
-  return new (session->mem_root) Item_func_concat_ws(*item_list);
+  return new (session->mem_root) Item_func_concat_ws(*session, *item_list);
 }
 
 
@@ -1684,16 +1271,6 @@ Create_func_conv::create(Session *session, Item *arg1, Item *arg2, Item *arg3)
   return new (session->mem_root) Item_func_conv(arg1, arg2, arg3);
 }
 
-
-Create_func_cos Create_func_cos::s_singleton;
-
-Item*
-Create_func_cos::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_cos(arg1);
-}
-
-
 Create_func_cot Create_func_cot::s_singleton;
 
 Item*
@@ -1701,7 +1278,7 @@ Create_func_cot::create(Session *session, Item *arg1)
 {
   Item *i1= new (session->mem_root) Item_int((char*) "1", 1, 1);
   Item *i2= new (session->mem_root) Item_func_tan(arg1);
-  return new (session->mem_root) Item_func_div(i1, i2);
+  return new (session->mem_root) Item_func_div(session, i1, i2);
 }
 
 Create_func_date_format Create_func_date_format::s_singleton;
@@ -1770,37 +1347,6 @@ Create_func_degrees::create(Session *session, Item *arg1)
                                              180/M_PI, 0.0);
 }
 
-
-Create_func_elt Create_func_elt::s_singleton;
-
-Item*
-Create_func_elt::create_native(Session *session, LEX_STRING name,
-                               List<Item> *item_list)
-{
-  int arg_count= 0;
-
-  if (item_list != NULL)
-    arg_count= item_list->elements;
-
-  if (arg_count < 2)
-  {
-    my_error(ER_WRONG_PARAMCOUNT_TO_FUNCTION, MYF(0), name.str);
-    return NULL;
-  }
-
-  return new (session->mem_root) Item_func_elt(*item_list);
-}
-
-
-Create_func_exp Create_func_exp::s_singleton;
-
-Item*
-Create_func_exp::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_exp(arg1);
-}
-
-
 Create_func_export_set Create_func_export_set::s_singleton;
 
 Item*
@@ -1883,25 +1429,6 @@ Create_func_find_in_set::create(Session *session, Item *arg1, Item *arg2)
   return new (session->mem_root) Item_func_find_in_set(arg1, arg2);
 }
 
-
-Create_func_floor Create_func_floor::s_singleton;
-
-Item*
-Create_func_floor::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_floor(arg1);
-}
-
-
-Create_func_format Create_func_format::s_singleton;
-
-Item*
-Create_func_format::create(Session *session, Item *arg1, Item *arg2)
-{
-  return new (session->mem_root) Item_func_format(arg1, arg2);
-}
-
-
 Create_func_found_rows Create_func_found_rows::s_singleton;
 
 Item*
@@ -2078,22 +1605,12 @@ Create_func_least::create_native(Session *session, LEX_STRING name,
   return new (session->mem_root) Item_func_min(*item_list);
 }
 
-
-Create_func_ln Create_func_ln::s_singleton;
-
-Item*
-Create_func_ln::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_ln(arg1);
-}
-
-
 Create_func_load_file Create_func_load_file::s_singleton;
 
 Item*
 Create_func_load_file::create(Session *session, Item *arg1)
 {
-  return new (session->mem_root) Item_load_file(arg1);
+  return new (session->mem_root) Item_load_file(*session, arg1);
 }
 
 
@@ -2137,68 +1654,12 @@ Create_func_locate::create_native(Session *session, LEX_STRING name,
   return func;
 }
 
-
-Create_func_log Create_func_log::s_singleton;
-
-Item*
-Create_func_log::create_native(Session *session, LEX_STRING name,
-                               List<Item> *item_list)
-{
-  Item *func= NULL;
-  int arg_count= 0;
-
-  if (item_list != NULL)
-    arg_count= item_list->elements;
-
-  switch (arg_count) {
-  case 1:
-  {
-    Item *param_1= item_list->pop();
-    func= new (session->mem_root) Item_func_log(param_1);
-    break;
-  }
-  case 2:
-  {
-    Item *param_1= item_list->pop();
-    Item *param_2= item_list->pop();
-    func= new (session->mem_root) Item_func_log(param_1, param_2);
-    break;
-  }
-  default:
-  {
-    my_error(ER_WRONG_PARAMCOUNT_TO_FUNCTION, MYF(0), name.str);
-    break;
-  }
-  }
-
-  return func;
-}
-
-
-Create_func_log10 Create_func_log10::s_singleton;
-
-Item*
-Create_func_log10::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_log10(arg1);
-}
-
-
-Create_func_log2 Create_func_log2::s_singleton;
-
-Item*
-Create_func_log2::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_log2(arg1);
-}
-
-
 Create_func_lpad Create_func_lpad::s_singleton;
 
 Item*
 Create_func_lpad::create(Session *session, Item *arg1, Item *arg2, Item *arg3)
 {
-  return new (session->mem_root) Item_func_lpad(arg1, arg2, arg3);
+  return new (session->mem_root) Item_func_lpad(*session, arg1, arg2, arg3);
 }
 
 
@@ -2223,7 +1684,7 @@ Create_func_makedate::create(Session *session, Item *arg1, Item *arg2)
 Create_func_make_set Create_func_make_set::s_singleton;
 
 Item*
-Create_func_make_set::create_native(Session *session, LEX_STRING name,
+Create_func_make_set::create_native(Session *session_arg, LEX_STRING name,
                                     List<Item> *item_list)
 {
   int arg_count= 0;
@@ -2238,7 +1699,7 @@ Create_func_make_set::create_native(Session *session, LEX_STRING name,
   }
 
   Item *param_1= item_list->pop();
-  return new (session->mem_root) Item_func_make_set(param_1, *item_list);
+  return new (session_arg->mem_root) Item_func_make_set(*session_arg, param_1, *item_list);
 }
 
 
@@ -2270,16 +1731,6 @@ Create_func_oct::create(Session *session, Item *arg1)
   return new (session->mem_root) Item_func_conv(arg1, i10, i8);
 }
 
-
-Create_func_ord Create_func_ord::s_singleton;
-
-Item*
-Create_func_ord::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_ord(arg1);
-}
-
-
 Create_func_period_add Create_func_period_add::s_singleton;
 
 Item*
@@ -2306,25 +1757,6 @@ Create_func_pi::create(Session *session)
   return new (session->mem_root) Item_static_float_func("pi()", M_PI, 6, 8);
 }
 
-
-Create_func_pow Create_func_pow::s_singleton;
-
-Item*
-Create_func_pow::create(Session *session, Item *arg1, Item *arg2)
-{
-  return new (session->mem_root) Item_func_pow(arg1, arg2);
-}
-
-
-Create_func_quote Create_func_quote::s_singleton;
-
-Item*
-Create_func_quote::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_quote(arg1);
-}
-
-
 Create_func_radians Create_func_radians::s_singleton;
 
 Item*
@@ -2334,42 +1766,6 @@ Create_func_radians::create(Session *session, Item *arg1)
                                              M_PI/180, 0.0);
 }
 
-
-Create_func_rand Create_func_rand::s_singleton;
-
-Item*
-Create_func_rand::create_native(Session *session, LEX_STRING name,
-                                List<Item> *item_list)
-{
-  Item *func= NULL;
-  int arg_count= 0;
-
-  if (item_list != NULL)
-    arg_count= item_list->elements;
-
-  switch (arg_count) {
-  case 0:
-  {
-    func= new (session->mem_root) Item_func_rand();
-    break;
-  }
-  case 1:
-  {
-    Item *param_1= item_list->pop();
-    func= new (session->mem_root) Item_func_rand(param_1);
-    break;
-  }
-  default:
-  {
-    my_error(ER_WRONG_PARAMCOUNT_TO_FUNCTION, MYF(0), name.str);
-    break;
-  }
-  }
-
-  return func;
-}
-
-
 Create_func_round Create_func_round::s_singleton;
 
 Item*
@@ -2422,7 +1818,7 @@ Create_func_rpad Create_func_rpad::s_singleton;
 Item*
 Create_func_rpad::create(Session *session, Item *arg1, Item *arg2, Item *arg3)
 {
-  return new (session->mem_root) Item_func_rpad(arg1, arg2, arg3);
+  return new (session->mem_root) Item_func_rpad(*session, arg1, arg2, arg3);
 }
 
 
@@ -2443,16 +1839,6 @@ Create_func_sign::create(Session *session, Item *arg1)
   return new (session->mem_root) Item_func_sign(arg1);
 }
 
-
-Create_func_sin Create_func_sin::s_singleton;
-
-Item*
-Create_func_sin::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_sin(arg1);
-}
-
-
 Create_func_space Create_func_space::s_singleton;
 
 Item*
@@ -2477,19 +1863,9 @@ Create_func_space::create(Session *session, Item *arg1)
     sp= new (session->mem_root) Item_string(" ", 1, cs, DERIVATION_COERCIBLE);
   }
 
-  return new (session->mem_root) Item_func_repeat(sp, arg1);
+  return new (session->mem_root) Item_func_repeat(*session, sp, arg1);
 }
 
-
-Create_func_sqrt Create_func_sqrt::s_singleton;
-
-Item*
-Create_func_sqrt::create(Session *session, Item *arg1)
-{
-  return new (session->mem_root) Item_func_sqrt(arg1);
-}
-
-
 Create_func_strcmp Create_func_strcmp::s_singleton;
 
 Item*
@@ -2593,19 +1969,10 @@ struct Native_func_registry
 
 static Native_func_registry func_array[] =
 {
-  { { C_STRING_WITH_LEN("ABS") }, BUILDER(Create_func_abs)},
-  { { C_STRING_WITH_LEN("ACOS") }, BUILDER(Create_func_acos)},
-  { { C_STRING_WITH_LEN("ASIN") }, BUILDER(Create_func_asin)},
-  { { C_STRING_WITH_LEN("ATAN") }, BUILDER(Create_func_atan)},
-  { { C_STRING_WITH_LEN("ATAN2") }, BUILDER(Create_func_atan)},
   { { C_STRING_WITH_LEN("BIN") }, BUILDER(Create_func_bin)},
-  { { C_STRING_WITH_LEN("CEIL") }, BUILDER(Create_func_ceiling)},
-  { { C_STRING_WITH_LEN("CEILING") }, BUILDER(Create_func_ceiling)},
-  { { C_STRING_WITH_LEN("COERCIBILITY") }, BUILDER(Create_func_coercibility)},
   { { C_STRING_WITH_LEN("CONCAT") }, BUILDER(Create_func_concat)},
   { { C_STRING_WITH_LEN("CONCAT_WS") }, BUILDER(Create_func_concat_ws)},
   { { C_STRING_WITH_LEN("CONV") }, BUILDER(Create_func_conv)},
-  { { C_STRING_WITH_LEN("COS") }, BUILDER(Create_func_cos)},
   { { C_STRING_WITH_LEN("COT") }, BUILDER(Create_func_cot)},
   { { C_STRING_WITH_LEN("DATEDIFF") }, BUILDER(Create_func_datediff)},
   { { C_STRING_WITH_LEN("DATE_FORMAT") }, BUILDER(Create_func_date_format)},
@@ -2614,13 +1981,9 @@ static Native_func_registry func_array[] =
   { { C_STRING_WITH_LEN("DAYOFWEEK") }, BUILDER(Create_func_dayofweek)},
   { { C_STRING_WITH_LEN("DAYOFYEAR") }, BUILDER(Create_func_dayofyear)},
   { { C_STRING_WITH_LEN("DEGREES") }, BUILDER(Create_func_degrees)},
-  { { C_STRING_WITH_LEN("ELT") }, BUILDER(Create_func_elt)},
-  { { C_STRING_WITH_LEN("EXP") }, BUILDER(Create_func_exp)},
   { { C_STRING_WITH_LEN("EXPORT_SET") }, BUILDER(Create_func_export_set)},
   { { C_STRING_WITH_LEN("FIELD") }, BUILDER(Create_func_field)},
   { { C_STRING_WITH_LEN("FIND_IN_SET") }, BUILDER(Create_func_find_in_set)},
-  { { C_STRING_WITH_LEN("FLOOR") }, BUILDER(Create_func_floor)},
-  { { C_STRING_WITH_LEN("FORMAT") }, BUILDER(Create_func_format)},
   { { C_STRING_WITH_LEN("FOUND_ROWS") }, BUILDER(Create_func_found_rows)},
   { { C_STRING_WITH_LEN("FROM_DAYS") }, BUILDER(Create_func_from_days)},
   { { C_STRING_WITH_LEN("FROM_UNIXTIME") }, BUILDER(Create_func_from_unixtime)},
@@ -2632,12 +1995,8 @@ static Native_func_registry func_array[] =
   { { C_STRING_WITH_LEN("LAST_INSERT_ID") }, BUILDER(Create_func_last_insert_id)},
   { { C_STRING_WITH_LEN("LCASE") }, BUILDER(Create_func_lcase)},
   { { C_STRING_WITH_LEN("LEAST") }, BUILDER(Create_func_least)},
-  { { C_STRING_WITH_LEN("LN") }, BUILDER(Create_func_ln)},
   { { C_STRING_WITH_LEN("LOAD_FILE") }, BUILDER(Create_func_load_file)},
   { { C_STRING_WITH_LEN("LOCATE") }, BUILDER(Create_func_locate)},
-  { { C_STRING_WITH_LEN("LOG") }, BUILDER(Create_func_log)},
-  { { C_STRING_WITH_LEN("LOG10") }, BUILDER(Create_func_log10)},
-  { { C_STRING_WITH_LEN("LOG2") }, BUILDER(Create_func_log2)},
   { { C_STRING_WITH_LEN("LOWER") }, BUILDER(Create_func_lcase)},
   { { C_STRING_WITH_LEN("LPAD") }, BUILDER(Create_func_lpad)},
   { { C_STRING_WITH_LEN("LTRIM") }, BUILDER(Create_func_ltrim)},
@@ -2646,23 +2005,16 @@ static Native_func_registry func_array[] =
   { { C_STRING_WITH_LEN("MONTHNAME") }, BUILDER(Create_func_monthname)},
   { { C_STRING_WITH_LEN("NULLIF") }, BUILDER(Create_func_nullif)},
   { { C_STRING_WITH_LEN("OCT") }, BUILDER(Create_func_oct)},
-  { { C_STRING_WITH_LEN("ORD") }, BUILDER(Create_func_ord)},
   { { C_STRING_WITH_LEN("PERIOD_ADD") }, BUILDER(Create_func_period_add)},
   { { C_STRING_WITH_LEN("PERIOD_DIFF") }, BUILDER(Create_func_period_diff)},
   { { C_STRING_WITH_LEN("PI") }, BUILDER(Create_func_pi)},
-  { { C_STRING_WITH_LEN("POW") }, BUILDER(Create_func_pow)},
-  { { C_STRING_WITH_LEN("POWER") }, BUILDER(Create_func_pow)},
-  { { C_STRING_WITH_LEN("QUOTE") }, BUILDER(Create_func_quote)},
   { { C_STRING_WITH_LEN("RADIANS") }, BUILDER(Create_func_radians)},
-  { { C_STRING_WITH_LEN("RAND") }, BUILDER(Create_func_rand)},
   { { C_STRING_WITH_LEN("ROUND") }, BUILDER(Create_func_round)},
   { { C_STRING_WITH_LEN("ROW_COUNT") }, BUILDER(Create_func_row_count)},
   { { C_STRING_WITH_LEN("RPAD") }, BUILDER(Create_func_rpad)},
   { { C_STRING_WITH_LEN("RTRIM") }, BUILDER(Create_func_rtrim)},
   { { C_STRING_WITH_LEN("SIGN") }, BUILDER(Create_func_sign)},
-  { { C_STRING_WITH_LEN("SIN") }, BUILDER(Create_func_sin)},
   { { C_STRING_WITH_LEN("SPACE") }, BUILDER(Create_func_space)},
-  { { C_STRING_WITH_LEN("SQRT") }, BUILDER(Create_func_sqrt)},
   { { C_STRING_WITH_LEN("STRCMP") }, BUILDER(Create_func_strcmp)},
   { { C_STRING_WITH_LEN("TAN") }, BUILDER(Create_func_tan)},
   { { C_STRING_WITH_LEN("TIME_FORMAT") }, BUILDER(Create_func_time_format)},
@@ -2675,8 +2027,6 @@ static Native_func_registry func_array[] =
   { {0, 0}, NULL}
 };
 
-static map<string, Native_func_registry *> native_functions_map;
-
 /*
   Load the hash table for native functions.
   Note: this code is not thread safe, and is intended to be used at server
@@ -2691,9 +2041,8 @@ int item_create_init()
   for (func= func_array; func->builder != NULL; func++)
   {
     func_name.assign(func->name.str, func->name.length);
-    transform(func_name.begin(), func_name.end(), func_name.begin(), ::tolower);
 
-    native_functions_map[func_name]= func;
+    FunctionContainer::getMap()[func_name]= func;
   }
 
   return 0;
@@ -2707,12 +2056,11 @@ find_native_function_builder(LEX_STRING name)
   Create_func *builder= NULL;
 
   string func_name(name.str, name.length);
-  transform(func_name.begin(), func_name.end(), func_name.begin(), ::tolower);
 
-  map<string, Native_func_registry *>::iterator func_iter=
-    native_functions_map.find(func_name);
+  NativeFunctionsMap::iterator func_iter=
+    FunctionContainer::getMap().find(func_name);
 
-  if (func_iter != native_functions_map.end())
+  if (func_iter != FunctionContainer::getMap().end())
   {
     func= (*func_iter).second;
     builder= func->builder;
