@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ITEM_EMPTY_STRING_H
+#define DRIZZLED_ITEM_EMPTY_STRING_H
 
 /**
   Item_empty_string -- is a utility class to put an item into List<Item>
@@ -27,20 +28,18 @@
 
 #include <drizzled/item/string.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
-class Item_empty_string : public Item_string
+class Item_empty_string :public Item_string
 {
 public:
-  Item_empty_string(const char* header, uint32_t length, const charset_info_st* cs= NULL) :
-    Item_string(str_ref(""), cs ? cs : &my_charset_utf8_general_ci)
-  { 
-    name= header; 
-    max_length= cs ? length * cs->mbmaxlen : length; 
-  }
-
-  void make_field(SendField*);
+  Item_empty_string(const char *header,uint32_t length, const CHARSET_INFO * cs= NULL) :
+    Item_string("",0, cs ? cs : &my_charset_utf8_general_ci)
+    { name= const_cast<char*>(header); max_length= cs ? length * cs->mbmaxlen : length; }
+  void make_field(SendField *field);
 };
 
 } /* namespace drizzled */
   
+#endif /* DRIZZLED_ITEM_EMPTY_STRING_H */
