@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,12 +17,13 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <drizzled/item/uint.h>
 #include <drizzled/item/int_with_ref.h>
 
-namespace drizzled {
+namespace drizzled
+{
 
 Item *Item_int_with_ref::clone_item()
 {
@@ -31,9 +32,9 @@ Item *Item_int_with_ref::clone_item()
     We need to evaluate the constant to make sure it works with
     parameter markers.
   */
-  return ref->unsigned_flag ?
+  return (ref->unsigned_flag ?
           new Item_uint(ref->name, ref->val_int(), ref->max_length) :
-          new Item_int(ref->name, ref->val_int(), ref->max_length);
+          new Item_int(ref->name, ref->val_int(), ref->max_length));
 }
 
 } /* namespace drizzled */
