@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,19 +17,22 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_ITEM_NUM_H
+#define DRIZZLED_ITEM_NUM_H
 
-#include <drizzled/item/basic_constant.h>
-#include <drizzled/type/decimal.h>
+#include "drizzled/item/basic_constant.h"
 
-namespace drizzled {
+namespace drizzled
+{
 
-class Item_num : public Item_basic_constant
+class Item_num: public Item_basic_constant
 {
 public:
+  Item_num() {}                               /* Remove gcc warning */
   virtual Item_num *neg()= 0;
-  Item *safe_charset_converter(const charset_info_st*);
+  Item *safe_charset_converter(const CHARSET_INFO * const tocs);
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_ITEM_NUM_H */
