@@ -21,6 +21,7 @@
   Sum functions (COUNT, MIN...)
 */
 #include "config.h"
+#include <cstdio>
 #include <math.h>
 #include <drizzled/sql_select.h>
 #include <drizzled/error.h>
@@ -521,7 +522,7 @@ Field *Item_sum::create_tmp_field(bool ,
         !convert_blob_length)
       return make_string_field(table);
     field= new Field_varstring(convert_blob_length, maybe_null,
-                               name, table->s, collation.collation);
+                               name, table->getMutableShare(), collation.collation);
     break;
   case DECIMAL_RESULT:
     field= new Field_decimal(max_length, maybe_null, name,
@@ -952,6 +953,7 @@ void Item_sum_distinct::fix_length_and_dec()
 {
   assert(args[0]->fixed);
 
+  null_value= maybe_null= true;
   table_field_type= args[0]->field_type();
 
   /* Adjust tmp table type according to the chosen aggregation type */
@@ -1023,11 +1025,11 @@ bool Item_sum_distinct::setup(Session *session)
   field_def.init_for_tmp_table(table_field_type, args[0]->max_length,
                                args[0]->decimals, args[0]->maybe_null);
 
-  if (! (table= create_virtual_tmp_table(session, field_list)))
+  if (! (table= session->create_virtual_tmp_table(field_list)))
     return(true);
 
   /* XXX: check that the case of CHAR(0) works OK */
-  tree_key_length= table->s->reclength - table->s->null_bytes;
+  tree_key_length= table->getShare()->getRecordLength() - table->getShare()->null_bytes;
 
   /*
     Unique handles all unique elements in a tree until they can't fit
@@ -1046,9 +1048,9 @@ bool Item_sum_distinct::setup(Session *session)
 
 bool Item_sum_distinct::add()
 {
-  args[0]->save_in_field(table->field[0], false);
+  args[0]->save_in_field(table->getField(0), false);
   is_evaluated= false;
-  if (!table->field[0]->is_null())
+  if (!table->getField(0)->is_null())
   {
     assert(tree);
     null_value= 0;
@@ -1056,7 +1058,7 @@ bool Item_sum_distinct::add()
       '0' values are also stored in the tree. This doesn't matter
       for SUM(DISTINCT), but is important for AVG(DISTINCT)
     */
-    return tree->unique_add(table->field[0]->ptr);
+    return tree->unique_add(table->getField(0)->ptr);
   }
   return 0;
 }
@@ -1064,9 +1066,9 @@ bool Item_sum_distinct::add()
 
 bool Item_sum_distinct::unique_walk_function(void *element)
 {
-  memcpy(table->field[0]->ptr, element, tree_key_length);
+  memcpy(table->getField(0)->ptr, element, tree_key_length);
   ++count;
-  val.traits->add(&val, table->field[0]);
+  val.traits->add(&val, table->getField(0));
   return 0;
 }
 
@@ -1108,7 +1110,7 @@ void Item_sum_distinct::calculate_val_and_count()
      */
     if (tree)
     {
-      table->field[0]->set_notnull();
+      table->getField(0)->set_notnull();
       tree->walk(item_sum_distinct_walk, (void*) this);
     }
     is_evaluated= true;
@@ -1255,7 +1257,7 @@ Field *Item_sum_avg::create_tmp_field(bool group, Table *table,
     */
     field= new Field_varstring(((hybrid_type == DECIMAL_RESULT) ?
                                 dec_bin_size : sizeof(double)) + sizeof(int64_t),
-                               0, name, table->s, &my_charset_bin);
+                               0, name, table->getMutableShare(), &my_charset_bin);
   }
   else if (hybrid_type == DECIMAL_RESULT)
     field= new Field_decimal(max_length, maybe_null, name,
@@ -1469,7 +1471,7 @@ Field *Item_sum_variance::create_tmp_field(bool group, Table *table,
       The easiest way is to do this is to store both value in a string
       and unpack on access.
     */
-    field= new Field_varstring(sizeof(double)*2 + sizeof(int64_t), 0, name, table->s, &my_charset_bin);
+    field= new Field_varstring(sizeof(double)*2 + sizeof(int64_t), 0, name, table->getMutableShare(), &my_charset_bin);
   }
   else
     field= new Field_double(max_length, maybe_null, name, decimals, true);
@@ -2492,8 +2494,8 @@ int simple_str_key_cmp(void* arg, unsigned char* key1, unsigned char* key2)
 int composite_key_cmp(void* arg, unsigned char* key1, unsigned char* key2)
 {
   Item_sum_count_distinct* item = (Item_sum_count_distinct*)arg;
-  Field **field    = item->table->field;
-  Field **field_end= field + item->table->s->fields;
+  Field **field    = item->table->getFields();
+  Field **field_end= field + item->table->getShare()->sizeFields();
   uint32_t *lengths=item->field_lengths;
   for (; field < field_end; ++field)
   {
@@ -2533,7 +2535,6 @@ void Item_sum_count_distinct::cleanup()
     is_evaluated= false;
     if (table)
     {
-      table->free_tmp_table(table->in_use);
       table= 0;
     }
     delete tmp_table_param;
@@ -2602,11 +2603,13 @@ bool Item_sum_count_distinct::setup(Session *session)
 				0,
 				(select_lex->options | session->options),
 				HA_POS_ERROR, (char*)"")))
+  {
     return true;
+  }
   table->cursor->extra(HA_EXTRA_NO_ROWS);		// Don't update rows
   table->no_rows=1;
 
-  if (table->s->db_type() == heap_engine)
+  if (table->getShare()->db_type() == heap_engine)
   {
     /*
       No blobs, otherwise it would have been MyISAM: set up a compare
@@ -2614,8 +2617,8 @@ bool Item_sum_count_distinct::setup(Session *session)
     */
     qsort_cmp2 compare_key;
     void* cmp_arg;
-    Field **field= table->field;
-    Field **field_end= field + table->s->fields;
+    Field **field= table->getFields();
+    Field **field_end= field + table->getShare()->sizeFields();
     bool all_binary= true;
 
     for (tree_key_length= 0; field < field_end; ++field)
@@ -2636,7 +2639,7 @@ bool Item_sum_count_distinct::setup(Session *session)
     }
     else
     {
-      if (table->s->fields == 1)
+      if (table->getShare()->sizeFields() == 1)
       {
         /*
           If we have only one field, which is the most common use of
@@ -2645,7 +2648,7 @@ bool Item_sum_count_distinct::setup(Session *session)
           about other fields.
         */
         compare_key= (qsort_cmp2) simple_str_key_cmp;
-        cmp_arg= (void*) table->field[0];
+        cmp_arg= (void*) table->getField(0);
         /* tree_key_length has been set already */
       }
       else
@@ -2653,8 +2656,8 @@ bool Item_sum_count_distinct::setup(Session *session)
         uint32_t *length;
         compare_key= (qsort_cmp2) composite_key_cmp;
         cmp_arg= (void*) this;
-        field_lengths= (uint32_t*) session->alloc(table->s->fields * sizeof(uint32_t));
-        for (tree_key_length= 0, length= field_lengths, field= table->field;
+        field_lengths= (uint32_t*) session->alloc(table->getShare()->sizeFields() * sizeof(uint32_t));
+        for (tree_key_length= 0, length= field_lengths, field= table->getFields();
              field < field_end; ++field, ++length)
         {
           *length= (*field)->pack_length();
@@ -2709,9 +2712,13 @@ bool Item_sum_count_distinct::add()
   copy_fields(tmp_table_param);
   copy_funcs(tmp_table_param->items_to_copy);
 
-  for (Field **field=table->field ; *field ; field++)
+  for (Field **field= table->getFields() ; *field ; field++)
+  {
     if ((*field)->is_real_null(0))
+    {
       return 0;					// Don't count NULL
+    }
+  }
 
   is_evaluated= false;
   if (tree)
@@ -2722,9 +2729,9 @@ bool Item_sum_count_distinct::add()
       bloat the tree without providing any valuable info. Besides,
       key_length used to initialize the tree didn't include space for them.
     */
-    return tree->unique_add(table->record[0] + table->s->null_bytes);
+    return tree->unique_add(table->record[0] + table->getShare()->null_bytes);
   }
-  if ((error= table->cursor->ha_write_row(table->record[0])) &&
+  if ((error= table->cursor->insertRecord(table->record[0])) &&
       table->cursor->is_fatal_error(error, HA_CHECK_DUP))
     return true;
   return false;
@@ -2811,7 +2818,7 @@ int group_concat_key_cmp_with_distinct(void* arg, const void* key1,
     */
     Field *field= item->get_tmp_table_field();
     int res;
-    uint32_t offset= field->offset(field->table->record[0])-table->s->null_bytes;
+    uint32_t offset= field->offset(field->getTable()->record[0])-table->getShare()->null_bytes;
     if((res= field->cmp((unsigned char*)key1 + offset, (unsigned char*)key2 + offset)))
       return res;
   }
@@ -2848,8 +2855,8 @@ int group_concat_key_cmp_with_order(void* arg, const void* key1,
     if (field && !item->const_item())
     {
       int res;
-      uint32_t offset= (field->offset(field->table->record[0]) -
-                    table->s->null_bytes);
+      uint32_t offset= (field->offset(field->getTable()->record[0]) -
+                    table->getShare()->null_bytes);
       if ((res= field->cmp((unsigned char*)key1 + offset, (unsigned char*)key2 + offset)))
         return (*order_item)->asc ? res : -res;
     }
@@ -2871,7 +2878,7 @@ int dump_leaf_key(unsigned char* key, uint32_t ,
                   Item_func_group_concat *item)
 {
   Table *table= item->table;
-  String tmp((char *)table->record[1], table->s->reclength,
+  String tmp((char *)table->getUpdateRecord(), table->getShare()->getRecordLength(),
              default_charset_info);
   String tmp2;
   String *result= &item->result;
@@ -2898,9 +2905,9 @@ int dump_leaf_key(unsigned char* key, uint32_t ,
         because it contains both order and arg list fields.
       */
       Field *field= (*arg)->get_tmp_table_field();
-      uint32_t offset= (field->offset(field->table->record[0]) -
-                    table->s->null_bytes);
-      assert(offset < table->s->reclength);
+      uint32_t offset= (field->offset(field->getTable()->record[0]) -
+                    table->getShare()->null_bytes);
+      assert(offset < table->getShare()->getRecordLength());
       res= field->val_str(&tmp, key + offset);
     }
     else
@@ -3031,7 +3038,7 @@ void Item_func_group_concat::cleanup()
   if (warning)
   {
     char warn_buff[DRIZZLE_ERRMSG_SIZE];
-    sprintf(warn_buff, ER(ER_CUT_VALUE_GROUP_CONCAT), count_cut_values);
+    snprintf(warn_buff, sizeof(warn_buff), ER(ER_CUT_VALUE_GROUP_CONCAT), count_cut_values);
     warning->set_msg(current_session, warn_buff);
     warning= 0;
   }
@@ -3047,7 +3054,6 @@ void Item_func_group_concat::cleanup()
     if (table)
     {
       Session *session= table->in_use;
-      table->free_tmp_table(session);
       table= 0;
       if (tree)
       {
@@ -3062,7 +3068,7 @@ void Item_func_group_concat::cleanup()
       if (warning)
       {
         char warn_buff[DRIZZLE_ERRMSG_SIZE];
-        sprintf(warn_buff, ER(ER_CUT_VALUE_GROUP_CONCAT), count_cut_values);
+        snprintf(warn_buff, sizeof(warn_buff), ER(ER_CUT_VALUE_GROUP_CONCAT), count_cut_values);
         warning->set_msg(session, warn_buff);
         warning= 0;
       }
@@ -3119,14 +3125,14 @@ bool Item_func_group_concat::add()
   {
     /* Filter out duplicate rows. */
     uint32_t count= unique_filter->elements_in_tree();
-    unique_filter->unique_add(table->record[0] + table->s->null_bytes);
+    unique_filter->unique_add(table->record[0] + table->getShare()->null_bytes);
     if (count == unique_filter->elements_in_tree())
       row_eligible= false;
   }
 
   TREE_ELEMENT *el= 0;                          // Only for safety
   if (row_eligible && tree)
-    el= tree_insert(tree, table->record[0] + table->s->null_bytes, 0,
+    el= tree_insert(tree, table->record[0] + table->getShare()->null_bytes, 0,
                     tree->custom_arg);
   /*
     If the row is not a duplicate (el->count == 1)
@@ -3135,7 +3141,7 @@ bool Item_func_group_concat::add()
   */
   if (row_eligible && !warning_for_row &&
       (!tree || (el->count == 1 && distinct && !arg_count_order)))
-    dump_leaf_key(table->record[0] + table->s->null_bytes, 1, this);
+    dump_leaf_key(table->record[0] + table->getShare()->null_bytes, 1, this);
 
   return 0;
 }
@@ -3256,7 +3262,10 @@ bool Item_func_group_concat::setup(Session *session)
                                 (order_st*) 0, 0, true,
                                 (select_lex->options | session->options),
                                 HA_POS_ERROR, (char*) "")))
+  {
     return(true);
+  }
+
   table->cursor->extra(HA_EXTRA_NO_ROWS);
   table->no_rows= 1;
 
@@ -3265,7 +3274,7 @@ bool Item_func_group_concat::setup(Session *session)
      Don't reserve space for NULLs: if any of gconcat arguments is NULL,
      the row is not added to the result.
   */
-  uint32_t tree_key_length= table->s->reclength - table->s->null_bytes;
+  uint32_t tree_key_length= table->getShare()->getRecordLength() - table->getShare()->null_bytes;
 
   if (arg_count_order)
   {
