@@ -23,7 +23,7 @@
 /* classes for sum functions */
 
 
-#include "drizzled/my_tree.h"
+#include "drizzled/tree.h"
 #include <drizzled/hybrid_type.h>
 #include <drizzled/item.h>
 #include <drizzled/item/field.h>
