@@ -69,7 +69,7 @@ public:
 };
 
 
-int find_ref_key(KEY *key, uint32_t key_count, unsigned char *record, Field *field,
+int find_ref_key(KeyInfo *key, uint32_t key_count, unsigned char *record, Field *field,
                  uint32_t *key_length, uint32_t *keypart);
 /**
   Copy part of a record that forms a key or key prefix to a buffer.
@@ -86,17 +86,16 @@ int find_ref_key(KEY *key, uint32_t key_count, unsigned char *record, Field *fie
   @param key_length  specifies length of all keyparts that will be copied
 */
 
-void key_copy(unsigned char *to_key, unsigned char *from_record, KEY *key_info, uint32_t key_length);
+void key_copy(unsigned char *to_key, unsigned char *from_record, KeyInfo *key_info, uint32_t key_length);
 void key_copy(std::basic_string<unsigned char> &to_key,
-              unsigned char *from_record, KEY *key_info, uint32_t key_length);
-void key_restore(unsigned char *to_record, unsigned char *from_key, KEY *key_info,
+              unsigned char *from_record, KeyInfo *key_info, uint32_t key_length);
+void key_restore(unsigned char *to_record, unsigned char *from_key, KeyInfo *key_info,
                  uint16_t key_length);
-void key_zero_nulls(unsigned char *tuple, KEY *key_info);
+void key_zero_nulls(unsigned char *tuple, KeyInfo *key_info);
 bool key_cmp_if_same(Table *form,const unsigned char *key,uint32_t index,uint32_t key_length);
 void key_unpack(String *to,Table *form,uint32_t index);
 bool is_key_used(Table *table, uint32_t idx, const MyBitmap *fields);
-int key_cmp(KEY_PART_INFO *key_part, const unsigned char *key, uint32_t key_length);
-extern "C" int key_rec_cmp(void *key_info, unsigned char *a, unsigned char *b);
+int key_cmp(KeyPartInfo *key_part, const unsigned char *key, uint32_t key_length);
 
 } /* namespace drizzled */
 
