@@ -88,21 +88,25 @@
 #include <algorithm>
 #include <functional>
 
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/condition_variable.hpp>
+
 using namespace std;
 
 namespace drizzled
 {
 
+static boost::mutex LOCK_global_read_lock;
+static boost::condition_variable COND_global_read_lock;
+
 /**
   @defgroup Locking Locking
   @{
 */
 
-extern HASH open_cache;
-
-static DRIZZLE_LOCK *get_lock_data(Session *session, Table **table,
-                                   uint32_t count,
-                                   bool should_lock, Table **write_locked);
+static DrizzleLock *get_lock_data(Session *session, Table **table,
+                                  uint32_t count,
+                                  bool should_lock, Table **write_locked);
 static int lock_external(Session *session, Table **table,uint32_t count);
 static int unlock_external(Session *session, Table **table,uint32_t count);
 static void print_lock_error(int error, const char *);
@@ -156,9 +160,9 @@ static int thr_lock_errno_to_mysql[]=
         lock request will set its lock type properly.
 */
 
-static void reset_lock_data_and_free(DRIZZLE_LOCK **mysql_lock)
+static void reset_lock_data_and_free(DrizzleLock **mysql_lock)
 {
-  DRIZZLE_LOCK *sql_lock= *mysql_lock;
+  DrizzleLock *sql_lock= *mysql_lock;
   THR_LOCK_DATA **ldata, **ldata_end;
 
   /* Clear the lock type of all lock data to avoid reusage. */
@@ -174,10 +178,10 @@ static void reset_lock_data_and_free(DRIZZLE_LOCK **mysql_lock)
 }
 
 
-DRIZZLE_LOCK *mysql_lock_tables(Session *session, Table **tables, uint32_t count,
+DrizzleLock *mysql_lock_tables(Session *session, Table **tables, uint32_t count,
                                 uint32_t flags, bool *need_reopen)
 {
-  DRIZZLE_LOCK *sql_lock;
+  DrizzleLock *sql_lock;
   Table *write_lock_used;
   vector<plugin::StorageEngine *> involved_engines;
   int rc;
@@ -223,7 +227,7 @@ DRIZZLE_LOCK *mysql_lock_tables(Session *session, Table **tables, uint32_t count
       set<size_t> involved_slots;
       for (size_t x= 1; x <= num_tables; x++, tables++)
       {
-        engine= (*tables)->cursor->engine;
+        engine= (*tables)->cursor->getEngine();
         if (involved_slots.count(engine->getId()) > 0)
           continue; /* already added to involved engines */
         involved_engines.push_back(engine);
@@ -348,7 +352,7 @@ static int lock_external(Session *session, Table **tables, uint32_t count)
 
     if ((error=(*tables)->cursor->ha_external_lock(session,lock_type)))
     {
-      print_lock_error(error, (*tables)->cursor->engine->getName().c_str());
+      print_lock_error(error, (*tables)->cursor->getEngine()->getName().c_str());
       while (--i)
       {
         tables--;
@@ -367,12 +371,12 @@ static int lock_external(Session *session, Table **tables, uint32_t count)
 }
 
 
-void mysql_unlock_tables(Session *session, DRIZZLE_LOCK *sql_lock)
+void mysql_unlock_tables(Session *session, DrizzleLock *sql_lock)
 {
   if (sql_lock->lock_count)
     thr_multi_unlock(sql_lock->locks,sql_lock->lock_count);
   if (sql_lock->table_count)
-    unlock_external(session,sql_lock->table,sql_lock->table_count);
+    unlock_external(session, sql_lock->table, sql_lock->table_count);
   free((unsigned char*) sql_lock);
   return;
 }
@@ -385,7 +389,7 @@ void mysql_unlock_tables(Session *session, DRIZZLE_LOCK *sql_lock)
 
 void mysql_unlock_some_tables(Session *session, Table **table, uint32_t count)
 {
-  DRIZZLE_LOCK *sql_lock;
+  DrizzleLock *sql_lock;
   Table *write_lock_used;
   if ((sql_lock= get_lock_data(session, table, count, false,
                                &write_lock_used)))
@@ -397,7 +401,7 @@ void mysql_unlock_some_tables(Session *session, Table **table, uint32_t count)
   unlock all tables locked for read.
 */
 
-void mysql_unlock_read_tables(Session *session, DRIZZLE_LOCK *sql_lock)
+void mysql_unlock_read_tables(Session *session, DrizzleLock *sql_lock)
 {
   uint32_t i,found;
 
@@ -483,14 +487,14 @@ void mysql_lock_remove(Session *session, Table *table)
 
 void mysql_lock_abort(Session *session, Table *table)
 {
-  DRIZZLE_LOCK *locked;
+  DrizzleLock *locked;
   Table *write_lock_used;
 
   if ((locked= get_lock_data(session, &table, 1, false,
                              &write_lock_used)))
   {
     for (uint32_t x= 0; x < locked->lock_count; x++)
-      thr_abort_locks(locked->locks[x]->lock);
+      locked->locks[x]->lock->abort_locks();
     free((unsigned char*) locked);
   }
 }
@@ -510,17 +514,16 @@ void mysql_lock_abort(Session *session, Table *table)
 
 bool mysql_lock_abort_for_thread(Session *session, Table *table)
 {
-  DRIZZLE_LOCK *locked;
+  DrizzleLock *locked;
   Table *write_lock_used;
   bool result= false;
 
   if ((locked= get_lock_data(session, &table, 1, false,
                              &write_lock_used)))
   {
-    for (uint32_t i=0; i < locked->lock_count; i++)
+    for (uint32_t i= 0; i < locked->lock_count; i++)
     {
-      if (thr_abort_locks_for_thread(locked->locks[i]->lock,
-                                     table->in_use->thread_id))
+      if (locked->locks[i]->lock->abort_locks_for_thread(table->in_use->thread_id))
         result= true;
     }
     free((unsigned char*) locked);
@@ -543,7 +546,7 @@ static int unlock_external(Session *session, Table **table,uint32_t count)
       if ((error=(*table)->cursor->ha_external_lock(session, F_UNLCK)))
       {
 	error_code=error;
-	print_lock_error(error_code, (*table)->cursor->engine->getName().c_str());
+	print_lock_error(error_code, (*table)->cursor->getEngine()->getName().c_str());
       }
     }
     table++;
@@ -563,16 +566,16 @@ static int unlock_external(Session *session, Table **table,uint32_t count)
   @param write_lock_used   Store pointer to last table with WRITE_ALLOW_WRITE
 */
 
-static DRIZZLE_LOCK *get_lock_data(Session *session, Table **table_ptr, uint32_t count,
+static DrizzleLock *get_lock_data(Session *session, Table **table_ptr, uint32_t count,
 				 bool should_lock, Table **write_lock_used)
 {
-  uint32_t i,tables,lock_count;
-  DRIZZLE_LOCK *sql_lock;
+  uint32_t tables,lock_count;
+  DrizzleLock *sql_lock;
   THR_LOCK_DATA **locks, **locks_buf, **locks_start;
   Table **to, **table_buf;
 
   *write_lock_used=0;
-  for (i= tables= lock_count= 0 ; i < count ; i++)
+  for (uint32_t i= tables= lock_count= 0 ; i < count ; i++)
   {
     Table *t= table_ptr[i];
 
@@ -589,16 +592,18 @@ static DRIZZLE_LOCK *get_lock_data(Session *session, Table **table_ptr, uint32_t
     update the table values. So the second part of the array is copied
     from the first part immediately before calling thr_multi_lock().
   */
-  if (!(sql_lock= (DRIZZLE_LOCK*)
+  if (!(sql_lock= (DrizzleLock*)
 	malloc(sizeof(*sql_lock) +
                sizeof(THR_LOCK_DATA*) * tables * 2 +
                sizeof(table_ptr) * lock_count)))
+  {
     return NULL;
+  }
   locks= locks_buf= sql_lock->locks= (THR_LOCK_DATA**) (sql_lock + 1);
   to= table_buf= sql_lock->table= (Table**) (locks + tables * 2);
   sql_lock->table_count= lock_count;
 
-  for (i=0 ; i < count ; i++)
+  for (uint32_t i= 0; i < count ; i++)
   {
     Table *table;
     enum thr_lock_type lock_type;
@@ -614,7 +619,7 @@ static DRIZZLE_LOCK *get_lock_data(Session *session, Table **table_ptr, uint32_t
       *write_lock_used=table;
       if (table->db_stat & HA_READ_ONLY)
       {
-	my_error(ER_OPEN_AS_READONLY,MYF(0),table->alias);
+	my_error(ER_OPEN_AS_READONLY, MYF(0), table->getAlias());
         /* Clear the lock type of the lock data that are stored already. */
         sql_lock->lock_count= locks - sql_lock->locks;
         reset_lock_data_and_free(&sql_lock);
@@ -680,26 +685,23 @@ static DRIZZLE_LOCK *get_lock_data(Session *session, Table **table_ptr, uint32_t
     > 0  table locked, but someone is using it
 */
 
-int lock_table_name(Session *session, TableList *table_list, bool check_in_use)
+static int lock_table_name(Session *session, TableList *table_list, bool check_in_use)
 {
-  Table *table;
-  char  key[MAX_DBKEY_LENGTH];
-  char *db= table_list->db;
-  uint32_t  key_length;
   bool  found_locked_table= false;
-  HASH_SEARCH_STATE state;
-
-  key_length= table_list->create_table_def_key(key);
+  TableIdentifier identifier(table_list->db, table_list->table_name);
+  const TableIdentifier::Key &key(identifier.getKey());
 
   if (check_in_use)
   {
     /* Only insert the table if we haven't insert it already */
-    for (table=(Table*) hash_first(&open_cache, (unsigned char*)key,
-                                   key_length, &state);
-         table ;
-         table = (Table*) hash_next(&open_cache,(unsigned char*) key,
-                                    key_length, &state))
+    TableOpenCacheRange ppp;
+
+    ppp= get_open_cache().equal_range(key);
+
+    for (TableOpenCache::const_iterator iter= ppp.first;
+         iter != ppp.second; ++iter)
     {
+      Table *table= (*iter).second;
       if (table->reginfo.lock_type < TL_WRITE)
       {
         if (table->in_use == session)
@@ -709,21 +711,24 @@ int lock_table_name(Session *session, TableList *table_list, bool check_in_use)
 
       if (table->in_use == session)
       {
-        table->s->version= 0;                  // Ensure no one can use this
-        table->locked_by_name= 1;
+        table->getMutableShare()->resetVersion();                  // Ensure no one can use this
+        table->locked_by_name= true;
         return 0;
       }
     }
   }
 
-  if (!(table= session->table_cache_insert_placeholder(key, key_length)))
+  Table *table;
+  if (!(table= session->table_cache_insert_placeholder(table_list->db, table_list->table_name)))
+  {
     return -1;
+  }
 
-  table_list->table=table;
+  table_list->table= table;
 
   /* Return 1 if table is in use */
-  return(test(remove_table_from_cache(session, db, table_list->table_name,
-             check_in_use ? RTFC_NO_FLAG : RTFC_WAIT_OTHER_THREAD_FLAG)));
+  return(test(remove_table_from_cache(session, identifier,
+				      check_in_use ? RTFC_NO_FLAG : RTFC_WAIT_OTHER_THREAD_FLAG)));
 }
 
 
@@ -731,7 +736,7 @@ void unlock_table_name(TableList *table_list)
 {
   if (table_list->table)
   {
-    hash_delete(&open_cache, (unsigned char*) table_list->table);
+    remove_table(table_list->table);
     broadcast_refresh();
   }
 }
@@ -744,11 +749,11 @@ static bool locked_named_table(TableList *table_list)
     Table *table= table_list->table;
     if (table)
     {
-      Table *save_next= table->next;
+      Table *save_next= table->getNext();
       bool result;
-      table->next= 0;
+      table->setNext(NULL);
       result= table_is_used(table_list->table, 0);
-      table->next= save_next;
+      table->setNext(save_next);
       if (result)
         return 1;
     }
@@ -757,11 +762,13 @@ static bool locked_named_table(TableList *table_list)
 }
 
 
-bool wait_for_locked_table_names(Session *session, TableList *table_list)
+static bool wait_for_locked_table_names(Session *session, TableList *table_list)
 {
   bool result= false;
 
-  safe_mutex_assert_owner(&LOCK_open);
+#if 0
+  assert(ownership of LOCK_open);
+#endif
 
   while (locked_named_table(table_list))
   {
@@ -770,8 +777,8 @@ bool wait_for_locked_table_names(Session *session, TableList *table_list)
       result=1;
       break;
     }
-    session->wait_for_condition(&LOCK_open, &COND_refresh);
-    pthread_mutex_lock(&LOCK_open); /* Wait for a table to unlock and then lock it */
+    session->wait_for_condition(LOCK_open, COND_refresh);
+    LOCK_open.lock(); /* Wait for a table to unlock and then lock it */
   }
   return result;
 }
@@ -791,7 +798,7 @@ bool wait_for_locked_table_names(Session *session, TableList *table_list)
     1	Fatal error (end of memory ?)
 */
 
-bool lock_table_names(Session *session, TableList *table_list)
+static bool lock_table_names(Session *session, TableList *table_list)
 {
   bool got_all_locks=1;
   TableList *lock_table;
@@ -799,7 +806,7 @@ bool lock_table_names(Session *session, TableList *table_list)
   for (lock_table= table_list; lock_table; lock_table= lock_table->next_local)
   {
     int got_lock;
-    if ((got_lock= lock_table_name(session,lock_table, true)) < 0)
+    if ((got_lock= lock_table_name(session, lock_table, true)) < 0)
       goto end;					// Fatal error
     if (got_lock)
       got_all_locks=0;				// Someone is using table
@@ -1002,14 +1009,16 @@ bool lock_global_read_lock(Session *session)
   if (!session->global_read_lock)
   {
     const char *old_message;
-    (void) pthread_mutex_lock(&LOCK_global_read_lock);
-    old_message=session->enter_cond(&COND_global_read_lock, &LOCK_global_read_lock,
+    LOCK_global_read_lock.lock();
+    old_message=session->enter_cond(COND_global_read_lock, LOCK_global_read_lock,
                                 "Waiting to get readlock");
 
     waiting_for_read_lock++;
+    boost::mutex::scoped_lock scopedLock(LOCK_global_read_lock, boost::adopt_lock_t());
     while (protect_against_global_read_lock && !session->killed)
-      pthread_cond_wait(&COND_global_read_lock, &LOCK_global_read_lock);
+      COND_global_read_lock.wait(scopedLock);
     waiting_for_read_lock--;
+    scopedLock.release();
     if (session->killed)
     {
       session->exit_cond(old_message);
@@ -1035,22 +1044,26 @@ void unlock_global_read_lock(Session *session)
 {
   uint32_t tmp;
 
-  pthread_mutex_lock(&LOCK_global_read_lock);
-  tmp= --global_read_lock;
-  if (session->global_read_lock == MADE_GLOBAL_READ_LOCK_BLOCK_COMMIT)
-    --global_read_lock_blocks_commit;
-  pthread_mutex_unlock(&LOCK_global_read_lock);
+  {
+    boost::mutex::scoped_lock scopedLock(LOCK_global_read_lock);
+    tmp= --global_read_lock;
+    if (session->global_read_lock == MADE_GLOBAL_READ_LOCK_BLOCK_COMMIT)
+      --global_read_lock_blocks_commit;
+  }
   /* Send the signal outside the mutex to avoid a context switch */
   if (!tmp)
   {
-    pthread_cond_broadcast(&COND_global_read_lock);
+    COND_global_read_lock.notify_all();
   }
   session->global_read_lock= 0;
 }
 
-#define must_wait (global_read_lock &&                             \
-                   (is_not_commit ||                               \
-                    global_read_lock_blocks_commit))
+static inline bool must_wait(bool is_not_commit)
+{
+  return (global_read_lock &&
+          (is_not_commit ||
+          global_read_lock_blocks_commit));
+}
 
 bool wait_if_global_read_lock(Session *session, bool abort_on_refresh,
                               bool is_not_commit)
@@ -1063,17 +1076,17 @@ bool wait_if_global_read_lock(Session *session, bool abort_on_refresh,
     threads could not close their tables. This would make a pretty
     deadlock.
   */
-  safe_mutex_assert_not_owner(&LOCK_open);
+  safe_mutex_assert_not_owner(LOCK_open.native_handle());
 
-  (void) pthread_mutex_lock(&LOCK_global_read_lock);
-  if ((need_exit_cond= must_wait))
+  LOCK_global_read_lock.lock();
+  if ((need_exit_cond= must_wait(is_not_commit)))
   {
     if (session->global_read_lock)		// This thread had the read locks
     {
       if (is_not_commit)
         my_message(ER_CANT_UPDATE_WITH_READLOCK,
                    ER(ER_CANT_UPDATE_WITH_READLOCK), MYF(0));
-      (void) pthread_mutex_unlock(&LOCK_global_read_lock);
+      LOCK_global_read_lock.unlock();
       /*
         We allow FLUSHer to COMMIT; we assume FLUSHer knows what it does.
         This allowance is needed to not break existing versions of innobackup
@@ -1081,12 +1094,14 @@ bool wait_if_global_read_lock(Session *session, bool abort_on_refresh,
       */
       return is_not_commit;
     }
-    old_message=session->enter_cond(&COND_global_read_lock, &LOCK_global_read_lock,
-				"Waiting for release of readlock");
-    while (must_wait && ! session->killed &&
+    old_message=session->enter_cond(COND_global_read_lock, LOCK_global_read_lock,
+                                    "Waiting for release of readlock");
+    while (must_wait(is_not_commit) && ! session->killed &&
 	   (!abort_on_refresh || session->version == refresh_version))
     {
-      (void) pthread_cond_wait(&COND_global_read_lock, &LOCK_global_read_lock);
+      boost::mutex::scoped_lock scoped(LOCK_global_read_lock, boost::adopt_lock_t());
+      COND_global_read_lock.wait(scoped);
+      scoped.release();
     }
     if (session->killed)
       result=1;
@@ -1100,7 +1115,7 @@ bool wait_if_global_read_lock(Session *session, bool abort_on_refresh,
   if (unlikely(need_exit_cond))
     session->exit_cond(old_message); // this unlocks LOCK_global_read_lock
   else
-    pthread_mutex_unlock(&LOCK_global_read_lock);
+    LOCK_global_read_lock.unlock();
   return result;
 }
 
@@ -1110,12 +1125,12 @@ void start_waiting_global_read_lock(Session *session)
   bool tmp;
   if (unlikely(session->global_read_lock))
     return;
-  (void) pthread_mutex_lock(&LOCK_global_read_lock);
+  LOCK_global_read_lock.lock();
   tmp= (!--protect_against_global_read_lock &&
         (waiting_for_read_lock || global_read_lock_blocks_commit));
-  (void) pthread_mutex_unlock(&LOCK_global_read_lock);
+  LOCK_global_read_lock.unlock();
   if (tmp)
-    pthread_cond_broadcast(&COND_global_read_lock);
+    COND_global_read_lock.notify_all();
   return;
 }
 
@@ -1130,13 +1145,17 @@ bool make_global_read_lock_block_commit(Session *session)
   */
   if (session->global_read_lock != GOT_GLOBAL_READ_LOCK)
     return false;
-  pthread_mutex_lock(&LOCK_global_read_lock);
+  LOCK_global_read_lock.lock();
   /* increment this BEFORE waiting on cond (otherwise race cond) */
   global_read_lock_blocks_commit++;
-  old_message= session->enter_cond(&COND_global_read_lock, &LOCK_global_read_lock,
-                               "Waiting for all running commits to finish");
+  old_message= session->enter_cond(COND_global_read_lock, LOCK_global_read_lock,
+                                   "Waiting for all running commits to finish");
   while (protect_against_global_read_lock && !session->killed)
-    pthread_cond_wait(&COND_global_read_lock, &LOCK_global_read_lock);
+  {
+    boost::mutex::scoped_lock scopedLock(LOCK_global_read_lock, boost::adopt_lock_t());
+    COND_global_read_lock.wait(scopedLock);
+    scopedLock.release();
+  }
   if ((error= test(session->killed)))
     global_read_lock_blocks_commit--; // undo what we did
   else
@@ -1167,8 +1186,8 @@ bool make_global_read_lock_block_commit(Session *session)
 
 void broadcast_refresh(void)
 {
-  pthread_cond_broadcast(&COND_refresh);
-  pthread_cond_broadcast(&COND_global_read_lock);
+  COND_refresh.notify_all();
+  COND_global_read_lock.notify_all();
 }
 
 
