@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,9 +18,10 @@
  */
 
 
-#pragma once
+#ifndef DRIZZLED_LOOKUP_SYMBOL_H
+#define DRIZZLED_LOOKUP_SYMBOL_H
 
-#include <drizzled/lex_symbol.h>
+#include "drizzled/lex_symbol.h"
 
 namespace drizzled
 {
@@ -30,3 +31,4 @@ const SYMBOL *lookup_symbol(const char *s, unsigned int len, bool function);
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_LOOKUP_SYMBOL_H */
