@@ -25,7 +25,6 @@
 using namespace std;
 using namespace drizzled;
 
-
 int main(int argc, char* argv[])
 {
   GOOGLE_PROTOBUF_VERIFY_VERSION;
@@ -44,8 +43,11 @@ int main(int argc, char* argv[])
   else
     file_name= argv[1];
 
-
   schema.set_name(argv[1]);
+  schema.mutable_engine()->set_name("filesystem");
+  schema.set_creation_timestamp(time(NULL));
+  schema.set_update_timestamp(time(NULL));
+  schema.set_collation("utf8_general_ci");
 
   fstream output(file_name.c_str(), ios::out | ios::trunc | ios::binary);
 
