@@ -34,10 +34,14 @@
 #include "drizzled/message/statement_transform.h"
 #include "drizzled/message/transaction.pb.h"
 #include "drizzled/message/table.pb.h"
+#include "drizzled/charset.h"
+#include "drizzled/charset_info.h"
+#include "drizzled/global_charset_info.h"
 
 #include <string>
 #include <vector>
 #include <sstream>
+#include <cstdio>
 
 using namespace std;
 
@@ -47,6 +51,71 @@ namespace drizzled
 namespace message
 {
 
+static void escapeEmbeddedQuotes(string &s, const char quote='\'')
+{
+  string::iterator it;
+
+  for (it= s.begin(); it != s.end(); ++it)
+  {
+    if (*it == quote)
+    {
+      it= s.insert(it, quote);
+      ++it;  // advance back to the quote
+    }
+  }
+}
+
+/* Incredibly similar to append_unescaped() in table.cc, but for std::string */
+static void append_escaped_string(std::string *res, const std::string &input, const char quote='\'')
+{
+  const char *pos= input.c_str();
+  const char *end= input.c_str()+input.length();
+  res->push_back(quote);
+
+  for (; pos != end ; pos++)
+  {
+    uint32_t mblen;
+    if (use_mb(default_charset_info) &&
+        (mblen= my_ismbchar(default_charset_info, pos, end)))
+    {
+      res->append(pos, mblen);
+      pos+= mblen - 1;
+      if (pos >= end)
+        break;
+      continue;
+    }
+
+    switch (*pos) {
+    case 0:				/* Must be escaped for 'mysql' */
+      res->push_back('\\');
+      res->push_back('0');
+      break;
+    case '\n':				/* Must be escaped for logs */
+      res->push_back('\\');
+      res->push_back('n');
+      break;
+    case '\r':
+      res->push_back('\\');		/* This gives better readability */
+      res->push_back('r');
+      break;
+    case '\\':
+      res->push_back('\\');		/* Because of the sql syntax */
+      res->push_back('\\');
+      break;
+    default:
+      if (*pos == quote) /* SQL syntax for quoting a quote */
+      {
+        res->push_back(quote);
+        res->push_back(quote);
+      }
+      else
+        res->push_back(*pos);
+      break;
+    }
+  }
+  res->push_back(quote);
+}
+
 enum TransformSqlError
 transformStatementToSql(const Statement &source,
                         vector<string> &sql_strings,
@@ -322,25 +391,41 @@ transformInsertRecordToSql(const InsertHeader &header,
 
     const FieldMetadata &field_metadata= header.field_metadata(x);
 
-    should_quote_field_value= shouldQuoteFieldValue(field_metadata.type());
+    if (record.is_null(x))
+    {
+      should_quote_field_value= false;
+    }
+    else 
+    {
+      should_quote_field_value= shouldQuoteFieldValue(field_metadata.type());
+    }
 
     if (should_quote_field_value)
       destination.push_back('\'');
 
-    if (field_metadata.type() == Table::Field::BLOB)
+    if (record.is_null(x))
     {
-      /* 
-        * We do this here because BLOB data is returned
-        * in a string correctly, but calling append()
-        * without a length will result in only the string
-        * up to a \0 being output here.
-        */
-      string raw_data(record.insert_value(x));
-      destination.append(raw_data.c_str(), raw_data.size());
+      destination.append("NULL");
     }
     else
     {
-      destination.append(record.insert_value(x));
+      if (field_metadata.type() == Table::Field::BLOB)
+      {
+        /*
+         * We do this here because BLOB data is returned
+         * in a string correctly, but calling append()
+         * without a length will result in only the string
+         * up to a \0 being output here.
+         */
+        string raw_data(record.insert_value(x));
+        destination.append(raw_data.c_str(), raw_data.size());
+      }
+      else
+      {
+        string tmp(record.insert_value(x));
+        escapeEmbeddedQuotes(tmp);
+        destination.append(tmp);
+      }
     }
 
     if (should_quote_field_value)
@@ -403,7 +488,9 @@ transformInsertStatementToSql(const InsertHeader &header,
       }
       else
       {
-        destination.append(data.record(x).insert_value(y));
+        string tmp(data.record(x).insert_value(y));
+        escapeEmbeddedQuotes(tmp);
+        destination.append(tmp);
       }
 
       if (should_quote_field_value)
@@ -467,25 +554,41 @@ transformUpdateRecordToSql(const UpdateHeader &header,
     destination.push_back(quoted_identifier);
     destination.push_back('=');
 
-    should_quote_field_value= shouldQuoteFieldValue(field_metadata.type());
+    if (record.is_null(x))
+    {
+      should_quote_field_value= false;
+    }
+    else 
+    {
+      should_quote_field_value= shouldQuoteFieldValue(field_metadata.type());
+    }    
 
     if (should_quote_field_value)
       destination.push_back('\'');
 
-    if (field_metadata.type() == Table::Field::BLOB)
+    if (record.is_null(x))
     {
-      /* 
-       * We do this here because BLOB data is returned
-       * in a string correctly, but calling append()
-       * without a length will result in only the string
-       * up to a \0 being output here.
-       */
-      string raw_data(record.after_value(x));
-      destination.append(raw_data.c_str(), raw_data.size());
+      destination.append("NULL");
     }
-    else
+    else 
     {
-      destination.append(record.after_value(x));
+      if (field_metadata.type() == Table::Field::BLOB)
+      {
+        /*
+         * We do this here because BLOB data is returned
+         * in a string correctly, but calling append()
+         * without a length will result in only the string
+         * up to a \0 being output here.
+         */
+        string raw_data(record.after_value(x));
+        destination.append(raw_data.c_str(), raw_data.size());
+      }
+      else 
+      {
+        string tmp(record.after_value(x));
+        escapeEmbeddedQuotes(tmp);
+        destination.append(tmp);
+      }
     }
 
     if (should_quote_field_value)
@@ -607,7 +710,9 @@ transformDeleteRecordToSql(const DeleteHeader &header,
     }
     else
     {
-      destination.append(record.key_value(x));
+      string tmp(record.key_value(x));
+      escapeEmbeddedQuotes(tmp);
+      destination.append(tmp);
     }
 
     if (should_quote_field_value)
@@ -676,7 +781,9 @@ transformDeleteStatementToSql(const DeleteHeader &header,
       }
       else
       {
-        destination.append(data.record(x).key_value(y));
+        string tmp(data.record(x).key_value(y));
+        escapeEmbeddedQuotes(tmp);
+        destination.append(tmp);
       }
 
       if (should_quote_field_value)
@@ -819,7 +926,7 @@ transformCreateTableStatementToSql(const CreateTableStatement &statement,
 enum TransformSqlError
 transformTableDefinitionToSql(const Table &table,
                               string &destination,
-                              enum TransformSqlVariant sql_variant)
+                              enum TransformSqlVariant sql_variant, bool with_schema)
 {
   char quoted_identifier= '`';
   if (sql_variant == ANSI)
@@ -831,9 +938,12 @@ transformTableDefinitionToSql(const Table &table,
     destination.append("TEMPORARY ", 10);
   
   destination.append("TABLE ", 6);
-  destination.push_back(quoted_identifier);
-  destination.append(table.name());
-  destination.push_back(quoted_identifier);
+  if (with_schema)
+  {
+    append_escaped_string(&destination, table.schema(), quoted_identifier);
+    destination.push_back('.');
+  }
+  append_escaped_string(&destination, table.name(), quoted_identifier);
   destination.append(" (\n", 3);
 
   enum TransformSqlError result= NONE;
@@ -845,8 +955,10 @@ transformTableDefinitionToSql(const Table &table,
     if (x != 0)
       destination.append(",\n", 2);
 
+    destination.append("  ");
+
     result= transformFieldDefinitionToSql(field, destination, sql_variant);
-    
+
     if (result != NONE)
       return result;
   }
@@ -868,24 +980,45 @@ transformTableDefinitionToSql(const Table &table,
     if (result != NONE)
       return result;
   }
+
+  size_t num_foreign_keys= table.fk_constraint_size();
+
+  if (num_foreign_keys > 0)
+    destination.append(",\n", 2);
+
+  for (size_t x= 0; x < num_foreign_keys; ++x)
+  {
+    const message::Table::ForeignKeyConstraint &fkey= table.fk_constraint(x);
+
+    if (x != 0)
+      destination.append(",\n", 2);
+
+    result= transformForeignKeyConstraintDefinitionToSql(fkey, table, destination, sql_variant);
+
+    if (result != NONE)
+      return result;
+  }
+
   destination.append("\n)", 2);
 
   /* Add ENGINE = " clause */
   if (table.has_engine())
   {
-    const Table::StorageEngine &engine= table.engine();
-    destination.append("\nENGINE = ", 10);
-    destination.append(engine.name());
+    destination.append(" ENGINE=", 8);
+    destination.append(table.engine().name());
 
-    size_t num_engine_options= engine.option_size();
+    size_t num_engine_options= table.engine().options_size();
+    if (num_engine_options > 0)
+      destination.append(" ", 1);
     for (size_t x= 0; x < num_engine_options; ++x)
     {
-      const Table::StorageEngine::EngineOption &option= engine.option(x);
-      destination.push_back('\n');
-      destination.append(option.option_name());
-      destination.append(" = ", 3);
-      destination.append(option.option_value());
-      destination.push_back('\n');
+      const Engine::Option &option= table.engine().options(x);
+      destination.append(option.name());
+      destination.append("='", 2);
+      destination.append(option.state());
+      destination.append("'", 1);
+      if(x != num_engine_options-1)
+        destination.append(", ", 2);
     }
   }
 
@@ -907,33 +1040,16 @@ transformTableOptionsToSql(const Table::TableOptions &options,
 
   if (options.has_comment())
   {
-    destination.append("\nCOMMENT = '", 12);
-    destination.append(options.comment());
-    destination.push_back('\'');
+    destination.append(" COMMENT=", 9);
+    append_escaped_string(&destination, options.comment());
   }
 
   if (options.has_collation())
   {
-    destination.append("\nCOLLATE = ", 11);
+    destination.append(" COLLATE = ", 11);
     destination.append(options.collation());
   }
 
-  if (options.has_auto_increment())
-  {
-    ss << options.auto_increment();
-    destination.append("\nAUTOINCREMENT_OFFSET = ", 24);
-    destination.append(ss.str());
-    ss.clear();
-  }
-  
-  if (options.has_row_type())
-  {
-    ss << options.row_type();
-    destination.append("\nROW_TYPE = ", 12);
-    destination.append(ss.str());
-    ss.clear();
-  }
-
   if (options.has_data_file_name())
   {
     destination.append("\nDATA_FILE_NAME = '", 19);
@@ -964,10 +1080,11 @@ transformTableOptionsToSql(const Table::TableOptions &options,
     ss.clear();
   }
 
-  if (options.has_auto_increment_value())
+  if (options.has_user_set_auto_increment_value()
+      && options.has_auto_increment_value())
   {
     ss << options.auto_increment_value();
-    destination.append("\nAUTO_INCREMENT = ", 18);
+    destination.append(" AUTO_INCREMENT=", 16);
     destination.append(ss.str());
     ss.clear();
   }
@@ -980,28 +1097,6 @@ transformTableOptionsToSql(const Table::TableOptions &options,
     ss.clear();
   }
 
-  if (options.has_key_block_size())
-  {
-    ss << options.key_block_size();
-    destination.append("\nKEY_BLOCK_SIZE = ", 18);
-    destination.append(ss.str());
-    ss.clear();
-  }
-
-  if (options.has_block_size())
-  {
-    ss << options.block_size();
-    destination.append("\nBLOCK_SIZE = ", 14);
-    destination.append(ss.str());
-    ss.clear();
-  }
-
-  if (options.has_pack_keys() &&
-      options.pack_keys())
-    destination.append("\nPACK_KEYS = TRUE", 17);
-  if (options.has_pack_record() &&
-      options.pack_record())
-    destination.append("\nPACK_RECORD = TRUE", 19);
   if (options.has_checksum() &&
       options.checksum())
     destination.append("\nCHECKSUM = TRUE", 16);
@@ -1022,17 +1117,24 @@ transformIndexDefinitionToSql(const Table::Index &index,
   if (sql_variant == ANSI)
     quoted_identifier= '"';
 
+  destination.append("  ", 2);
+
   if (index.is_primary())
     destination.append("PRIMARY ", 8);
   else if (index.is_unique())
     destination.append("UNIQUE ", 7);
 
   destination.append("KEY ", 4);
-  destination.push_back(quoted_identifier);
-  destination.append(index.name());
-  destination.push_back(quoted_identifier);
-  destination.append(" (", 2);
-  
+  if (! (index.is_primary() && index.name().compare("PRIMARY")==0))
+  {
+    destination.push_back(quoted_identifier);
+    destination.append(index.name());
+    destination.push_back(quoted_identifier);
+    destination.append(" (", 2);
+  }
+  else
+    destination.append("(", 1);
+
   size_t num_parts= index.index_part_size();
   for (size_t x= 0; x < num_parts; ++x)
   {
@@ -1056,18 +1158,11 @@ transformIndexDefinitionToSql(const Table::Index &index,
     {
       if (part.has_compare_length())
       {
-        size_t compare_length_in_chars= part.compare_length();
-        
-        /* hack: compare_length() is bytes, not chars, but
-         * only for VARCHAR. Ass. */
-        if (field.type() == Table::Field::VARCHAR)
-          compare_length_in_chars/= 4;
-
-        if (compare_length_in_chars != field.string_options().length())
+        if (part.compare_length() != field.string_options().length())
         {
           stringstream ss;
           destination.push_back('(');
-          ss << compare_length_in_chars;
+          ss << part.compare_length();
           destination.append(ss.str());
           destination.push_back(')');
         }
@@ -1076,6 +1171,116 @@ transformIndexDefinitionToSql(const Table::Index &index,
   }
   destination.push_back(')');
 
+  switch (index.type())
+  {
+  case Table::Index::UNKNOWN_INDEX:
+    break;
+  case Table::Index::BTREE:
+    destination.append(" USING BTREE", 12);
+    break;
+  case Table::Index::RTREE:
+    destination.append(" USING RTREE", 12);
+    break;
+  case Table::Index::HASH:
+    destination.append(" USING HASH", 11);
+    break;
+  case Table::Index::FULLTEXT:
+    destination.append(" USING FULLTEXT", 15);
+    break;
+  }
+
+  if (index.has_comment())
+  {
+    destination.append(" COMMENT ", 9);
+    append_escaped_string(&destination, index.comment());
+  }
+
+  return NONE;
+}
+
+static void transformForeignKeyOptionToSql(Table::ForeignKeyConstraint::ForeignKeyOption opt, string &destination)
+{
+  switch (opt)
+  {
+  case Table::ForeignKeyConstraint::OPTION_UNDEF:
+    break;
+  case Table::ForeignKeyConstraint::OPTION_RESTRICT:
+    destination.append("RESTRICT");
+    break;
+  case Table::ForeignKeyConstraint::OPTION_CASCADE:
+    destination.append("CASCADE");
+    break;
+  case Table::ForeignKeyConstraint::OPTION_SET_NULL:
+    destination.append("SET NULL");
+    break;
+  case Table::ForeignKeyConstraint::OPTION_NO_ACTION:
+    destination.append("NO ACTION");
+    break;
+  case Table::ForeignKeyConstraint::OPTION_DEFAULT:
+    destination.append("SET DEFAULT");
+    break;
+  }
+}
+
+enum TransformSqlError
+transformForeignKeyConstraintDefinitionToSql(const Table::ForeignKeyConstraint &fkey,
+                                             const Table &,
+                                             string &destination,
+                                             enum TransformSqlVariant sql_variant)
+{
+  char quoted_identifier= '`';
+  if (sql_variant == ANSI)
+    quoted_identifier= '"';
+
+  destination.append("  ", 2);
+
+  if (fkey.has_name())
+  {
+    destination.append("CONSTRAINT ", 11);
+    append_escaped_string(&destination, fkey.name(), quoted_identifier);
+    destination.append(" ", 1);
+  }
+
+  destination.append("FOREIGN KEY (", 13);
+
+  for (ssize_t x= 0; x < fkey.column_names_size(); ++x)
+  {
+    if (x != 0)
+      destination.append(", ");
+
+    append_escaped_string(&destination, fkey.column_names(x),
+                          quoted_identifier);
+  }
+
+  destination.append(") REFERENCES ", 13);
+
+  append_escaped_string(&destination, fkey.references_table_name(),
+                        quoted_identifier);
+  destination.append(" (", 2);
+
+  for (ssize_t x= 0; x < fkey.references_columns_size(); ++x)
+  {
+    if (x != 0)
+      destination.append(", ");
+
+    append_escaped_string(&destination, fkey.references_columns(x),
+                          quoted_identifier);
+  }
+
+  destination.push_back(')');
+
+  if (fkey.update_option() != Table::ForeignKeyConstraint::OPTION_UNDEF)
+  {
+    destination.append(" ON UPDATE ", 11);
+    transformForeignKeyOptionToSql(fkey.update_option(), destination);
+  }
+
+  if (fkey.delete_option() != Table::ForeignKeyConstraint::OPTION_UNDEF)
+  {
+    destination.append(" ON DELETE ", 11);
+    transformForeignKeyOptionToSql(fkey.delete_option(), destination);
+  }
+
   return NONE;
 }
 
@@ -1085,12 +1290,17 @@ transformFieldDefinitionToSql(const Table::Field &field,
                               enum TransformSqlVariant sql_variant)
 {
   char quoted_identifier= '`';
+  char quoted_default;
+
   if (sql_variant == ANSI)
     quoted_identifier= '"';
 
-  destination.push_back(quoted_identifier);
-  destination.append(field.name());
-  destination.push_back(quoted_identifier);
+  if (sql_variant == DRIZZLE)
+    quoted_default= '\'';
+  else
+    quoted_default= quoted_identifier;
+
+  append_escaped_string(&destination, field.name(), quoted_identifier);
 
   Table::Field::FieldType field_type= field.type();
 
@@ -1098,25 +1308,44 @@ transformFieldDefinitionToSql(const Table::Field &field,
   {
     case Table::Field::DOUBLE:
     destination.append(" DOUBLE", 7);
+    if (field.has_numeric_options()
+        && field.numeric_options().has_precision())
+    {
+      stringstream ss;
+      ss << "(" << field.numeric_options().precision() << ",";
+      ss << field.numeric_options().scale() << ")";
+      destination.append(ss.str());
+    }
     break;
   case Table::Field::VARCHAR:
     {
-      destination.append(" VARCHAR(", 9);
+      if (field.string_options().has_collation()
+          && field.string_options().collation().compare("binary") == 0)
+        destination.append(" VARBINARY(", 11);
+      else
+        destination.append(" VARCHAR(", 9);
+
       stringstream ss;
       ss << field.string_options().length() << ")";
       destination.append(ss.str());
     }
     break;
   case Table::Field::BLOB:
-    destination.append(" BLOB", 5);
+    {
+      if (field.string_options().has_collation()
+          && field.string_options().collation().compare("binary") == 0)
+        destination.append(" BLOB", 5);
+      else
+        destination.append(" TEXT", 5);
+    }
     break;
   case Table::Field::ENUM:
     {
-      size_t num_field_values= field.set_options().field_value_size();
+      size_t num_field_values= field.enumeration_values().field_value_size();
       destination.append(" ENUM(", 6);
       for (size_t x= 0; x < num_field_values; ++x)
       {
-        const string &type= field.set_options().field_value(x);
+        const string &type= field.enumeration_values().field_value(x);
 
         if (x != 0)
           destination.push_back(',');
@@ -1165,13 +1394,24 @@ transformFieldDefinitionToSql(const Table::Field &field,
     }
   }
 
-
-  if (! (field.has_constraints() &&
-         field.constraints().is_nullable()))
+  if (field.type() == Table::Field::BLOB ||
+      field.type() == Table::Field::VARCHAR)
   {
-    destination.append(" NOT", 4);
+    if (field.string_options().has_collation()
+        && field.string_options().collation().compare("binary"))
+    {
+      destination.append(" COLLATE ", 9);
+      destination.append(field.string_options().collation());
+    }
   }
-  destination.append(" NULL", 5);
+
+  if (field.has_constraints() &&
+      ! field.constraints().is_nullable())
+  {
+    destination.append(" NOT NULL", 9);
+  }
+  else if (field.type() == Table::Field::TIMESTAMP)
+    destination.append(" NULL", 5);
 
   if (field.type() == Table::Field::INTEGER || 
       field.type() == Table::Field::BIGINT)
@@ -1184,45 +1424,49 @@ transformFieldDefinitionToSql(const Table::Field &field,
     }
   }
 
-  if (field.type() == Table::Field::BLOB ||
-      field.type() == Table::Field::VARCHAR)
-  {
-    if (field.string_options().has_collation())
-    {
-      destination.append(" COLLATE ", 9);
-      destination.append(field.string_options().collation());
-    }
-  }
-
   if (field.options().has_default_value())
   {
     destination.append(" DEFAULT ", 9);
-    destination.push_back(quoted_identifier);
-    destination.append(field.options().default_value());
-    destination.push_back(quoted_identifier);
+    append_escaped_string(&destination, field.options().default_value());
   }
-
-  if (field.options().has_default_bin_value())
+  else if (field.options().has_default_expression())
+  {
+    destination.append(" DEFAULT ", 9);
+    destination.append(field.options().default_expression());
+  }
+  else if (field.options().has_default_bin_value())
   {
     const string &v= field.options().default_bin_value();
-    destination.append(" DEFAULT 0x", 11);
-    for (size_t x= 0; x < v.length(); x++)
+    if (v.length() == 0)
+      destination.append(" DEFAULT ''", 11);
+    else
     {
-      printf("%.2x", *(v.c_str() + x));
+      destination.append(" DEFAULT 0x", 11);
+      for (size_t x= 0; x < v.length(); x++)
+      {
+        char hex[3];
+        snprintf(hex, sizeof(hex), "%.2X", *(v.c_str() + x));
+        destination.append(hex, 2);
+      }
     }
   }
+  else if (field.options().has_default_null()
+           && field.options().default_null()
+           && field.type() != Table::Field::BLOB)
+  {
+    destination.append(" DEFAULT NULL", 13);
+  }
 
-  if (field.type() == Table::Field::TIMESTAMP)
-    if (field.timestamp_options().has_auto_updates() &&
-        field.timestamp_options().auto_updates())
-      destination.append(" ON UPDATE CURRENT_TIMESTAMP", 28);
+  if (field.has_options() && field.options().has_update_expression())
+  {
+    destination.append(" ON UPDATE ", 11);
+    destination.append(field.options().update_expression());
+  }
 
   if (field.has_comment())
   {
     destination.append(" COMMENT ", 9);
-    destination.push_back(quoted_identifier);
-    destination.append(field.comment());
-    destination.push_back(quoted_identifier);
+    append_escaped_string(&destination, field.comment(), quoted_default);
   }
   return NONE;
 }
@@ -1235,7 +1479,6 @@ bool shouldQuoteFieldValue(Table::Field::FieldType in_type)
   case Table::Field::DECIMAL:
   case Table::Field::INTEGER:
   case Table::Field::BIGINT:
-  case Table::Field::ENUM:
     return false;
   default:
     return true;
@@ -1274,5 +1517,43 @@ Table::Field::FieldType internalFieldTypeToFieldProtoType(enum enum_field_types
   return Table::Field::INTEGER; /* unreachable */
 }
 
+bool transactionContainsBulkSegment(const Transaction &transaction)
+{
+  size_t num_statements= transaction.statement_size();
+  if (num_statements == 0)
+    return false;
+
+  /*
+   * Only INSERT, UPDATE, and DELETE statements can possibly
+   * have bulk segments.  So, we loop through the statements
+   * checking for segment_id > 1 in those specific submessages.
+   */
+  size_t x;
+  for (x= 0; x < num_statements; ++x)
+  {
+    const Statement &statement= transaction.statement(x);
+    Statement::Type type= statement.type();
+
+    switch (type)
+    {
+      case Statement::INSERT:
+        if (statement.insert_data().segment_id() > 1)
+          return true;
+        break;
+      case Statement::UPDATE:
+        if (statement.update_data().segment_id() > 1)
+          return true;
+        break;
+      case Statement::DELETE:
+        if (statement.delete_data().segment_id() > 1)
+          return true;
+        break;
+      default:
+        break;
+    }
+  }
+  return false;
+}
+
 } /* namespace message */
 } /* namespace drizzled */
