@@ -43,6 +43,7 @@ namespace drizzled
 namespace message
 {
 /* some forward declarations */
+class Transaction;
 class Statement;
 class InsertHeader;
 class InsertData;
@@ -388,7 +389,8 @@ transformCreateTableStatementToSql(const CreateTableStatement &statement,
 enum TransformSqlError
 transformTableDefinitionToSql(const Table &table,
                               std::string &destination,
-                              enum TransformSqlVariant sql_variant= DRIZZLE);
+                              enum TransformSqlVariant sql_variant= DRIZZLE,
+                              bool with_schema= true);
 
 /**
  * Appends to the supplied string an SQL expression
@@ -432,6 +434,26 @@ transformIndexDefinitionToSql(const Table::Index &index,
                               std::string &destination,
                               enum TransformSqlVariant sql_variant= DRIZZLE);
 
+/**
+ * Appends to the supplied string an SQL expression
+ * representing the foreign key attributes.  The built string
+ * corresponds to the SQL in a CREATE TABLE statement.
+ *
+ * @param[in]   Foreign Key Constraint message
+ * @param[in]   Table containing this foregin key (used to get field names...)
+ * @param[out]  String to append to
+ *
+ * @retval
+ *  NONE if successful transformation
+ * @retval
+ *  Error code (see enum TransformSqlError definition) if failure
+ */
+enum TransformSqlError
+transformForeignKeyConstraintDefinitionToSql(const Table::ForeignKeyConstraint &fkey,
+                                             const Table &table,
+                                             std::string &destination,
+                                             enum TransformSqlVariant sql_variant = DRIZZLE);
+
 /**
  * Appends to the supplied string an SQL expression
  * representing the field's attributes.  The built string
@@ -460,6 +482,15 @@ bool shouldQuoteFieldValue(Table::Field::FieldType in_type);
 
 drizzled::message::Table::Field::FieldType internalFieldTypeToFieldProtoType(enum enum_field_types type);
 
+/**
+ * Returns true if the transaction contains any Statement
+ * messages which are not end segments (i.e. a bulk statement has
+ * previously been sent to replicators).
+ *
+ * @param The transaction to check
+ */
+bool transactionContainsBulkSegment(const Transaction &transaction);
+
 } /* namespace message */
 } /* namespace drizzled */
 
