@@ -17,37 +17,22 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <boost/algorithm/string.hpp>
-#include <drizzled/module/context.h>
-#include <drizzled/module/option_map.h>
-#include <drizzled/module/module.h>
-#include <drizzled/drizzled.h>
-#include <drizzled/sys_var.h>
+#include "config.h"
+#include "drizzled/module/context.h"
+#include "drizzled/module/option_map.h"
+#include "drizzled/module/module.h"
+#include "drizzled/drizzled.h"
 
-namespace drizzled {
-namespace module {
+namespace drizzled
+{
+
+namespace module
+{
 
 module::option_map Context::getOptions()
 {
   return module::option_map(module->getName(), getVariablesMap());
 }
 
-void Context::registerVariable(sys_var *var)
-{
-  var->setName(prepend_name(module->getName(), var->getName()));
-  module->addMySysVar(var);
-  add_sys_var_to_list(var);
-}
-
-std::string Context::prepend_name(std::string module_name,
-                                  const std::string &var_name)
-{
-  module_name.push_back('_');
-  module_name.append(var_name);
-  std::replace(module_name.begin(), module_name.end(), '-', '_');
-  return boost::to_lower_copy(module_name);
-}
-
 } /* namespace module */
 } /* namespace drizzled */
