@@ -17,21 +17,30 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
 #include "option_map.h"
 
-namespace drizzled {
-namespace module {
+
+namespace drizzled
+{
+namespace module
+{
 
 namespace po= boost::program_options;
 
 option_map::option_map(const std::string &module_name_in,
                        const po::variables_map &vm_in) :
-  vm(vm_in),
-  module_name(module_name_in)
+  module_name(module_name_in),
+  vm(vm_in)
 { }
 
+option_map::option_map(const option_map &old) :
+  module_name(old.module_name),
+  vm(old.vm)
+{ }
+
+
 } /* namespace module */
 } /* namespace drizzled */
 
