@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *  Copyright (C) 2010 Joseph Daly <skinny.moey@gmail.com> 
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,18 +18,27 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 #include <string>
-#include <drizzled/named_savepoint.h>
-#include <drizzled/message/transaction.pb.h>
+#include "drizzled/named_savepoint.h"
+#include "drizzled/message/transaction.pb.h"
 
 namespace drizzled
 {
   NamedSavepoint::NamedSavepoint(const NamedSavepoint &other)
   {
-    name= other.getName();
-    resource_contexts= other.getResourceContexts();
-    transaction_message= other.getTransactionMessage() ? new message::Transaction(*other.getTransactionMessage()) : NULL;
+    name.assign(other.getName());
+    const TransactionContext::ResourceContexts &other_resource_contexts= other.getResourceContexts();
+    resource_contexts.assign(other_resource_contexts.begin(),
+                             other_resource_contexts.end());
+    if (other.getTransactionMessage() != NULL)
+    {
+      transaction_message= new message::Transaction(*other.getTransactionMessage());
+    }
+    else 
+    {
+      transaction_message= NULL;
+    }
   }
 
   NamedSavepoint::~NamedSavepoint()
