@@ -29,6 +29,11 @@
 namespace drizzled
 {
 
+namespace message
+{
+class Transaction;
+}
+
 /**
  * This is a class which stores information about
  * a named savepoint in a transaction
@@ -41,11 +46,14 @@ public:
    */
   NamedSavepoint(const char *in_name, size_t in_name_length) :
     name(in_name, in_name_length),
-    resource_contexts()
-  {}
-  ~NamedSavepoint()
+    resource_contexts(),
+    transaction_message(NULL)
   {}
 
+  ~NamedSavepoint();
+
+  NamedSavepoint(const NamedSavepoint &other);
+
   void setResourceContexts(TransactionContext::ResourceContexts &new_contexts)
   {
     resource_contexts.assign(new_contexts.begin(), new_contexts.end());
@@ -66,12 +74,13 @@ public:
   {
     return name;
   }
-  NamedSavepoint(const NamedSavepoint &other)
+  message::Transaction *getTransactionMessage() const
   {
-    name.assign(other.getName());
-    const TransactionContext::ResourceContexts &other_resource_contexts= other.getResourceContexts();
-    resource_contexts.assign(other_resource_contexts.begin(),
-                             other_resource_contexts.end());
+    return transaction_message;
+  }
+  void setTransactionMessage(message::Transaction *in_transaction_message)
+  {
+    transaction_message= in_transaction_message;
   }
   NamedSavepoint &operator=(const NamedSavepoint &other)
   {
@@ -87,6 +96,7 @@ public:
 private:
   std::string name;
   TransactionContext::ResourceContexts resource_contexts;
+  message::Transaction *transaction_message;
   NamedSavepoint();
 };
 
