@@ -65,6 +65,16 @@ struct nested_join_st
   table_map sj_corr_tables;
 
   List<Item> sj_outer_expr_list;
+
+  /**
+     True if this join nest node is completely covered by the query execution
+     plan. This means two things.
+
+     1. All tables on its @c join_list are covered by the plan.
+
+     2. All child join nest nodes are fully covered.
+   */
+  bool is_fully_covered() const { return join_list.elements == counter_; }
 };
 
 } /* namespace drizzled */
