@@ -63,43 +63,26 @@ public:
     the 'LOCK_TABLES' chapter of the MySQL manual.
     See also lock_tables() for details.
   */
-  DRIZZLE_LOCK *lock;
+  DrizzleLock *lock;
 
   /*
     CREATE-SELECT keeps an extra lock for the table being
     created. This field is used to keep the extra lock available for
     lower level routines, which would otherwise miss that lock.
    */
-  DRIZZLE_LOCK *extra_lock;
+  DrizzleLock *extra_lock;
 
   uint64_t version;
   uint32_t current_tablenr;
 
-  /*
-    Flags with information about the open tables state.
-  */
-  bool backups_available;
-
   /*
     This constructor serves for creation of Open_tables_state instances
     which are used as backup storage.
   */
-  Open_tables_state() : backups_available(false) { }
+  Open_tables_state() { }
   virtual ~Open_tables_state() {}
 
   Open_tables_state(uint64_t version_arg);
-
-  void set_open_tables_state(Open_tables_state *state)
-  {
-    *this= *state;
-  }
-
-  void reset_open_tables_state()
-  {
-    open_tables= temporary_tables= derived_tables= NULL;
-    extra_lock= lock= NULL;
-    backups_available= false;
-  }
 };
 
 } /* namespace drizzled */
