@@ -18,12 +18,25 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_OPTIMIZER_ACCESS_METHOD_H
+#define DRIZZLED_OPTIMIZER_ACCESS_METHOD_H
 
-#include <drizzled/common_fwd.h>
+#include "drizzled/definitions.h"
+#include "drizzled/error.h"
+#include "drizzled/sql_parse.h"
+#include "drizzled/sql_base.h"
+#include "drizzled/show.h"
 
-namespace drizzled {
-namespace optimizer {
+
+namespace drizzled
+{
+
+/* Forward declarations */
+class Table;
+class JoinTable;
+
+namespace optimizer
+{
 
 /**
  * @class AccessMethod
@@ -32,7 +45,8 @@ namespace optimizer {
 class AccessMethod
 {
 public:
-  typedef boost::shared_ptr<AccessMethod> ptr;
+
+  AccessMethod() {}
 
   virtual ~AccessMethod() {}
 
@@ -43,9 +57,13 @@ public:
    * @param[in] join_tab
    * @return true on failure; false on success
    */
-  virtual void getStats(Table&, JoinTable&)= 0;
+  virtual bool getStats(Table *table,
+                        JoinTable *join_tab)= 0;
+
 };
 
 } /* end namespace optimizer */
+
 } /* end namespace drizzled */
 
+#endif /* DRIZZLED_OPTIMIZER_ACCESS_METHOD_H */
