@@ -18,24 +18,25 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <drizzled/join_table.h>
-#include <drizzled/optimizer/access_method/const.h>
-#include <drizzled/sql_select.h>
-#include <drizzled/table.h>
+#include "config.h"
+#include "drizzled/session.h"
+#include "drizzled/join_table.h"
+#include "drizzled/sql_select.h"
+#include "drizzled/optimizer/access_method/const.h"
 
-namespace drizzled {
+using namespace drizzled;
 
-void optimizer::Const::getStats(Table& table, JoinTable& join_tab)
+bool optimizer::Const::getStats(Table *table,
+                                JoinTable *join_tab)
 {
-  table.status= STATUS_NO_RECORD;
-  join_tab.read_first_record= join_read_const;
-  join_tab.read_record.read_record= join_no_more_records;
-  if (table.covering_keys.test(join_tab.ref.key) && not table.no_keyread)
+  table->status= STATUS_NO_RECORD;
+  join_tab->read_first_record= join_read_const;
+  join_tab->read_record.read_record= join_no_more_records;
+  if (table->covering_keys.test(join_tab->ref.key) &&
+      ! table->no_keyread)
   {
-    table.key_read= 1;
-    table.cursor->extra(HA_EXTRA_KEYREAD);
+    table->key_read= 1;
+    table->cursor->extra(HA_EXTRA_KEYREAD);
   }
-}
-
+  return false;
 }
