@@ -18,12 +18,17 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_OPTIMIZER_ACCESS_METHOD_CONST_H
+#define DRIZZLED_OPTIMIZER_ACCESS_METHOD_CONST_H
 
-#include <drizzled/optimizer/access_method.h>
+#include "drizzled/optimizer/access_method.h"
 
-namespace drizzled {
-namespace optimizer {
+class Session;
+
+namespace drizzled
+{
+namespace optimizer
+{
 
 /**
  * @class Const
@@ -33,8 +38,15 @@ namespace optimizer {
 class Const : public AccessMethod
 {
 public:
-  void getStats(Table&, JoinTable&);
+  Const() {}
+
+  bool getStats(Table *table,
+                JoinTable *join_tab);
+
 };
 
 } /* end namespace optimizer */
+
 } /* end namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_ACCESS_METHOD_CONST_H */
