@@ -18,17 +18,19 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <drizzled/join_table.h>
-#include <drizzled/sql_select.h>
-#include <drizzled/optimizer/access_method/system.h>
-#include <drizzled/table.h>
+#include "config.h"
+#include "drizzled/session.h"
+#include "drizzled/join_table.h"
+#include "drizzled/sql_select.h"
+#include "drizzled/optimizer/access_method/system.h"
 
 using namespace drizzled;
 
-void optimizer::System::getStats(Table& table, JoinTable& join_tab)
+bool optimizer::System::getStats(Table *table,
+                                 JoinTable *join_tab)
 {
-  table.status= STATUS_NO_RECORD;
-  join_tab.read_first_record= join_read_system;
-  join_tab.read_record.read_record= join_no_more_records;
+  table->status= STATUS_NO_RECORD;
+  join_tab->read_first_record= join_read_system;
+  join_tab->read_record.read_record= join_no_more_records;
+  return false;
 }
