@@ -18,13 +18,26 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_OPTIMIZER_ACCESS_METHOD_FACTORY_H
+#define DRIZZLED_OPTIMIZER_ACCESS_METHOD_FACTORY_H
 
-#include <drizzled/join_table.h>
-#include <drizzled/optimizer/access_method.h>
+#include "drizzled/definitions.h"
+#include "drizzled/error.h"
+#include "drizzled/sql_parse.h"
+#include "drizzled/sql_base.h"
+#include "drizzled/join_table.h"
+#include "drizzled/optimizer/access_method.h"
 
-namespace drizzled {
-namespace optimizer {
+#include <boost/shared_ptr.hpp>
+
+/* Forward declarations */
+class Table;
+class JoinTable;
+
+namespace drizzled
+{
+namespace optimizer
+{
 
 /**
  * @class AccessMethodFactory
@@ -33,9 +46,26 @@ namespace optimizer {
 class AccessMethodFactory
 {
 public:
-  static AccessMethod::ptr create(access_method);
+
+  static AccessMethodFactory &singleton()
+  {
+    static AccessMethodFactory fact;
+    return fact;
+  }
+
+  boost::shared_ptr<AccessMethod> createAccessMethod(enum access_method type);
+
+private:
+  
+  AccessMethodFactory() {}
+
+  ~AccessMethodFactory() {}
+
+  AccessMethodFactory(const AccessMethodFactory&);
 };
 
 } /* end namespace optimizer */
+
 } /* end namespace drizzled */
 
+#endif /* DRIZZLED_OPTIMIZER_ACCESS_METHOD_FACTORY_H */
