@@ -68,8 +68,8 @@ class QuickGroupMinMaxSelect : public QuickSelectInterface
 private:
 
   Cursor *cursor; /**< The Cursor used to get data. */
-  JOIN *join; /**< Descriptor of the current query */
-  KEY *index_info; /**< The index chosen for data access */
+  Join *join; /**< Descriptor of the current query */
+  KeyInfo *index_info; /**< The index chosen for data access */
   unsigned char *record; /**< Buffer where the next record is returned. */
   unsigned char *tmp_record; /**< Temporary storage for next_min(), next_max(). */
   unsigned char *group_prefix; /**< Key prefix consisting of the GROUP fields. */
@@ -79,7 +79,7 @@ private:
   bool have_min; /**< Specify whether we are computing */
   bool have_max; /**< a MIN, a MAX, or both. */
   bool seen_first_key; /**< Denotes whether the first key was retrieved.*/
-  KEY_PART_INFO *min_max_arg_part; /** The keypart of the only argument field of all MIN/MAX functions. */
+  KeyPartInfo *min_max_arg_part; /** The keypart of the only argument field of all MIN/MAX functions. */
   uint32_t min_max_arg_len; /**< The length of the MIN/MAX argument field */
   unsigned char *key_infix; /**< Infix of constants from equality predicates. */
   uint32_t key_infix_len;
@@ -217,14 +217,14 @@ private:
    * DESCRIPTION
    * The method iterates through all MIN functions and updates the result value
    * of each function by calling Item_sum::reset(), which in turn picks the new
-   * result value from this->head->record[0], previously updated by
+   * result value from this->head->getInsertRecord(), previously updated by
    * next_min(). The updated value is stored in a member variable of each of the
    * Item_sum objects, depending on the value type.
    *
    * IMPLEMENTATION
    * The update must be done separately for MIN and MAX, immediately after
    * next_min() was called and before next_max() is called, because both MIN and
-   * MAX take their result value from the same buffer this->head->record[0]
+   * MAX take their result value from the same buffer this->head->getInsertRecord()
    * (i.e.  this->record).
    *
    * RETURN
@@ -241,14 +241,14 @@ private:
    * DESCRIPTION
    * The method iterates through all MAX functions and updates the result value
    * of each function by calling Item_sum::reset(), which in turn picks the new
-   * result value from this->head->record[0], previously updated by
+   * result value from this->head->getInsertRecord(), previously updated by
    * next_max(). The updated value is stored in a member variable of each of the
    * Item_sum objects, depending on the value type.
    *
    * IMPLEMENTATION
    * The update must be done separately for MIN and MAX, immediately after
    * next_max() was called, because both MIN and MAX take their result value
-   * from the same buffer this->head->record[0] (i.e.  this->record).
+   * from the same buffer this->head->getInsertRecord() (i.e.  this->record).
    *
    * RETURN
    * None
@@ -281,14 +281,14 @@ public:
      None
    */
   QuickGroupMinMaxSelect(Table *table, 
-                         JOIN *join, 
+                         Join *join, 
                          bool have_min,
                          bool have_max, 
-                         KEY_PART_INFO *min_max_arg_part,
+                         KeyPartInfo *min_max_arg_part,
                          uint32_t group_prefix_len, 
                          uint32_t group_key_parts,
                          uint32_t used_key_parts, 
-                         KEY *index_info,
+                         KeyInfo *index_info,
                          uint use_index, 
                          double read_cost, 
                          ha_rows records,
