@@ -69,7 +69,7 @@ optimizer::QuickIndexMergeSelect::push_quick_back(optimizer::QuickRangeSelect *q
     processed separately.
   */
   if (head->cursor->primary_key_is_clustered() &&
-      quick_sel_range->index == head->s->primary_key)
+      quick_sel_range->index == head->getShare()->getPrimaryKey())
   {
     pk_quick_select= quick_sel_range;
   }
@@ -93,7 +93,7 @@ optimizer::QuickIndexMergeSelect::~QuickIndexMergeSelect()
            DeletePtr());
   quick_selects.clear();
   delete pk_quick_select;
-  free_root(&alloc,MYF(0));
+  alloc.free_root(MYF(0));
   return;
 }
 
@@ -141,7 +141,7 @@ int optimizer::QuickIndexMergeSelect::read_keys_and_merge()
         break;
 
       if (cur_quick->cursor->inited != Cursor::NONE)
-        cur_quick->cursor->ha_index_end();
+        cur_quick->cursor->endIndexScan();
       if (cur_quick->init() || cur_quick->reset())
         return 0;
     }
@@ -177,7 +177,7 @@ int optimizer::QuickIndexMergeSelect::read_keys_and_merge()
   /* index_merge currently doesn't support "using index" at all */
   cursor->extra(HA_EXTRA_NO_KEYREAD);
   /* start table scan */
-  init_read_record(&read_record, session, head, (optimizer::SqlSelect*) 0, 1, 1);
+  read_record.init_read_record(session, head, (optimizer::SqlSelect*) 0, 1, 1);
   return result;
 }
 
@@ -192,7 +192,7 @@ int optimizer::QuickIndexMergeSelect::get_next()
   if ((result= read_record.read_record(&read_record)) == -1)
   {
     result= HA_ERR_END_OF_FILE;
-    end_read_record(&read_record);
+    read_record.end_read_record();
     /* All rows from Unique have been retrieved, do a clustered PK scan */
     if (pk_quick_select)
     {
@@ -264,14 +264,14 @@ void optimizer::QuickIndexMergeSelect::add_keys_and_lengths(String *key_names,
       used_lengths->append(',');
     }
 
-    KEY *key_info= head->key_info + (*it)->index;
+    KeyInfo *key_info= head->key_info + (*it)->index;
     key_names->append(key_info->name);
     length= internal::int64_t2str((*it)->max_used_key_length, buf, 10) - buf;
     used_lengths->append(buf, length);
   }
   if (pk_quick_select)
   {
-    KEY *key_info= head->key_info + pk_quick_select->index;
+    KeyInfo *key_info= head->key_info + pk_quick_select->index;
     key_names->append(',');
     key_names->append(key_info->name);
     length= internal::int64_t2str(pk_quick_select->max_used_key_length, buf, 10) - buf;
