@@ -30,7 +30,7 @@
 namespace drizzled
 {
 
-class JOIN;
+class Join;
 class RorIntersectReadPlan; 
 typedef class Item COND;
 
@@ -39,13 +39,11 @@ namespace internal
 typedef struct st_io_cache IO_CACHE;
 }
 
-typedef struct st_handler_buffer HANDLER_BUFFER;
-
 typedef struct st_key_part
 {
   uint16_t key;
   uint16_t part;
-  /* See KEY_PART_INFO for meaning of the next two: */
+  /* See KeyPartInfo for meaning of the next two: */
   uint16_t store_length;
   uint16_t length;
   uint8_t null_bit;
