@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008-2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2008-2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,16 +17,16 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <drizzled/sql_base.h>
-#include <drizzled/sql_select.h>
-#include <drizzled/memory/sql_alloc.h>
-#include <drizzled/optimizer/range.h>
-#include <drizzled/optimizer/range_param.h>
-#include <drizzled/optimizer/sel_arg.h>
-#include <drizzled/optimizer/sel_tree.h>
-#include <drizzled/optimizer/sel_imerge.h>
+#include "drizzled/sql_base.h"
+#include "drizzled/sql_select.h"
+#include "drizzled/memory/sql_alloc.h"
+#include "drizzled/optimizer/range.h"
+#include "drizzled/optimizer/range_param.h"
+#include "drizzled/optimizer/sel_arg.h"
+#include "drizzled/optimizer/sel_tree.h"
+#include "drizzled/optimizer/sel_imerge.h"
 
 using namespace std;
 using namespace drizzled;
@@ -34,19 +34,31 @@ using namespace drizzled;
 static optimizer::SEL_ARG *key_or(optimizer::RangeParameter *param, optimizer::SEL_ARG *key1, optimizer::SEL_ARG *key2);
 static bool eq_tree(optimizer::SEL_ARG* a, optimizer::SEL_ARG *b);
 
-bool optimizer::sel_trees_can_be_ored(const SEL_TREE& tree1, const SEL_TREE& tree2, const RangeParameter& param)
+bool optimizer::sel_trees_can_be_ored(optimizer::SEL_TREE *tree1, 
+                                      optimizer::SEL_TREE *tree2,
+                                      optimizer::RangeParameter* param)
 {
-  key_map common_keys= tree1.keys_map;
-  common_keys&= tree2.keys_map;
+  key_map common_keys= tree1->keys_map;
+  common_keys&= tree2->keys_map;
 
   if (common_keys.none())
+  {
     return false;
+  }
 
   /* trees have a common key, check if they refer to same key part */
-  for (uint32_t key_no= 0; key_no < param.keys; key_no++)
+  optimizer::SEL_ARG **key1,**key2;
+  for (uint32_t key_no=0; key_no < param->keys; key_no++)
   {
-    if (common_keys.test(key_no) && tree1.keys[key_no]->part == tree2.keys[key_no]->part)
-      return true;
+    if (common_keys.test(key_no))
+    {
+      key1= tree1->keys + key_no;
+      key2= tree2->keys + key_no;
+      if ((*key1)->part == (*key2)->part)
+      {
+        return true;
+      }
+    }
   }
   return false;
 }
@@ -75,13 +87,15 @@ bool optimizer::sel_trees_can_be_ored(const SEL_TREE& tree1, const SEL_TREE& tre
     other Error, both passed lists are unusable
 */
 
-static int imerge_list_or_list(optimizer::RangeParameter *param, List<optimizer::SEL_IMERGE> *im1, List<optimizer::SEL_IMERGE> *im2)
+static int imerge_list_or_list(optimizer::RangeParameter *param,
+                               List<optimizer::SEL_IMERGE> *im1,
+                               List<optimizer::SEL_IMERGE> *im2)
 {
-  optimizer::SEL_IMERGE *imerge= &im1->front();
-  im1->clear();
+  optimizer::SEL_IMERGE *imerge= im1->head();
+  im1->empty();
   im1->push_back(imerge);
 
-  return imerge->or_sel_imerge_with_checks(*param, im2->front());
+  return imerge->or_sel_imerge_with_checks(param, im2->head());
 }
 
 
@@ -93,45 +107,63 @@ static int imerge_list_or_list(optimizer::RangeParameter *param, List<optimizer:
      other Error
  */
 
-static int imerge_list_or_tree(optimizer::RangeParameter& param, List<optimizer::SEL_IMERGE>& im1, optimizer::SEL_TREE& tree)
+static int imerge_list_or_tree(optimizer::RangeParameter *param,
+                               List<optimizer::SEL_IMERGE> *im1,
+                               optimizer::SEL_TREE *tree)
 {
-  List_iterator<optimizer::SEL_IMERGE> it(im1.begin());
-  while (optimizer::SEL_IMERGE* imerge= it++)
+  optimizer::SEL_IMERGE *imerge= NULL;
+  List_iterator<optimizer::SEL_IMERGE> it(*im1);
+  while ((imerge= it++))
   {
     if (imerge->or_sel_tree_with_checks(param, tree))
       it.remove();
   }
-  return im1.is_empty();
+  return im1->is_empty();
 }
 
 
-optimizer::SEL_TREE* optimizer::tree_or(optimizer::RangeParameter *param, optimizer::SEL_TREE *tree1, optimizer::SEL_TREE *tree2)
+optimizer::SEL_TREE *
+optimizer::tree_or(optimizer::RangeParameter *param,
+                   optimizer::SEL_TREE *tree1,
+                   optimizer::SEL_TREE *tree2)
 {
   if (! tree1 || ! tree2)
+  {
     return 0;
+  }
 
-  if (tree1->type == SEL_TREE::IMPOSSIBLE || tree2->type == SEL_TREE::ALWAYS)
+  if (tree1->type == optimizer::SEL_TREE::IMPOSSIBLE || tree2->type == optimizer::SEL_TREE::ALWAYS)
+  {
     return tree2;
+  }
 
-  if (tree2->type == SEL_TREE::IMPOSSIBLE || tree1->type == SEL_TREE::ALWAYS)
+  if (tree2->type == optimizer::SEL_TREE::IMPOSSIBLE || tree1->type == optimizer::SEL_TREE::ALWAYS)
+  {
     return tree1;
+  }
 
-  if (tree1->type == SEL_TREE::MAYBE)
+  if (tree1->type == optimizer::SEL_TREE::MAYBE)
+  {
     return tree1; // Can't use this
+  }
 
-  if (tree2->type == SEL_TREE::MAYBE)
+  if (tree2->type == optimizer::SEL_TREE::MAYBE)
+  {
     return tree2;
+  }
 
-  SEL_TREE *result= NULL;
+  optimizer::SEL_TREE *result= NULL;
   key_map  result_keys;
   result_keys.reset();
-  if (sel_trees_can_be_ored(*tree1, *tree2, *param))
+  if (sel_trees_can_be_ored(tree1, tree2, param))
   {
     /* Join the trees key per key */
-    SEL_ARG** key1= tree1->keys;
-    SEL_ARG** key2= tree2->keys;
-    SEL_ARG** end= key1+param->keys;
-    for (; key1 != end;  key1++, key2++)
+    optimizer::SEL_ARG **key1= NULL;
+    optimizer::SEL_ARG **key2= NULL;
+    optimizer::SEL_ARG **end= NULL;
+    for (key1= tree1->keys,key2= tree2->keys,end= key1+param->keys;
+         key1 != end; 
+         key1++, key2++)
     {
       *key1= key_or(param, *key1, *key2);
       if (*key1)
@@ -148,21 +180,36 @@ optimizer::SEL_TREE* optimizer::tree_or(optimizer::RangeParameter *param, optimi
     /* ok, two trees have KEY type but cannot be used without index merge */
     if (tree1->merges.is_empty() && tree2->merges.is_empty())
     {
-      if (param->remove_jump_scans && (remove_nonrange_trees(param, tree1) || remove_nonrange_trees(param, tree2)))
-        return new SEL_TREE(SEL_TREE::ALWAYS);
+      if (param->remove_jump_scans)
+      {
+        bool no_trees= optimizer::remove_nonrange_trees(param, tree1);
+        no_trees= no_trees || optimizer::remove_nonrange_trees(param, tree2);
+        if (no_trees)
+        {
+          return (new optimizer::SEL_TREE(optimizer::SEL_TREE::ALWAYS));
+        }
+      }
+      optimizer::SEL_IMERGE *merge= NULL;
       /* both trees are "range" trees, produce new index merge structure */
-			result= new SEL_TREE();
-			SEL_IMERGE* merge= new SEL_IMERGE();
-			result->merges.push_back(merge);
-      merge->or_sel_tree(param, tree1);
-      merge->or_sel_tree(param, tree2);
-      result->type= tree1->type;
+      if (! (result= new optimizer::SEL_TREE()) || 
+          ! (merge= new optimizer::SEL_IMERGE()) ||
+          (result->merges.push_back(merge)) ||
+          (merge->or_sel_tree(param, tree1)) ||
+          (merge->or_sel_tree(param, tree2)))
+      {
+        result= NULL;
+      }
+      else
+      {
+        result->type= tree1->type;
+      }
     }
     else if (!tree1->merges.is_empty() && !tree2->merges.is_empty())
     {
-      result= imerge_list_or_list(param, &tree1->merges, &tree2->merges)
-        ? new SEL_TREE(SEL_TREE::ALWAYS)
-        : tree1;
+      if (imerge_list_or_list(param, &tree1->merges, &tree2->merges))
+        result= new optimizer::SEL_TREE(optimizer::SEL_TREE::ALWAYS);
+      else
+        result= tree1;
     }
     else
     {
@@ -170,12 +217,13 @@ optimizer::SEL_TREE* optimizer::tree_or(optimizer::RangeParameter *param, optimi
       if (tree1->merges.is_empty())
         std::swap(tree1, tree2);
 
-      if (param->remove_jump_scans && remove_nonrange_trees(param, tree2))
-         return new SEL_TREE(SEL_TREE::ALWAYS);
+      if (param->remove_jump_scans && optimizer::remove_nonrange_trees(param, tree2))
+         return(new optimizer::SEL_TREE(optimizer::SEL_TREE::ALWAYS));
       /* add tree2 to tree1->merges, checking if it collapses to ALWAYS */
-      result= imerge_list_or_tree(*param, tree1->merges, *tree2)
-        ? new SEL_TREE(SEL_TREE::ALWAYS)
-        : tree1;
+      if (imerge_list_or_tree(param, &tree1->merges, tree2))
+        result= new optimizer::SEL_TREE(optimizer::SEL_TREE::ALWAYS);
+      else
+        result= tree1;
     }
   }
   return result;
@@ -257,7 +305,8 @@ key_or(optimizer::RangeParameter *param, optimizer::SEL_ARG *key1, optimizer::SE
         optimizer::SEL_ARG *key2_next=key2->next;
         if (key2_shared)
         {
-          key2=new optimizer::SEL_ARG(*key2);
+          if (! (key2=new optimizer::SEL_ARG(*key2)))
+            return 0;		// out of memory
           key2->increment_use_count(key1->use_count+1);
           key2->next= key2_next; // New copy of key2
         }
@@ -348,6 +397,8 @@ key_or(optimizer::RangeParameter *param, optimizer::SEL_ARG *key1, optimizer::SE
     if (cmp >= 0 && tmp->cmp_min_to_min(key2) < 0)
     {						// tmp.min <= x < key2.min
       optimizer::SEL_ARG *new_arg= tmp->clone_first(key2);
+      if (! new_arg)
+        return 0;				// OOM
       if ((new_arg->next_key_part= key1->next_key_part))
         new_arg->increment_use_count(key1->use_count+1);
       tmp->copy_min_to_min(key2);
@@ -361,6 +412,8 @@ key_or(optimizer::RangeParameter *param, optimizer::SEL_ARG *key1, optimizer::SE
       if (tmp->cmp_min_to_min(&key) > 0)
       {						// key.min <= x < tmp.min
         optimizer::SEL_ARG *new_arg= key.clone_first(tmp);
+        if (! new_arg)
+          return 0;				// OOM
         if ((new_arg->next_key_part=key.next_key_part))
           new_arg->increment_use_count(key1->use_count+1);
         key1= key1->insert(new_arg);
@@ -394,6 +447,8 @@ key_or(optimizer::RangeParameter *param, optimizer::SEL_ARG *key1, optimizer::SE
       else
       {
         optimizer::SEL_ARG *new_arg= tmp->clone_last(&key); // tmp.min <= x <= key.max
+        if (! new_arg)
+          return 0;				// OOM
         tmp->copy_max_to_min(&key);
         tmp->increment_use_count(key1->use_count+1);
         /* Increment key count as it may be used for next loop */
@@ -483,7 +538,7 @@ end:
 */
 bool optimizer::remove_nonrange_trees(optimizer::RangeParameter *param, optimizer::SEL_TREE *tree)
 {
-  bool res= true;
+  bool res= false;
   for (uint32_t i= 0; i < param->keys; i++)
   {
     if (tree->keys[i])
@@ -494,10 +549,10 @@ bool optimizer::remove_nonrange_trees(optimizer::RangeParameter *param, optimize
         tree->keys_map.reset(i);
       }
       else
-        res= false;
+        res= true;
     }
   }
-  return res;
+  return ! res;
 }
 
 
@@ -505,7 +560,9 @@ bool optimizer::remove_nonrange_trees(optimizer::RangeParameter *param, optimize
 static bool eq_tree(optimizer::SEL_ARG *a, optimizer::SEL_ARG *b)
 {
   if (a == b)
+  {
     return true;
+  }
 
   if (! a || ! b || ! a->is_same(b))
   {
