@@ -135,7 +135,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
   for (TableList *tl= tables; tl; tl= tl->next_leaf)
   {
     TableList *embedded= NULL;
-    for (embedded= tl; embedded; embedded= embedded->embedding)
+    for (embedded= tl; embedded; embedded= embedded->getEmbedding())
     {
       if (embedded->on_expr)
         break;
@@ -239,7 +239,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
 
               ref.key_buff= key_buff;
               Item_field *item_field= (Item_field*) (expr->real_item());
-              Table *table= item_field->field->table;
+              Table *table= item_field->field->getTable();
 
               /*
                  Look for a partial key that can be used for optimization.
@@ -261,7 +261,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
                 const_result= 0;
                 break;
               }
-              error= table->cursor->ha_index_init(static_cast<uint32_t>(ref.key), 1);
+              error= table->cursor->startIndexScan(static_cast<uint32_t>(ref.key), 1);
 
               if (! ref.key_length)
               {
@@ -351,7 +351,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
                 table->key_read= 0;
                 table->cursor->extra(HA_EXTRA_NO_KEYREAD);
               }
-              table->cursor->ha_index_end();
+              table->cursor->endIndexScan();
               if (error)
               {
                 if (error == HA_ERR_KEY_NOT_FOUND || error == HA_ERR_END_OF_FILE)
@@ -407,7 +407,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
 
               ref.key_buff= key_buff;
               Item_field *item_field= (Item_field*) (expr->real_item());
-              Table *table= item_field->field->table;
+              Table *table= item_field->field->getTable();
 
               /*
                  Look for a partial key that can be used for optimization.
@@ -429,7 +429,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
                 const_result= 0;
                 break;
               }
-              error= table->cursor->ha_index_init(static_cast<uint32_t>(ref.key), 1);
+              error= table->cursor->startIndexScan(static_cast<uint32_t>(ref.key), 1);
 
               if (! ref.key_length)
               {
@@ -459,7 +459,7 @@ int optimizer::sum_query(TableList *tables, List<Item> &all_fields, COND *conds)
                 table->key_read= 0;
                 table->cursor->extra(HA_EXTRA_NO_KEYREAD);
               }
-              table->cursor->ha_index_end();
+              table->cursor->endIndexScan();
               if (error)
               {
                 if (error == HA_ERR_KEY_NOT_FOUND || error == HA_ERR_END_OF_FILE)
@@ -647,8 +647,8 @@ bool optimizer::simple_pred(Item_func *func_item, Item **args, bool &inv_order)
 */
 static bool matching_cond(bool max_fl,
                           table_reference_st *ref,
-                          KEY *keyinfo,
-                          KEY_PART_INFO *field_part,
+                          KeyInfo *keyinfo,
+                          KeyPartInfo *field_part,
                           COND *cond,
                           key_part_map *key_part_used,
                           uint32_t *range_fl,
@@ -662,7 +662,7 @@ static bool matching_cond(bool max_fl,
 
   field->setWriteSet();
 
-  if (! (cond->used_tables() & field->table->map))
+  if (! (cond->used_tables() & field->getTable()->map))
   {
     /* Condition doesn't restrict the used table */
     return 1;
@@ -748,7 +748,7 @@ static bool matching_cond(bool max_fl,
 
   /* Check if field is part of the tested partial key */
   unsigned char *key_ptr= ref->key_buff;
-  KEY_PART_INFO *part= NULL;
+  KeyPartInfo *part= NULL;
   for (part= keyinfo->key_part; ; key_ptr+= part++->store_length)
 
   {
@@ -907,16 +907,16 @@ static bool find_key_for_maxmin(bool max_fl,
     return 0; // Not key field
   }
 
-  Table *table= field->table;
+  Table *table= field->getTable();
   uint32_t idx= 0;
 
-  KEY *keyinfo,*keyinfo_end= NULL;
-  for (keyinfo= table->key_info, keyinfo_end= keyinfo+table->s->keys;
+  KeyInfo *keyinfo,*keyinfo_end= NULL;
+  for (keyinfo= table->key_info, keyinfo_end= keyinfo+table->getShare()->sizeKeys();
        keyinfo != keyinfo_end;
        keyinfo++,idx++)
   {
-    KEY_PART_INFO *part= NULL;
-    KEY_PART_INFO *part_end= NULL;
+    KeyPartInfo *part= NULL;
+    KeyPartInfo *part_end= NULL;
     key_part_map key_part_to_use= 0;
     /*
       Perform a check if index is not disabled by ALTER Table
@@ -938,7 +938,7 @@ static bool find_key_for_maxmin(bool max_fl,
       }
 
       /* Check whether the index component is partial */
-      Field *part_field= table->field[part->fieldnr-1];
+      Field *part_field= table->getField(part->fieldnr-1);
       part_field->setWriteSet();
 
       if ((part_field->flags & BLOB_FLAG) ||
@@ -1028,7 +1028,7 @@ static int reckey_in_range(bool max_fl,
                            uint32_t range_fl,
                            uint32_t prefix_len)
 {
-  if (key_cmp_if_same(field->table, ref->key_buff, ref->key, prefix_len))
+  if (key_cmp_if_same(field->getTable(), ref->key_buff, ref->key, prefix_len))
   {
     return 1;
   }
@@ -1069,7 +1069,7 @@ static int maxmin_in_range(bool max_fl, Field* field, COND *cond)
     return 0;
   }
 
-  if (cond->used_tables() != field->table->map)
+  if (cond->used_tables() != field->getTable()->map)
   {
     return 0;
   }
