@@ -28,8 +28,6 @@
 #include <drizzled/function/str/right.h>
 #include <drizzled/function/str/set_collation.h>
 #include <drizzled/function/str/trim.h>
-#include <drizzled/function/str/user.h>
-
 #include <drizzled/function/time/curdate.h>
 #include <drizzled/function/time/date_add_interval.h>
 #include <drizzled/function/time/dayofmonth.h>
@@ -70,7 +68,6 @@
 #include <drizzled/statement/analyze.h>
 #include <drizzled/statement/change_schema.h>
 #include <drizzled/statement/check.h>
-#include <drizzled/statement/checksum.h>
 #include <drizzled/statement/commit.h>
 #include <drizzled/statement/create_index.h>
 #include <drizzled/statement/create_schema.h>
