@@ -20,13 +20,15 @@
 #ifndef DRIZZLED_PLUGIN_H
 #define DRIZZLED_PLUGIN_H
 
-#include <drizzled/lex_string.h>
-#include <drizzled/xid.h>
-#include <drizzled/plugin/manifest.h>
-#include <drizzled/plugin/module.h>
+#include "drizzled/module/manifest.h"
+#include "drizzled/module/module.h"
 #include "drizzled/plugin/version.h"
+#include "drizzled/module/context.h"
 #include "drizzled/definitions.h"
 
+#include "drizzled/lex_string.h"
+#include "drizzled/xid.h"
+#include <boost/program_options.hpp>
 
 namespace drizzled
 {
@@ -42,12 +44,8 @@ struct charset_info_st;
 
 class sys_var;
 typedef drizzle_lex_string LEX_STRING;
-struct my_option;
+struct option;
 
-extern char *opt_plugin_add;
-extern char *opt_plugin_remove;
-extern char *opt_plugin_load;
-extern char *opt_plugin_dir_ptr;
 extern char opt_plugin_dir[FN_REFLEN];
 
 namespace plugin { class StorageEngine; }
@@ -55,18 +53,18 @@ namespace plugin { class StorageEngine; }
 /*
   Macros for beginning and ending plugin declarations. Between
   DRIZZLE_DECLARE_PLUGIN and DRIZZLE_DECLARE_PLUGIN_END there should
-  be a plugin::Manifest for each plugin to be declared.
+  be a module::Manifest for each plugin to be declared.
 */
 
 
 #define PANDORA_CPP_NAME(x) _drizzled_ ## x ## _plugin_
 #define PANDORA_PLUGIN_NAME(x) PANDORA_CPP_NAME(x)
 #define DRIZZLE_DECLARE_PLUGIN \
-  ::drizzled::plugin::Manifest PANDORA_PLUGIN_NAME(PANDORA_MODULE_NAME)= 
+  ::drizzled::module::Manifest PANDORA_PLUGIN_NAME(PANDORA_MODULE_NAME)= 
 
 
 #define DRIZZLE_DECLARE_PLUGIN_END
-#define DRIZZLE_PLUGIN(init,deinit,system) \
+#define DRIZZLE_PLUGIN(init,system,options) \
   DRIZZLE_DECLARE_PLUGIN \
   { \
     DRIZZLE_VERSION_ID, \
@@ -75,7 +73,7 @@ namespace plugin { class StorageEngine; }
     STRINGIFY_ARG(PANDORA_MODULE_AUTHOR), \
     STRINGIFY_ARG(PANDORA_MODULE_TITLE), \
     PANDORA_MODULE_LICENSE, \
-    init, deinit, system, NULL \
+    init, system, options \
   } 
 
 
@@ -86,7 +84,7 @@ enum enum_mysql_show_type
 {
   SHOW_UNDEF, SHOW_BOOL, SHOW_INT, SHOW_LONG,
   SHOW_LONGLONG, SHOW_CHAR, SHOW_CHAR_PTR,
-  SHOW_FUNC, SHOW_KEY_CACHE_LONG, SHOW_KEY_CACHE_LONGLONG,
+  SHOW_FUNC,
   SHOW_LONG_STATUS, SHOW_DOUBLE_STATUS,
   SHOW_MY_BOOL, SHOW_HA_ROWS, SHOW_SYS, SHOW_INT_NOFLUSH,
   SHOW_LONGLONG_STATUS, SHOW_DOUBLE, SHOW_SIZE
@@ -187,7 +185,7 @@ typedef void (*mysql_var_update_func)(Session *session,
   mysql_var_check_func check;   \
   mysql_var_update_func update
 
-#define DRIZZLE_SYSVAR_NAME(name) mysql_sysvar_ ## name
+#define DRIZZLE_SYSVAR_NAME(name) drizzle_sysvar_ ## name
 #define DRIZZLE_SYSVAR(name) \
   ((drizzle_sys_var *)(&(DRIZZLE_SYSVAR_NAME(name))))
 
@@ -355,7 +353,7 @@ struct drizzle_sys_var
   DRIZZLE_PLUGIN_VAR_HEADER;
 };
 
-void plugin_opt_set_limits(my_option *options, const drizzle_sys_var *opt);
+void plugin_opt_set_limits(option *options, const drizzle_sys_var *opt);
 
 struct drizzle_value
 {
@@ -370,15 +368,10 @@ struct drizzle_value
   Miscellaneous functions for plugin implementors
 */
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-extern bool plugin_init(plugin::Registry &registry,
-                        int *argc, char **argv,
-                        bool skip_init);
-extern void plugin_shutdown(plugin::Registry &plugins);
-extern void my_print_help_inc_plugins(my_option *options);
+extern bool plugin_init(module::Registry &registry,
+                        boost::program_options::options_description &long_options);
+extern bool plugin_finalize(module::Registry &registry);
+extern void my_print_help_inc_plugins(option *options);
 extern bool plugin_is_ready(const LEX_STRING *name, int type);
 extern void plugin_sessionvar_init(Session *session);
 extern void plugin_sessionvar_cleanup(Session *session);
@@ -390,8 +383,11 @@ void set_session_proc_info(Session *session, const char *info);
 const char *get_session_proc_info(Session *session);
 int64_t session_test_options(const Session *session, int64_t test_options);
 int session_sql_command(const Session *session);
-int session_tx_isolation(const Session *session);
+enum_tx_isolation session_tx_isolation(const Session *session);
 
+void compose_plugin_add(std::vector<std::string> options);
+void compose_plugin_remove(std::vector<std::string> options);
+void notify_plugin_load(std::string in_plugin_load);
 
 
 /**
@@ -425,56 +421,7 @@ int mysql_tmpfile(const char *prefix);
 int session_killed(const Session *session);
 
 
-/**
-  Return the thread id of a user thread
-
-  @param session  user thread connection handle
-  @return  thread id
-*/
-unsigned long session_get_thread_id(const Session *session);
-
 const charset_info_st *session_charset(Session *session);
-int session_non_transactional_update(const Session *session);
-void session_mark_transaction_to_rollback(Session *session, bool all);
-
-
-/**
-  Allocate memory in the connection's local memory pool
-
-  @details
-  When properly used in place of @c malloc(), this can significantly
-  improve concurrency. Don't use this or related functions to allocate
-  large chunks of memory. Use for temporary storage only. The memory
-  will be freed automatically at the end of the statement; no explicit
-  code is required to prevent memory leaks.
-
-  @see alloc_root()
-*/
-void *session_alloc(Session *session, unsigned int size);
-/**
-  @see session_alloc()
-*/
-void *session_calloc(Session *session, unsigned int size);
-/**
-  @see session_alloc()
-*/
-char *session_strdup(Session *session, const char *str);
-/**
-  @see session_alloc()
-*/
-char *session_strmake(Session *session, const char *str, unsigned int size);
-/**
-  @see session_alloc()
-*/
-void *session_memdup(Session *session, const void* str, unsigned int size);
-
-/**
-  Get the XID for this connection's transaction
-
-  @param session  user thread connection handle
-  @param xid  location where identifier is stored
-*/
-void session_get_xid(const Session *session, DRIZZLE_XID *xid);
 
 /**
   Invalidate the query cache for a given table.
@@ -488,10 +435,6 @@ void mysql_query_cache_invalidate4(Session *session,
                                    const char *key, unsigned int key_length,
                                    int using_trx);
 
-#ifdef __cplusplus
-}
-#endif
-
 } /* namespace drizzled */
 
 #endif /* DRIZZLED_PLUGIN_H */
