@@ -19,22 +19,30 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_PLUGIN_DAEMON_H
+#define DRIZZLED_PLUGIN_DAEMON_H
 
-#include <drizzled/plugin/plugin.h>
+#include "drizzled/plugin/plugin.h"
 
-#include <drizzled/visibility.h>
-
-namespace drizzled {
-namespace plugin {
-
-class DRIZZLED_API Daemon : public Plugin
+namespace drizzled
 {
+class Session;
+
+namespace plugin
+{
+
+class Daemon : public Plugin
+{
+  Daemon();
+  Daemon(const Daemon &);
+  Daemon& operator=(const Daemon &);
 public:
   explicit Daemon(std::string name_arg)
     : Plugin(name_arg, "Daemon")
   {}
 
+  virtual ~Daemon() {}
+
   /**
    * Standard plugin system registration hooks
    */
@@ -50,3 +58,4 @@ public:
 } /* namespace plugin */
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_PLUGIN_DAEMON_H */
