@@ -19,10 +19,11 @@
 
 #include "config.h"
 #include "drizzled/plugin/error_message.h"
-#include "drizzled/plugin/registry.h"
 
 #include "drizzled/gettext.h"
 
+#include <cstdio>
+#include <algorithm>
 #include <vector>
 
 using namespace std;
@@ -89,7 +90,7 @@ public:
 
 
 bool plugin::ErrorMessage::vprintf(Session *session, int priority,
-                                 char const *format, va_list ap)
+                                   char const *format, va_list ap)
 {
 
   /* check to see if any errmsg plugin has been loaded
