@@ -21,29 +21,34 @@
  * 2010-05-12
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string>
 #include <vector>
 
-#include <drizzled/session.h>
-#include <drizzled/table_list.h>
-#include <drizzled/table/instance.h>
-#include <drizzled/module/registry.h>
-#include <drizzled/plugin/event_observer.h>
+#include "drizzled/session.h"
+#include "drizzled/table_list.h"
+#include "drizzled/table_share.h"
+#include "drizzled/module/registry.h"
+#include "drizzled/plugin/event_observer.h"
 #include <drizzled/util/functors.h>
 #include <algorithm>
 
-namespace drizzled {
-namespace plugin {
 
-  EventObserverVector all_event_plugins;
 
-  const EventObserverVector &EventObserver::getEventObservers()
-  {
-    return all_event_plugins;
-  }
+using namespace std;
 
+namespace drizzled
+{
+
+namespace plugin
+{
+
+/*============================*/
+  // Basic plugin registration stuff.
+  static vector<EventObserver *> all_event_plugins;
+
+  //---------
   bool EventObserver::addPlugin(EventObserver *handler)
   {
     if (handler != NULL)
@@ -51,10 +56,11 @@ namespace plugin {
     return false;
   }
 
+  //---------
   void EventObserver::removePlugin(EventObserver *handler)
   {
     if (handler != NULL)
-      all_event_plugins.erase(std::find(all_event_plugins.begin(), all_event_plugins.end(), handler));
+      all_event_plugins.erase(find(all_event_plugins.begin(), all_event_plugins.end(), handler));
   }
 
 
@@ -70,26 +76,28 @@ namespace plugin {
   {
 
   public:
-    typedef std::multimap<uint32_t, EventObserver *> ObserverMap;
+    typedef multimap<uint32_t, EventObserver *> ObserverMap;
 
   private:
     /* A list of lists indexed by event type. */
-    std::vector<ObserverMap *> event_observer_lists;
+    vector<ObserverMap *> event_observer_lists;
 
   public:
 
     EventObserverList()
     {
-			// Initialize the list with NULL pointers.
-			event_observer_lists.assign(EventObserver::MAX_EVENT_COUNT, NULL);
+      uint32_t i;
+
+      event_observer_lists.reserve(EventObserver::MAX_EVENT_COUNT);
+      for (i=0; i < EventObserver::MAX_EVENT_COUNT; i++)
+      {
+        event_observer_lists[i]= NULL;
+      }
     }
 
     ~EventObserverList()
     {
-      std::for_each(event_observer_lists.begin(),
-                    event_observer_lists.end(),
-                    DeletePtr());
-      event_observer_lists.clear();
+      clearAllObservers();
     }
 
     /* Add the observer to the observer list for the even, positioning it if required.
@@ -118,9 +126,9 @@ namespace plugin {
       /* If positioned then check if the position is already taken. */
       if (position) 
       {
-        if (observers->count(event_pos))
+        if (observers->find(event_pos) != observers->end())
         {
-          errmsg_printf(error::WARN,
+          errmsg_printf(ERRMSG_LVL_WARN,
                         _("EventObserverList::addEventObserver() Duplicate event position %d for event '%s' from EventObserver plugin '%s'"),
                         position,
                         EventObserver::eventName(event), 
@@ -128,7 +136,16 @@ namespace plugin {
         }
       }
 
-      observers->insert(std::pair<uint32_t, EventObserver *>(event_pos, eventObserver) );
+      observers->insert(pair<uint32_t, EventObserver *>(event_pos, eventObserver) );
+    }
+
+    /* Remove all observer from all lists. */
+    void clearAllObservers()
+    {
+      for_each(event_observer_lists.begin(),
+               event_observer_lists.end(),
+               DeletePtr());
+      event_observer_lists.clear();
     }
 
 
@@ -157,7 +174,7 @@ namespace plugin {
   /* For each EventObserver plugin call its registerTableEventsDo() meathod so that it can
    * register what events, if any, it is interested in on this table.
    */ 
-  class RegisterTableEventsIterate : public std::unary_function<EventObserver *, void>
+  class RegisterTableEventsIterate : public unary_function<EventObserver *, void>
   {
     TableShare &table_share;
     EventObserverList &observers;
@@ -185,20 +202,22 @@ namespace plugin {
 
     observers= table_share.getTableObservers();
 
-    if (observers != NULL) 
-		{
-			errmsg_printf(error::WARN,
-									_("EventObserver::registerTableEvents(): Table already has events registered on it: probable programming error."));
-			table_share.setTableObservers(NULL);
-      delete observers;
+    if (observers == NULL) 
+    {
+      observers= new EventObserverList();
+      table_share.setTableObservers(observers);
+    } 
+    else 
+    {
+      /* Calling registerTableEvents() for a table that already has
+       * events registered on it is probably a programming error.
+     */
+      observers->clearAllObservers();
     }
 
-		observers= new EventObserverList();
-		table_share.setTableObservers(observers);
- 
 
-    std::for_each(all_event_plugins.begin(), all_event_plugins.end(),
-                  RegisterTableEventsIterate(table_share, *observers));
+    for_each(all_event_plugins.begin(), all_event_plugins.end(),
+             RegisterTableEventsIterate(table_share, *observers));
 
   }
 
@@ -206,14 +225,17 @@ namespace plugin {
   /* Cleanup before freeing the TableShare object. */
   void EventObserver::deregisterTableEvents(TableShare &table_share)
   {
-   if (all_event_plugins.empty())
+    if (all_event_plugins.empty())
       return;
 
-    EventObserverList *observers= table_share.getTableObservers();
+    EventObserverList *observers;
+
+    observers= table_share.getTableObservers();
 
     if (observers) 
     {
       table_share.setTableObservers(NULL);
+      observers->clearAllObservers();
       delete observers;
     }
   }
@@ -227,7 +249,7 @@ namespace plugin {
   /* For each EventObserver plugin call its registerSchemaEventsDo() meathod so that it can
    * register what events, if any, it is interested in on the schema.
    */ 
-  class RegisterSchemaEventsIterate : public std::unary_function<EventObserver *, void>
+  class RegisterSchemaEventsIterate : public unary_function<EventObserver *, void>
   {
     const std::string &db;
     EventObserverList &observers;
@@ -251,21 +273,40 @@ namespace plugin {
   {
     if (all_event_plugins.empty())
       return;
-    EventObserverList *observers= session.getSchemaObservers(db);
-    if (observers == NULL) 
-      observers= session.setSchemaObservers(db, new EventObserverList());
 
-    std::for_each(all_event_plugins.begin(), all_event_plugins.end(),
-                  RegisterSchemaEventsIterate(db, *observers));
+    EventObserverList *observers;
+
+    observers= session.getSchemaObservers(db);
+
+    if (observers == NULL) 
+    {
+      observers= new EventObserverList();
+      session.setSchemaObservers(db, observers);
+    }
+
+    for_each(all_event_plugins.begin(), all_event_plugins.end(),
+             RegisterSchemaEventsIterate(db, *observers));
 
   }
 
   //----------
   /* Cleanup before freeing the Session object. */
-  void EventObserver::deregisterSchemaEvents(EventObserverList *observers)
+  void EventObserver::deregisterSchemaEvents(Session &session, const std::string &db)
   {
+    if (all_event_plugins.empty())
+      return;
+
+    EventObserverList *observers;
+
+    observers= session.getSchemaObservers(db);
+
+    if (observers) 
+    {
+      session.setSchemaObservers(db, NULL);
+      observers->clearAllObservers();
       delete observers;
-   }
+    }
+  }
 
   /*========================================================*/
   /*             Session Event Observer handling:           */
@@ -275,7 +316,7 @@ namespace plugin {
   /* For each EventObserver plugin call its registerSessionEventsDo() meathod so that it can
    * register what events, if any, it is interested in on this session.
    */ 
-  class RegisterSessionEventsIterate : public std::unary_function<EventObserver *, void>
+  class RegisterSessionEventsIterate : public unary_function<EventObserver *, void>
   {
     Session &session;
     EventObserverList &observers;
@@ -301,38 +342,49 @@ namespace plugin {
     EventObserverList *observers;
 
     observers= session.getSessionObservers();
-		if (observers) { // This should not happed
-			errmsg_printf(error::WARN,
-									_("EventObserver::registerSessionEvents(): Session already has events registered on it: probable programming error."));
-			session.setSessionObservers(NULL);
-			delete observers;
-		}
 
-	observers= new EventObserverList();
-	session.setSessionObservers(observers);
+    if (observers == NULL) 
+    {
+      observers= new EventObserverList();
+      session.setSessionObservers(observers);
+    }
 
-  std::for_each(all_event_plugins.begin(), all_event_plugins.end(),
-                RegisterSessionEventsIterate(session, *observers));
+    observers->clearAllObservers();
+
+    for_each(all_event_plugins.begin(), all_event_plugins.end(),
+             RegisterSessionEventsIterate(session, *observers));
 
   }
 
   //----------
   /* Cleanup before freeing the session object. */
-  void EventObserver::deregisterSessionEvents(EventObserverList *observers)
+  void EventObserver::deregisterSessionEvents(Session &session)
   {
+    if (all_event_plugins.empty())
+      return;
+
+    EventObserverList *observers;
+
+    observers= session.getSessionObservers();
+
+    if (observers) 
+    {
+      session.setSessionObservers(NULL);
+      observers->clearAllObservers();
       delete observers;
+    }
   }
 
 
   /* Event observer list iterator: */
   //----------
-  class EventIterate : public std::unary_function<std::pair<uint32_t, EventObserver *>, bool>
+  class EventIterate : public unary_function<pair<uint32_t, EventObserver *>, bool>
   {
     EventData &data;
 
   public:
     EventIterate(EventData &data_arg) :
-      std::unary_function<std::pair<uint32_t, EventObserver *>, bool>(),
+      unary_function<pair<uint32_t, EventObserver *>, bool>(),
       data(data_arg)
     {}
 
@@ -343,7 +395,7 @@ namespace plugin {
       {
         /* TRANSLATORS: The leading word "EventObserver" is the name
           of the plugin api, and so should not be translated. */
-        errmsg_printf(error::ERROR,
+        errmsg_printf(ERRMSG_LVL_ERROR,
                       _("EventIterate event handler '%s' failed for event '%s'"),
                       handler.second->getName().c_str(), handler.second->eventName(data.event));
 
@@ -363,9 +415,6 @@ namespace plugin {
   {
     EventObserverList::ObserverMap *eventObservers;
 
-    if (observerList == NULL)
-      return false; // Nobody was interested in the event. :(
-
     eventObservers = observerList->getObservers(event);
 
     if (eventObservers == NULL)
@@ -373,8 +422,8 @@ namespace plugin {
 
     /* Use find_if instead of foreach so that we can collect return codes */
     EventObserverList::ObserverMap::iterator iter=
-      std::find_if(eventObservers->begin(), eventObservers->end(),
-                   EventIterate(*this)); 
+      find_if(eventObservers->begin(), eventObservers->end(),
+              EventIterate(*this)); 
     /* If iter is == end() here, that means that all of the plugins returned
      * false, which in this case means they all succeeded. Since we want to 
      * return false on success, we return the value of the two being !=.
@@ -390,11 +439,6 @@ namespace plugin {
     return EventData::callEventObservers();
   }
 
-  bool SessionEventData::hasEvents(Session &in_session)
-  {
-    return (in_session.getSessionObservers() != NULL);
-  }
-
   //--------
   bool SchemaEventData::callEventObservers()
   {
@@ -411,21 +455,16 @@ namespace plugin {
   //--------
   bool TableEventData::callEventObservers()
   {
-    observerList= table.getMutableShare()->getTableObservers();
+    observerList= table.s->getTableObservers();
 
     return EventData::callEventObservers();
   }
 
-  bool TableEventData::hasEvents(Table &in_table)
-  {
-    return (in_table.getMutableShare()->getTableObservers() != NULL);
-  }
-
   /*==========================================================*/
   /* Static meathods called by drizzle to notify interested plugins 
-   * of a schema event.
+   * of a schema event,
  */
-  bool EventObserver::beforeDropTable(Session &session, const drizzled::identifier::Table &table)
+  bool EventObserver::beforeDropTable(Session &session, const drizzled::TableIdentifier &table)
   {
     if (all_event_plugins.empty())
       return false;
@@ -434,7 +473,7 @@ namespace plugin {
     return eventData.callEventObservers();
   }
 
-  bool EventObserver::afterDropTable(Session &session, const drizzled::identifier::Table &table, int err)
+  bool EventObserver::afterDropTable(Session &session, const drizzled::TableIdentifier &table, int err)
   {
     if (all_event_plugins.empty())
       return false;
@@ -443,7 +482,7 @@ namespace plugin {
     return eventData.callEventObservers();
   }
 
-  bool EventObserver::beforeRenameTable(Session &session, const drizzled::identifier::Table &from, const drizzled::identifier::Table &to)
+  bool EventObserver::beforeRenameTable(Session &session, const drizzled::TableIdentifier &from, const drizzled::TableIdentifier &to)
   {
     if (all_event_plugins.empty())
       return false;
@@ -452,7 +491,7 @@ namespace plugin {
     return eventData.callEventObservers();
   }
 
-  bool EventObserver::afterRenameTable(Session &session, const drizzled::identifier::Table &from, const drizzled::identifier::Table &to, int err)
+  bool EventObserver::afterRenameTable(Session &session, const drizzled::TableIdentifier &from, const drizzled::TableIdentifier &to, int err)
   {
     if (all_event_plugins.empty())
       return false;
@@ -463,13 +502,11 @@ namespace plugin {
 
   /*==========================================================*/
   /* Static meathods called by drizzle to notify interested plugins 
-   * of a table event.
-   *
-   * A quick test is done first to see if there are any interested observers.
+   * of a table event,
  */
   bool EventObserver::beforeInsertRecord(Table &table, unsigned char *buf)
   {
-    if (all_event_plugins.empty() || !TableEventData::hasEvents(table))
+    if (all_event_plugins.empty())
       return false;
 
     BeforeInsertRecordEventData eventData(*(table.in_use), table, buf);
@@ -478,7 +515,7 @@ namespace plugin {
 
   bool EventObserver::afterInsertRecord(Table &table, const unsigned char *buf, int err)
   {
-    if (all_event_plugins.empty() || !TableEventData::hasEvents(table))
+    if (all_event_plugins.empty())
       return false;
 
     AfterInsertRecordEventData eventData(*(table.in_use), table, buf, err);
@@ -487,7 +524,7 @@ namespace plugin {
 
   bool EventObserver::beforeDeleteRecord(Table &table, const unsigned char *buf)
   {
-    if (all_event_plugins.empty() || !TableEventData::hasEvents(table))
+    if (all_event_plugins.empty())
       return false;
 
     BeforeDeleteRecordEventData eventData(*(table.in_use), table, buf);
@@ -496,7 +533,7 @@ namespace plugin {
 
   bool EventObserver::afterDeleteRecord(Table &table, const unsigned char *buf, int err)
   {
-    if (all_event_plugins.empty() || !TableEventData::hasEvents(table))
+    if (all_event_plugins.empty())
       return false;
 
     AfterDeleteRecordEventData eventData(*(table.in_use), table, buf, err);
@@ -505,7 +542,7 @@ namespace plugin {
 
   bool EventObserver::beforeUpdateRecord(Table &table, const unsigned char *old_data, unsigned char *new_data)
   {
-    if (all_event_plugins.empty() || !TableEventData::hasEvents(table))
+    if (all_event_plugins.empty())
       return false;
 
     BeforeUpdateRecordEventData eventData(*(table.in_use), table, old_data, new_data);
@@ -514,7 +551,7 @@ namespace plugin {
 
   bool EventObserver::afterUpdateRecord(Table &table, const unsigned char *old_data, unsigned char *new_data, int err)
   {
-    if (all_event_plugins.empty() || !TableEventData::hasEvents(table))
+    if (all_event_plugins.empty())
       return false;
 
     AfterUpdateRecordEventData eventData(*(table.in_use), table, old_data, new_data, err);
@@ -523,13 +560,11 @@ namespace plugin {
 
   /*==========================================================*/
   /* Static meathods called by drizzle to notify interested plugins 
-   * of a session event.
-   *
-   * A quick test is done first to see if there are any interested observers.
-*/
+   * of a session event,
+ */
   bool EventObserver::beforeCreateDatabase(Session &session, const std::string &db)
   {
-    if (all_event_plugins.empty() || !SessionEventData::hasEvents(session))
+    if (all_event_plugins.empty())
       return false;
 
     BeforeCreateDatabaseEventData eventData(session, db);
@@ -538,7 +573,7 @@ namespace plugin {
 
   bool EventObserver::afterCreateDatabase(Session &session, const std::string &db, int err)
   {
-    if (all_event_plugins.empty() || !SessionEventData::hasEvents(session))
+    if (all_event_plugins.empty())
       return false;
 
     AfterCreateDatabaseEventData eventData(session, db, err);
@@ -547,7 +582,7 @@ namespace plugin {
 
   bool EventObserver::beforeDropDatabase(Session &session, const std::string &db)
   {
-    if (all_event_plugins.empty() || !SessionEventData::hasEvents(session))
+    if (all_event_plugins.empty())
       return false;
 
     BeforeDropDatabaseEventData eventData(session, db);
@@ -556,49 +591,13 @@ namespace plugin {
 
   bool EventObserver::afterDropDatabase(Session &session, const std::string &db, int err)
   {
-    if (all_event_plugins.empty() || !SessionEventData::hasEvents(session))
+    if (all_event_plugins.empty())
       return false;
 
     AfterDropDatabaseEventData eventData(session, db, err);
     return eventData.callEventObservers();
   }
 
-  bool EventObserver::connectSession(Session &session)
-  {
-    if (all_event_plugins.empty() || !SessionEventData::hasEvents(session))
-      return false;
-
-    ConnectSessionEventData eventData(session);
-    return eventData.callEventObservers();
-  }
-
-  bool EventObserver::disconnectSession(Session &session)
-  {
-    if (all_event_plugins.empty() || !SessionEventData::hasEvents(session))
-      return false;
-
-    DisconnectSessionEventData eventData(session);
-    return eventData.callEventObservers();
-  }
-
-  bool EventObserver::beforeStatement(Session &session)
-  {
-    if (all_event_plugins.empty() || !SessionEventData::hasEvents(session))
-      return false;
-
-    BeforeStatementEventData eventData(session);
-    return eventData.callEventObservers();
-  }
-
-  bool EventObserver::afterStatement(Session &session)
-  {
-    if (all_event_plugins.empty() || !SessionEventData::hasEvents(session))
-      return false;
-
-    AfterStatementEventData eventData(session);
-    return eventData.callEventObservers();
-  }
-
 
 } /* namespace plugin */
 } /* namespace drizzled */
