@@ -23,6 +23,7 @@
 #include <drizzled/plugin/listen_tcp.h>
 #include <drizzled/errmsg_print.h>
 
+#include <cstdio>
 #include <unistd.h>
 #include <sys/socket.h>
 #include <fcntl.h>
@@ -192,24 +193,24 @@ bool plugin::ListenTcp::getFileDescriptors(std::vector<int> &fds)
         break;
       }
 
-      errmsg_printf(ERRMSG_LVL_INFO, _("Retrying bind() on %u"), getPort());
+      errmsg_printf(ERRMSG_LVL_INFO, _("Retrying bind() on %u\n"), getPort());
       this_wait= retry * retry / 3 + 1;
       sleep(this_wait);
     }
 
     if (ret < 0)
     {
-      errmsg_printf(ERRMSG_LVL_ERROR, _("bind() failed with errno: %d"),
+      errmsg_printf(ERRMSG_LVL_ERROR, _("bind() failed with errno: %d\n"),
                     errno);
       errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("Do you already have another drizzled running?"));
+                    _("Do you already have another drizzled running?\n"));
       return true;
     }
 
     if (listen(fd, (int) back_log) < 0)
     {
       errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("listen() failed with errno %d"), errno);
+                    _("listen() failed with errno %d\n"), errno);
       return true;
     }
 
