@@ -2,6 +2,7 @@
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
  *  Copyright (C) 2008 Sun Microsystems
+ *  Copyright (C) 2010 Djellel Eddine Difallah
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,10 +20,11 @@
 
 #include "config.h"
 #include "drizzled/plugin/query_cache.h"
-#include "drizzled/plugin/registry.h"
+#include "drizzled/errmsg_print.h"
 
 #include "drizzled/gettext.h"
 
+#include <algorithm>
 #include <vector>
 
 class Session;
@@ -31,127 +33,150 @@ using namespace std;
 
 namespace drizzled
 {
-
-vector<plugin::QueryCache *> all_query_cache;
+typedef vector<plugin::QueryCache *> QueryCaches;
+QueryCaches all_query_cache;
 
 /* Namespaces are here to prevent global symbol clashes with these classes */
 
-class TryFetchAndSendIterate
+class IsCachedIterate
  : public unary_function<plugin::QueryCache *, bool>
 {
   Session *session;
-  bool is_transactional;
 public:
-  TryFetchAndSendIterate(Session *session_arg, bool is_transactional_arg) :
+  IsCachedIterate(Session* session_arg) :
     unary_function<plugin::QueryCache *, bool>(),
-    session(session_arg), is_transactional(is_transactional_arg) { }
+    session(session_arg) { }
 
   inline result_type operator()(argument_type handler)
   {
-    if (handler->tryFetchAndSend(session, is_transactional))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("qcache plugin '%s' try_fetch_and_send() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
+    return handler->doIsCached(session);
   }
 };
 
-class SetIterate
+bool plugin::QueryCache::isCached(Session *session)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            IsCachedIterate(session));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+
+class SendCachedResultsetIterate
  : public unary_function<plugin::QueryCache *, bool>
 {
   Session *session;
-  bool is_transactional;
 public:
-  SetIterate(Session *session_arg, bool is_transactional_arg) :
+  SendCachedResultsetIterate(Session *session_arg) :
     unary_function<plugin::QueryCache *, bool>(),
-    session(session_arg), is_transactional(is_transactional_arg) { }
+    session(session_arg) { }
 
   inline result_type operator()(argument_type handler)
   {
-
-    if (handler->set(session, is_transactional))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR, _("qcache plugin '%s' set() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
+    return handler->doSendCachedResultset(session);
   }
 };
-
-class InvalidateTableIterate
- : public unary_function<plugin::QueryCache *, bool>
+bool plugin::QueryCache::sendCachedResultset(Session *session)
 {
-  Session *session;
-  bool is_transactional;
-public:
-  InvalidateTableIterate(Session *session_arg, bool is_transactional_arg) :
-    unary_function<plugin::QueryCache *, bool>(),
-    session(session_arg), is_transactional(is_transactional_arg) { }
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            SendCachedResultsetIterate(session));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
 
-  inline result_type operator()(argument_type handler)
-  {
-
-    if (handler->invalidateTable(session, is_transactional))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("qcache plugin '%s' invalidateTable() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
-  }
-};
-
-
-class InvalidateDbIterate
- : public unary_function<plugin::QueryCache *, bool>
-{
-  Session *session;
-  const char *dbname;
-  bool is_transactional;
-public:
-  InvalidateDbIterate(Session *session_arg, const char *dbname_arg,
-                      bool is_transactional_arg) :
-    unary_function<plugin::QueryCache *, bool>(),
-    session(session_arg), dbname(dbname_arg),
-    is_transactional(is_transactional_arg) { }
-
-  inline result_type operator()(argument_type handler)
-  {
-    if (handler->invalidateDb(session, dbname, is_transactional))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("qcache plugin '%s' invalidateDb() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
-  }
-};
-
-class FlushIterate
+class PrepareResultsetIterate
  : public unary_function<plugin::QueryCache *, bool>
 {
   Session *session;
 public:
-  FlushIterate(Session *session_arg) :
+  PrepareResultsetIterate(Session *session_arg) :
     unary_function<plugin::QueryCache *, bool>(), session(session_arg) { }
 
   inline result_type operator()(argument_type handler)
   {
-    if (handler->flush(session))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR, _("qcache plugin '%s' flush() failed"),
-                    handler->getName().c_str());
-      return true;
-    }
-    return false;
+    return handler->doPrepareResultset(session);
   }
 };
+bool plugin::QueryCache::prepareResultset(Session *session)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            PrepareResultsetIterate(session));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+class SetResultsetIterate
+ : public unary_function<plugin::QueryCache *, bool>
+{
+  Session *session;
+public:
+  SetResultsetIterate(Session *session_arg) :
+    unary_function<plugin::QueryCache *, bool>(),
+    session(session_arg) { }
+
+  inline result_type operator()(argument_type handler)
+  {
+    return handler->doSetResultset(session);
+  }
+};
+
+bool plugin::QueryCache::setResultset(Session *session)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            SetResultsetIterate(session));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+class InsertRecordIterate
+ : public unary_function<plugin::QueryCache *, bool>
+{
+  Session *session;
+  List<Item> &item;
+public:
+  InsertRecordIterate(Session *session_arg, List<Item> &item_arg) :
+    unary_function<plugin::QueryCache *, bool>(),
+    session(session_arg), item(item_arg) { }
+
+  inline result_type operator()(argument_type handler)
+  {
+    return handler->doInsertRecord(session, item);
+  }
+};
+bool plugin::QueryCache::insertRecord(Session *session, List<Item> &items)
+{
+  /* Use find_if instead of foreach so that we can collect return codes */
+  QueryCaches::iterator iter=
+    find_if(all_query_cache.begin(), all_query_cache.end(),
+            InsertRecordIterate(session, items));
+  /* If iter is == end() here, that means that all of the plugins returned
+   * false, which in this case means they all succeeded. Since we want to 
+   * return false on success, we return the value of the two being != 
+   */
+  return iter != all_query_cache.end();
+}
+
+
 
 bool plugin::QueryCache::addPlugin(plugin::QueryCache *handler)
 {
@@ -165,73 +190,4 @@ void plugin::QueryCache::removePlugin(plugin::QueryCache *handler)
                         handler));
 }
 
-
-bool plugin::QueryCache::tryFetchAndSendDo(Session *session,
-                                           bool transactional)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            TryFetchAndSendIterate(session, transactional));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
-bool plugin::QueryCache::setDo(Session *session, bool transactional)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            SetIterate(session, transactional));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
-bool plugin::QueryCache::invalidateTableDo(Session *session,
-                                         bool transactional)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            InvalidateTableIterate(session, transactional));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
-bool plugin::QueryCache::invalidateDbDo(Session *session, const char *dbname,
-                                        bool transactional)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            InvalidateDbIterate(session, dbname, transactional));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
-bool plugin::QueryCache::flushDo(Session *session)
-{
-  /* Use find_if instead of foreach so that we can collect return codes */
-  vector<plugin::QueryCache *>::iterator iter=
-    find_if(all_query_cache.begin(), all_query_cache.end(),
-            FlushIterate(session));
-  /* If iter is == end() here, that means that all of the plugins returned
-   * false, which in this case means they all succeeded. Since we want to 
-   * return false on success, we return the value of the two being != 
-   */
-  return iter != all_query_cache.end();
-}
-
 } /* namespace drizzled */
