@@ -1,9 +1,8 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Definitions required for Query Cache plugin
- *
  *  Copyright (C) 2008 Sun Microsystems, Toru Maesaka
+ *  Copyright (C) 2010 Djellel Eddine Difallah
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,55 +21,60 @@
 #ifndef DRIZZLED_PLUGIN_QUERY_CACHE_H
 #define DRIZZLED_PLUGIN_QUERY_CACHE_H
 
+#include "drizzled/plugin.h"
 #include "drizzled/plugin/plugin.h"
-
-class Session;
+#include <drizzled/sql_list.h>
 
 namespace drizzled
 {
+class Session;
+class select_result;
+
 namespace plugin
 {
 
 /* 
   This is the API that a qcache plugin must implement.
-  it should implement each of these function pointers.
-  if a function pointer is NULL (not loaded), that's ok.
-
-  Return:
-    false = success
-    true  = failure
 */
+
 class QueryCache : public Plugin
 {
+private:  
+  
   QueryCache();
   QueryCache(const QueryCache &);
   QueryCache& operator=(const QueryCache &);
-public:
+
+public:  
+
   explicit QueryCache(std::string name_arg)
     : Plugin(name_arg, "QueryCache")
   {}
 
   virtual ~QueryCache() {}
-  /* Lookup the cache and transmit the data back to the client */
-  virtual bool tryFetchAndSend(Session *session,
-                               bool is_transactional)= 0;
 
-  virtual bool set(Session *session, bool is_transactional)= 0;
-  virtual bool invalidateTable(Session *session, bool is_transactional)= 0;
-  virtual bool invalidateDb(Session *session, const char *db_name,
-                            bool transactional)= 0;
-  virtual bool flush(Session *session)= 0;
+  /* these are the Query Cache interface functions */
+
+  /* Lookup the cache and transmit the data back to the client */
+  virtual bool doIsCached(Session* session)= 0;  
+  /* Lookup the cache and transmit the data back to the client */
+  virtual bool doSendCachedResultset(Session *session)= 0;
+  /* Send the current Resultset to the cache */
+  virtual bool doSetResultset(Session *session)= 0;
+  /* initiate a new Resultset (header) */
+  virtual bool doPrepareResultset(Session *session)= 0;
+  /* push a record to the current Resultset */
+  virtual bool doInsertRecord(Session *session, List<Item> &item)= 0;
 
   static bool addPlugin(QueryCache *handler);
   static void removePlugin(QueryCache *handler);
 
   /* These are the functions called by the rest of the Drizzle server */
-  static bool tryFetchAndSendDo(Session *session, bool transactional);
-  static bool setDo(Session *session, bool transactional);
-  static bool invalidateTableDo(Session *session, bool transactional);
-  static bool invalidateDbDo(Session *session, const char *db_name,
-                            bool transactional);
-  static bool flushDo(Session *session);
+  static bool isCached(Session *session);
+  static bool sendCachedResultset(Session *session);
+  static bool prepareResultset(Session *session);
+  static bool setResultset(Session *session);
+  static bool insertRecord(Session *session, List<Item> &item);
 };
 
 } /* namespace plugin */
