@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2010 Jay Pipes
+ *  Copyright (c) 2010 Jay Pipes
  *
  *  Authors:
  *
@@ -21,7 +21,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_PLUGIN_REPLICATION_H
+#define DRIZZLED_PLUGIN_REPLICATION_H
 
 /**
  * @file Common structs and enums for the replication API
@@ -42,3 +43,4 @@ enum ReplicationReturnCode
 } /* namespace plugin */
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_PLUGIN_REPLICATION_H */
