@@ -25,13 +25,13 @@
 #include <string>
 #include <vector>
 #include <set>
+#include <fstream>
 #include <algorithm>
 #include <functional>
 
 #include <google/protobuf/io/zero_copy_stream.h>
 #include <google/protobuf/io/zero_copy_stream_impl.h>
 
-#include "drizzled/my_hash.h"
 #include "drizzled/cached_directory.h"
 
 #include <drizzled/definitions.h>
@@ -47,12 +47,14 @@
 #include "drizzled/xid.h"
 #include "drizzled/sql_table.h"
 #include "drizzled/global_charset_info.h"
-#include "drizzled/plugin/authorization.h"
 #include "drizzled/charset.h"
 #include "drizzled/internal/my_sys.h"
 #include "drizzled/db.h"
 
 #include <drizzled/table_proto.h>
+#include <drizzled/plugin/event_observer.h>
+
+#include <boost/algorithm/string/compare.hpp>
 
 static bool shutdown_has_begun= false; // Once we put in the container for the vector/etc for engines this will go away.
 
@@ -67,23 +69,27 @@ namespace plugin
 static EngineVector vector_of_engines;
 static EngineVector vector_of_schema_engines;
 
+const std::string DEFAULT_STRING("default");
 const std::string UNKNOWN_STRING("UNKNOWN");
 const std::string DEFAULT_DEFINITION_FILE_EXT(".dfe");
 
 static std::set<std::string> set_of_table_definition_ext;
 
+EngineVector &StorageEngine::getSchemaEngines()
+{
+  return vector_of_schema_engines;
+}
+
 StorageEngine::StorageEngine(const string name_arg,
-                             const bitset<HTON_BIT_SIZE> &flags_arg)
-    : Plugin(name_arg, "StorageEngine"),
-      MonitoredInTransaction(), /* This gives the storage engine a "slot" or ID */
-      flags(flags_arg)
+                             const bitset<HTON_BIT_SIZE> &flags_arg) :
+  Plugin(name_arg, "StorageEngine"),
+  MonitoredInTransaction(), /* This gives the storage engine a "slot" or ID */
+  flags(flags_arg)
 {
-  pthread_mutex_init(&proto_cache_mutex, NULL);
 }
 
 StorageEngine::~StorageEngine()
 {
-  pthread_mutex_destroy(&proto_cache_mutex);
 }
 
 void StorageEngine::setTransactionReadWrite(Session& session)
@@ -92,24 +98,28 @@ void StorageEngine::setTransactionReadWrite(Session& session)
   statement_ctx.markModifiedNonTransData();
 }
 
-int StorageEngine::doRenameTable(Session *,
-                                 const char *from,
-                                 const char *to)
+
+int StorageEngine::renameTable(Session &session, const TableIdentifier &from, const TableIdentifier &to)
 {
-  int error= 0;
-  for (const char **ext= bas_ext(); *ext ; ext++)
+  int error;
+  setTransactionReadWrite(session);
+
+  if (unlikely(plugin::EventObserver::beforeRenameTable(session, from, to)))
   {
-    if (rename_file_ext(from, to, *ext))
+    error= ER_EVENT_OBSERVER_PLUGIN;
+  }
+  else
+  {
+    error =  doRenameTable(session, from, to);
+    if (unlikely(plugin::EventObserver::afterRenameTable(session, from, to, error)))
     {
-      if ((error=errno) != ENOENT)
-	break;
-      error= 0;
+      error= ER_EVENT_OBSERVER_PLUGIN;
     }
   }
+  
   return error;
 }
 
-
 /**
   Delete all files with extension from bas_ext().
 
@@ -125,8 +135,8 @@ int StorageEngine::doRenameTable(Session *,
   @retval
     !0  Error
 */
-int StorageEngine::doDropTable(Session&,
-                               const string &table_path)
+int StorageEngine::doDropTable(Session&, const TableIdentifier &identifier)
+                               
 {
   int error= 0;
   int enoent_or_zero= ENOENT;                   // Error if no file was deleted
@@ -134,43 +144,23 @@ int StorageEngine::doDropTable(Session&,
 
   for (const char **ext= bas_ext(); *ext ; ext++)
   {
-    internal::fn_format(buff, table_path.c_str(), "", *ext,
-              MY_UNPACK_FILENAME|MY_APPEND_EXT);
+    internal::fn_format(buff, identifier.getPath().c_str(), "", *ext,
+                        MY_UNPACK_FILENAME|MY_APPEND_EXT);
     if (internal::my_delete_with_symlink(buff, MYF(0)))
     {
       if ((error= errno) != ENOENT)
-	break;
+        break;
     }
     else
+    {
       enoent_or_zero= 0;                        // No error for ENOENT
+    }
+
     error= enoent_or_zero;
   }
   return error;
 }
 
-const char *StorageEngine::checkLowercaseNames(const char *path,
-                                                       char *tmp_path)
-{
-  if (flags.test(HTON_BIT_FILE_BASED))
-    return path;
-
-  /* Ensure that table Cursor get path in lower case */
-  if (tmp_path != path)
-    strcpy(tmp_path, path);
-
-  /*
-    we only should turn into lowercase database/table part
-    so start the process after homedirectory
-  */
-  if (strstr(tmp_path, drizzle_tmpdir) == tmp_path)
-    my_casedn_str(files_charset_info, tmp_path + strlen(drizzle_tmpdir));
-  else
-    my_casedn_str(files_charset_info, tmp_path + drizzle_data_home_len);
-
-  return tmp_path;
-}
-
-
 bool StorageEngine::addPlugin(StorageEngine *engine)
 {
 
@@ -202,30 +192,25 @@ void StorageEngine::removePlugin(StorageEngine *)
 class FindEngineByName
   : public unary_function<StorageEngine *, bool>
 {
-  const string target;
+  const string &predicate;
+
 public:
-  explicit FindEngineByName(const string target_arg)
-    : target(target_arg)
-  {}
+  explicit FindEngineByName(const string &target_arg) :
+    predicate(target_arg)
+  {
+  }
+
   result_type operator() (argument_type engine)
   {
-    string engine_name(engine->getName());
-
-    transform(engine_name.begin(), engine_name.end(),
-              engine_name.begin(), ::tolower);
-    return engine_name == target;
+    return boost::iequals(engine->getName(), predicate);
   }
 };
 
-StorageEngine *StorageEngine::findByName(string find_str)
+StorageEngine *StorageEngine::findByName(const string &predicate)
 {
-  transform(find_str.begin(), find_str.end(),
-            find_str.begin(), ::tolower);
-
-  
   EngineVector::iterator iter= find_if(vector_of_engines.begin(),
                                        vector_of_engines.end(),
-                                       FindEngineByName(find_str));
+                                       FindEngineByName(predicate));
   if (iter != vector_of_engines.end())
   {
     StorageEngine *engine= *iter;
@@ -236,19 +221,14 @@ StorageEngine *StorageEngine::findByName(string find_str)
   return NULL;
 }
 
-StorageEngine *StorageEngine::findByName(Session& session,
-                                                         string find_str)
+StorageEngine *StorageEngine::findByName(Session& session, const string &predicate)
 {
-  
-  transform(find_str.begin(), find_str.end(),
-            find_str.begin(), ::tolower);
-
-  if (find_str.compare("default") == 0)
+  if (boost::iequals(predicate, DEFAULT_STRING))
     return session.getDefaultStorageEngine();
 
   EngineVector::iterator iter= find_if(vector_of_engines.begin(),
                                        vector_of_engines.end(),
-                                       FindEngineByName(find_str));
+                                       FindEngineByName(predicate));
   if (iter != vector_of_engines.end())
   {
     StorageEngine *engine= *iter;
@@ -259,8 +239,7 @@ StorageEngine *StorageEngine::findByName(Session& session,
   return NULL;
 }
 
-class StorageEngineCloseConnection
-: public unary_function<StorageEngine *, void>
+class StorageEngineCloseConnection : public unary_function<StorageEngine *, void>
 {
   Session *session;
 public:
@@ -306,53 +285,78 @@ bool StorageEngine::flushLogs(StorageEngine *engine)
 class StorageEngineGetTableDefinition: public unary_function<StorageEngine *,bool>
 {
   Session& session;
-  const char* path;
-  const char *db;
-  const char *table_name;
-  const bool is_tmp;
-  message::Table *table_message;
-  int *err;
+  const TableIdentifier &identifier;
+  message::Table &table_message;
+  int &err;
 
 public:
   StorageEngineGetTableDefinition(Session& session_arg,
-                                  const char* path_arg,
-                                  const char *db_arg,
-                                  const char *table_name_arg,
-                                  const bool is_tmp_arg,
-                                  message::Table *table_message_arg,
-                                  int *err_arg) :
+                                  const TableIdentifier &identifier_arg,
+                                  message::Table &table_message_arg,
+                                  int &err_arg) :
     session(session_arg), 
-    path(path_arg), 
-    db(db_arg),
-    table_name(table_name_arg),
-    is_tmp(is_tmp_arg),
+    identifier(identifier_arg),
     table_message(table_message_arg), 
     err(err_arg) {}
 
   result_type operator() (argument_type engine)
   {
-    int ret= engine->doGetTableDefinition(session,
-                                          path, 
-                                          db,
-                                          table_name,
-                                          is_tmp,
-                                          table_message);
+    int ret= engine->doGetTableDefinition(session, identifier, table_message);
 
     if (ret != ENOENT)
-      *err= ret;
+      err= ret;
 
-    return *err == EEXIST || *err != ENOENT;
+    return err == EEXIST || err != ENOENT;
+  }
+};
+
+class StorageEngineDoesTableExist: public unary_function<StorageEngine *, bool>
+{
+  Session& session;
+  const TableIdentifier &identifier;
+
+public:
+  StorageEngineDoesTableExist(Session& session_arg, const TableIdentifier &identifier_arg) :
+    session(session_arg), 
+    identifier(identifier_arg) 
+  { }
+
+  result_type operator() (argument_type engine)
+  {
+    return engine->doDoesTableExist(session, identifier);
   }
 };
 
 /**
   Utility method which hides some of the details of getTableDefinition()
 */
-bool plugin::StorageEngine::doesTableExist(Session& session,
-                                           TableIdentifier &identifier,
+bool plugin::StorageEngine::doesTableExist(Session &session,
+                                           const TableIdentifier &identifier,
                                            bool include_temporary_tables)
 {
-  return (plugin::StorageEngine::getTableDefinition(session, identifier, NULL, include_temporary_tables) == EEXIST);
+  if (include_temporary_tables)
+  {
+    if (session.doDoesTableExist(identifier))
+      return true;
+  }
+
+  EngineVector::iterator iter=
+    find_if(vector_of_engines.begin(), vector_of_engines.end(),
+            StorageEngineDoesTableExist(session, identifier));
+
+  if (iter == vector_of_engines.end())
+  {
+    return false;
+  }
+
+  return true;
+}
+
+bool plugin::StorageEngine::doDoesTableExist(Session&, const drizzled::TableIdentifier&)
+{
+  cerr << " Engine was called for doDoesTableExist() and does not implement it: " << this->getName() << "\n";
+  assert(0);
+  return false;
 }
 
 /**
@@ -361,34 +365,21 @@ bool plugin::StorageEngine::doesTableExist(Session& session,
   or any dropped tables that need to be removed from disk
 */
 int StorageEngine::getTableDefinition(Session& session,
-                                      TableIdentifier &identifier,
-                                      message::Table *table_message,
+                                      const TableIdentifier &identifier,
+                                      message::Table &table_message,
                                       bool include_temporary_tables)
-{
-  return getTableDefinition(session,
-                            identifier.getPath(), identifier.getDBName(), identifier.getTableName(), identifier.isTmp(),
-                            table_message, include_temporary_tables);
-}
-
-int StorageEngine::getTableDefinition(Session& session,
-                                              const char* path,
-                                              const char *schema_name,
-                                              const char *table_name,
-                                              const bool,
-                                              message::Table *table_message,
-                                              bool include_temporary_tables)
 {
   int err= ENOENT;
 
   if (include_temporary_tables)
   {
-    if (session.doGetTableDefinition(path, schema_name, table_name, false, table_message) == EEXIST)
+    if (session.doGetTableDefinition(identifier, table_message) == EEXIST)
       return EEXIST;
   }
 
   EngineVector::iterator iter=
     find_if(vector_of_engines.begin(), vector_of_engines.end(),
-            StorageEngineGetTableDefinition(session, path, NULL, NULL, true, table_message, &err));
+            StorageEngineGetTableDefinition(session, identifier, table_message, err));
 
   if (iter == vector_of_engines.end())
   {
@@ -428,82 +419,75 @@ handle_error(uint32_t ,
   return true;
 }
 
-class DropTable : 
-  public unary_function<StorageEngine *, void>
-{
-  uint64_t &success_count;
-  TableIdentifier &identifier;
-  Session &session;
-
-public:
-
-  DropTable(Session &session_arg, TableIdentifier &arg, uint64_t &count_arg) :
-    success_count(count_arg),
-    identifier(arg),
-    session(session_arg)
-  {
-  }
-
-  result_type operator() (argument_type engine)
-  {
-    // @todo someday check that at least one engine said "true"
-    std::string path(identifier.getPath());
-    bool success= engine->doDropTable(session, path);
-
-    if (success)
-      success_count++;
-  }
-};
-
-
 /**
    returns ENOENT if the file doesn't exists.
 */
 int StorageEngine::dropTable(Session& session,
-                             TableIdentifier &identifier)
+                             const TableIdentifier &identifier)
 {
   int error= 0;
   int error_proto;
   message::Table src_proto;
-  StorageEngine* engine;
+  StorageEngine *engine;
 
-  error_proto= StorageEngine::getTableDefinition(session,
-                                                 identifier,
-                                                 &src_proto);
+  error_proto= StorageEngine::getTableDefinition(session, identifier, src_proto);
 
   if (error_proto == ER_CORRUPT_TABLE_DEFINITION)
   {
-    my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0),
-             src_proto.InitializationErrorString().c_str());
+    string error_message;
+
+    error_message.append(const_cast<TableIdentifier &>(identifier).getSQLPath());
+    error_message.append(" : ");
+    error_message.append(src_proto.InitializationErrorString());
+
+    my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0), error_message.c_str());
+
     return ER_CORRUPT_TABLE_DEFINITION;
   }
 
   engine= StorageEngine::findByName(session, src_proto.engine().name());
 
-  if (engine)
+  if (not engine)
   {
-    std::string path(identifier.getPath());
-    engine->setTransactionReadWrite(session);
-    error= engine->doDropTable(session, path);
+    my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0), const_cast<TableIdentifier &>(identifier).getSQLPath().c_str());
 
-    if (not error)
-    {
-      if (not engine->check_flag(HTON_BIT_HAS_DATA_DICTIONARY))
-      {
-        uint64_t counter; // @todo We need to refactor to check that.
-
-        for_each(vector_of_schema_engines.begin(), vector_of_schema_engines.end(),
-                 DropTable(session, identifier, counter));
-      }
-    }
+    return ER_CORRUPT_TABLE_DEFINITION;
   }
 
+  error= StorageEngine::dropTable(session, *engine, identifier);
+
   if (error_proto && error == 0)
     return 0;
 
   return error;
 }
 
+int StorageEngine::dropTable(Session& session,
+                             StorageEngine &engine,
+                             const TableIdentifier &identifier)
+{
+  int error;
+
+  engine.setTransactionReadWrite(session);
+  
+  if (unlikely(plugin::EventObserver::beforeDropTable(session, identifier)))
+  {
+    error= ER_EVENT_OBSERVER_PLUGIN;
+  }
+  else
+  {
+    error= engine.doDropTable(session, identifier);
+    if (unlikely(plugin::EventObserver::afterDropTable(session, identifier, error)))
+    {
+      error= ER_EVENT_OBSERVER_PLUGIN;
+    }
+  }
+
+
+  return error;
+}
+
+
 /**
   Initiates table-file and calls appropriate database-creator.
 
@@ -511,325 +495,92 @@ int StorageEngine::dropTable(Session& session,
    0  ok
   @retval
    1  error
-
-   @todo refactor to remove goto
 */
-int StorageEngine::createTable(Session& session,
-                               TableIdentifier &identifier,
-                               bool update_create_info,
+int StorageEngine::createTable(Session &session,
+                               const TableIdentifier &identifier,
                                message::Table& table_message)
 {
   int error= 1;
   Table table;
-  TableShare share(identifier.getDBName(), 0, identifier.getTableName(), identifier.getPath());
+  TableShare share(identifier);
   message::Table tmp_proto;
 
-  if (parse_table_proto(session, table_message, &share))
-    goto err;
-
-  if (open_table_from_share(&session, &share, "", 0, 0,
-                            &table))
-    goto err;
-
-  if (update_create_info)
-    table.updateCreateInfo(&table_message);
-
-  /* Check for legal operations against the Engine using the proto (if used) */
-  if (table_message.type() == message::Table::TEMPORARY &&
-      share.storage_engine->check_flag(HTON_BIT_TEMPORARY_NOT_SUPPORTED) == true)
-  {
-    error= HA_ERR_UNSUPPORTED;
-    goto err2;
+  if (share.parse_table_proto(session, table_message) || share.open_table_from_share(&session, identifier, "", 0, 0, table))
+  { 
+    // @note Error occured, we should probably do a little more here.
   }
-  else if (table_message.type() != message::Table::TEMPORARY &&
-           share.storage_engine->check_flag(HTON_BIT_TEMPORARY_ONLY) == true)
+  else
   {
-    error= HA_ERR_UNSUPPORTED;
-    goto err2;
-  }
-
-  {
-    char name_buff[FN_REFLEN];
-    const char *table_name_arg;
-
-    table_name_arg= share.storage_engine->checkLowercaseNames(identifier.getPath(), name_buff);
-
-    if (not share.storage_engine->check_flag(HTON_BIT_HAS_DATA_DICTIONARY))
+    /* Check for legal operations against the Engine using the proto (if used) */
+    if (table_message.type() == message::Table::TEMPORARY &&
+        share.storage_engine->check_flag(HTON_BIT_TEMPORARY_NOT_SUPPORTED) == true)
     {
-      int protoerr= StorageEngine::writeDefinitionFromPath(identifier, table_message);
+      error= HA_ERR_UNSUPPORTED;
+    }
+    else if (table_message.type() != message::Table::TEMPORARY &&
+             share.storage_engine->check_flag(HTON_BIT_TEMPORARY_ONLY) == true)
+    {
+      error= HA_ERR_UNSUPPORTED;
+    }
+    else
+    {
+      share.storage_engine->setTransactionReadWrite(session);
 
-      if (protoerr)
-      {
-        error= protoerr;
-        goto err2;
-      }
+      error= share.storage_engine->doCreateTable(session,
+                                                 table,
+                                                 identifier,
+                                                 table_message);
     }
 
-    share.storage_engine->setTransactionReadWrite(session);
+    if (error)
+    {
+      my_error(ER_CANT_CREATE_TABLE, MYF(ME_BELL+ME_WAITTANG), const_cast<TableIdentifier &>(identifier).getSQLPath().c_str(), error);
+    }
 
-    error= share.storage_engine->doCreateTable(&session,
-                                               table_name_arg,
-                                               table,
-                                               table_message);
+    table.delete_table();
   }
 
-err2:
-  if (error)
-  {
-    if (not share.storage_engine->check_flag(HTON_BIT_HAS_DATA_DICTIONARY))
-      plugin::StorageEngine::deleteDefinitionFromPath(identifier);
-
-    my_error(ER_CANT_CREATE_TABLE, MYF(ME_BELL+ME_WAITTANG), identifier.getSQLPath().c_str(), error);
-  }
-
-  table.closefrm(false);
-
-err:
-  share.free_table_share();
   return(error != 0);
 }
 
-Cursor *StorageEngine::getCursor(TableShare &share, memory::Root *alloc)
+Cursor *StorageEngine::getCursor(TableShare &share)
 {
-  return create(share, alloc);
+  return create(share);
 }
 
-/**
-  TODO -> Remove this to force all engines to implement their own file. Solves the "we only looked at dfe" problem.
-*/
-void StorageEngine::doGetTableNames(CachedDirectory&, string&, set<string>&)
-{ }
-
-class AddTableName : 
+class AddTableIdentifier : 
   public unary_function<StorageEngine *, void>
 {
-  string db;
-  CachedDirectory& directory;
-  TableNameList &set_of_names;
+  CachedDirectory &directory;
+  const SchemaIdentifier &identifier;
+  TableIdentifiers &set_of_identifiers;
 
 public:
 
-  AddTableName(CachedDirectory& directory_arg, const string& database_name, set<string>& of_names) :
+  AddTableIdentifier(CachedDirectory &directory_arg, const SchemaIdentifier &identifier_arg, TableIdentifiers &of_names) :
     directory(directory_arg),
-    set_of_names(of_names)
-  {
-    db= database_name;
-  }
-
-  result_type operator() (argument_type engine)
-  {
-    engine->doGetTableNames(directory, db, set_of_names);
-  }
-};
-
-class AddSchemaNames : 
-  public unary_function<StorageEngine *, void>
-{
-  SchemaNameList &set_of_names;
-
-public:
-
-  AddSchemaNames(set<string>& of_names) :
-    set_of_names(of_names)
+    identifier(identifier_arg),
+    set_of_identifiers(of_names)
   {
   }
 
   result_type operator() (argument_type engine)
   {
-    engine->doGetSchemaNames(set_of_names);
+    engine->doGetTableIdentifiers(directory, identifier, set_of_identifiers);
   }
 };
 
-void StorageEngine::getSchemaNames(SchemaNameList &set_of_names)
+
+void StorageEngine::getIdentifiers(Session &session, const SchemaIdentifier &schema_identifier, TableIdentifiers &set_of_identifiers)
 {
-  // Add hook here for engines to register schema.
-  for_each(vector_of_schema_engines.begin(), vector_of_schema_engines.end(),
-           AddSchemaNames(set_of_names));
+  static SchemaIdentifier INFORMATION_SCHEMA_IDENTIFIER("information_schema");
+  static SchemaIdentifier DATA_DICTIONARY_IDENTIFIER("data_dictionary");
 
-  plugin::Authorization::pruneSchemaNames(current_session->getSecurityContext(),
-                                          set_of_names);
-}
+  CachedDirectory directory(schema_identifier.getPath(), set_of_table_definition_ext);
 
-class StorageEngineGetSchemaDefinition: public unary_function<StorageEngine *, bool>
-{
-  const std::string &schema_name;
-  message::Schema &schema_proto;
-
-public:
-  StorageEngineGetSchemaDefinition(const std::string &schema_name_arg,
-                                  message::Schema &schema_proto_arg) :
-    schema_name(schema_name_arg),
-    schema_proto(schema_proto_arg) 
+  if (schema_identifier == INFORMATION_SCHEMA_IDENTIFIER)
   { }
-
-  result_type operator() (argument_type engine)
-  {
-    return engine->doGetSchemaDefinition(schema_name, schema_proto);
-  }
-};
-
-/*
-  Return value is "if parsed"
-*/
-bool StorageEngine::getSchemaDefinition(const std::string &schema_name, message::Schema &proto)
-{
-  proto.Clear();
-
-  EngineVector::iterator iter=
-    find_if(vector_of_schema_engines.begin(), vector_of_schema_engines.end(),
-            StorageEngineGetSchemaDefinition(schema_name, proto));
-
-  if (iter != vector_of_schema_engines.end())
-  {
-    return true;
-  }
-
-  return false;
-}
-
-bool StorageEngine::doesSchemaExist(const std::string &schema_name)
-{
-  message::Schema proto;
-
-  return StorageEngine::getSchemaDefinition(schema_name, proto);
-}
-
-
-const CHARSET_INFO *StorageEngine::getSchemaCollation(const std::string &schema_name)
-{
-  message::Schema schmema_proto;
-  bool found;
-
-  found= StorageEngine::getSchemaDefinition(schema_name, schmema_proto);
-
-  if (found && schmema_proto.has_collation())
-  {
-    const string buffer= schmema_proto.collation();
-    const CHARSET_INFO* cs= get_charset_by_name(buffer.c_str());
-
-    if (not cs)
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("Error while loading database options: '%s':"), schema_name.c_str());
-      errmsg_printf(ERRMSG_LVL_ERROR, ER(ER_UNKNOWN_COLLATION), buffer.c_str());
-
-      return default_charset_info;
-    }
-
-    return cs;
-  }
-
-  return default_charset_info;
-}
-
-class CreateSchema : 
-  public unary_function<StorageEngine *, void>
-{
-  const drizzled::message::Schema &schema_message;
-
-public:
-
-  CreateSchema(const drizzled::message::Schema &arg) :
-    schema_message(arg)
-  {
-  }
-
-  result_type operator() (argument_type engine)
-  {
-    // @todo eomeday check that at least one engine said "true"
-    (void)engine->doCreateSchema(schema_message);
-  }
-};
-
-bool StorageEngine::createSchema(const drizzled::message::Schema &schema_message)
-{
-  // Add hook here for engines to register schema.
-  for_each(vector_of_schema_engines.begin(), vector_of_schema_engines.end(),
-           CreateSchema(schema_message));
-
-  return true;
-}
-
-class DropSchema : 
-  public unary_function<StorageEngine *, void>
-{
-  uint64_t &success_count;
-  const string &schema_name;
-
-public:
-
-  DropSchema(const string &arg, uint64_t &count_arg) :
-    success_count(count_arg),
-    schema_name(arg)
-  {
-  }
-
-  result_type operator() (argument_type engine)
-  {
-    // @todo someday check that at least one engine said "true"
-    bool success= engine->doDropSchema(schema_name);
-
-    if (success)
-      success_count++;
-  }
-};
-
-bool StorageEngine::dropSchema(const string &schema_name)
-{
-  uint64_t counter= 0;
-  // Add hook here for engines to register schema.
-  for_each(vector_of_schema_engines.begin(), vector_of_schema_engines.end(),
-           DropSchema(schema_name, counter));
-
-  return counter ? true : false;
-}
-
-class AlterSchema : 
-  public unary_function<StorageEngine *, void>
-{
-  uint64_t &success_count;
-  const drizzled::message::Schema &schema_message;
-
-public:
-
-  AlterSchema(const drizzled::message::Schema &arg, uint64_t &count_arg) :
-    success_count(count_arg),
-    schema_message(arg)
-  {
-  }
-
-  result_type operator() (argument_type engine)
-  {
-    // @todo eomeday check that at least one engine said "true"
-    bool success= engine->doAlterSchema(schema_message);
-
-    if (success)
-      success_count++;
-  }
-};
-
-bool StorageEngine::alterSchema(const drizzled::message::Schema &schema_message)
-{
-  uint64_t success_count= 0;
-
-  for_each(vector_of_schema_engines.begin(), vector_of_schema_engines.end(),
-           AlterSchema(schema_message, success_count));
-
-  return success_count ? true : false;
-}
-
-
-void StorageEngine::getTableNames(const string &schema_name, TableNameList &set_of_names)
-{
-  char tmp_path[FN_REFLEN];
-
-  build_table_filename(tmp_path, sizeof(tmp_path), schema_name.c_str(), "", false);
-
-  CachedDirectory directory(tmp_path, set_of_table_definition_ext);
-
-  if (not schema_name.compare("information_schema"))
-  { }
-  else if (not schema_name.compare("data_dictionary"))
+  else if (schema_identifier == DATA_DICTIONARY_IDENTIFIER)
   { }
   else
   {
@@ -837,7 +588,7 @@ void StorageEngine::getTableNames(const string &schema_name, TableNameList &set_
     {
       errno= directory.getError();
       if (errno == ENOENT)
-        my_error(ER_BAD_DB_ERROR, MYF(ME_BELL+ME_WAITTANG), schema_name.c_str());
+        my_error(ER_BAD_DB_ERROR, MYF(ME_BELL+ME_WAITTANG), const_cast<SchemaIdentifier &>(schema_identifier).getSQLPath().c_str());
       else
         my_error(ER_CANT_READ_DIR, MYF(ME_BELL+ME_WAITTANG), directory.getPath(), errno);
       return;
@@ -845,41 +596,50 @@ void StorageEngine::getTableNames(const string &schema_name, TableNameList &set_
   }
 
   for_each(vector_of_engines.begin(), vector_of_engines.end(),
-           AddTableName(directory, schema_name, set_of_names));
-
-  Session *session= current_session;
-
-  session->doGetTableNames(directory, schema_name, set_of_names);
+           AddTableIdentifier(directory, schema_identifier, set_of_identifiers));
 
+  session.doGetTableIdentifiers(directory, schema_identifier, set_of_identifiers);
 }
 
+class DropTable: public unary_function<TableIdentifier&, bool>
+{
+  Session &session;
+  StorageEngine *engine;
+
+public:
+
+  DropTable(Session &session_arg, StorageEngine *engine_arg) :
+    session(session_arg),
+    engine(engine_arg)
+  { }
+
+  result_type operator() (argument_type identifier)
+  {
+    return engine->doDropTable(session, identifier) == 0;
+  } 
+};
+
 /* This will later be converted to TableIdentifiers */
 class DropTables: public unary_function<StorageEngine *, void>
 {
   Session &session;
-  TableNameList &set_of_names;
+  TableIdentifiers &table_identifiers;
 
 public:
 
-  DropTables(Session &session_arg, set<string>& of_names) :
+  DropTables(Session &session_arg, TableIdentifiers &table_identifiers_arg) :
     session(session_arg),
-    set_of_names(of_names)
+    table_identifiers(table_identifiers_arg)
   { }
 
   result_type operator() (argument_type engine)
   {
-
-    for (TableNameList::iterator iter= set_of_names.begin();
-         iter != set_of_names.end();
-         iter++)
-    {
-      int error= engine->doDropTable(session, *iter);
-
-      // On a return of zero we know we found and deleted the table. So we
-      // remove it from our search.
-      if (not error)
-        set_of_names.erase(iter);
-    }
+    // True returning from DropTable means the table has been successfully
+    // deleted, so it should be removed from the list of tables to drop
+    table_identifiers.erase(remove_if(table_identifiers.begin(),
+                                      table_identifiers.end(),
+                                      DropTable(session, engine)),
+                            table_identifiers.end());
   }
 };
 
@@ -891,7 +651,7 @@ public:
 void StorageEngine::removeLostTemporaryTables(Session &session, const char *directory)
 {
   CachedDirectory dir(directory, set_of_table_definition_ext);
-  set<string> set_of_table_names;
+  TableIdentifiers table_identifiers;
 
   if (dir.fail())
   {
@@ -917,11 +677,16 @@ void StorageEngine::removeLostTemporaryTables(Session &session, const char *dire
     path+= directory;
     path+= FN_LIBCHAR;
     path+= entry->filename;
-    set_of_table_names.insert(path);
+    message::Table definition;
+    if (StorageEngine::readTableFile(path, definition))
+    {
+      TableIdentifier identifier(definition.schema(), definition.name(), path);
+      table_identifiers.push_back(identifier);
+    }
   }
 
   for_each(vector_of_engines.begin(), vector_of_engines.end(),
-           DropTables(session, set_of_table_names));
+           DropTables(session, table_identifiers));
   
   /*
     Now we just clean up anything that might left over.
@@ -962,7 +727,7 @@ void StorageEngine::removeLostTemporaryTables(Session &session, const char *dire
   @note
     In case of delete table it's only safe to use the following parts of
     the 'table' structure:
-    - table->s->path
+    - table->getShare()->path
     - table->alias
 */
 void StorageEngine::print_error(int error, myf errflag, Table &table)
@@ -999,15 +764,8 @@ void StorageEngine::print_error(int error, myf errflag, Table *table)
     {
       const char *err_msg= ER(ER_DUP_ENTRY_WITH_KEY_NAME);
 
-      if (key_nr == 0 &&
-          (table->key_info[0].key_part[0].field->flags &
-           AUTO_INCREMENT_FLAG)
-          && (current_session)->lex->sql_command == SQLCOM_ALTER_TABLE)
-      {
-        err_msg= ER(ER_DUP_ENTRY_AUTOINCREMENT_CASE);
-      }
-
       print_keydup_error(key_nr, err_msg, *table);
+
       return;
     }
     textno=ER_DUP_KEY;
@@ -1034,7 +792,7 @@ void StorageEngine::print_error(int error, myf errflag, Table *table)
         str.length(max_length-4);
         str.append(STRING_WITH_LEN("..."));
       }
-      my_error(ER_FOREIGN_DUPLICATE_KEY, MYF(0), table->s->table_name.str,
+      my_error(ER_FOREIGN_DUPLICATE_KEY, MYF(0), table->getShare()->getTableName(),
         str.c_ptr(), key_nr+1);
       return;
     }
@@ -1112,8 +870,8 @@ void StorageEngine::print_error(int error, myf errflag, Table *table)
     break;
   case HA_ERR_NO_SUCH_TABLE:
     assert(table);
-    my_error(ER_NO_SUCH_TABLE, MYF(0), table->s->getSchemaName(),
-             table->s->table_name.str);
+    my_error(ER_NO_SUCH_TABLE, MYF(0), table->getShare()->getSchemaName(),
+             table->getShare()->getTableName());
     return;
   case HA_ERR_RBR_LOGGING_FAILED:
     textno= ER_BINLOG_ROW_LOGGING_FAILED;
@@ -1169,7 +927,7 @@ void StorageEngine::print_error(int error, myf errflag, Table *table)
       return;
     }
   }
-  my_error(textno, errflag, table->s->table_name.str, error);
+  my_error(textno, errflag, table->getShare()->getTableName(), error);
 }
 
 
@@ -1216,7 +974,7 @@ void StorageEngine::print_keydup_error(uint32_t key_nr, const char *msg, Table &
 }
 
 
-int StorageEngine::deleteDefinitionFromPath(TableIdentifier &identifier)
+int StorageEngine::deleteDefinitionFromPath(const TableIdentifier &identifier)
 {
   string path(identifier.getPath());
 
@@ -1225,40 +983,104 @@ int StorageEngine::deleteDefinitionFromPath(TableIdentifier &identifier)
   return internal::my_delete(path.c_str(), MYF(0));
 }
 
-int StorageEngine::renameDefinitionFromPath(TableIdentifier &dest, TableIdentifier &src)
+int StorageEngine::renameDefinitionFromPath(const TableIdentifier &dest, const TableIdentifier &src)
 {
+  message::Table table_message;
   string src_path(src.getPath());
   string dest_path(dest.getPath());
 
   src_path.append(DEFAULT_DEFINITION_FILE_EXT);
   dest_path.append(DEFAULT_DEFINITION_FILE_EXT);
 
-  return internal::my_rename(src_path.c_str(), dest_path.c_str(), MYF(MY_WME));
+  bool was_read= StorageEngine::readTableFile(src_path.c_str(), table_message);
+
+  if (not was_read)
+  {
+    return ENOENT;
+  }
+
+  dest.copyToTableMessage(table_message);
+
+  int error= StorageEngine::writeDefinitionFromPath(dest, table_message);
+
+  if (not error)
+  {
+    if (unlink(src_path.c_str()))
+      perror(src_path.c_str());
+  }
+
+  return error;
 }
 
-int StorageEngine::writeDefinitionFromPath(TableIdentifier &identifier, message::Table &table_message)
+int StorageEngine::writeDefinitionFromPath(const TableIdentifier &identifier, message::Table &table_message)
 {
+  char definition_file_tmp[FN_REFLEN];
   string file_name(identifier.getPath());
 
   file_name.append(DEFAULT_DEFINITION_FILE_EXT);
 
-  int fd= open(file_name.c_str(), O_RDWR|O_CREAT|O_TRUNC, internal::my_umask);
+  snprintf(definition_file_tmp, sizeof(definition_file_tmp), "%sXXXXXX", file_name.c_str());
+
+  int fd= mkstemp(definition_file_tmp);
 
   if (fd == -1)
+  {
+    perror(definition_file_tmp);
     return errno;
+  }
 
   google::protobuf::io::ZeroCopyOutputStream* output=
     new google::protobuf::io::FileOutputStream(fd);
 
-  if (table_message.SerializeToZeroCopyStream(output) == false)
+  bool success;
+
+  try {
+    success= table_message.SerializeToZeroCopyStream(output);
+  }
+  catch (...)
   {
+    success= false;
+  }
+
+  if (not success)
+  {
+    my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0),
+             table_message.InitializationErrorString().c_str());
     delete output;
-    close(fd);
-    return errno;
+
+    if (close(fd) == -1)
+      perror(definition_file_tmp);
+
+    if (unlink(definition_file_tmp) == -1)
+      perror(definition_file_tmp);
+
+    return ER_CORRUPT_TABLE_DEFINITION;
   }
 
   delete output;
-  close(fd);
+
+  if (close(fd) == -1)
+  {
+    int error= errno;
+    perror(definition_file_tmp);
+
+    if (unlink(definition_file_tmp))
+      perror(definition_file_tmp);
+
+    return error;
+  }
+
+  if (rename(definition_file_tmp, file_name.c_str()) == -1)
+  {
+    int error= errno;
+    perror(definition_file_tmp);
+
+    if (unlink(definition_file_tmp))
+      perror(definition_file_tmp);
+
+    return error;
+  }
+
   return 0;
 }
 
@@ -1281,7 +1103,7 @@ public:
 /**
   @note on success table can be created.
 */
-bool StorageEngine::canCreateTable(drizzled::TableIdentifier &identifier)
+bool StorageEngine::canCreateTable(const TableIdentifier &identifier)
 {
   EngineVector::iterator iter=
     find_if(vector_of_engines.begin(), vector_of_engines.end(),
@@ -1295,5 +1117,33 @@ bool StorageEngine::canCreateTable(drizzled::TableIdentifier &identifier)
   return false;
 }
 
+bool StorageEngine::readTableFile(const std::string &path, message::Table &table_message)
+{
+  fstream input(path.c_str(), ios::in | ios::binary);
+
+  if (input.good())
+  {
+    try {
+      if (table_message.ParseFromIstream(&input))
+      {
+        return true;
+      }
+    }
+    catch (...)
+    {
+      my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0),
+               table_message.InitializationErrorString().empty() ? "": table_message.InitializationErrorString().c_str());
+    }
+  }
+  else
+  {
+    perror(path.c_str());
+  }
+
+  return false;
+}
+
+
+
 } /* namespace plugin */
 } /* namespace drizzled */
