@@ -22,7 +22,6 @@
 #include <drizzled/plugin/table_function.h>
 #include <drizzled/table_function_container.h>
 #include <drizzled/gettext.h>
-#include "drizzled/plugin/registry.h"
 #include "drizzled/global_charset_info.h"
 #include "drizzled/session.h"
 #include "drizzled/current_session.h"
@@ -38,11 +37,14 @@ static TableFunctionContainer table_functions;
 
 void plugin::TableFunction::init()
 {
-  drizzled::message::Table::StorageEngine *engine;
+  drizzled::message::Engine *engine;
   drizzled::message::Table::TableOptions *table_options;
 
-  proto.set_name(identifier.getTableName());
+  proto.set_name(getTableLabel());
+  proto.set_schema(identifier.getSchemaName());
   proto.set_type(drizzled::message::Table::FUNCTION);
+  proto.set_creation_timestamp(0);
+  proto.set_update_timestamp(0);
 
   table_options= proto.mutable_options();
   table_options->set_collation_id(default_charset_info->number);
@@ -89,9 +91,9 @@ void plugin::TableFunction::add_field(const char *label,
 }
 
 void plugin::TableFunction::add_field(const char *label,
-                              TableFunction::ColumnType type,
-                              uint32_t field_length,
-                              bool is_default_null)
+                                      TableFunction::ColumnType type,
+                                      uint32_t field_length,
+                                      bool is_default_null)
 {
   drizzled::message::Table::Field *field;
   drizzled::message::Table::Field::FieldOptions *field_options;
@@ -113,23 +115,34 @@ void plugin::TableFunction::add_field(const char *label,
     field_options->set_default_null(false);
     field_constraints->set_is_nullable(false);
   case TableFunction::STRING:
+  {
     drizzled::message::Table::Field::StringFieldOptions *string_field_options;
     field->set_type(drizzled::message::Table::Field::VARCHAR);
 
     string_field_options= field->mutable_string_options();
     string_field_options->set_length(field_length);
+  }
+    break;
+  case TableFunction::VARBINARY:
+  {
+    drizzled::message::Table::Field::StringFieldOptions *string_field_options;
+    field->set_type(drizzled::message::Table::Field::VARCHAR);
 
+    string_field_options= field->mutable_string_options();
+    string_field_options->set_length(field_length);
+    string_field_options->set_collation(my_charset_bin.csname);
+    string_field_options->set_collation_id(my_charset_bin.number);
+  }
     break;
   case TableFunction::NUMBER: // Currently NUMBER always has a value
     field->set_type(drizzled::message::Table::Field::BIGINT);
-    field_options->set_default_null(false);
-    field_constraints->set_is_nullable(false);
     break;
   }
 }
 
 plugin::TableFunction::Generator::Generator(Field **arg) :
-  columns(arg)
+  columns(arg),
+  session(current_session)
 {
   scs= system_charset_info;
 }
@@ -154,12 +167,14 @@ bool plugin::TableFunction::Generator::sub_populate(uint32_t field_size)
 void plugin::TableFunction::Generator::push(uint64_t arg)
 {
   (*columns_iterator)->store(static_cast<int64_t>(arg), true);
+  (*columns_iterator)->set_notnull();
   columns_iterator++;
 }
 
 void plugin::TableFunction::Generator::push(int64_t arg)
 {
   (*columns_iterator)->store(arg, false);
+  (*columns_iterator)->set_notnull();
   columns_iterator++;
 }
 
@@ -170,9 +185,6 @@ void plugin::TableFunction::Generator::push(const char *arg, uint32_t length)
   assert(arg);
   length= length ? length : strlen(arg);
 
-  if (not length)
-    return push();
-
   (*columns_iterator)->store(arg, length, scs);
   (*columns_iterator)->set_notnull();
   columns_iterator++;
@@ -180,7 +192,9 @@ void plugin::TableFunction::Generator::push(const char *arg, uint32_t length)
 
 void plugin::TableFunction::Generator::push()
 {
-  assert((*columns_iterator)->type()  == DRIZZLE_TYPE_VARCHAR);
+#if 0 // @note this needs to be rewritten such that a drizzled::Field object can determine if it should ever be null
+  assert((*columns_iterator)->getTable()->getShare()->getTableProto()->field((*columns_iterator)->getTable()->getFields() - columns_iterator).constraints().is_nullable());
+#endif
   (*columns_iterator)->set_null();
   columns_iterator++;
 }
@@ -194,11 +208,11 @@ void plugin::TableFunction::Generator::push(bool arg)
 {
   if (arg)
   {
-    (*columns_iterator)->store("TRUE", 4, scs);
+    (*columns_iterator)->store("YES", 3, scs);
   }
   else
   {
-    (*columns_iterator)->store("FALSE", 5, scs);
+    (*columns_iterator)->store("NO", 2, scs);
   }
 
   columns_iterator++;
@@ -206,14 +220,12 @@ void plugin::TableFunction::Generator::push(bool arg)
 
 bool plugin::TableFunction::Generator::isWild(const std::string &predicate)
 {
-  Session *session= current_session;
-
-  if (not session->lex->wild)
+  if (not getSession().lex->wild)
     return false;
 
   bool match= wild_case_compare(system_charset_info,
                                 predicate.c_str(),
-                                session->lex->wild->ptr());
+                                getSession().lex->wild->ptr());
 
   return match;
 }
