@@ -26,7 +26,7 @@
 #include <drizzled/definitions.h>
 #include "drizzled/plugin.h"
 #include "drizzled/plugin/plugin.h"
-#include "drizzled/table_identifier.h"
+#include "drizzled/identifier.h"
 #include "drizzled/message/table.pb.h"
 #include "drizzled/charset.h"
 #include "drizzled/field.h"
@@ -44,6 +44,20 @@ extern int wild_case_compare(const CHARSET_INFO * const cs,
 namespace plugin
 {
 
+// Not thread safe, but plugins are just loaded in a single thread right
+// now.
+static const char *local_string_append(const char *arg1, const char *arg2)
+{
+  static char buffer[1024];
+  char *buffer_ptr= buffer;
+  strcpy(buffer_ptr, arg1);
+  buffer_ptr+= strlen(arg1);
+  buffer_ptr[0]= '-';
+  buffer_ptr++;
+  strcpy(buffer_ptr, arg2);
+
+  return buffer;
+}
 
 class TableFunction : public Plugin
 {
@@ -54,7 +68,7 @@ class TableFunction : public Plugin
   message::Table proto;
   TableIdentifier identifier;
   std::string local_path;
-  std::string local_schema;
+  std::string original_table_label;
 
   void setName(); // init name
 
@@ -62,8 +76,9 @@ class TableFunction : public Plugin
 
 public:
   TableFunction(const char *schema_arg, const char *table_arg) :
-    Plugin(table_arg, "TableFunction"),
-    identifier(schema_arg, table_arg)
+    Plugin(local_string_append(schema_arg, table_arg) , "TableFunction"),
+    identifier(schema_arg, table_arg),
+    original_table_label(table_arg)
   {
     init();
   }
@@ -80,13 +95,22 @@ public:
   enum ColumnType {
     BOOLEAN,
     NUMBER,
-    STRING
+    STRING,
+    VARBINARY
   };
 
   class Generator 
   {
     Field **columns;
     Field **columns_iterator;
+    Session *session;
+
+  protected:
+
+    drizzled::Session &getSession()
+    {
+      return *session;
+    }
 
   public:
     const CHARSET_INFO *scs;
@@ -120,28 +144,24 @@ public:
     arg.CopyFrom(proto);
   }
 
+  const std::string &getTableLabel()
+  { 
+    return original_table_label;
+  }
+
+  const std::string &getIdentifierTableName()
+  { 
+    return identifier.getTableName();
+  }
+
   const std::string &getSchemaHome()
   { 
-    if (local_schema.length() == 0)
-    {
-      local_schema= identifier.getSchemaName();
-      std::transform(local_schema.begin(), local_schema.end(),
-                     local_schema.begin(), ::tolower);
-    }
-
-    return local_schema;
+    return identifier.getSchemaName();
   }
 
   const std::string &getPath()
   { 
-    if (local_path.length() == 0)
-    {
-      local_path= identifier.getPath();
-      std::transform(local_path.begin(), local_path.end(),
-                     local_path.begin(), ::tolower);
-    }
-
-    return local_path;
+    return identifier.getPath();
   }
 
   virtual Generator *generator(Field **arg);
@@ -151,7 +171,7 @@ public:
                  uint32_t length= 0);
 
   void add_field(const char *label,
-                 uint32_t field_length= 64);
+                 uint32_t field_length= MAXIMUM_IDENTIFIER_LENGTH);
 
   void add_field(const char *label,
                  TableFunction::ColumnType type,
