@@ -2,10 +2,11 @@
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
  *  Copyright (C) 2008-2009 Sun Microsystems
+ *  Copyright (c) 2010 Jay Pipes
  *
  *  Authors:
  *
- *    Jay Pipes <joinfu@sun.com>
+ *    Jay Pipes <jaypipes@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
 #ifndef DRIZZLED_PLUGIN_TRANSACTION_REPLICATOR_H
 #define DRIZZLED_PLUGIN_TRANSACTION_REPLICATOR_H
 
-#include "drizzled/atomics.h"
+#include "drizzled/plugin/replication.h"
 #include "drizzled/plugin/plugin.h"
 
 /**
@@ -36,7 +37,6 @@
  * An applier is responsible for applying events, not a replicator...
  */
 
-
 namespace drizzled
 {
 namespace message
@@ -45,6 +45,8 @@ namespace message
   class Statement;
 }
 
+class Session;
+
 namespace plugin
 {
 
@@ -58,12 +60,10 @@ class TransactionReplicator : public Plugin
   TransactionReplicator();
   TransactionReplicator(const TransactionReplicator &);
   TransactionReplicator& operator=(const TransactionReplicator &);
-  atomic<bool> is_enabled;
 public:
   explicit TransactionReplicator(std::string name_arg)
     : Plugin(name_arg, "TransactionReplicator")
   {
-    is_enabled= true;
   }
   virtual ~TransactionReplicator() {}
 
@@ -83,25 +83,11 @@ public:
    * @param Pointer to the applier of the command message
    * @param Transaction message to be replicated
    */
-  virtual void replicate(TransactionApplier *in_applier, 
-                         message::Transaction &to_replicate)= 0;
+  virtual ReplicationReturnCode replicate(TransactionApplier *in_applier, 
+                                          Session &session,
+                                          message::Transaction &to_replicate)= 0;
   static bool addPlugin(TransactionReplicator *replicator);
   static void removePlugin(TransactionReplicator *replicator);
-
-  virtual bool isEnabled() const
-  {
-    return is_enabled;
-  }
-
-  virtual void enable()
-  {
-    is_enabled= true;
-  }
-
-  virtual void disable()
-  {
-    is_enabled= false;
-  }
 };
 
 } /* namespace plugin */
