@@ -21,10 +21,10 @@
 #define DRIZZLED_PLUGIN_VERSION_H
 
 /* Machine comsumable version of the server which was built against. */
-#define DRIZZLE_VERSION_ID 2010031347
+#define DRIZZLE_VERSION_ID 2010091802
 
 /* A release comment about the branch that was built */
-#define DRIZZLE_RELEASE_COMMENT "trunk"
+#define DRIZZLE_RELEASE_COMMENT "drizzle"
 
 #endif /* DRIZZLED_PLUGIN_VERSION_H */
 
