@@ -20,7 +20,6 @@
 
 #include "config.h"
 
-#include "drizzled/my_hash.h"
 #include "drizzled/cached_directory.h"
 
 #include <drizzled/definitions.h>
@@ -30,8 +29,6 @@
 #include <drizzled/plugin/xa_storage_engine.h>
 #include "drizzled/xid.h"
 
-#include "drizzled/hash.h"
-
 #include <string>
 #include <vector>
 #include <algorithm>
