@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,9 +18,8 @@
  */
 
 
-#pragma once
-
-#include <drizzled/select_to_file.h>
+#ifndef DRIZZLED_SELECT_DUMP_H
+#define DRIZZLED_SELECT_DUMP_H
 
 namespace drizzled
 {
@@ -34,3 +33,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_SELECT_DUMP_H */
