@@ -22,6 +22,7 @@
 #define DRIZZLED_SELECT_SEND_H
 
 #include <drizzled/plugin/client.h>
+#include <drizzled/plugin/query_cache.h>
 #include <drizzled/plugin/transactional_storage_engine.h>
 
 namespace drizzled
@@ -111,6 +112,10 @@ public:
         break;
       }
     }
+    /* Insert this record to the Resultset into the cache */
+    if (session->query_cache_key != "" && session->getResultsetMessage() != NULL)
+      plugin::QueryCache::insertRecord(session, items);
+
     session->sent_row_count++;
     if (session->is_error())
       return true;
