@@ -46,24 +46,26 @@
 #include "drizzled/db.h"
 #include "drizzled/pthread_globals.h"
 #include "drizzled/transaction_services.h"
+#include "drizzled/drizzled.h"
+
+#include "drizzled/table_share_instance.h"
 
 #include "plugin/myisam/myisam.h"
 #include "drizzled/internal/iocache.h"
+#include "drizzled/internal/thread_var.h"
+#include "drizzled/plugin/event_observer.h"
 
 #include <fcntl.h>
 #include <algorithm>
 #include <climits>
+#include "boost/filesystem.hpp" 
 
 using namespace std;
+
+namespace fs=boost::filesystem;
 namespace drizzled
 {
 
-extern "C"
-{
-  unsigned char *get_var_key(user_var_entry *entry, size_t *length, bool );
-  void free_user_var(user_var_entry *entry);
-}
-
 /*
   The following is used to initialise Table_ident with a internal
   table name
@@ -72,25 +74,6 @@ char internal_table_name[2]= "*";
 char empty_c_string[1]= {0};    /* used for not defined db */
 
 const char * const Session::DEFAULT_WHERE= "field list";
-extern pthread_key_t THR_Session;
-extern pthread_key_t THR_Mem_root;
-extern uint32_t max_used_connections;
-extern atomic<uint32_t> connection_count;
-
-
-/****************************************************************************
-** User variables
-****************************************************************************/
-unsigned char *get_var_key(user_var_entry *entry, size_t *length, bool )
-{
-  *length= entry->name.length;
-  return (unsigned char*) entry->name.str;
-}
-
-void free_user_var(user_var_entry *entry)
-{
-  delete entry;
-}
 
 bool Key_part_spec::operator==(const Key_part_spec& other) const
 {
@@ -99,19 +82,20 @@ bool Key_part_spec::operator==(const Key_part_spec& other) const
          !strcmp(field_name.str, other.field_name.str);
 }
 
-Open_tables_state::Open_tables_state(uint64_t version_arg)
-  :version(version_arg), backups_available(false)
+Open_tables_state::Open_tables_state(uint64_t version_arg) :
+  version(version_arg)
 {
-  reset_open_tables_state();
+  open_tables= temporary_tables= derived_tables= NULL;
+  extra_lock= lock= NULL;
 }
 
 /*
   The following functions form part of the C plugin API
 */
-extern "C" int mysql_tmpfile(const char *prefix)
+int mysql_tmpfile(const char *prefix)
 {
   char filename[FN_REFLEN];
-  int fd = internal::create_temp_file(filename, drizzle_tmpdir, prefix, MYF(MY_WME));
+  int fd = internal::create_temp_file(filename, drizzle_tmpdir.c_str(), prefix, MYF(MY_WME));
   if (fd >= 0) {
     unlink(filename);
   }
@@ -119,7 +103,6 @@ extern "C" int mysql_tmpfile(const char *prefix)
   return fd;
 }
 
-extern "C"
 int session_tablespace_op(const Session *session)
 {
   return test(session->tablespace_op);
@@ -133,13 +116,11 @@ int session_tablespace_op(const Session *session)
 
    @see Session::set_proc_info
  */
-extern "C" void
-set_session_proc_info(Session *session, const char *info)
+void set_session_proc_info(Session *session, const char *info)
 {
   session->set_proc_info(info);
 }
 
-extern "C"
 const char *get_session_proc_info(Session *session)
 {
   return session->get_proc_info();
@@ -156,26 +137,22 @@ ResourceContext *Session::getResourceContext(const plugin::MonitoredInTransactio
   return &ha_data[monitored->getId()].resource_context[index];
 }
 
-extern "C"
 int64_t session_test_options(const Session *session, int64_t test_options)
 {
   return session->options & test_options;
 }
 
-extern "C"
 int session_sql_command(const Session *session)
 {
   return (int) session->lex->sql_command;
 }
 
-extern "C"
-int session_tx_isolation(const Session *session)
+enum_tx_isolation session_tx_isolation(const Session *session)
 {
-  return (int) session->variables.tx_isolation;
+  return (enum_tx_isolation)session->variables.tx_isolation;
 }
 
-Session::Session(plugin::Client *client_arg)
-  :
+Session::Session(plugin::Client *client_arg) :
   Open_tables_state(refresh_version),
   mem_root(&main_mem_root),
   lex(&main_lex),
@@ -202,7 +179,9 @@ Session::Session(plugin::Client *client_arg)
   m_lip(NULL),
   cached_table(0),
   transaction_message(NULL),
-  statement_message(NULL)
+  statement_message(NULL),
+  session_event_observers(NULL),
+  use_usage(false)
 {
   memset(process_list_info, 0, PROCESS_LIST_WIDTH);
   client->setSession(this);
@@ -214,7 +193,7 @@ Session::Session(plugin::Client *client_arg)
   */
   memory::init_sql_alloc(&main_mem_root, memory::ROOT_MIN_BLOCK_SIZE, 0);
   thread_stack= NULL;
-  count_cuted_fields= CHECK_FIELD_IGNORE;
+  count_cuted_fields= CHECK_FIELD_ERROR_FOR_NULL;
   killed= NOT_KILLED;
   col_access= 0;
   tmp_table= 0;
@@ -233,9 +212,13 @@ Session::Session(plugin::Client *client_arg)
   query_id= 0;
   warn_query_id= 0;
   mysys_var= 0;
+  scoreboard_index= -1;
   dbug_sentry=Session_SENTRY_MAGIC;
-  cleanup_done= abort_on_warning= no_warnings_for_error= false;
-  pthread_mutex_init(&LOCK_delete, MY_MUTEX_INIT_FAST);
+  cleanup_done= abort_on_warning= no_warnings_for_error= false;  
+
+  /* query_cache init */
+  query_cache_key= "";
+  resultset= NULL;
 
   /* Variables with default values */
   proc_info="login";
@@ -267,15 +250,14 @@ Session::Session(plugin::Client *client_arg)
 
   /* Initialize sub structures */
   memory::init_sql_alloc(&warn_root, WARN_ALLOC_BLOCK_SIZE, WARN_ALLOC_PREALLOC_SIZE);
-  hash_init(&user_vars, system_charset_info, USER_VARS_HASH_SIZE, 0, 0,
-	    (hash_get_key) get_var_key,
-	    (hash_free_key) free_user_var, 0);
 
   substitute_null_with_insert_id = false;
-  thr_lock_info_init(&lock_info); /* safety: will be reset after start */
+  lock_info.init(); /* safety: will be reset after start */
   thr_lock_owner_init(&main_lock_id, &lock_info);
 
   m_internal_handler= NULL;
+  
+  plugin::EventObserver::registerSessionEvents(*this); 
 }
 
 void Session::free_items()
@@ -310,50 +292,37 @@ bool Session::handle_error(uint32_t sql_errno, const char *message,
   return false;                                 // 'false', as per coding style
 }
 
+void Session::setAbort(bool arg)
+{
+  mysys_var->abort= arg;
+}
+
+void Session::lockOnSys()
+{
+  if (not mysys_var)
+    return;
+
+  setAbort(true);
+  boost::mutex::scoped_lock scopedLock(mysys_var->mutex);
+  if (mysys_var->current_cond)
+  {
+    mysys_var->current_mutex->lock();
+    mysys_var->current_cond->notify_all();
+    mysys_var->current_mutex->unlock();
+  }
+}
+
 void Session::pop_internal_handler()
 {
   assert(m_internal_handler != NULL);
   m_internal_handler= NULL;
 }
 
-#if defined(__cplusplus)
-extern "C" {
-#endif
-
-void *session_alloc(Session *session, unsigned int size)
+void Session::get_xid(DRIZZLE_XID *xid)
 {
-  return session->alloc(size);
+  *xid = *(DRIZZLE_XID *) &transaction.xid_state.xid;
 }
 
-void *session_calloc(Session *session, unsigned int size)
-{
-  return session->calloc(size);
-}
-
-char *session_strdup(Session *session, const char *str)
-{
-  return session->strdup(str);
-}
-
-char *session_strmake(Session *session, const char *str, unsigned int size)
-{
-  return session->strmake(str, size);
-}
-
-void *session_memdup(Session *session, const void* str, unsigned int size)
-{
-  return session->memdup(str, size);
-}
-
-void session_get_xid(const Session *session, DRIZZLE_XID *xid)
-{
-  *xid = *(DRIZZLE_XID *) &session->transaction.xid_state.xid;
-}
-
-#if defined(__cplusplus)
-}
-#endif
-
 /* Do operations that may take a long time */
 
 void Session::cleanup(void)
@@ -369,10 +338,20 @@ void Session::cleanup(void)
 #endif
   {
     TransactionServices &transaction_services= TransactionServices::singleton();
-    transaction_services.ha_rollback_trans(this, true);
+    transaction_services.rollbackTransaction(this, true);
     xid_cache_delete(&transaction.xid_state);
   }
-  hash_free(&user_vars);
+
+  for (UserVars::iterator iter= user_vars.begin();
+       iter != user_vars.end();
+       iter++)
+  {
+    user_var_entry *entry= (*iter).second;
+    delete entry;
+  }
+  user_vars.clear();
+
+
   close_temporary_tables();
 
   if (global_read_lock)
@@ -383,8 +362,7 @@ void Session::cleanup(void)
 
 Session::~Session()
 {
-  Session_CHECK_SENTRY(this);
-  add_to_status(&global_status_var, &status_var);
+  this->checkSentry();
 
   if (client->isConnected())
   {
@@ -406,71 +384,30 @@ Session::~Session()
   plugin::StorageEngine::closeConnection(this);
   plugin_sessionvar_cleanup(this);
 
-  free_root(&warn_root,MYF(0));
+  warn_root.free_root(MYF(0));
   mysys_var=0;					// Safety (shouldn't be needed)
   dbug_sentry= Session_SENTRY_GONE;
 
-  free_root(&main_mem_root, MYF(0));
-  pthread_setspecific(THR_Session,  0);
+  main_mem_root.free_root(MYF(0));
+  currentMemRoot().release();
+  currentSession().release();
 
   plugin::Logging::postEndDo(this);
+  plugin::EventObserver::deregisterSessionEvents(*this); 
+
+  for (PropertyMap::iterator iter= life_properties.begin(); iter != life_properties.end(); iter++)
+  {
+    delete (*iter).second;
+  }
+  life_properties.clear();
 
   /* Ensure that no one is using Session */
-  pthread_mutex_unlock(&LOCK_delete);
-  pthread_mutex_destroy(&LOCK_delete);
-}
-
-/*
-  Add all status variables to another status variable array
-
-  SYNOPSIS
-   add_to_status()
-   to_var       add to this array
-   from_var     from this array
-
-  NOTES
-    This function assumes that all variables are long/ulong.
-    If this assumption will change, then we have to explictely add
-    the other variables after the while loop
-*/
-void add_to_status(system_status_var *to_var, system_status_var *from_var)
-{
-  ulong *end= (ulong*) ((unsigned char*) to_var +
-                        offsetof(system_status_var, last_system_status_var) +
-			sizeof(ulong));
-  ulong *to= (ulong*) to_var, *from= (ulong*) from_var;
-
-  while (to != end)
-    *(to++)+= *(from++);
-}
-
-/*
-  Add the difference between two status variable arrays to another one.
-
-  SYNOPSIS
-    add_diff_to_status
-    to_var       add to this array
-    from_var     from this array
-    dec_var      minus this array
-
-  NOTE
-    This function assumes that all variables are long/ulong.
-*/
-void add_diff_to_status(system_status_var *to_var, system_status_var *from_var,
-                        system_status_var *dec_var)
-{
-  ulong *end= (ulong*) ((unsigned char*) to_var + offsetof(system_status_var,
-						  last_system_status_var) +
-			sizeof(ulong));
-  ulong *to= (ulong*) to_var, *from= (ulong*) from_var, *dec= (ulong*) dec_var;
-
-  while (to != end)
-    *(to++)+= *(from++) - *(dec++);
+  LOCK_delete.unlock();
 }
 
 void Session::awake(Session::killed_state state_to_set)
 {
-  Session_CHECK_SENTRY(this);
+  this->checkSentry();
   safe_mutex_assert_owner(&LOCK_delete);
 
   killed= state_to_set;
@@ -481,8 +418,9 @@ void Session::awake(Session::killed_state state_to_set)
   }
   if (mysys_var)
   {
-    pthread_mutex_lock(&mysys_var->mutex);
+    boost::mutex::scoped_lock scopedLock(mysys_var->mutex);
     /*
+      "
       This broadcast could be up in the air if the victim thread
       exits the cond in the time between read and broadcast, but that is
       ok since all we want to do is to make the victim thread get out
@@ -503,11 +441,10 @@ void Session::awake(Session::killed_state state_to_set)
     */
     if (mysys_var->current_cond && mysys_var->current_mutex)
     {
-      pthread_mutex_lock(mysys_var->current_mutex);
-      pthread_cond_broadcast(mysys_var->current_cond);
-      pthread_mutex_unlock(mysys_var->current_mutex);
+      mysys_var->current_mutex->lock();
+      mysys_var->current_cond->notify_all();
+      mysys_var->current_mutex->unlock();
     }
-    pthread_mutex_unlock(&mysys_var->mutex);
   }
 }
 
@@ -523,9 +460,11 @@ bool Session::storeGlobals()
   */
   assert(thread_stack);
 
-  if (pthread_setspecific(THR_Session,  this) ||
-      pthread_setspecific(THR_Mem_root, &mem_root))
-    return true;
+  currentSession().release();
+  currentSession().reset(this);
+
+  currentMemRoot().release();
+  currentMemRoot().reset(&mem_root);
 
   mysys_var=my_thread_var;
 
@@ -534,13 +473,13 @@ bool Session::storeGlobals()
     This allows us to move Session to different threads if needed.
   */
   mysys_var->id= thread_id;
-  real_id= pthread_self();                      // For debugging
 
   /*
     We have to call thr_lock_info_init() again here as Session may have been
     created in another thread
   */
-  thr_lock_info_init(&lock_info);
+  lock_info.init();
+
   return false;
 }
 
@@ -560,10 +499,12 @@ void Session::prepareForQueries()
   command= COM_SLEEP;
   set_time();
 
-  reset_root_defaults(mem_root, variables.query_alloc_block_size,
-                      variables.query_prealloc_size);
+  mem_root->reset_root_defaults(variables.query_alloc_block_size,
+                                variables.query_prealloc_size);
   transaction.xid_state.xid.null();
   transaction.xid_state.in_session=1;
+  if (use_usage)
+    resetUsage();
 }
 
 bool Session::initGlobals()
@@ -571,7 +512,7 @@ bool Session::initGlobals()
   if (storeGlobals())
   {
     disconnect(ER_OUT_OF_RESOURCES, true);
-    statistic_increment(aborted_connects, &LOCK_status);
+    status_var.aborted_connects++;
     return true;
   }
   return false;
@@ -603,14 +544,17 @@ bool Session::schedule()
 
   connection_count.increment();
 
-  if (connection_count > max_used_connections)
-    max_used_connections= connection_count;
+  if (connection_count > current_global_counters.max_used_connections)
+  {
+    current_global_counters.max_used_connections= connection_count;
+  }
 
+  current_global_counters.connections++;
   thread_id= variables.pseudo_thread_id= global_thread_id++;
 
-  pthread_mutex_lock(&LOCK_thread_count);
+  LOCK_thread_count.lock();
   getSessionList().push_back(this);
-  pthread_mutex_unlock(&LOCK_thread_count);
+  LOCK_thread_count.unlock();
 
   if (scheduler->addSession(this))
   {
@@ -619,7 +563,7 @@ bool Session::schedule()
 
     killed= Session::KILL_CONNECTION;
 
-    statistic_increment(aborted_connects, &LOCK_status);
+    status_var.aborted_connects++;
 
     /* Can't use my_error() since store_globals has not been called. */
     /* TODO replace will better error message */
@@ -633,14 +577,12 @@ bool Session::schedule()
 }
 
 
-const char* Session::enter_cond(pthread_cond_t *cond,
-                                pthread_mutex_t* mutex,
-                                const char* msg)
+const char* Session::enter_cond(boost::condition_variable &cond, boost::mutex &mutex, const char* msg)
 {
   const char* old_msg = get_proc_info();
   safe_mutex_assert_owner(mutex);
-  mysys_var->current_mutex = mutex;
-  mysys_var->current_cond = cond;
+  mysys_var->current_mutex = &mutex;
+  mysys_var->current_cond = &cond;
   this->set_proc_info(msg);
   return old_msg;
 }
@@ -653,12 +595,11 @@ void Session::exit_cond(const char* old_msg)
     locked (if that would not be the case, you'll get a deadlock if someone
     does a Session::awake() on you).
   */
-  pthread_mutex_unlock(mysys_var->current_mutex);
-  pthread_mutex_lock(&mysys_var->mutex);
+  mysys_var->current_mutex->unlock();
+  boost::mutex::scoped_lock scopedLock(mysys_var->mutex);
   mysys_var->current_mutex = 0;
   mysys_var->current_cond = 0;
   this->set_proc_info(old_msg);
-  pthread_mutex_unlock(&mysys_var->mutex);
 }
 
 bool Session::authenticate()
@@ -667,7 +608,8 @@ bool Session::authenticate()
   if (client->authenticate())
     return false;
 
-  statistic_increment(aborted_connects, &LOCK_status);
+  status_var.aborted_connects++;
+
   return true;
 }
 
@@ -680,6 +622,7 @@ bool Session::checkUser(const char *passwd, uint32_t passwd_len, const char *in_
 
   if (is_authenticated != true)
   {
+    status_var.access_denied++;
     /* isAuthenticated has pushed the error message */
     return false;
   }
@@ -687,7 +630,8 @@ bool Session::checkUser(const char *passwd, uint32_t passwd_len, const char *in_
   /* Change database if necessary */
   if (in_db && in_db[0])
   {
-    if (mysql_change_db(this, in_db))
+    SchemaIdentifier identifier(in_db);
+    if (mysql_change_db(this, identifier))
     {
       /* mysql_change_db() has pushed the error message. */
       return false;
@@ -718,6 +662,9 @@ bool Session::executeStatement()
   if (client->readCommand(&l_packet, &packet_length) == false)
     return false;
 
+  if (killed == KILL_CONNECTION)
+    return false;
+
   if (packet_length == 0)
     return true;
 
@@ -771,7 +718,7 @@ bool Session::endTransaction(enum enum_mysql_completiontype completion)
        * (Which of course should never happen...)
        */
       server_status&= ~SERVER_STATUS_IN_TRANS;
-      if (transaction_services.ha_commit_trans(this, true))
+      if (transaction_services.commitTransaction(this, true))
         result= false;
       options&= ~(OPTION_BEGIN);
       break;
@@ -788,7 +735,7 @@ bool Session::endTransaction(enum enum_mysql_completiontype completion)
     case ROLLBACK_AND_CHAIN:
     {
       server_status&= ~SERVER_STATUS_IN_TRANS;
-      if (transaction_services.ha_rollback_trans(this, true))
+      if (transaction_services.rollbackTransaction(this, true))
         result= false;
       options&= ~(OPTION_BEGIN);
       if (result == true && (completion == ROLLBACK_AND_CHAIN))
@@ -821,7 +768,7 @@ bool Session::endActiveTransaction()
   if (options & (OPTION_NOT_AUTOCOMMIT | OPTION_BEGIN))
   {
     server_status&= ~SERVER_STATUS_IN_TRANS;
-    if (transaction_services.ha_commit_trans(this, true))
+    if (transaction_services.commitTransaction(this, true))
       result= false;
   }
   options&= ~(OPTION_BEGIN);
@@ -872,6 +819,14 @@ void Session::cleanup_after_query()
   free_items();
   /* Reset where. */
   where= Session::DEFAULT_WHERE;
+
+  /* Reset the temporary shares we built */
+  for (std::vector<TableShareInstance *>::iterator iter= temporary_shares.begin();
+       iter != temporary_shares.end(); iter++)
+  {
+    delete *iter;
+  }
+  temporary_shares.clear();
 }
 
 /**
@@ -898,7 +853,7 @@ LEX_STRING *Session::make_lex_string(LEX_STRING *lex_str,
   if (allocate_lex_string)
     if (!(lex_str= (LEX_STRING *)alloc(sizeof(LEX_STRING))))
       return 0;
-  if (!(lex_str->str= strmake_root(mem_root, str, length)))
+  if (!(lex_str->str= mem_root->strmake_root(str, length)))
     return 0;
   lex_str->length= length;
   return lex_str;
@@ -1047,20 +1002,25 @@ static int create_file(Session *session, char *path, file_exchange *exchange, in
 
   if (!internal::dirname_length(exchange->file_name))
   {
-    strcpy(path, drizzle_real_data_home);
+    strcpy(path, getDataHomeCatalog().c_str());
+    strncat(path, "/", 1);
     if (! session->db.empty())
-      strncat(path, session->db.c_str(), FN_REFLEN-strlen(drizzle_real_data_home)-1);
+      strncat(path, session->db.c_str(), FN_REFLEN-getDataHomeCatalog().size());
     (void) internal::fn_format(path, exchange->file_name, path, "", option);
   }
   else
-    (void) internal::fn_format(path, exchange->file_name, drizzle_real_data_home, "", option);
+    (void) internal::fn_format(path, exchange->file_name, getDataHomeCatalog().c_str(), "", option);
 
-  if (opt_secure_file_priv &&
-      strncmp(opt_secure_file_priv, path, strlen(opt_secure_file_priv)))
+  if (opt_secure_file_priv)
   {
-    /* Write only allowed to dir or subdir specified by secure_file_priv */
-    my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--secure-file-priv");
-    return -1;
+    fs::path secure_file_path(fs::system_complete(fs::path(opt_secure_file_priv)));
+    fs::path target_path(fs::system_complete(fs::path(path)));
+    if (target_path.file_string().substr(0, secure_file_path.file_string().size()) != secure_file_path.file_string())
+    {
+      /* Write only allowed to dir or subdir specified by secure_file_priv */
+      my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--secure-file-priv");
+      return -1;
+    }
   }
 
   if (!access(path, F_OK))
@@ -1142,13 +1102,6 @@ select_export::prepare(List<Item> &list, Select_Lex_Unit *u)
   return 0;
 }
 
-
-#define NEED_ESCAPING(x) ((int) (unsigned char) (x) == escape_char    || \
-                          (enclosed ? (int) (unsigned char) (x) == field_sep_char      \
-                                    : (int) (unsigned char) (x) == field_term_char) || \
-                          (int) (unsigned char) (x) == line_sep_char  || \
-                          !(x))
-
 bool select_export::send_data(List<Item> &items)
 {
   char buff[MAX_FIELD_WIDTH],null_buff[2],space[MAX_FIELD_WIDTH];
@@ -1265,11 +1218,11 @@ bool select_export::send_data(List<Item> &items)
             assert before the loop makes that sure.
           */
 
-          if ((NEED_ESCAPING(*pos) ||
+          if ((needs_escaping(*pos, enclosed) ||
                (check_second_byte &&
                 my_mbcharlen(character_set_client, (unsigned char) *pos) == 2 &&
                 pos + 1 < end &&
-                NEED_ESCAPING(pos[1]))) &&
+                needs_escaping(pos[1], enclosed))) &&
               /*
                 Don't escape field_term_char by doubling - doubling is only
                 valid for ENCLOSED BY characters:
@@ -1545,6 +1498,8 @@ void Session::end_statement()
 {
   /* Cleanup SQL processing state to reuse this statement in next query. */
   lex_end(lex);
+  query_cache_key= ""; // reset the cache key
+  resetResultsetMessage();
 }
 
 bool Session::copy_db_to(char **p_db, size_t *p_db_length)
@@ -1595,38 +1550,10 @@ void Session::set_status_var_init()
 }
 
 
-/****************************************************************************
-  Handling of open and locked tables states.
-
-  This is used when we want to open/lock (and then close) some tables when
-  we already have a set of tables open and locked. We use these methods for
-  access to mysql.proc table to find definitions of stored routines.
-****************************************************************************/
-
-void Session::reset_n_backup_open_tables_state(Open_tables_state *backup)
-{
-  backup->set_open_tables_state(this);
-  reset_open_tables_state();
-  backups_available= false;
-}
-
-
-void Session::restore_backup_open_tables_state(Open_tables_state *backup)
-{
-  /*
-    Before we will throw away current open tables state we want
-    to be sure that it was properly cleaned up.
-  */
-  assert(open_tables == 0 && temporary_tables == 0 &&
-              derived_tables == 0 &&
-              lock == 0);
-  set_open_tables_state(backup);
-}
-
-bool Session::set_db(const char *new_db, size_t length)
+bool Session::set_db(const std::string &new_db)
 {
   /* Do not reallocate memory if current chunk is big enough. */
-  if (length)
+  if (new_db.length())
     db= new_db;
   else
     db.clear();
@@ -1643,37 +1570,17 @@ bool Session::set_db(const char *new_db, size_t length)
   @retval 0 the user thread is active
   @retval 1 the user thread has been killed
 */
-extern "C" int session_killed(const Session *session)
+int session_killed(const Session *session)
 {
   return(session->killed);
 }
 
-/**
-  Return the session id of a user session
-  @param pointer to Session object
-  @return session's id
-*/
-extern "C" unsigned long session_get_thread_id(const Session *session)
-{
-  return (unsigned long) session->getSessionId();
-}
-
 
 const struct charset_info_st *session_charset(Session *session)
 {
   return(session->charset());
 }
 
-int session_non_transactional_update(const Session *session)
-{
-  return(session->transaction.all.hasModifiedNonTransData());
-}
-
-void session_mark_transaction_to_rollback(Session *session, bool all)
-{
-  mark_transaction_to_rollback(session, all);
-}
-
 /**
   Mark transaction to rollback and mark error as fatal to a sub-statement.
 
@@ -1696,7 +1603,9 @@ void Session::disconnect(uint32_t errcode, bool should_lock)
 
   /* If necessary, log any aborted or unauthorized connections */
   if (killed || client->wasAborted())
-    statistic_increment(aborted_threads, &LOCK_status);
+  {
+    status_var.aborted_threads++;
+  }
 
   if (client->wasAborted())
   {
@@ -1715,7 +1624,7 @@ void Session::disconnect(uint32_t errcode, bool should_lock)
 
   /* Close out our connection to the client */
   if (should_lock)
-    (void) pthread_mutex_lock(&LOCK_thread_count);
+    LOCK_thread_count.lock();
   killed= Session::KILL_CONNECTION;
   if (client->isConnected())
   {
@@ -1727,7 +1636,7 @@ void Session::disconnect(uint32_t errcode, bool should_lock)
     client->close();
   }
   if (should_lock)
-    (void) pthread_mutex_unlock(&LOCK_thread_count);
+    (void) LOCK_thread_count.unlock();
 }
 
 void Session::reset_for_next_command()
@@ -1760,13 +1669,13 @@ void Session::close_temporary_tables()
   Table *table;
   Table *tmp_next;
 
-  if (!temporary_tables)
+  if (not temporary_tables)
     return;
 
   for (table= temporary_tables; table; table= tmp_next)
   {
-    tmp_next= table->next;
-    close_temporary(table);
+    tmp_next= table->getNext();
+    nukeTable(table);
   }
   temporary_tables= NULL;
 }
@@ -1777,11 +1686,13 @@ void Session::close_temporary_tables()
 
 void Session::close_temporary_table(Table *table)
 {
-  if (table->prev)
+  if (table->getPrev())
   {
-    table->prev->next= table->next;
-    if (table->prev->next)
-      table->next->prev= table->prev;
+    table->getPrev()->setNext(table->getNext());
+    if (table->getPrev()->getNext())
+    {
+      table->getNext()->setPrev(table->getPrev());
+    }
   }
   else
   {
@@ -1792,82 +1703,77 @@ void Session::close_temporary_table(Table *table)
       passing non-zero value to end_slave via rli->save_temporary_tables
       when no temp tables opened, see an invariant below.
     */
-    temporary_tables= table->next;
+    temporary_tables= table->getNext();
     if (temporary_tables)
-      table->next->prev= NULL;
+    {
+      table->getNext()->setPrev(NULL);
+    }
   }
-  close_temporary(table);
+  nukeTable(table);
 }
 
 /*
-  Close and delete a temporary table
+  Close and drop a temporary table
 
   NOTE
   This dosn't unlink table from session->temporary
   If this is needed, use close_temporary_table()
 */
 
-void Session::close_temporary(Table *table)
+void Session::nukeTable(Table *table)
 {
-  plugin::StorageEngine *table_type= table->s->db_type();
+  plugin::StorageEngine *table_type= table->getShare()->db_type();
 
   table->free_io_cache();
-  table->closefrm(false);
+  table->delete_table();
 
-  rm_temporary_table(table_type, table->s->path.str);
+  TableIdentifier identifier(table->getShare()->getSchemaName(), table->getShare()->getTableName(), table->getShare()->getPath());
+  rm_temporary_table(table_type, identifier);
 
-  table->s->free_table_share();
+  delete table->getMutableShare();
 
   /* This makes me sad, but we're allocating it via malloc */
-  free(table);
+  delete table;
 }
 
 /** Clear most status variables. */
 extern time_t flush_status_time;
-extern uint32_t max_used_connections;
 
 void Session::refresh_status()
 {
-  pthread_mutex_lock(&LOCK_status);
-
-  /* Add thread's status variabes to global status */
-  add_to_status(&global_status_var, &status_var);
-
   /* Reset thread's status variables */
   memset(&status_var, 0, sizeof(status_var));
 
-  /* Reset some global variables */
-  reset_status_vars();
-
-  /* Reset the counters of all key caches (default and named). */
-  reset_key_cache_counters();
   flush_status_time= time((time_t*) 0);
-  max_used_connections= 1; /* We set it to one, because we know we exist */
-  pthread_mutex_unlock(&LOCK_status);
+  current_global_counters.max_used_connections= 1; /* We set it to one, because we know we exist */
+  current_global_counters.connections= 0;
 }
 
 user_var_entry *Session::getVariable(LEX_STRING &name, bool create_if_not_exists)
 {
   user_var_entry *entry= NULL;
+  UserVarsRange ppp= user_vars.equal_range(std::string(name.str, name.length));
 
-  entry= (user_var_entry*) hash_search(&user_vars, (unsigned char*) name.str, name.length);
+  for (UserVars::iterator iter= ppp.first;
+         iter != ppp.second; ++iter)
+  {
+    entry= (*iter).second;
+  }
 
   if ((entry == NULL) && create_if_not_exists)
   {
-    if (!hash_inited(&user_vars))
-      return NULL;
     entry= new (nothrow) user_var_entry(name.str, query_id);
 
     if (entry == NULL)
       return NULL;
 
-    if (my_hash_insert(&user_vars, (unsigned char*) entry))
-    {
-      assert(1);
-      free((char*) entry);
-      return 0;
-    }
+    std::pair<UserVars::iterator, bool> returnable= user_vars.insert(make_pair(std::string(name.str, name.length), entry));
 
+    if (not returnable.second)
+    {
+      delete entry;
+      return NULL;
+    }
   }
 
   return entry;
@@ -1875,7 +1781,7 @@ user_var_entry *Session::getVariable(LEX_STRING &name, bool create_if_not_exists
 
 void Session::mark_temp_tables_as_free_for_reuse()
 {
-  for (Table *table= temporary_tables ; table ; table= table->next)
+  for (Table *table= temporary_tables ; table ; table= table->getNext())
   {
     if (table->query_id == query_id)
     {
@@ -1887,7 +1793,7 @@ void Session::mark_temp_tables_as_free_for_reuse()
 
 void Session::mark_used_tables_as_free_for_reuse(Table *table)
 {
-  for (; table ; table= table->next)
+  for (; table ; table= table->getNext())
   {
     if (table->query_id == query_id)
     {
@@ -1908,33 +1814,8 @@ void Session::mark_used_tables_as_free_for_reuse(Table *table)
 */
 void Session::close_thread_tables()
 {
-  Table *table;
-
-  /*
-    We are assuming here that session->derived_tables contains ONLY derived
-    tables for this substatement. i.e. instead of approach which uses
-    query_id matching for determining which of the derived tables belong
-    to this substatement we rely on the ability of substatements to
-    save/restore session->derived_tables during their execution.
-
-    TODO: Probably even better approach is to simply associate list of
-          derived tables with (sub-)statement instead of thread and destroy
-          them at the end of its execution.
-  */
   if (derived_tables)
-  {
-    Table *next;
-    /*
-      Close all derived tables generated in queries like
-      SELECT * FROM (SELECT * FROM t1)
-    */
-    for (table= derived_tables ; table ; table= next)
-    {
-      next= table->next;
-      table->free_tmp_table(this);
-    }
-    derived_tables= 0;
-  }
+    derived_tables= NULL; // They should all be invalid by this point
 
   /*
     Mark all temporary tables used by this statement as free for reuse.
@@ -1948,11 +1829,10 @@ void Session::close_thread_tables()
     does not belong to statement for which we do close_thread_tables()).
     TODO: This should be fixed in later releases.
    */
-  if (backups_available == false)
   {
     TransactionServices &transaction_services= TransactionServices::singleton();
     main_da.can_overwrite_status= true;
-    transaction_services.ha_autocommit_or_rollback(this, is_error());
+    transaction_services.autocommitOrRollback(this, is_error());
     main_da.can_overwrite_status= false;
     transaction.stmt.reset();
   }
@@ -2006,9 +1886,9 @@ bool Session::openTablesLock(TableList *tables)
     if (open_tables_from_list(&tables, &counter))
       return true;
 
-    if (!lock_tables(tables, counter, &need_reopen))
+    if (not lock_tables(tables, counter, &need_reopen))
       break;
-    if (!need_reopen)
+    if (not need_reopen)
       return true;
     close_tables_for_reopen(&tables);
   }
@@ -2027,16 +1907,27 @@ bool Session::openTables(TableList *tables, uint32_t flags)
   assert(ret == false);
   if (open_tables_from_list(&tables, &counter, flags) ||
       mysql_handle_derived(lex, &mysql_derived_prepare))
+  {
     return true;
+  }
   return false;
 }
 
-bool Session::rm_temporary_table(TableIdentifier &identifier)
+/*
+  @note "best_effort" is used in cases were if a failure occurred on this
+  operation it would not be surprising because we are only removing because there
+  might be an issue (lame engines).
+*/
+
+bool Session::rm_temporary_table(TableIdentifier &identifier, bool best_effort)
 {
-  if (not plugin::StorageEngine::dropTable(*this, identifier))
+  if (plugin::StorageEngine::dropTable(*this, identifier))
   {
-    errmsg_printf(ERRMSG_LVL_WARN, _("Could not remove temporary table: '%s', error: %d"),
-                  identifier.getPath(), errno);
+    if (not best_effort)
+    {
+      errmsg_printf(ERRMSG_LVL_WARN, _("Could not remove temporary table: '%s', error: %d"),
+                    identifier.getSQLPath().c_str(), errno);
+    }
 
     return true;
   }
@@ -2044,22 +1935,130 @@ bool Session::rm_temporary_table(TableIdentifier &identifier)
   return false;
 }
 
-bool Session::rm_temporary_table(plugin::StorageEngine *base, const char *path)
+bool Session::rm_temporary_table(plugin::StorageEngine *base, TableIdentifier &identifier)
 {
-  bool error= false;
-
   assert(base);
 
-  if (delete_table_proto_file(path))
-    error= true;
-
-  if (base->doDropTable(*this, path))
+  if (plugin::StorageEngine::dropTable(*this, *base, identifier))
   {
-    error= true;
     errmsg_printf(ERRMSG_LVL_WARN, _("Could not remove temporary table: '%s', error: %d"),
-                  path, errno);
+                  identifier.getSQLPath().c_str(), errno);
+
+    return true;
   }
-  return error;
+
+  return false;
+}
+
+/**
+  @note this will be removed, I am looking through Hudson to see if it is finding
+  any tables that are missed during cleanup.
+*/
+void Session::dumpTemporaryTableNames(const char *foo)
+{
+  Table *table;
+
+  if (not temporary_tables)
+    return;
+
+  cerr << "Begin Run: " << foo << "\n";
+  for (table= temporary_tables; table; table= table->getNext())
+  {
+    bool have_proto= false;
+
+    message::Table *proto= table->getShare()->getTableProto();
+    if (table->getShare()->getTableProto())
+      have_proto= true;
+
+    const char *answer= have_proto ? "true" : "false";
+
+    if (have_proto)
+    {
+      cerr << "\tTable Name " << table->getShare()->getSchemaName() << "." << table->getShare()->getTableName() << " : " << answer << "\n";
+      cerr << "\t\t Proto " << proto->schema() << " " << proto->name() << "\n";
+    }
+    else
+      cerr << "\tTabl;e Name " << table->getShare()->getSchemaName() << "." << table->getShare()->getTableName() << " : " << answer << "\n";
+  }
+}
+
+bool Session::storeTableMessage(const TableIdentifier &identifier, message::Table &table_message)
+{
+  table_message_cache.insert(make_pair(identifier.getPath(), table_message));
+
+  return true;
+}
+
+bool Session::removeTableMessage(const TableIdentifier &identifier)
+{
+  TableMessageCache::iterator iter;
+
+  iter= table_message_cache.find(identifier.getPath());
+
+  if (iter == table_message_cache.end())
+    return false;
+
+  table_message_cache.erase(iter);
+
+  return true;
+}
+
+bool Session::getTableMessage(const TableIdentifier &identifier, message::Table &table_message)
+{
+  TableMessageCache::iterator iter;
+
+  iter= table_message_cache.find(identifier.getPath());
+
+  if (iter == table_message_cache.end())
+    return false;
+
+  table_message.CopyFrom(((*iter).second));
+
+  return true;
+}
+
+bool Session::doesTableMessageExist(const TableIdentifier &identifier)
+{
+  TableMessageCache::iterator iter;
+
+  iter= table_message_cache.find(identifier.getPath());
+
+  if (iter == table_message_cache.end())
+  {
+    return false;
+  }
+
+  return true;
+}
+
+bool Session::renameTableMessage(const TableIdentifier &from, const TableIdentifier &to)
+{
+  TableMessageCache::iterator iter;
+
+  table_message_cache[to.getPath()]= table_message_cache[from.getPath()];
+
+  iter= table_message_cache.find(to.getPath());
+
+  if (iter == table_message_cache.end())
+  {
+    return false;
+  }
+
+  (*iter).second.set_schema(to.getSchemaName());
+  (*iter).second.set_name(to.getTableName());
+
+  return true;
+}
+
+TableShareInstance *Session::getTemporaryShare(TableIdentifier::Type type_arg)
+{
+  temporary_shares.push_back(new TableShareInstance(type_arg)); // This will not go into the tableshare cache, so no key is used.
+
+  TableShareInstance *tmp_share= temporary_shares.back();
+
+  assert(tmp_share);
+
+  return tmp_share;
 }
 
 } /* namespace drizzled */
