@@ -31,10 +31,14 @@
 #include <drizzled/sql_error.h>
 #include <drizzled/file_exchange.h>
 #include <drizzled/select_result_interceptor.h>
+#include <drizzled/statistics_variables.h>
 #include <drizzled/xid.h>
 #include "drizzled/query_id.h"
 #include "drizzled/named_savepoint.h"
 #include "drizzled/transaction_context.h"
+#include "drizzled/util/storable.h"
+
+#include "drizzled/my_hash.h"
 
 #include <netdb.h>
 #include <map>
@@ -42,6 +46,8 @@
 #include <bitset>
 #include <deque>
 
+#include "drizzled/internal/getrusage.h"
+
 #include <drizzled/security_context.h>
 #include <drizzled/open_tables_state.h>
 
@@ -50,6 +56,10 @@
 
 #include <drizzled/plugin/authorization.h>
 
+#include <boost/unordered_map.hpp>
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/condition_variable.hpp>
+
 #define MIN_HANDSHAKE_SIZE      6
 
 namespace drizzled
@@ -59,11 +69,14 @@ namespace plugin
 {
 class Client;
 class Scheduler;
+class EventObserverList;
 }
+
 namespace message
 {
 class Transaction;
 class Statement;
+class Resultset;
 }
 namespace internal
 {
@@ -75,6 +88,8 @@ class user_var_entry;
 class CopyField;
 class Table_ident;
 
+class TableShareInstance;
+
 extern char internal_table_name[2];
 extern char empty_c_string[1];
 extern const char **errmesg;
@@ -103,7 +118,7 @@ typedef std::map <std::string, message::Table> ProtoCache;
       of the INSERT ... ON DUPLICATE KEY UPDATE no matter whether the row
       was actually changed or not.
 */
-typedef struct st_copy_info 
+struct CopyInfo 
 {
   ha_rows records; /**< Number of processed records */
   ha_rows deleted; /**< Number of deleted records */
@@ -118,15 +133,38 @@ typedef struct st_copy_info
   List<Item> *update_fields;
   List<Item> *update_values;
   /* for VIEW ... WITH CHECK OPTION */
-} COPY_INFO;
 
-typedef struct drizzled_lock_st
+  CopyInfo() :
+    records(0),
+    deleted(0),
+    updated(0),
+    copied(0),
+    error_count(0),
+    touched(0),
+    escape_char(0),
+    last_errno(0),
+    ignore(0),
+    update_fields(0),
+    update_values(0)
+  { }
+
+};
+
+struct DrizzleLock
 {
   Table **table;
   uint32_t table_count;
   uint32_t lock_count;
   THR_LOCK_DATA **locks;
-} DRIZZLE_LOCK;
+
+  DrizzleLock() :
+    table(0),
+    table_count(0),
+    lock_count(0),
+    locks(0)
+  { }
+
+};
 
 } /* namespace drizzled */
 
@@ -142,8 +180,6 @@ class Time_zone;
 #define Session_SENTRY_MAGIC 0xfeedd1ff
 #define Session_SENTRY_GONE  0xdeadbeef
 
-#define Session_CHECK_SENTRY(session) assert(session->dbug_sentry == Session_SENTRY_MAGIC)
-
 struct system_variables
 {
   system_variables() {};
@@ -224,82 +260,8 @@ extern struct system_variables global_system_variables;
 namespace drizzled
 {
 
-/**
- * Per-session local status counters
- */
-typedef struct system_status_var
-{
-  uint64_t bytes_received;
-  uint64_t bytes_sent;
-  ulong com_other;
-  ulong com_stat[(uint32_t) SQLCOM_END];
-  ulong created_tmp_disk_tables;
-  ulong created_tmp_tables;
-  ulong ha_commit_count;
-  ulong ha_delete_count;
-  ulong ha_read_first_count;
-  ulong ha_read_last_count;
-  ulong ha_read_key_count;
-  ulong ha_read_next_count;
-  ulong ha_read_prev_count;
-  ulong ha_read_rnd_count;
-  ulong ha_read_rnd_next_count;
-  ulong ha_rollback_count;
-  ulong ha_update_count;
-  ulong ha_write_count;
-  ulong ha_prepare_count;
-  ulong ha_savepoint_count;
-  ulong ha_savepoint_rollback_count;
-
-  /* KEY_CACHE parts. These are copies of the original */
-  ulong key_blocks_changed;
-  ulong key_blocks_used;
-  ulong key_cache_r_requests;
-  ulong key_cache_read;
-  ulong key_cache_w_requests;
-  ulong key_cache_write;
-  /* END OF KEY_CACHE parts */
-
-  ulong net_big_packet_count;
-  ulong select_full_join_count;
-  ulong select_full_range_join_count;
-  ulong select_range_count;
-  ulong select_range_check_count;
-  ulong select_scan_count;
-  ulong long_query_count;
-  ulong filesort_merge_passes;
-  ulong filesort_range_count;
-  ulong filesort_rows;
-  ulong filesort_scan_count;
-  /*
-    Number of statements sent from the client
-  */
-  ulong questions;
-
-  /*
-    IMPORTANT!
-    SEE last_system_status_var DEFINITION BELOW.
-
-    Below 'last_system_status_var' are all variables which doesn't make any
-    sense to add to the /global/ status variable counter.
-  */
-  double last_query_cost;
-} system_status_var;
-
-/*
-  This is used for 'SHOW STATUS'. It must be updated to the last ulong
-  variable in system_status_var which is makes sens to add to the global
-  counter
-*/
-
-#define last_system_status_var questions
-
 void mark_transaction_to_rollback(Session *session, bool all);
 
-extern pthread_mutex_t LOCK_xid_cache;
-extern HASH xid_cache;
-
-
 /**
   Storage engine specific thread local data.
 */
@@ -355,6 +317,9 @@ struct Ha_data
 class Session : public Open_tables_state
 {
 public:
+  // Plugin storage in Session.
+  typedef boost::unordered_map<std::string, util::Storable *, util::insensitive_hash, util::insensitive_equal_to> PropertyMap;
+
   /*
     MARK_COLUMNS_NONE:  Means mark_used_colums is not set and no indicator to
                         handler of fields used is set
@@ -369,31 +334,31 @@ public:
   enum enum_mark_columns mark_used_columns;
   inline void* alloc(size_t size)
   {
-    return alloc_root(mem_root,size);
+    return mem_root->alloc_root(size);
   }
   inline void* calloc(size_t size)
   {
     void *ptr;
-    if ((ptr= alloc_root(mem_root,size)))
+    if ((ptr= mem_root->alloc_root(size)))
       memset(ptr, 0, size);
     return ptr;
   }
   inline char *strdup(const char *str)
   {
-    return strdup_root(mem_root,str);
+    return mem_root->strdup_root(str);
   }
   inline char *strmake(const char *str, size_t size)
   {
-    return strmake_root(mem_root,str,size);
+    return mem_root->strmake_root(str,size);
   }
   inline void *memdup(const void *str, size_t size)
   {
-    return memdup_root(mem_root,str,size);
+    return mem_root->memdup_root(str, size);
   }
   inline void *memdup_w_gap(const void *str, size_t size, uint32_t gap)
   {
     void *ptr;
-    if ((ptr= alloc_root(mem_root,size+gap)))
+    if ((ptr= mem_root->alloc_root(size + gap)))
       memcpy(ptr,str,size);
     return ptr;
   }
@@ -405,6 +370,12 @@ public:
    */
   Item *free_list;
   memory::Root *mem_root; /**< Pointer to current memroot */
+
+
+  memory::Root *getMemRoot()
+  {
+    return mem_root;
+  }
   /**
    * Uniquely identifies each statement object in thread scope; change during
    * statement lifetime.
@@ -413,6 +384,11 @@ public:
    */
   uint32_t id;
   LEX *lex; /**< parse tree descriptor */
+
+  LEX *getLex() 
+  {
+    return lex;
+  }
   /** query associated with this statement */
   std::string query;
 
@@ -429,7 +405,8 @@ public:
     only responsible for freeing this member.
   */
   std::string db;
-
+  /* current cache key */
+  std::string query_cache_key;
   /**
     Constant for Session::where initialization in the beginning of every query.
 
@@ -442,14 +419,30 @@ public:
   plugin::Client *client; /**< Pointer to client object */
   plugin::Scheduler *scheduler; /**< Pointer to scheduler object */
   void *scheduler_arg; /**< Pointer to the optional scheduler argument */
-  HASH user_vars; /**< Hash of user variables defined during the session's lifetime */
+private:
+  typedef boost::unordered_map< std::string, user_var_entry *, util::insensitive_hash, util::insensitive_equal_to> UserVars;
+  typedef std::pair< UserVars::iterator, UserVars::iterator > UserVarsRange;
+  UserVars user_vars; /**< Hash of user variables defined during the session's lifetime */
+
+public:
   struct system_variables variables; /**< Mutable local variables local to the session */
   struct system_status_var status_var; /**< Session-local status counters */
-  struct system_status_var *initial_status_var; /* used by show status */
   THR_LOCK_INFO lock_info; /**< Locking information for this session */
   THR_LOCK_OWNER main_lock_id; /**< To use for conventional queries */
   THR_LOCK_OWNER *lock_id; /**< If not main_lock_id, points to the lock_id of a cursor. */
-  pthread_mutex_t LOCK_delete; /**< Locked before session is deleted */
+private:
+  boost::mutex LOCK_delete; /**< Locked before session is deleted */
+public:
+
+  void lockForDelete()
+  {
+    LOCK_delete.lock();
+  }
+
+  void unlockForDelete()
+  {
+    LOCK_delete.unlock();
+  }
 
   /**
    * A peek into the query string for the session. This is a best effort
@@ -465,6 +458,13 @@ public:
 
 private:
   SecurityContext security_ctx;
+
+  int32_t scoreboard_index;
+
+  inline void checkSentry() const
+  {
+    assert(this->dbug_sentry == Session_SENTRY_MAGIC);
+  }
 public:
   const SecurityContext& getSecurityContext() const
   {
@@ -476,6 +476,16 @@ public:
     return security_ctx;
   }
 
+  int32_t getScoreboardIndex()
+  {
+    return scoreboard_index;
+  }
+
+  void setScoreboardIndex(int32_t in_scoreboard_index)
+  {
+    scoreboard_index= in_scoreboard_index;
+  }
+
   /**
    * Is this session viewable by the current user?
    */
@@ -499,7 +509,19 @@ public:
     chapter 'Miscellaneous functions', for functions GET_LOCK, RELEASE_LOCK.
   */
   uint32_t dbug_sentry; /**< watch for memory corruption */
+private:
   internal::st_my_thread_var *mysys_var;
+public:
+
+  internal::st_my_thread_var *getThreadVar()
+  {
+    return mysys_var;
+  }
+
+  void resetThreadVar()
+  {
+    mysys_var= NULL;
+  }
   /**
    * Type of current query: COM_STMT_PREPARE, COM_QUERY, etc. Set from
    * first byte of the packet in executeStatement()
@@ -662,7 +684,6 @@ public:
     create_sort_index(); may differ from examined_row_count.
   */
   uint32_t row_count;
-  pthread_t real_id; /**< For debugging */
   uint64_t thread_id;
   uint32_t tmp_table;
   uint32_t global_read_lock;
@@ -986,7 +1007,7 @@ public:
     enter_cond(); this mutex is then released by exit_cond().
     Usage must be: lock mutex; enter_cond(); your code; exit_cond().
   */
-  const char* enter_cond(pthread_cond_t *cond, pthread_mutex_t* mutex, const char* msg);
+  const char* enter_cond(boost::condition_variable &cond, boost::mutex &mutex, const char* msg);
   void exit_cond(const char* old_msg);
 
   inline time_t query_start() { return start_time; }
@@ -1100,9 +1121,10 @@ public:
   {
     return (abort_on_warning);
   }
+
+  void setAbort(bool arg);
+  void lockOnSys();
   void set_status_var_init();
-  void reset_n_backup_open_tables_state(Open_tables_state *backup);
-  void restore_backup_open_tables_state(Open_tables_state *backup);
 
   /**
     Set the current database; use deep copy of C-string.
@@ -1125,7 +1147,7 @@ public:
       @retval false Success
       @retval true  Out-of-memory error
   */
-  bool set_db(const char *new_db, size_t new_db_len);
+  bool set_db(const std::string &new_db);
 
   /*
     Copy the current database to the argument. Use the current arena to
@@ -1223,7 +1245,15 @@ public:
   {
     return statement_message;
   }
-
+  
+  /**
+   * Returns a pointer to the current Resulset message for this
+   * Session, or NULL if no active message.
+   */
+  message::Resultset *getResultsetMessage() const
+  {
+    return resultset;
+  }
   /**
    * Sets the active transaction message used by the ReplicationServices
    * component.
@@ -1245,11 +1275,77 @@ public:
   {
     statement_message= in_message;
   }
+
+  /**
+   * Sets the active Resultset message used by the Query Cache
+   * plugin.
+   *
+   * @param[in] Pointer to the message
+   */
+  void setResultsetMessage(message::Resultset *in_message)
+  {
+    resultset= in_message;
+  }
+  /**
+   * reset the active Resultset message used by the Query Cache
+   * plugin.
+   */
+
+  void resetResultsetMessage()
+  { 
+    resultset= NULL;
+  }
+
 private:
   /** Pointers to memory managed by the ReplicationServices component */
   message::Transaction *transaction_message;
   message::Statement *statement_message;
-  /** Microsecond timestamp of when Session connected */
+  /* Pointer to the current resultset of Select query */
+  message::Resultset *resultset;
+  plugin::EventObserverList *session_event_observers;
+  
+  /* Schema observers are mapped to databases. */
+  std::map<std::string, plugin::EventObserverList *> schema_event_observers;
+
+ 
+public:
+  plugin::EventObserverList *getSessionObservers() 
+  { 
+    return session_event_observers;
+  }
+  
+  void setSessionObservers(plugin::EventObserverList *observers) 
+  { 
+    session_event_observers= observers;
+  }
+  
+  /* For schema event observers there is one set of observers per database. */
+  plugin::EventObserverList *getSchemaObservers(const std::string &db_name) 
+  { 
+    std::map<std::string, plugin::EventObserverList *>::iterator it;
+    
+    it= schema_event_observers.find(db_name);
+    if (it == schema_event_observers.end())
+      return NULL;
+      
+    return it->second;
+  }
+  
+  void setSchemaObservers(const std::string &db_name, plugin::EventObserverList *observers) 
+  { 
+    std::map<std::string, plugin::EventObserverList *>::iterator it;
+
+    it= schema_event_observers.find(db_name);
+    if (it != schema_event_observers.end())
+      schema_event_observers.erase(it);;
+
+    if (observers)
+      schema_event_observers[db_name] = observers;
+  }
+  
+  
+ private:
+ /** Microsecond timestamp of when Session connected */
   uint64_t connect_microseconds;
   const char *proc_info;
 
@@ -1351,7 +1447,7 @@ public:
   void close_old_data_files(bool morph_locks= false,
                             bool send_refresh= false);
   void close_open_tables();
-  void close_data_files_and_morph_locks(const char *db, const char *table_name);
+  void close_data_files_and_morph_locks(TableIdentifier &identifier);
 
 private:
   bool free_cached_table();
@@ -1408,38 +1504,55 @@ public:
   Table *openTable(TableList *table_list, bool *refresh, uint32_t flags= 0);
 
   void unlink_open_table(Table *find);
-  void drop_open_table(Table *table, const char *db_name,
-                       const char *table_name);
+  void drop_open_table(Table *table, TableIdentifier &identifier);
   void close_cached_table(Table *table);
 
   /* Create a lock in the cache */
-  Table *table_cache_insert_placeholder(const char *key, uint32_t key_length);
-  bool lock_table_name_if_not_cached(const char *db,
-                                     const char *table_name, Table **table);
+  Table *table_cache_insert_placeholder(const char *db_name, const char *table_name);
+  bool lock_table_name_if_not_cached(TableIdentifier &identifier, Table **table);
+
+  typedef boost::unordered_map<std::string, message::Table, util::insensitive_hash, util::insensitive_equal_to> TableMessageCache;
+  TableMessageCache table_message_cache;
+
+  bool storeTableMessage(const TableIdentifier &identifier, message::Table &table_message);
+  bool removeTableMessage(const TableIdentifier &identifier);
+  bool getTableMessage(const TableIdentifier &identifier, message::Table &table_message);
+  bool doesTableMessageExist(const TableIdentifier &identifier);
+  bool renameTableMessage(const TableIdentifier &from, const TableIdentifier &to);
 
   /* Work with temporary tables */
   Table *find_temporary_table(TableList *table_list);
   Table *find_temporary_table(const char *db, const char *table_name);
+  Table *find_temporary_table(TableIdentifier &identifier);
+
   void doGetTableNames(CachedDirectory &directory,
-                       const std::string& db_name,
+                       const SchemaIdentifier &schema_identifier,
                        std::set<std::string>& set_of_names);
-  int doGetTableDefinition(const char *path,
-                           const char *db,
-                           const char *table_name,
-                           const bool is_tmp,
-                           message::Table *table_proto);
+  void doGetTableNames(const SchemaIdentifier &schema_identifier,
+                       std::set<std::string>& set_of_names);
+
+  void doGetTableIdentifiers(CachedDirectory &directory,
+                             const SchemaIdentifier &schema_identifier,
+                             TableIdentifiers &set_of_identifiers);
+  void doGetTableIdentifiers(const SchemaIdentifier &schema_identifier,
+                             TableIdentifiers &set_of_identifiers);
+
+  int doGetTableDefinition(const drizzled::TableIdentifier &identifier,
+                           message::Table &table_proto);
+  bool doDoesTableExist(const drizzled::TableIdentifier &identifier);
 
   void close_temporary_tables();
   void close_temporary_table(Table *table);
   // The method below just handles the de-allocation of the table. In
   // a better memory type world, this would not be needed.
 private:
-  void close_temporary(Table *table);
+  void nukeTable(Table *table);
 public:
 
+  void dumpTemporaryTableNames(const char *id);
   int drop_temporary_table(TableList *table_list);
-  bool rm_temporary_table(plugin::StorageEngine *base, const char *path);
-  bool rm_temporary_table(TableIdentifier &identifier);
+  bool rm_temporary_table(plugin::StorageEngine *base, TableIdentifier &identifier);
+  bool rm_temporary_table(TableIdentifier &identifier, bool best_effort= false);
   Table *open_temporary_table(TableIdentifier &identifier,
                               bool link_in_list= true);
 
@@ -1448,10 +1561,24 @@ public:
   bool reopen_name_locked_table(TableList* table_list, bool link_in);
   bool close_cached_tables(TableList *tables, bool wait_for_refresh, bool wait_for_placeholders);
 
-  void wait_for_condition(pthread_mutex_t *mutex, pthread_cond_t *cond);
+  void wait_for_condition(boost::mutex &mutex, boost::condition_variable &cond);
   int setup_conds(TableList *leaves, COND **conds);
   int lock_tables(TableList *tables, uint32_t count, bool *need_reopen);
 
+  Table *create_virtual_tmp_table(List<CreateField> &field_list);
+  
+  drizzled::util::Storable *getProperty(const std::string &arg)
+  {
+    return life_properties[arg];
+  }
+
+  template<class T>
+  bool setProperty(const std::string &arg, T *value)
+  {
+    life_properties[arg]= value;
+
+    return true;
+  }
 
   /**
     Return the default storage engine
@@ -1470,9 +1597,41 @@ public:
 
   static void unlink(Session *session);
 
+  void get_xid(DRIZZLE_XID *xid); // Innodb only
+
+  TableShareInstance *getTemporaryShare(TableIdentifier::Type type_arg);
+
+private:
+  bool resetUsage()
+  {
+    if (getrusage(RUSAGE_THREAD, &usage))
+    {
+      return false;
+    }
+
+    return true;
+  }
+public:
+
+  void setUsage(bool arg)
+  {
+    use_usage= arg;
+  }
+
+  const struct rusage &getUsage()
+  {
+    return usage;
+  }
+
+private:
+  // This lives throughout the life of Session
+  bool use_usage;
+  PropertyMap life_properties;
+  std::vector<TableShareInstance *> temporary_shares;
+  struct rusage usage;
 };
 
-class JOIN;
+class Join;
 
 #define ESCAPE_CHARS "ntrb0ZN" // keep synchronous with READ_INFO::unescape
 
@@ -1498,7 +1657,7 @@ namespace drizzled
  * A structure used to describe sort information
  * for a field or item used in ORDER BY.
  */
-typedef struct st_sort_field 
+struct SortField 
 {
   Field *field;	/**< Field to sort */
   Item	*item; /**< Item if not sorting fields */
@@ -1507,16 +1666,18 @@ typedef struct st_sort_field
   Item_result result_type; /**< Type of item */
   bool reverse; /**< if descending sort */
   bool need_strxnfrm;	/**< If we have to use strxnfrm() */
-} SORT_FIELD;
 
-typedef struct st_sort_buffer 
-{
-  uint32_t index;	/* 0 or 1 */
-  uint32_t sort_orders;
-  uint32_t change_pos; /* If sort-fields changed */
-  char **buff;
-  SORT_FIELD *sortorder;
-} SORT_BUFFER;
+  SortField() :
+    field(0),
+    item(0),
+    length(0),
+    suffix_length(0),
+    result_type(STRING_RESULT),
+    reverse(0),
+    need_strxnfrm(0)
+  { }
+
+};
 
 } /* namespace drizzled */
 
@@ -1525,7 +1686,7 @@ typedef struct st_sort_buffer
 #include <drizzled/table_ident.h>
 #include <drizzled/user_var_entry.h>
 #include <drizzled/unique.h>
-#include <drizzled/my_var.h>
+#include <drizzled/var.h>
 #include <drizzled/select_dumpvar.h>
 
 namespace drizzled
@@ -1549,12 +1710,6 @@ static const std::bitset<CF_BIT_SIZE> CF_STATUS_COMMAND(1 << CF_BIT_STATUS_COMMA
 static const std::bitset<CF_BIT_SIZE> CF_SHOW_TABLE_COMMAND(1 << CF_BIT_SHOW_TABLE_COMMAND);
 static const std::bitset<CF_BIT_SIZE> CF_WRITE_LOGS_COMMAND(1 << CF_BIT_WRITE_LOGS_COMMAND);
 
-/* Functions in sql_class.cc */
-void add_to_status(system_status_var *to_var, system_status_var *from_var);
-
-void add_diff_to_status(system_status_var *to_var, system_status_var *from_var,
-                        system_status_var *dec_var);
-
 } /* namespace drizzled */
 
 #endif /* DRIZZLED_SESSION_H */
