@@ -29,7 +29,7 @@
   - If the variable is thread specific, add it to 'system_variables' struct.
     If not, add it to mysqld.cc and an declaration in 'mysql_priv.h'
   - If the variable should be changed from the command line, add a definition
-    of it in the my_option structure list in mysqld.cc
+    of it in the option structure list in mysqld.cc
   - Don't forget to initialize new fields in global_system_variables and
     max_system_variables!
 
@@ -44,7 +44,7 @@
 */
 
 #include "config.h"
-#include "drizzled/my_getopt.h"
+#include "drizzled/option.h"
 #include <drizzled/error.h>
 #include <drizzled/gettext.h>
 #include <drizzled/tztime.h>
@@ -64,6 +64,7 @@
 #include "drizzled/charset.h"
 #include "drizzled/transaction_services.h"
 
+#include <cstdio>
 #include <map>
 #include <algorithm>
 
@@ -80,7 +81,7 @@ extern bool timed_mutexes;
 extern plugin::StorageEngine *myisam_engine;
 extern bool timed_mutexes;
 
-extern struct my_option my_long_options[];
+extern struct option my_long_options[];
 extern const CHARSET_INFO *character_set_filesystem;
 extern size_t my_thread_stack_size;
 
@@ -143,7 +144,7 @@ static sys_var_session_uint32_t	sys_completion_type(&vars, "completion_type",
                                                     fix_completion_type);
 static sys_var_collation_sv
 sys_collation_server(&vars, "collation_server", &system_variables::collation_server, &default_charset_info);
-static sys_var_const_str       sys_datadir(&vars, "datadir", drizzle_real_data_home);
+static sys_var_const_str_ptr       sys_datadir(&vars, "datadir", getDatadirPtr());
 
 static sys_var_session_uint64_t	sys_join_buffer_size(&vars, "join_buffer_size",
                                                      &system_variables::join_buff_size);
@@ -199,6 +200,10 @@ static sys_var_readonly sys_tmpdir(&vars, "tmpdir", OPT_GLOBAL, SHOW_CHAR, get_t
 
 static sys_var_const_str_ptr sys_secure_file_priv(&vars, "secure_file_priv",
                                              &opt_secure_file_priv);
+
+static sys_var_const_str_ptr sys_scheduler(&vars, "scheduler",
+                                           (char**)&opt_scheduler);
+
 static sys_var_uint32_t_ptr  sys_server_id(&vars, "server_id", &server_id,
                                            fix_server_id);
 
@@ -399,9 +404,8 @@ static int check_completion_type(Session *, set_var *var)
 static void fix_session_mem_root(Session *session, sql_var_t type)
 {
   if (type != OPT_GLOBAL)
-    reset_root_defaults(session->mem_root,
-                        session->variables.query_alloc_block_size,
-                        session->variables.query_prealloc_size);
+    session->mem_root->reset_root_defaults(session->variables.query_alloc_block_size,
+                                           session->variables.query_prealloc_size);
 }
 
 
@@ -430,7 +434,7 @@ bool throw_bounds_warning(Session *session, bool fixed, bool unsignd,
 }
 
 uint64_t fix_unsigned(Session *session, uint64_t num,
-                              const struct my_option *option_limits)
+                              const struct option *option_limits)
 {
   bool fixed= false;
   uint64_t out= getopt_ull_limit_value(num, option_limits, &fixed);
@@ -441,7 +445,7 @@ uint64_t fix_unsigned(Session *session, uint64_t num,
 
 
 static size_t fix_size_t(Session *session, size_t num,
-                           const struct my_option *option_limits)
+                           const struct option *option_limits)
 {
   bool fixed= false;
   size_t out= (size_t)getopt_ull_limit_value(num, option_limits, &fixed);
@@ -500,7 +504,7 @@ bool sys_var_uint32_t_ptr::check(Session *, set_var *var)
 bool sys_var_uint32_t_ptr::update(Session *session, set_var *var)
 {
   uint32_t tmp= var->save_result.uint32_t_value;
-  pthread_mutex_lock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.lock();
   if (option_limits)
   {
     uint32_t newvalue= (uint32_t) fix_unsigned(session, tmp, option_limits);
@@ -509,7 +513,7 @@ bool sys_var_uint32_t_ptr::update(Session *session, set_var *var)
   }
   else
     *value= (uint32_t) tmp;
-  pthread_mutex_unlock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.unlock();
   return 0;
 }
 
@@ -517,17 +521,17 @@ bool sys_var_uint32_t_ptr::update(Session *session, set_var *var)
 void sys_var_uint32_t_ptr::set_default(Session *, sql_var_t)
 {
   bool not_used;
-  pthread_mutex_lock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.lock();
   *value= (uint32_t)getopt_ull_limit_value((uint32_t) option_limits->def_value,
                                            option_limits, &not_used);
-  pthread_mutex_unlock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.unlock();
 }
 
 
 bool sys_var_uint64_t_ptr::update(Session *session, set_var *var)
 {
   uint64_t tmp= var->save_result.uint64_t_value;
-  pthread_mutex_lock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.lock();
   if (option_limits)
   {
     uint64_t newvalue= (uint64_t) fix_unsigned(session, tmp, option_limits);
@@ -536,7 +540,7 @@ bool sys_var_uint64_t_ptr::update(Session *session, set_var *var)
   }
   else
     *value= (uint64_t) tmp;
-  pthread_mutex_unlock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.unlock();
   return 0;
 }
 
@@ -544,22 +548,22 @@ bool sys_var_uint64_t_ptr::update(Session *session, set_var *var)
 void sys_var_uint64_t_ptr::set_default(Session *, sql_var_t)
 {
   bool not_used;
-  pthread_mutex_lock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.lock();
   *value= getopt_ull_limit_value((uint64_t) option_limits->def_value,
                                  option_limits, &not_used);
-  pthread_mutex_unlock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.unlock();
 }
 
 
 bool sys_var_size_t_ptr::update(Session *session, set_var *var)
 {
   size_t tmp= var->save_result.size_t_value;
-  pthread_mutex_lock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.lock();
   if (option_limits)
     *value= fix_size_t(session, tmp, option_limits);
   else
     *value= tmp;
-  pthread_mutex_unlock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.unlock();
   return 0;
 }
 
@@ -567,10 +571,10 @@ bool sys_var_size_t_ptr::update(Session *session, set_var *var)
 void sys_var_size_t_ptr::set_default(Session *, sql_var_t)
 {
   bool not_used;
-  pthread_mutex_lock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.lock();
   *value= (size_t)getopt_ull_limit_value((size_t) option_limits->def_value,
                                          option_limits, &not_used);
-  pthread_mutex_unlock(&LOCK_global_system_variables);
+  LOCK_global_system_variables.unlock();
 }
 
 bool sys_var_bool_ptr::update(Session *, set_var *var)
@@ -661,9 +665,9 @@ bool sys_var_session_ha_rows::update(Session *session, set_var *var)
   if (var->type == OPT_GLOBAL)
   {
     /* Lock is needed to make things safe on 32 bit systems */
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     global_system_variables.*offset= (ha_rows) tmp;
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.*offset= (ha_rows) tmp;
@@ -677,11 +681,11 @@ void sys_var_session_ha_rows::set_default(Session *session, sql_var_t type)
   {
     bool not_used;
     /* We will not come here if option_limits is not set */
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     global_system_variables.*offset=
       (ha_rows) getopt_ull_limit_value((ha_rows) option_limits->def_value,
                                        option_limits, &not_used);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.*offset= global_system_variables.*offset;
@@ -718,9 +722,9 @@ bool sys_var_session_uint64_t::update(Session *session,  set_var *var)
   if (var->type == OPT_GLOBAL)
   {
     /* Lock is needed to make things safe on 32 bit systems */
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     global_system_variables.*offset= (uint64_t) tmp;
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.*offset= (uint64_t) tmp;
@@ -733,11 +737,11 @@ void sys_var_session_uint64_t::set_default(Session *session, sql_var_t type)
   if (type == OPT_GLOBAL)
   {
     bool not_used;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     global_system_variables.*offset=
       getopt_ull_limit_value((uint64_t) option_limits->def_value,
                              option_limits, &not_used);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.*offset= global_system_variables.*offset;
@@ -771,9 +775,9 @@ bool sys_var_session_size_t::update(Session *session,  set_var *var)
   if (var->type == OPT_GLOBAL)
   {
     /* Lock is needed to make things safe on 32 bit systems */
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     global_system_variables.*offset= tmp;
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.*offset= tmp;
@@ -786,11 +790,11 @@ void sys_var_session_size_t::set_default(Session *session, sql_var_t type)
   if (type == OPT_GLOBAL)
   {
     bool not_used;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     global_system_variables.*offset=
       (size_t)getopt_ull_limit_value((size_t) option_limits->def_value,
                                      option_limits, &not_used);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.*offset= global_system_variables.*offset;
@@ -900,56 +904,56 @@ Item *sys_var::item(Session *session, sql_var_t var_type, const LEX_STRING *base
   case SHOW_INT:
   {
     uint32_t value;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     value= *(uint*) value_ptr(session, var_type, base);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
     return new Item_uint((uint64_t) value);
   }
   case SHOW_LONGLONG:
   {
     int64_t value;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     value= *(int64_t*) value_ptr(session, var_type, base);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
     return new Item_int(value);
   }
   case SHOW_DOUBLE:
   {
     double value;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     value= *(double*) value_ptr(session, var_type, base);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
     /* 6, as this is for now only used with microseconds */
     return new Item_float(value, 6);
   }
   case SHOW_HA_ROWS:
   {
     ha_rows value;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     value= *(ha_rows*) value_ptr(session, var_type, base);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
     return new Item_int((uint64_t) value);
   }
   case SHOW_SIZE:
   {
     size_t value;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     value= *(size_t*) value_ptr(session, var_type, base);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
     return new Item_int((uint64_t) value);
   }
   case SHOW_MY_BOOL:
   {
     int32_t value;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     value= *(bool*) value_ptr(session, var_type, base);
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
     return new Item_int(value,1);
   }
   case SHOW_CHAR_PTR:
   {
     Item *tmp;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     char *str= *(char**) value_ptr(session, var_type, base);
     if (str)
     {
@@ -962,13 +966,13 @@ Item *sys_var::item(Session *session, sql_var_t var_type, const LEX_STRING *base
       tmp= new Item_null();
       tmp->collation.set(system_charset_info, DERIVATION_SYSCONST);
     }
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
     return tmp;
   }
   case SHOW_CHAR:
   {
     Item *tmp;
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     char *str= (char*) value_ptr(session, var_type, base);
     if (str)
       tmp= new Item_string(str, strlen(str),
@@ -978,7 +982,7 @@ Item *sys_var::item(Session *session, sql_var_t var_type, const LEX_STRING *base
       tmp= new Item_null();
       tmp->collation.set(system_charset_info, DERIVATION_SYSCONST);
     }
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
     return tmp;
   }
   default:
@@ -1181,9 +1185,9 @@ bool sys_var_session_time_zone::update(Session *session, set_var *var)
   /* We are using Time_zone object found during check() phase. */
   if (var->type == OPT_GLOBAL)
   {
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     global_system_variables.time_zone= var->save_result.time_zone;
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.time_zone= var->save_result.time_zone;
@@ -1218,7 +1222,7 @@ unsigned char *sys_var_session_time_zone::value_ptr(Session *session,
 
 void sys_var_session_time_zone::set_default(Session *session, sql_var_t type)
 {
- pthread_mutex_lock(&LOCK_global_system_variables);
+ LOCK_global_system_variables.lock();
  if (type == OPT_GLOBAL)
  {
    if (default_tz_name)
@@ -1235,7 +1239,7 @@ void sys_var_session_time_zone::set_default(Session *session, sql_var_t type)
  }
  else
    session->variables.time_zone= global_system_variables.time_zone;
- pthread_mutex_unlock(&LOCK_global_system_variables);
+ LOCK_global_system_variables.unlock();
 }
 
 
@@ -1324,9 +1328,9 @@ bool sys_var_microseconds::update(Session *session, set_var *var)
   microseconds= (int64_t) (num * 1000000.0 + 0.5);
   if (var->type == OPT_GLOBAL)
   {
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     (global_system_variables.*offset)= microseconds;
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.*offset= microseconds;
@@ -1339,9 +1343,9 @@ void sys_var_microseconds::set_default(Session *session, sql_var_t type)
   int64_t microseconds= (int64_t) (option_limits->def_value * 1000000.0);
   if (type == OPT_GLOBAL)
   {
-    pthread_mutex_lock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.lock();
     global_system_variables.*offset= microseconds;
-    pthread_mutex_unlock(&LOCK_global_system_variables);
+    LOCK_global_system_variables.unlock();
   }
   else
     session->variables.*offset= microseconds;
@@ -1381,7 +1385,7 @@ static bool set_option_autocommit(Session *session, set_var *var)
       session->options&= ~(uint64_t) (OPTION_BEGIN);
       session->server_status|= SERVER_STATUS_AUTOCOMMIT;
       TransactionServices &transaction_services= TransactionServices::singleton();
-      if (transaction_services.ha_commit_trans(session, true))
+      if (transaction_services.commitTransaction(session, true))
         return 1;
     }
     else
@@ -1431,8 +1435,8 @@ static unsigned char *get_error_count(Session *session)
 */
 static unsigned char *get_tmpdir(Session *)
 {
-  assert(drizzle_tmpdir);
-  return (unsigned char*)drizzle_tmpdir;
+  assert(drizzle_tmpdir.size());
+  return (unsigned char*)drizzle_tmpdir.c_str();
 }
 
 /****************************************************************************
@@ -1455,7 +1459,7 @@ static unsigned char *get_tmpdir(Session *)
     ptr		pointer to option structure
 */
 
-static struct my_option *find_option(struct my_option *opt, const char *name)
+static struct option *find_option(struct option *opt, const char *name)
 {
   uint32_t length=strlen(name);
   for (; opt->name; opt++)
@@ -1488,10 +1492,10 @@ static struct my_option *find_option(struct my_option *opt, const char *name)
 */
 
 
-int mysql_add_sys_var_chain(sys_var *first, struct my_option *long_options)
+int mysql_add_sys_var_chain(sys_var *first, struct option *long_options)
 {
   sys_var *var;
-  /* A write lock should be held on LOCK_system_variables_hash */
+  /* @todo for future A write lock should be held on LOCK_system_variables_hash */
 
   for (var= first; var; var= var->getNext())
   {
@@ -1626,7 +1630,7 @@ int set_var_init()
   return(0);
 
 error:
-  fprintf(stderr, "failed to initialize system variables");
+   errmsg_printf(ERRMSG_LVL_ERROR, _("Failed to initialize system variables"));
   return(1);
 }
 
