@@ -31,7 +31,12 @@ void table_cache_free(void);
 bool table_cache_init(void);
 uint32_t cached_open_tables(void);
 uint32_t cached_table_definitions(void);
-HASH *get_open_cache();
+
+typedef boost::unordered_multimap< TableIdentifier::Key, Table *> TableOpenCache;
+typedef std::pair< TableOpenCache::const_iterator, TableOpenCache::const_iterator > TableOpenCacheRange;
+
+TableOpenCache &get_open_cache();
+void remove_table(Table *arg);
 
 void kill_drizzle(void);
 
@@ -89,7 +94,8 @@ enum enum_resolution_type {
   RESOLVED_WITH_NO_ALIAS,
   RESOLVED_AGAINST_ALIAS
 };
-Item ** find_item_in_list(Item *item, List<Item> &items, uint32_t *counter,
+Item ** find_item_in_list(Session *session,
+                          Item *item, List<Item> &items, uint32_t *counter,
                           find_item_error_report_type report_error,
                           enum_resolution_type *resolution);
 bool insert_fields(Session *session, Name_resolution_context *context,
@@ -129,15 +135,14 @@ TableList *find_table_in_list(TableList *table,
                                const char *table_name);
 TableList *unique_table(TableList *table, TableList *table_list,
                         bool check_alias= false);
-void remove_db_from_cache(const std::string schema_name);
+void remove_db_from_cache(const SchemaIdentifier &schema_identifier);
 
 /* bits for last argument to remove_table_from_cache() */
 #define RTFC_NO_FLAG                0x0000
 #define RTFC_OWNED_BY_Session_FLAG      0x0001
 #define RTFC_WAIT_OTHER_THREAD_FLAG 0x0002
 #define RTFC_CHECK_KILLED_FLAG      0x0004
-bool remove_table_from_cache(Session *session, const char *db, const char *table,
-                             uint32_t flags);
+bool remove_table_from_cache(Session *session, TableIdentifier &identifier, uint32_t flags);
 
 void mem_alloc_error(size_t size);
 
