@@ -49,7 +49,7 @@ bool mysql_delete(Session *session, TableList *table_list, COND *conds,
   int		error;
   Table		*table;
   optimizer::SqlSelect *select= NULL;
-  READ_RECORD	info;
+  ReadRecord	info;
   bool          using_limit=limit != HA_POS_ERROR;
   bool		transactional_table, const_cond;
   bool          const_cond_result;
@@ -80,7 +80,6 @@ bool mysql_delete(Session *session, TableList *table_list, COND *conds,
     List<Item>   fields;
     List<Item>   all_fields;
 
-    memset(&tables, 0, sizeof(tables));
     tables.table = table;
     tables.alias = table_list->alias;
 
@@ -188,7 +187,7 @@ bool mysql_delete(Session *session, TableList *table_list, COND *conds,
   if (order && order->elements)
   {
     uint32_t         length= 0;
-    SORT_FIELD  *sortorder;
+    SortField  *sortorder;
     ha_rows examined_rows;
 
     if ((!select || table->quick_keys.none()) && limit != HA_POS_ERROR)
@@ -197,7 +196,6 @@ bool mysql_delete(Session *session, TableList *table_list, COND *conds,
     if (usable_index == MAX_KEY)
     {
       table->sort.io_cache= new internal::IO_CACHE;
-      memset(table->sort.io_cache, 0, sizeof(internal::IO_CACHE));
 
 
       if (!(sortorder= make_unireg_sortorder((order_st*) order->first,
@@ -230,9 +228,13 @@ bool mysql_delete(Session *session, TableList *table_list, COND *conds,
   }
 
   if (usable_index==MAX_KEY)
-    init_read_record(&info,session,table,select,1,1);
+  {
+    info.init_read_record(session,table,select,1,1);
+  }
   else
-    init_read_record_idx(&info, session, table, 1, usable_index);
+  {
+    info.init_read_record_idx(session, table, 1, usable_index);
+  }
 
   session->set_proc_info("updating");
 
@@ -244,7 +246,7 @@ bool mysql_delete(Session *session, TableList *table_list, COND *conds,
     // session->is_error() is tested to disallow delete row on error
     if (!(select && select->skip_record())&& ! session->is_error() )
     {
-      if (!(error= table->cursor->ha_delete_row(table->record[0])))
+      if (!(error= table->cursor->deleteRecord(table->getInsertRecord())))
       {
 	deleted++;
 	if (!--limit && using_limit)
@@ -276,7 +278,7 @@ bool mysql_delete(Session *session, TableList *table_list, COND *conds,
     error= 1;					// Aborted
 
   session->set_proc_info("end");
-  end_read_record(&info);
+  info.end_read_record();
 
 cleanup:
 
@@ -321,6 +323,7 @@ cleanup:
     session->main_da.reset_diagnostics_area();    
     session->my_ok((ha_rows) session->row_count_func);
   }
+  session->status_var.deleted_row_count+= deleted;
   return (error >= 0 || session->is_error());
 
 err:
@@ -396,7 +399,7 @@ bool mysql_truncate(Session& session, TableList *table_list)
     Safety, in case the engine ignored ha_enable_transaction(false)
     above. Also clears session->transaction.*.
   */
-  error= transaction_services.ha_autocommit_or_rollback(&session, error);
+  error= transaction_services.autocommitOrRollback(&session, error);
   session.options= save_options;
 
   return error;
