@@ -17,6 +17,7 @@
 /* Insert of records */
 
 #include "config.h"
+#include <cstdio>
 #include <drizzled/sql_select.h>
 #include <drizzled/show.h>
 #include <drizzled/error.h>
@@ -67,7 +68,7 @@ static int check_insert_fields(Session *session, TableList *table_list,
 
   if (fields.elements == 0 && values.elements != 0)
   {
-    if (values.elements != table->s->fields)
+    if (values.elements != table->getShare()->sizeFields())
     {
       my_error(ER_WRONG_VALUE_COUNT_ON_ROW, MYF(0), 1L);
       return -1;
@@ -120,8 +121,10 @@ static int check_insert_fields(Session *session, TableList *table_list,
     if (table->timestamp_field)	// Don't automaticly set timestamp if used
     {
       if (table->timestamp_field->isWriteSet())
+      {
         clear_timestamp_auto_bits(table->timestamp_field_type,
                                   TIMESTAMP_AUTO_SET_ON_INSERT);
+      }
       else
       {
         table->setWriteSet(table->timestamp_field->field_index);
@@ -176,10 +179,15 @@ static int check_update_fields(Session *session, TableList *insert_table_list,
   {
     /* Don't set timestamp column if this is modified. */
     if (table->timestamp_field->isWriteSet())
+    {
       clear_timestamp_auto_bits(table->timestamp_field_type,
                                 TIMESTAMP_AUTO_SET_ON_UPDATE);
+    }
+
     if (timestamp_mark)
+    {
       table->setWriteSet(table->timestamp_field->field_index);
+    }
   }
   return 0;
 }
@@ -230,7 +238,7 @@ bool mysql_insert(Session *session,TableList *table_list,
   uint32_t value_count;
   ulong counter = 1;
   uint64_t id;
-  COPY_INFO info;
+  CopyInfo info;
   Table *table= 0;
   List_iterator_fast<List_item> its(values_list);
   List_item *values;
@@ -309,7 +317,6 @@ bool mysql_insert(Session *session,TableList *table_list,
   /*
     Fill in the given fields and dump it to the table cursor
   */
-  memset(&info, 0, sizeof(info));
   info.ignore= ignore;
   info.handle_duplicates=duplic;
   info.update_fields= &update_fields;
@@ -320,10 +327,8 @@ bool mysql_insert(Session *session,TableList *table_list,
     For single line insert, generate an error if try to set a NOT NULL field
     to NULL.
   */
-  session->count_cuted_fields= ((values_list.elements == 1 &&
-                                 !ignore) ?
-                                CHECK_FIELD_ERROR_FOR_NULL :
-                                CHECK_FIELD_WARN);
+  session->count_cuted_fields= ignore ? CHECK_FIELD_WARN : CHECK_FIELD_ERROR_FOR_NULL;
+
   session->cuted_fields = 0L;
   table->next_number_field=table->found_next_number_field;
 
@@ -369,7 +374,7 @@ bool mysql_insert(Session *session,TableList *table_list,
     {
       table->restoreRecordAsDefault();	// Get empty record
 
-      if (fill_record(session, table->field, *values))
+      if (fill_record(session, table->getFields(), *values))
       {
 	if (values_list.elements != 1 && ! session->is_error())
 	{
@@ -458,15 +463,16 @@ bool mysql_insert(Session *session,TableList *table_list,
   {
     char buff[160];
     if (ignore)
-      sprintf(buff, ER(ER_INSERT_INFO), (ulong) info.records,
+      snprintf(buff, sizeof(buff), ER(ER_INSERT_INFO), (ulong) info.records,
               (ulong) (info.records - info.copied), (ulong) session->cuted_fields);
     else
-      sprintf(buff, ER(ER_INSERT_INFO), (ulong) info.records,
+      snprintf(buff, sizeof(buff), ER(ER_INSERT_INFO), (ulong) info.records,
 	      (ulong) (info.deleted + info.updated), (ulong) session->cuted_fields);
     session->row_count_func= info.copied + info.deleted + info.updated;
     session->my_ok((ulong) session->row_count_func,
                    info.copied + info.deleted + info.touched, id, buff);
   }
+  session->status_var.inserted_row_count+= session->row_count_func;
   session->abort_on_warning= 0;
   DRIZZLE_INSERT_DONE(0, session->row_count_func);
   return false;
@@ -672,7 +678,7 @@ bool mysql_prepare_insert(Session *session, TableList *table_list,
 
 static int last_uniq_key(Table *table,uint32_t keynr)
 {
-  while (++keynr < table->s->keys)
+  while (++keynr < table->getShare()->sizeKeys())
     if (table->key_info[keynr].flags & HA_NOSAME)
       return 0;
   return 1;
@@ -687,7 +693,7 @@ static int last_uniq_key(Table *table,uint32_t keynr)
      write_record()
       session   - thread context
       table - table to which record should be written
-      info  - COPY_INFO structure describing handling of duplicates
+      info  - CopyInfo structure describing handling of duplicates
               and which is used for counting number of records inserted
               and deleted.
 
@@ -706,10 +712,10 @@ static int last_uniq_key(Table *table,uint32_t keynr)
 */
 
 
-int write_record(Session *session, Table *table,COPY_INFO *info)
+int write_record(Session *session, Table *table,CopyInfo *info)
 {
   int error;
-  char *key=0;
+  std::vector<unsigned char> key;
   MyBitmap *save_read_set, *save_write_set;
   uint64_t prev_insert_id= table->cursor->next_insert_id;
   uint64_t insert_id_for_cur_row= 0;
@@ -721,7 +727,7 @@ int write_record(Session *session, Table *table,COPY_INFO *info)
 
   if (info->handle_duplicates == DUP_REPLACE || info->handle_duplicates == DUP_UPDATE)
   {
-    while ((error=table->cursor->ha_write_row(table->record[0])))
+    while ((error=table->cursor->insertRecord(table->getInsertRecord())))
     {
       uint32_t key_nr;
       /*
@@ -764,12 +770,12 @@ int write_record(Session *session, Table *table,COPY_INFO *info)
       */
       if (info->handle_duplicates == DUP_REPLACE &&
           table->next_number_field &&
-          key_nr == table->s->next_number_index &&
+          key_nr == table->getShare()->next_number_index &&
 	  (insert_id_for_cur_row > 0))
 	goto err;
       if (table->cursor->getEngine()->check_flag(HTON_BIT_DUPLICATE_POS))
       {
-	if (table->cursor->rnd_pos(table->record[1],table->cursor->dup_ref))
+	if (table->cursor->rnd_pos(table->getUpdateRecord(),table->cursor->dup_ref))
 	  goto err;
       }
       else
@@ -780,17 +786,13 @@ int write_record(Session *session, Table *table,COPY_INFO *info)
 	  goto err;
 	}
 
-	if (!key)
+	if (not key.size())
 	{
-	  if (!(key=(char*) malloc(table->s->max_unique_length)))
-	  {
-	    error=ENOMEM;
-	    goto err;
-	  }
+          key.resize(table->getShare()->max_unique_length);
 	}
-	key_copy((unsigned char*) key,table->record[0],table->key_info+key_nr,0);
-	if ((error=(table->cursor->index_read_idx_map(table->record[1],key_nr,
-                                                    (unsigned char*) key, HA_WHOLE_KEY,
+	key_copy(&key[0], table->getInsertRecord(), table->key_info+key_nr, 0);
+	if ((error=(table->cursor->index_read_idx_map(table->getUpdateRecord(),key_nr,
+                                                    &key[0], HA_WHOLE_KEY,
                                                     HA_READ_KEY_EXACT))))
 	  goto err;
       }
@@ -801,7 +803,7 @@ int write_record(Session *session, Table *table,COPY_INFO *info)
           that matches, is updated. If update causes a conflict again,
           an error is returned
         */
-	assert(table->insert_values != NULL);
+	assert(table->insert_values.size());
         table->storeRecordAsInsert();
         table->restoreRecord();
         assert(info->update_fields->elements ==
@@ -820,8 +822,8 @@ int write_record(Session *session, Table *table,COPY_INFO *info)
              !bitmap_is_subset(table->write_set, table->read_set)) ||
             table->compare_record())
         {
-          if ((error=table->cursor->ha_update_row(table->record[1],
-                                                table->record[0])) &&
+          if ((error=table->cursor->updateRecord(table->getUpdateRecord(),
+                                                table->getInsertRecord())) &&
               error != HA_ERR_RECORD_IS_THE_SAME)
           {
             if (info->ignore &&
@@ -875,8 +877,8 @@ int write_record(Session *session, Table *table,COPY_INFO *info)
             (table->timestamp_field_type == TIMESTAMP_NO_AUTO_SET ||
              table->timestamp_field_type == TIMESTAMP_AUTO_SET_ON_BOTH))
         {
-          if ((error=table->cursor->ha_update_row(table->record[1],
-					        table->record[0])) &&
+          if ((error=table->cursor->updateRecord(table->getUpdateRecord(),
+					        table->getInsertRecord())) &&
               error != HA_ERR_RECORD_IS_THE_SAME)
             goto err;
           if (error != HA_ERR_RECORD_IS_THE_SAME)
@@ -892,7 +894,7 @@ int write_record(Session *session, Table *table,COPY_INFO *info)
         }
         else
         {
-          if ((error=table->cursor->ha_delete_row(table->record[1])))
+          if ((error=table->cursor->deleteRecord(table->getUpdateRecord())))
             goto err;
           info->deleted++;
           if (!table->cursor->has_transactions())
@@ -910,7 +912,7 @@ int write_record(Session *session, Table *table,COPY_INFO *info)
         table->write_set != save_write_set)
       table->column_bitmaps_set(save_read_set, save_write_set);
   }
-  else if ((error=table->cursor->ha_write_row(table->record[0])))
+  else if ((error=table->cursor->insertRecord(table->getInsertRecord())))
   {
     if (!info->ignore ||
         table->cursor->is_fatal_error(error, HA_CHECK_DUP))
@@ -924,8 +926,6 @@ after_n_copied_inc:
   session->record_first_successful_insert_id_in_cur_stmt(table->cursor->insert_id_for_cur_row);
 
 gok_or_after_err:
-  if (key)
-    free(key);
   if (!table->cursor->has_transactions())
     session->transaction.stmt.markModifiedNonTransData();
   return(0);
@@ -939,8 +939,6 @@ err:
 
 before_err:
   table->cursor->restore_auto_increment(prev_insert_id);
-  if (key)
-    free(key);
   table->column_bitmaps_set(save_read_set, save_write_set);
   return(1);
 }
@@ -955,7 +953,7 @@ int check_that_all_fields_are_given_values(Session *session, Table *entry,
 {
   int err= 0;
 
-  for (Field **field=entry->field ; *field ; field++)
+  for (Field **field=entry->getFields() ; *field ; field++)
   {
     if (((*field)->isWriteSet()) == false)
     {
@@ -1043,12 +1041,11 @@ select_insert::select_insert(TableList *table_list_par, Table *table_par,
                              List<Item> *update_fields,
                              List<Item> *update_values,
                              enum_duplicates duplic,
-                             bool ignore_check_option_errors)
-  :table_list(table_list_par), table(table_par), fields(fields_par),
-   autoinc_value_of_last_inserted_row(0),
-   insert_into_view(table_list_par && 0 != 0)
+                             bool ignore_check_option_errors) :
+  table_list(table_list_par), table(table_par), fields(fields_par),
+  autoinc_value_of_last_inserted_row(0),
+  insert_into_view(table_list_par && 0 != 0)
 {
-  memset(&info, 0, sizeof(info));
   info.handle_duplicates= duplic;
   info.ignore= ignore_check_option_errors;
   info.update_fields= update_fields;
@@ -1257,6 +1254,7 @@ bool select_insert::send_data(List<Item> &values)
   plugin::TransactionalStorageEngine::releaseTemporaryLatches(session);
 
   error= write_record(session, table, &info);
+  table->auto_increment_field_not_null= false;
 
   if (!error)
   {
@@ -1297,7 +1295,7 @@ void select_insert::store_values(List<Item> &values)
   if (fields->elements)
     fill_record(session, *fields, values, true);
   else
-    fill_record(session, table->field, values, true);
+    fill_record(session, table->getFields(), values, true);
 }
 
 void select_insert::send_error(uint32_t errcode,const char *err)
@@ -1325,7 +1323,7 @@ bool select_insert::send_eof()
   {
     /*
       We must invalidate the table in the query cache before binlog writing
-      and ha_autocommit_or_rollback.
+      and autocommitOrRollback.
     */
     if (session->transaction.stmt.hasModifiedNonTransData())
       session->transaction.all.markModifiedNonTransData();
@@ -1343,10 +1341,10 @@ bool select_insert::send_eof()
   }
   char buff[160];
   if (info.ignore)
-    sprintf(buff, ER(ER_INSERT_INFO), (ulong) info.records,
+    snprintf(buff, sizeof(buff), ER(ER_INSERT_INFO), (ulong) info.records,
 	    (ulong) (info.records - info.copied), (ulong) session->cuted_fields);
   else
-    sprintf(buff, ER(ER_INSERT_INFO), (ulong) info.records,
+    snprintf(buff, sizeof(buff), ER(ER_INSERT_INFO), (ulong) info.records,
 	    (ulong) (info.deleted+info.updated), (ulong) session->cuted_fields);
   session->row_count_func= info.copied + info.deleted + info.updated;
 
@@ -1357,6 +1355,7 @@ bool select_insert::send_eof()
      (info.copied ? autoinc_value_of_last_inserted_row : 0));
   session->my_ok((ulong) session->row_count_func,
                  info.copied + info.deleted + info.touched, id, buff);
+  session->status_var.inserted_row_count+= session->row_count_func; 
   DRIZZLE_INSERT_SELECT_DONE(0, session->row_count_func);
   return 0;
 }
@@ -1426,7 +1425,7 @@ void select_insert::abort() {
       items        in     List of items which should be used to produce rest
                           of fields for the table (corresponding fields will
                           be added to the end of alter_info->create_list)
-      lock         out    Pointer to the DRIZZLE_LOCK object for table created
+      lock         out    Pointer to the DrizzleLock object for table created
                           (or open temporary table) will be returned in this
                           parameter. Since this table is not included in
                           Session::lock caller is responsible for explicitly
@@ -1458,11 +1457,11 @@ static Table *create_table_from_items(Session *session, HA_CREATE_INFO *create_i
                                       AlterInfo *alter_info,
                                       List<Item> *items,
                                       bool is_if_not_exists,
-                                      DRIZZLE_LOCK **lock,
+                                      DrizzleLock **lock,
 				      TableIdentifier &identifier)
 {
   Table tmp_table;		// Used during 'CreateField()'
-  TableShare share;
+  TableShare share(message::Table::INTERNAL);
   Table *table= 0;
   uint32_t select_field_count= items->elements;
   /* Add selected items to field list */
@@ -1471,10 +1470,7 @@ static Table *create_table_from_items(Session *session, HA_CREATE_INFO *create_i
   Field *tmp_field;
   bool not_used;
 
-  bool lex_identified_temp_table= (table_proto.type() == message::Table::TEMPORARY);
-
-  if (not (lex_identified_temp_table) &&
-      create_table->table->db_stat)
+  if (not (identifier.isTmp()) && create_table->table->db_stat)
   {
     /* Table already exists and was open at openTablesLock() stage. */
     if (is_if_not_exists)
@@ -1490,17 +1486,16 @@ static Table *create_table_from_items(Session *session, HA_CREATE_INFO *create_i
     return NULL;
   }
 
-  tmp_table.alias= 0;
   tmp_table.timestamp_field= 0;
-  tmp_table.s= &share;
+  tmp_table.setShare(&share);
 
-  tmp_table.s->db_create_options=0;
-  tmp_table.s->blob_ptr_size= portable_sizeof_char_ptr;
+  tmp_table.getMutableShare()->db_create_options= 0;
+  tmp_table.getMutableShare()->blob_ptr_size= portable_sizeof_char_ptr;
 
   if (not table_proto.engine().name().compare("MyISAM"))
-    tmp_table.s->db_low_byte_first= true;
+    tmp_table.getMutableShare()->db_low_byte_first= true;
   else if (not table_proto.engine().name().compare("MEMORY"))
-    tmp_table.s->db_low_byte_first= true;
+    tmp_table.getMutableShare()->db_low_byte_first= true;
 
   tmp_table.null_row= false;
   tmp_table.maybe_null= false;
@@ -1545,8 +1540,7 @@ static Table *create_table_from_items(Session *session, HA_CREATE_INFO *create_i
 				       select_field_count,
 				       is_if_not_exists))
     {
-      if (create_info->table_existed &&
-          !(lex_identified_temp_table))
+      if (create_info->table_existed && not identifier.isTmp())
       {
         /*
           This means that someone created table underneath server
@@ -1557,16 +1551,16 @@ static Table *create_table_from_items(Session *session, HA_CREATE_INFO *create_i
         return NULL;
       }
 
-      if (not lex_identified_temp_table)
+      if (not identifier.isTmp())
       {
-        pthread_mutex_lock(&LOCK_open); /* CREATE TABLE... has found that the table already exists for insert and is adapting to use it */
+        LOCK_open.lock(); /* CREATE TABLE... has found that the table already exists for insert and is adapting to use it */
         if (session->reopen_name_locked_table(create_table, false))
         {
           quick_rm_table(*session, identifier);
         }
         else
           table= create_table->table;
-        pthread_mutex_unlock(&LOCK_open);
+        LOCK_open.unlock();
       }
       else
       {
@@ -1597,8 +1591,8 @@ static Table *create_table_from_items(Session *session, HA_CREATE_INFO *create_i
       *lock= 0;
     }
 
-    if (!create_info->table_existed)
-      session->drop_open_table(table, create_table->db, create_table->table_name);
+    if (not create_info->table_existed)
+      session->drop_open_table(table, identifier);
     return NULL;
   }
 
@@ -1609,9 +1603,7 @@ static Table *create_table_from_items(Session *session, HA_CREATE_INFO *create_i
 int
 select_create::prepare(List<Item> &values, Select_Lex_Unit *u)
 {
-  bool lex_identified_temp_table= (table_proto.type() == message::Table::TEMPORARY);
-
-  DRIZZLE_LOCK *extra_lock= NULL;
+  DrizzleLock *extra_lock= NULL;
   /*
     For replication, the CREATE-SELECT statement is written
     in two pieces: the first transaction messsage contains 
@@ -1629,13 +1621,15 @@ select_create::prepare(List<Item> &values, Select_Lex_Unit *u)
 					  alter_info, &values,
 					  is_if_not_exists,
 					  &extra_lock, identifier)))
+  {
     return(-1);				// abort() deletes table
+  }
 
   if (extra_lock)
   {
     assert(m_plock == NULL);
 
-    if (lex_identified_temp_table)
+    if (identifier.isTmp())
       m_plock= &m_lock;
     else
       m_plock= &session->extra_lock;
@@ -1643,14 +1637,14 @@ select_create::prepare(List<Item> &values, Select_Lex_Unit *u)
     *m_plock= extra_lock;
   }
 
-  if (table->s->fields < values.elements)
+  if (table->getShare()->sizeFields() < values.elements)
   {
     my_error(ER_WRONG_VALUE_COUNT_ON_ROW, MYF(0), 1);
     return(-1);
   }
 
  /* First field to copy */
-  field= table->field+table->s->fields - values.elements;
+  field= table->getFields() + table->getShare()->sizeFields() - values.elements;
 
   /* Mark all fields that are given values */
   for (Field **f= field ; *f ; f++)
@@ -1714,10 +1708,10 @@ bool select_create::send_eof()
       tables.  This can fail, but we should unlock the table
       nevertheless.
     */
-    if (!table->s->tmp_table)
+    if (!table->getShare()->getType())
     {
       TransactionServices &transaction_services= TransactionServices::singleton();
-      transaction_services.ha_autocommit_or_rollback(session, 0);
+      transaction_services.autocommitOrRollback(session, 0);
       (void) session->endActiveTransaction();
     }
 
@@ -1764,8 +1758,8 @@ void select_create::abort()
   {
     table->cursor->extra(HA_EXTRA_NO_IGNORE_DUP_KEY);
     table->cursor->extra(HA_EXTRA_WRITE_CANNOT_REPLACE);
-    if (!create_info->table_existed)
-      session->drop_open_table(table, create_table->db, create_table->table_name);
+    if (not create_info->table_existed)
+      session->drop_open_table(table, identifier);
     table= NULL;                                    // Safety
   }
 }
