@@ -26,6 +26,7 @@
 #include "drizzled/lookup_symbol.h"
 #include "drizzled/index_hint.h"
 
+#include <cstdio>
 #include <ctype.h>
 
 using namespace std;
@@ -258,6 +259,10 @@ void lex_start(Session *session)
 
   lex->is_lex_started= true;
   lex->statement= NULL;
+  
+  lex->is_cross= false;
+
+  lex->reset();
 }
 
 void lex_end(LEX *lex)
@@ -273,6 +278,7 @@ void lex_end(LEX *lex)
   delete lex->result;
 
   lex->result= 0;
+  lex->setCacheable(true);
 
   if (lex->statement) 
     delete lex->statement;
@@ -1857,8 +1863,16 @@ void Query_tables_list::reset_query_tables_list(bool init)
     for this.
 */
 LEX::LEX()
-  :result(0), yacc_yyss(0), yacc_yyvs(0),
-   sql_command(SQLCOM_END), option_type(OPT_DEFAULT), is_lex_started(0)
+  :
+    result(0), 
+    yacc_yyss(0), 
+    yacc_yyvs(0),
+    charset(NULL),
+    sql_command(SQLCOM_END), 
+    option_type(OPT_DEFAULT), 
+    is_lex_started(0),
+    cacheable(true),
+    sum_expr_used(false)
 {
   reset_query_tables_list(true);
   statement= NULL;
