@@ -258,7 +258,7 @@ public:
     return memory::sql_alloc(size);
   }
   static void *operator new(size_t size, memory::Root *mem_root)
-  { return (void*) alloc_root(mem_root, (uint32_t) size); }
+  { return (void*) mem_root->alloc_root((uint32_t) size); }
   static void operator delete(void *, size_t)
   {  }
   static void operator delete(void *, memory::Root *)
@@ -305,7 +305,7 @@ private:
 */
 class Session;
 class select_result;
-class JOIN;
+class Join;
 class select_union;
 class Select_Lex_Unit: public Select_Lex_Node {
 protected:
@@ -422,7 +422,7 @@ public:
   List<Item> item_list;  /* list of fields & expressions */
   List<String> interval_list;
   bool is_item_list_lookup;
-  JOIN *join; /* after JOIN::prepare it is pointer to corresponding JOIN */
+  Join *join; /* after Join::prepare it is pointer to corresponding JOIN */
   List<TableList> top_join_list; /* join list of the top level          */
   List<TableList> *join_list;    /* list for the currently parsed join  */
   TableList *embedding;          /* table embedding to the above list   */
@@ -514,6 +514,7 @@ public:
           defined as SUM_FUNC_USED.
   */
   std::bitset<2> full_group_by_flag;
+
   void init_query();
   void init_select();
   Select_Lex_Unit* master_unit();
@@ -929,6 +930,33 @@ public:
     }
     return false;
   }
+  bool is_cross; // CROSS keyword was used
+  bool isCacheable()
+  {
+    return cacheable;
+  }
+  void setCacheable(bool val)
+  {
+    cacheable= val;
+  }
+
+  void reset()
+  {
+    sum_expr_used= false;
+  }
+
+  void setSumExprUsed()
+  {
+    sum_expr_used= true;
+  }
+
+  bool isSumExprUsed()
+  {
+    return sum_expr_used;
+  }
+private: 
+  bool cacheable;
+  bool sum_expr_used;
 };
 
 extern void lex_start(Session *session);
