@@ -31,7 +31,9 @@
 #include <fcntl.h>
 #include <algorithm>
 #include <climits>
+#include <boost/filesystem.hpp>
 
+namespace fs=boost::filesystem;
 using namespace std;
 namespace drizzled
 {
@@ -85,12 +87,12 @@ public:
   void set_io_cache_arg(void* arg) { cache.arg = arg; }
 };
 
-static int read_fixed_length(Session *session, COPY_INFO &info, TableList *table_list,
+static int read_fixed_length(Session *session, CopyInfo &info, TableList *table_list,
                              List<Item> &fields_vars, List<Item> &set_fields,
                              List<Item> &set_values, READ_INFO &read_info,
 			     uint32_t skip_lines,
 			     bool ignore_check_option_errors);
-static int read_sep_field(Session *session, COPY_INFO &info, TableList *table_list,
+static int read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
                           List<Item> &fields_vars, List<Item> &set_fields,
                           List<Item> &set_values, READ_INFO &read_info,
 			  String &enclosed, uint32_t skip_lines,
@@ -178,7 +180,7 @@ int mysql_load(Session *session,file_exchange *ex,TableList *table_list,
   if (!fields_vars.elements)
   {
     Field **field;
-    for (field=table->field; *field ; field++)
+    for (field= table->getFields(); *field ; field++)
       fields_vars.push_back(new Item_field(*field));
     table->setWriteSet();
     table->timestamp_field_type= TIMESTAMP_NO_AUTO_SET;
@@ -258,22 +260,27 @@ int mysql_load(Session *session,file_exchange *ex,TableList *table_list,
 #endif
     if (!internal::dirname_length(ex->file_name))
     {
-      strcpy(name, drizzle_real_data_home);
-      strncat(name, tdb, FN_REFLEN-strlen(drizzle_real_data_home)-1);
+      strcpy(name, getDataHomeCatalog().c_str());
+      strncat(name, "/", 1);
+      strncat(name, tdb, FN_REFLEN-getDataHomeCatalog().size());
       (void) internal::fn_format(name, ex->file_name, name, "",
 		       MY_RELATIVE_PATH | MY_UNPACK_FILENAME);
     }
     else
     {
-      (void) internal::fn_format(name, ex->file_name, drizzle_real_data_home, "",
+      (void) internal::fn_format(name, ex->file_name, getDataHomeCatalog().c_str(), "",
 		       MY_RELATIVE_PATH | MY_UNPACK_FILENAME);
 
-      if (opt_secure_file_priv &&
-          strncmp(opt_secure_file_priv, name, strlen(opt_secure_file_priv)))
+      if (opt_secure_file_priv)
       {
-        /* Read only allowed from within dir specified by secure_file_priv */
-        my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--secure-file-priv");
-        return(true);
+        fs::path secure_file_path(fs::system_complete(fs::path(opt_secure_file_priv)));
+        fs::path target_path(fs::system_complete(fs::path(name)));
+        if (target_path.file_string().substr(0, secure_file_path.file_string().size()) != secure_file_path.file_string())
+        {
+          /* Read only allowed from within dir specified by secure_file_priv */
+          my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0), "--secure-file-priv");
+          return(true);
+        }
       }
 
       struct stat stat_info;
@@ -302,15 +309,16 @@ int mysql_load(Session *session,file_exchange *ex,TableList *table_list,
     }
   }
 
-  COPY_INFO info;
+  CopyInfo info;
   memset(&info, 0, sizeof(info));
   info.ignore= ignore;
   info.handle_duplicates=handle_duplicates;
   info.escape_char=escaped->length() ? (*escaped)[0] : INT_MAX;
 
+  SchemaIdentifier identifier(session->db);
   READ_INFO read_info(file, tot_length,
-                      ex->cs ? ex->cs : plugin::StorageEngine::getSchemaCollation(session->db.c_str()),
-		      *field_term,*ex->line_start, *ex->line_term, *enclosed,
+                      ex->cs ? ex->cs : plugin::StorageEngine::getSchemaCollation(identifier),
+		      *field_term, *ex->line_start, *ex->line_term, *enclosed,
 		      info.escape_char, is_fifo);
   if (read_info.error)
   {
@@ -376,7 +384,7 @@ int mysql_load(Session *session,file_exchange *ex,TableList *table_list,
     internal::my_close(file,MYF(0));
   free_blobs(table);				/* if pack_blob was used */
   table->copy_blobs=0;
-  session->count_cuted_fields= CHECK_FIELD_IGNORE;
+  session->count_cuted_fields= CHECK_FIELD_ERROR_FOR_NULL;
   /*
      simulated killing in the middle of per-row loop
      must be effective for binlogging
@@ -387,8 +395,8 @@ int mysql_load(Session *session,file_exchange *ex,TableList *table_list,
     error= -1;				// Error on read
     goto err;
   }
-  sprintf(name, ER(ER_LOAD_INFO), (uint32_t) info.records, (uint32_t) info.deleted,
-	  (uint32_t) (info.records - info.copied), (uint32_t) session->cuted_fields);
+  snprintf(name, sizeof(name), ER(ER_LOAD_INFO), info.records, info.deleted,
+	   (info.records - info.copied), session->cuted_fields);
 
   if (session->transaction.stmt.hasModifiedNonTransData())
     session->transaction.all.markModifiedNonTransData();
@@ -410,7 +418,7 @@ err:
 ****************************************************************************/
 
 static int
-read_fixed_length(Session *session, COPY_INFO &info, TableList *table_list,
+read_fixed_length(Session *session, CopyInfo &info, TableList *table_list,
                   List<Item> &fields_vars, List<Item> &set_fields,
                   List<Item> &set_values, READ_INFO &read_info,
                   uint32_t skip_lines, bool ignore_check_option_errors)
@@ -529,7 +537,7 @@ read_fixed_length(Session *session, COPY_INFO &info, TableList *table_list,
 
 
 static int
-read_sep_field(Session *session, COPY_INFO &info, TableList *table_list,
+read_sep_field(Session *session, CopyInfo &info, TableList *table_list,
                List<Item> &fields_vars, List<Item> &set_fields,
                List<Item> &set_values, READ_INFO &read_info,
 	       String &enclosed, uint32_t skip_lines,
