@@ -21,6 +21,7 @@
 #include <drizzled/error.h>
 #include <drizzled/nested_join.h>
 #include <drizzled/query_id.h>
+#include "drizzled/transaction_services.h"
 #include <drizzled/sql_parse.h>
 #include <drizzled/data_home.h>
 #include <drizzled/sql_base.h>
@@ -40,10 +41,10 @@
 #include "drizzled/probes.h"
 #include "drizzled/session_list.h"
 #include "drizzled/global_charset_info.h"
-#include "drizzled/transaction_services.h"
 
 #include "drizzled/plugin/logging.h"
 #include "drizzled/plugin/query_rewrite.h"
+#include "drizzled/plugin/query_cache.h"
 #include "drizzled/plugin/authorization.h"
 #include "drizzled/optimizer/explain_plan.h"
 #include "drizzled/pthread_globals.h"
@@ -182,7 +183,7 @@ bool dispatch_command(enum enum_server_command command, Session *session,
     break;
   /* Increase id and count all other statements. */
   default:
-    statistic_increment(session->status_var.questions, &LOCK_status);
+    session->status_var.questions++;
     query_id.next();
   }
 
@@ -195,10 +196,17 @@ bool dispatch_command(enum enum_server_command command, Session *session,
   switch (command) {
   case COM_INIT_DB:
   {
-    status_var_increment(session->status_var.com_stat[SQLCOM_CHANGE_DB]);
+    if (packet_length == 0)
+    {
+      my_message(ER_NO_DB_ERROR, ER(ER_NO_DB_ERROR), MYF(0));
+      break;
+    }
+
     string tmp(packet, packet_length);
 
-    if (not mysql_change_db(session, tmp))
+    SchemaIdentifier identifier(tmp);
+
+    if (not mysql_change_db(session, identifier))
     {
       session->my_ok();
     }
@@ -224,7 +232,7 @@ bool dispatch_command(enum enum_server_command command, Session *session,
     break;
   case COM_SHUTDOWN:
   {
-    status_var_increment(session->status_var.com_other);
+    session->status_var.com_other++;
     session->my_eof();
     session->close_thread_tables();			// Free before kill
     kill_drizzle();
@@ -232,7 +240,7 @@ bool dispatch_command(enum enum_server_command command, Session *session,
     break;
   }
   case COM_PING:
-    status_var_increment(session->status_var.com_other);
+    session->status_var.com_other++;
     session->my_ok();				// Tell client we are alive
     break;
   case COM_SLEEP:
@@ -246,7 +254,7 @@ bool dispatch_command(enum enum_server_command command, Session *session,
   /* If commit fails, we should be able to reset the OK status. */
   session->main_da.can_overwrite_status= true;
   TransactionServices &transaction_services= TransactionServices::singleton();
-  transaction_services.ha_autocommit_or_rollback(session, session->is_error());
+  transaction_services.autocommitOrRollback(session, session->is_error());
   session->main_da.can_overwrite_status= false;
 
   session->transaction.stmt.reset();
@@ -261,7 +269,7 @@ bool dispatch_command(enum enum_server_command command, Session *session,
   if (session->killed == Session::KILL_QUERY || session->killed == Session::KILL_BAD_DATA)
   {
     session->killed= Session::NOT_KILLED;
-    session->mysys_var->abort= 0;
+    session->setAbort(false);
   }
 
   /* Can not be true, but do not take chances in production. */
@@ -307,7 +315,7 @@ bool dispatch_command(enum enum_server_command command, Session *session,
   session->query.clear();
 
   session->set_proc_info(NULL);
-  free_root(session->mem_root,MYF(memory::KEEP_PREALLOC));
+  session->mem_root->free_root(MYF(memory::KEEP_PREALLOC));
 
   if (DRIZZLE_QUERY_DONE_ENABLED() || DRIZZLE_COMMAND_DONE_ENABLED())
   {
@@ -465,15 +473,11 @@ mysql_execute_command(Session *session)
     drizzle_reset_errors(session, 0);
   }
 
-  status_var_increment(session->status_var.com_stat[lex->sql_command]);
-
   assert(session->transaction.stmt.hasModifiedNonTransData() == false);
 
   /* now we are ready to execute the statement */
   res= lex->statement->execute();
-
   session->set_proc_info("query end");
-
   /*
     The return value for ROW_COUNT() is "implementation dependent" if the
     statement is not DELETE, INSERT or UPDATE, but -1 is what JDBC and ODBC
@@ -487,7 +491,6 @@ mysql_execute_command(Session *session)
 
   return (res || session->is_error());
 }
-
 bool execute_sqlcom_select(Session *session, TableList *all_tables)
 {
   LEX	*lex= session->lex;
@@ -500,7 +503,7 @@ bool execute_sqlcom_select(Session *session, TableList *all_tables)
       param->select_limit=
         new Item_int((uint64_t) session->variables.select_limit);
   }
-  if (!(res= session->openTablesLock(all_tables)))
+  if (not (res= session->openTablesLock(all_tables)))
   {
     if (lex->describe)
     {
@@ -535,7 +538,13 @@ bool execute_sqlcom_select(Session *session, TableList *all_tables)
     {
       if (!result && !(result= new select_send()))
         return true;
+
+      /* Init the Query Cache plugin */
+      plugin::QueryCache::prepareResultset(session); 
       res= handle_select(session, lex, result, 0);
+      /* Send the Resultset to the cache */
+      plugin::QueryCache::setResultset(session); 
+
       if (result != lex->result)
         delete result;
     }
@@ -689,7 +698,7 @@ void create_select_for_variable(const char *var_name)
   */
   if ((var= get_system_var(session, OPT_SESSION, tmp, null_lex_string)))
   {
-    end+= sprintf(buff, "@@session.%s", var_name);
+    end+= snprintf(buff, sizeof(buff), "@@session.%s", var_name);
     var->set_name(buff, end-buff, system_charset_info);
     session->add_item_to_list(var);
   }
@@ -707,15 +716,25 @@ void create_select_for_variable(const char *var_name)
 
 void mysql_parse(Session *session, const char *inBuf, uint32_t length)
 {
+  uint64_t start_time= my_getsystime();
   lex_start(session);
   session->reset_for_next_command();
-
+  /* Check if the Query is Cached if and return true if yes
+   * TODO the plugin has to make sure that the query is cacheble
+   * by setting the query_safe_cache param to TRUE
+   */
+  bool res= true;
+  if (plugin::QueryCache::isCached(session))
+  {
+    res= plugin::QueryCache::sendCachedResultset(session);
+  }
+  if (not res)
+  {
+    return;
+  }
   LEX *lex= session->lex;
-
   Lex_input_stream lip(session, inBuf, length);
-
   bool err= parse_sql(session, &lip);
-
   if (!err)
   {
     {
@@ -724,8 +743,17 @@ void mysql_parse(Session *session, const char *inBuf, uint32_t length)
         DRIZZLE_QUERY_EXEC_START(session->query.c_str(),
                                  session->thread_id,
                                  const_cast<const char *>(session->db.empty() ? "" : session->db.c_str()));
+        // Implement Views here --Brian
         /* Actually execute the query */
-        mysql_execute_command(session);
+        try 
+        {
+          mysql_execute_command(session);
+        }
+        catch (...)
+        {
+          // Just try to catch any random failures that could have come
+          // during execution.
+        }
         DRIZZLE_QUERY_EXEC_DONE(0);
       }
     }
@@ -734,13 +762,11 @@ void mysql_parse(Session *session, const char *inBuf, uint32_t length)
   {
     assert(session->is_error());
   }
-
   lex->unit.cleanup();
   session->set_proc_info("freeing items");
   session->end_statement();
   session->cleanup_after_query();
-
-  return;
+  session->status_var.execution_time_nsec+= my_getsystime() - start_time;
 }
 
 
@@ -890,11 +916,17 @@ TableList *Select_Lex::add_table_to_list(Session *session,
     return NULL;
   }
 
-  if (table->is_derived_table() == false && table->db.str &&
-      check_db_name(&table->db))
+  if (table->is_derived_table() == false && table->db.str)
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), table->db.str);
-    return NULL;
+    my_casedn_str(files_charset_info, table->db.str);
+
+    SchemaIdentifier schema_identifier(string(table->db.str));
+    if (not check_db_name(session, schema_identifier))
+    {
+
+      my_error(ER_WRONG_DB_NAME, MYF(0), table->db.str);
+      return NULL;
+    }
   }
 
   if (!alias)					/* Alias is case sensitive */
@@ -910,19 +942,20 @@ TableList *Select_Lex::add_table_to_list(Session *session,
   }
   if (!(ptr = (TableList *) session->calloc(sizeof(TableList))))
     return NULL;
+
   if (table->db.str)
   {
-    ptr->is_fqtn= true;
+    ptr->setIsFqtn(true);
     ptr->db= table->db.str;
     ptr->db_length= table->db.length;
   }
   else if (lex->copy_db_to(&ptr->db, &ptr->db_length))
     return NULL;
   else
-    ptr->is_fqtn= false;
+    ptr->setIsFqtn(false);
 
   ptr->alias= alias_str;
-  ptr->is_alias= alias ? true : false;
+  ptr->setIsAlias(alias ? true : false);
   if (table->table.length)
     table->table.length= my_casedn_str(files_charset_info, table->table.str);
   ptr->table_name=table->table.str;
@@ -943,7 +976,7 @@ TableList *Select_Lex::add_table_to_list(Session *session,
 	 tables=tables->next_local)
     {
       if (!my_strcasecmp(table_alias_charset, alias_str, tables->alias) &&
-	  !strcmp(ptr->db, tables->db))
+	  !strcasecmp(ptr->db, tables->db))
       {
 	my_error(ER_NONUNIQ_TABLE, MYF(0), alias_str);
 	return NULL;
@@ -1012,12 +1045,11 @@ bool Select_Lex::init_nested_join(Session *session)
   if (!(ptr= (TableList*) session->calloc(ALIGN_SIZE(sizeof(TableList))+
                                        sizeof(nested_join_st))))
     return true;
-  nested_join= ptr->nested_join=
-    ((nested_join_st*) ((unsigned char*) ptr + ALIGN_SIZE(sizeof(TableList))));
-
+  ptr->setNestedJoin(((nested_join_st*) ((unsigned char*) ptr + ALIGN_SIZE(sizeof(TableList)))));
+  nested_join= ptr->getNestedJoin();
   join_list->push_front(ptr);
-  ptr->embedding= embedding;
-  ptr->join_list= join_list;
+  ptr->setEmbedding(embedding);
+  ptr->setJoinList(join_list);
   ptr->alias= (char*) "(nested_join)";
   embedding= ptr;
   join_list= &nested_join->join_list;
@@ -1047,15 +1079,15 @@ TableList *Select_Lex::end_nested_join(Session *)
 
   assert(embedding);
   ptr= embedding;
-  join_list= ptr->join_list;
-  embedding= ptr->embedding;
-  nested_join= ptr->nested_join;
+  join_list= ptr->getJoinList();
+  embedding= ptr->getEmbedding();
+  nested_join= ptr->getNestedJoin();
   if (nested_join->join_list.elements == 1)
   {
     TableList *embedded= nested_join->join_list.head();
     join_list->pop();
-    embedded->join_list= join_list;
-    embedded->embedding= embedding;
+    embedded->setJoinList(join_list);
+    embedded->setEmbedding(embedding);
     join_list->push_front(embedded);
     ptr= embedded;
   }
@@ -1088,13 +1120,12 @@ TableList *Select_Lex::nest_last_join(Session *session)
   List<TableList> *embedded_list;
 
   if (!(ptr= (TableList*) session->calloc(ALIGN_SIZE(sizeof(TableList))+
-                                       sizeof(nested_join_st))))
+                                          sizeof(nested_join_st))))
     return NULL;
-  nested_join= ptr->nested_join=
-    ((nested_join_st*) ((unsigned char*) ptr + ALIGN_SIZE(sizeof(TableList))));
-
-  ptr->embedding= embedding;
-  ptr->join_list= join_list;
+  ptr->setNestedJoin(((nested_join_st*) ((unsigned char*) ptr + ALIGN_SIZE(sizeof(TableList)))));
+  nested_join= ptr->getNestedJoin();
+  ptr->setEmbedding(embedding);
+  ptr->setJoinList(join_list);
   ptr->alias= (char*) "(nest_last_join)";
   embedded_list= &nested_join->join_list;
   embedded_list->empty();
@@ -1102,8 +1133,8 @@ TableList *Select_Lex::nest_last_join(Session *session)
   for (uint32_t i=0; i < 2; i++)
   {
     TableList *table= join_list->pop();
-    table->join_list= embedded_list;
-    table->embedding= ptr;
+    table->setJoinList(embedded_list);
+    table->setEmbedding(ptr);
     embedded_list->push_back(table);
     if (table->natural_join)
     {
@@ -1139,8 +1170,8 @@ TableList *Select_Lex::nest_last_join(Session *session)
 void Select_Lex::add_joined_table(TableList *table)
 {
   join_list->push_front(table);
-  table->join_list= join_list;
-  table->embedding= embedding;
+  table->setJoinList(join_list);
+  table->setEmbedding(embedding);
 }
 
 
@@ -1398,18 +1429,18 @@ kill_one_thread(Session *, ulong id, bool only_kill_query)
 {
   Session *tmp= NULL;
   uint32_t error= ER_NO_SUCH_THREAD;
-  pthread_mutex_lock(&LOCK_thread_count); // For unlink from list
+  LOCK_thread_count.lock(); // For unlink from list
   
   for (SessionList::iterator it= getSessionList().begin(); it != getSessionList().end(); ++it )
   {
     if ((*it)->thread_id == id)
     {
       tmp= *it;
-      pthread_mutex_lock(&tmp->LOCK_delete);	// Lock from delete
+      tmp->lockForDelete();
       break;
     }
   }
-  pthread_mutex_unlock(&LOCK_thread_count);
+  LOCK_thread_count.unlock();
   if (tmp)
   {
 
@@ -1419,7 +1450,7 @@ kill_one_thread(Session *, ulong id, bool only_kill_query)
       error= 0;
     }
 
-    pthread_mutex_unlock(&tmp->LOCK_delete);
+    tmp->unlockForDelete();
   }
   return(error);
 }
@@ -1589,7 +1620,13 @@ bool create_table_precheck(TableIdentifier &identifier)
 {
   if (not plugin::StorageEngine::canCreateTable(identifier))
   {
-    my_error(ER_DBACCESS_DENIED_ERROR, MYF(0), "", "", identifier.getSchemaName());
+    my_error(ER_DBACCESS_DENIED_ERROR, MYF(0), "", "", identifier.getSchemaName().c_str());
+    return true;
+  }
+
+  if (not plugin::StorageEngine::doesSchemaExist(identifier))
+  {
+    my_error(ER_BAD_DB_ERROR, MYF(0), identifier.getSchemaName().c_str());
     return true;
   }
 
