@@ -36,7 +36,9 @@ public:
   AlterSchema(Session *in_session)
     :
       CreateSchema(in_session)
-  { }
+  { 
+    schema_message.set_update_timestamp(time(NULL));
+  }
 
   bool execute();
 };
