@@ -39,10 +39,11 @@ namespace statement
 class AlterTable : public CreateTable
 {
 public:
-  AlterTable(Session *in_session)
-    :
-      CreateTable(in_session)
-  { }
+  AlterTable(Session *in_session) :
+    CreateTable(in_session)
+  { 
+    create_table_message.set_update_timestamp(time(NULL));
+  }
 
   bool execute();
 };
@@ -50,8 +51,11 @@ public:
 } /* namespace statement */
 
 
-bool alter_table(Session *session, char *new_db, char *new_name,
+bool alter_table(Session *session,
+                 drizzled::TableIdentifier &original_table_identifier,
+                 drizzled::TableIdentifier &new_table_identifier,
                  HA_CREATE_INFO *create_info,
+                 const message::Table &original_proto,
                  message::Table &create_proto,
                  TableList *table_list,
                  AlterInfo *alter_info,
