@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,23 +18,25 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_STATEMENT_COMMIT_H
+#define DRIZZLED_STATEMENT_COMMIT_H
 
 #include <drizzled/statement.h>
 
-namespace drizzled {
-namespace statement {
+namespace drizzled
+{
+class Session;
+
+namespace statement
+{
 
 class Commit : public Statement
 {
 public:
-  Commit(Session *in_session, bool tx_chain_arg, bool tx_release_arg) :
-    Statement(in_session),
-    tx_chain(tx_chain_arg),
-    tx_release(tx_release_arg)
-  {
-    set_command(SQLCOM_COMMIT);
-  }
+  Commit(Session *in_session)
+    :
+      Statement(in_session)
+  {}
 
   bool execute();
 
@@ -49,3 +51,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_STATEMENT_COMMIT_H */
