@@ -23,7 +23,9 @@
 #include <drizzled/lock.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/create_table.h>
-#include <drizzled/table_identifier.h>
+#include <drizzled/identifier.h>
+
+#include <iostream>
 
 namespace drizzled
 {
@@ -39,17 +41,17 @@ bool statement::CreateTable::execute()
   bool res= false;
   bool link_to_local= false;
   bool lex_identified_temp_table= 
-    create_table_proto.type() == message::Table::TEMPORARY;
+    create_table_message.type() == message::Table::TEMPORARY;
 
   if (is_engine_set)
   {
     create_info.db_type= 
-      plugin::StorageEngine::findByName(*session, create_table_proto.engine().name());
+      plugin::StorageEngine::findByName(*session, create_table_message.engine().name());
 
     if (create_info.db_type == NULL)
     {
       my_error(ER_UNKNOWN_STORAGE_ENGINE, MYF(0), 
-               create_table_proto.name().c_str());
+               create_table_message.engine().name().c_str());
 
       return true;
     }
@@ -59,16 +61,17 @@ bool statement::CreateTable::execute()
     create_info.db_type= session->getDefaultStorageEngine();
   }
 
+  if (not validateCreateTableOption())
+  {
+    return true;
+  }
 
   /* 
     Now we set the name in our Table proto so that it will match 
     create_info.db_type.
   */
   {
-    message::Table::StorageEngine *protoengine;
-
-    protoengine= create_table_proto.mutable_engine();
-    protoengine->set_name(create_info.db_type->getName());
+    create_table_message.mutable_engine()->set_name(create_info.db_type->getName());
   }
 
 
@@ -89,9 +92,12 @@ bool statement::CreateTable::execute()
     Now that we have the engine, we can figure out the table identifier. We need the engine in order
     to determine if the table is transactional or not if it is temp.
   */
+
+  create_table_message.set_schema(create_table->db);
+
   TableIdentifier new_table_identifier(create_table->db,
                                        create_table->table_name,
-                                       create_table_proto.type() != message::Table::TEMPORARY ? STANDARD_TABLE : TEMP_TABLE);
+                                       create_table_message.type());
 
   if (create_table_precheck(new_table_identifier))
   {
@@ -133,7 +139,7 @@ bool statement::CreateTable::execute()
     if (not lex_identified_temp_table)
     {
       session->lex->link_first_table_back(create_table, link_to_local);
-      create_table->create= true;
+      create_table->setCreate(true);
     }
 
     if (not (res= session->openTablesLock(session->lex->query_tables)))
@@ -167,7 +173,7 @@ bool statement::CreateTable::execute()
       if ((result= new select_create(create_table,
                                      is_if_not_exists,
                                      &create_info,
-                                     create_table_proto,
+                                     create_table_message,
                                      &alter_info,
                                      select_lex->item_list,
                                      session->lex->duplicates,
@@ -197,7 +203,7 @@ bool statement::CreateTable::execute()
                                    new_table_identifier,
                                    create_table, 
                                    select_tables,
-                                   create_table_proto,
+                                   create_table_message,
                                    is_if_not_exists,
                                    is_engine_set);
     }
@@ -206,7 +212,7 @@ bool statement::CreateTable::execute()
 
       for (int32_t x= 0; x < alter_info.alter_proto.added_field_size(); x++)
       {
-        message::Table::Field *field= create_table_proto.add_field();
+        message::Table::Field *field= create_table_message.add_field();
 
         *field= alter_info.alter_proto.added_field(x);
       }
@@ -214,12 +220,13 @@ bool statement::CreateTable::execute()
       res= mysql_create_table(session, 
                               new_table_identifier,
                               &create_info,
-                              create_table_proto,
+                              create_table_message,
                               &alter_info, 
                               false, 
                               0,
                               is_if_not_exists);
     }
+
     if (not res)
     {
       session->my_ok();
@@ -235,5 +242,30 @@ bool statement::CreateTable::execute()
   return res;
 }
 
+bool statement::CreateTable::validateCreateTableOption()
+{
+  bool rc= true;
+  size_t num_engine_options= create_table_message.engine().options_size();
+
+  assert(create_info.db_type);
+
+  for (size_t y= 0; y < num_engine_options; ++y)
+  {
+    bool valid= create_info.db_type->validateCreateTableOption(create_table_message.engine().options(y).name(),
+                                                               create_table_message.engine().options(y).state());
+
+    if (not valid)
+    {
+      my_error(ER_UNKNOWN_ENGINE_OPTION, MYF(0),
+               create_table_message.engine().options(y).name().c_str(),
+               create_table_message.engine().options(y).state().c_str());
+
+      rc= false;
+    }
+  }
+
+  return rc;
+}
+
 } /* namespace drizzled */
 
