@@ -23,6 +23,7 @@
 #include <drizzled/session.h>
 #include <drizzled/statement/drop_schema.h>
 #include <drizzled/db.h>
+#include <drizzled/plugin/event_observer.h>
 
 #include <string>
 
@@ -37,9 +38,10 @@ bool statement::DropSchema::execute()
   {
     return true;
   }
-  if (check_db_name(&session->lex->name))
+  SchemaIdentifier schema_identifier(string(session->lex->name.str, session->lex->name.length));
+  if (not check_db_name(session, schema_identifier))
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), session->lex->name.str);
+    my_error(ER_WRONG_DB_NAME, MYF(0), schema_identifier.getSQLPath().c_str());
     return false;
   }
   if (session->inTransaction())
@@ -49,7 +51,23 @@ bool statement::DropSchema::execute()
         MYF(0));
     return true;
   }
-  bool res= mysql_rm_db(session, session->lex->name.str, drop_if_exists);
+  
+  bool res = true;
+  if (unlikely(plugin::EventObserver::beforeDropDatabase(*session, schema_identifier.getSQLPath()))) 
+  {
+    my_error(ER_EVENT_OBSERVER_PLUGIN, MYF(0), schema_identifier.getSQLPath().c_str());
+  }
+  else
+  {
+    res= mysql_rm_db(session, schema_identifier, drop_if_exists);
+    if (unlikely(plugin::EventObserver::afterDropDatabase(*session, schema_identifier.getSQLPath(), res)))
+    {
+      my_error(ER_EVENT_OBSERVER_PLUGIN, MYF(0), schema_identifier.getSQLPath().c_str());
+      res = false;
+    }
+
+  }
+
   return res;
 }
 
