@@ -59,9 +59,9 @@ static bool mysql_rm_table(Session *session, TableList *tables, bool if_exists,
 
   /* mark for close and remove all cached entries */
 
-  if (! drop_temporary)
+  if (not drop_temporary)
   {
-    if (!(need_start_waiting= !wait_if_global_read_lock(session, false, true)))
+    if (not (need_start_waiting= !wait_if_global_read_lock(session, false, true)))
       return true;
   }
 
@@ -88,13 +88,15 @@ bool statement::DropTable::execute()
   TableList *first_table= (TableList *) session->lex->select_lex.table_list.first;
   TableList *all_tables= session->lex->query_tables;
   assert(first_table == all_tables && first_table != 0);
-  if (! drop_temporary)
+
+  if (not drop_temporary)
   {
-    if (! session->endActiveTransaction())
+    if (not session->endActiveTransaction())
     {
       return true;
     }
   }
+
   /* DDL and binlog write order protected by LOCK_open */
   bool res= mysql_rm_table(session,
                            first_table,
