@@ -71,10 +71,10 @@ bool statement::RenameTable::renameTables(TableList *table_list)
   if (wait_if_global_read_lock(session, 0, 1))
     return true;
 
-  pthread_mutex_lock(&LOCK_open); /* Rename table lock for exclusive access */
+  LOCK_open.lock(); /* Rename table lock for exclusive access */
   if (lock_table_names_exclusively(session, table_list))
   {
-    pthread_mutex_unlock(&LOCK_open);
+    LOCK_open.unlock();
     goto err;
   }
 
@@ -105,7 +105,7 @@ bool statement::RenameTable::renameTables(TableList *table_list)
     higher concurrency - query_cache_invalidate can take minutes to
     complete.
   */
-  pthread_mutex_unlock(&LOCK_open);
+  LOCK_open.unlock();
 
   /* Lets hope this doesn't fail as the result will be messy */
   if (! error)
@@ -114,9 +114,9 @@ bool statement::RenameTable::renameTables(TableList *table_list)
     session->my_ok();
   }
 
-  pthread_mutex_lock(&LOCK_open); /* unlock all tables held */
+  LOCK_open.lock(); /* unlock all tables held */
   unlock_table_names(table_list, NULL);
-  pthread_mutex_unlock(&LOCK_open);
+  LOCK_open.unlock();
 
 err:
   start_waiting_global_read_lock(session);
@@ -154,9 +154,9 @@ bool statement::RenameTable::rename(TableList *ren_table,
   plugin::StorageEngine *engine= NULL;
   message::Table table_proto;
 
-  TableIdentifier old_identifier(ren_table->db, old_alias, STANDARD_TABLE);
+  TableIdentifier old_identifier(ren_table->db, old_alias, message::Table::STANDARD);
 
-  if (plugin::StorageEngine::getTableDefinition(*session, old_identifier, &table_proto) != EEXIST)
+  if (plugin::StorageEngine::getTableDefinition(*session, old_identifier, table_proto) != EEXIST)
   {
     my_error(ER_NO_SUCH_TABLE, MYF(0), ren_table->db, old_alias);
     return true;
@@ -164,16 +164,14 @@ bool statement::RenameTable::rename(TableList *ren_table,
 
   engine= plugin::StorageEngine::findByName(*session, table_proto.engine().name());
 
-  TableIdentifier new_identifier(new_db, new_alias, STANDARD_TABLE);
+  TableIdentifier new_identifier(new_db, new_alias, message::Table::STANDARD);
   if (plugin::StorageEngine::doesTableExist(*session, new_identifier))
   {
     my_error(ER_TABLE_EXISTS_ERROR, MYF(0), new_alias);
     return 1; // This can't be skipped
   }
 
-  rc= mysql_rename_table(engine,
-                         ren_table->db, old_alias,
-                         new_db, new_alias, 0);
+  rc= mysql_rename_table(*session, engine, old_identifier, new_identifier);
   if (rc && ! skip_error)
     return true;
 
