@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/rollback.h>
@@ -26,27 +26,15 @@
 namespace drizzled
 {
 
-namespace statement
+bool statement::Rollback::execute()
 {
-
-Rollback::Rollback(Session *in_session, bool tx_chain_arg, bool tx_release_arg) :
-  Statement(in_session),
-  tx_chain(tx_chain_arg),
-  tx_release(tx_release_arg)
-  {
-    set_command(SQLCOM_ROLLBACK);
-  }
-
-bool Rollback::execute()
-{
-  if (not session().endTransaction(tx_release ? ROLLBACK_RELEASE : tx_chain ? ROLLBACK_AND_CHAIN : ROLLBACK))
+  if (! session->endTransaction(tx_release ? ROLLBACK_RELEASE : tx_chain ? ROLLBACK_AND_CHAIN : ROLLBACK))
   {
     return true;
   }
-  session().my_ok();
-
+  session->my_ok();
   return false;
 }
 
-} /* namespace statement */
 } /* namespace drizzled */
+
