@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2009 Sun Microsystems, Inc.
+ *  Copyright (C) 2009 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,26 +18,21 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
+#include "config.h"
 #include <drizzled/session.h>
 #include <drizzled/statement/select.h>
-#include <drizzled/sql_lex.h>
-#include <drizzled/statistics_variables.h>
 
-namespace drizzled {
-namespace statement {
-
-Select::Select(Session *in_session) :
-  Statement(in_session)
-  {
-    set_command(SQLCOM_SELECT);
-  }
-
-bool Select::execute()
+namespace drizzled
 {
-  session().status_var.last_query_cost= 0.0;
-  return execute_sqlcom_select(&session(), lex().query_tables);
+
+bool statement::Select::execute()
+{
+  TableList *all_tables= session->lex->query_tables;
+  session->status_var.last_query_cost= 0.0;
+  bool res= execute_sqlcom_select(session, all_tables);
+
+  return res;
 }
 
-} /* namespace statement */
 } /* namespace drizzled */
+
