@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *  Copyright (C) 2010 Joseph Daly 
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,22 +18,25 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_STATUS_HELPER_H
+#define DRIZZLED_STATUS_HELPER_H
 
-#include <drizzled/plugin.h>
-#include <drizzled/statistics_variables.h>
+#include "drizzled/plugin.h"
+#include "drizzled/statistics_variables.h"
 
-#include <drizzled/visibility.h>
+namespace drizzled
+{
 
-namespace drizzled {
-
-class DRIZZLED_API StatusHelper 
+class StatusHelper 
 {
 public:
 
-  static std::string fillHelper(system_status_var*, const char* value, SHOW_TYPE);
-  static drizzle_show_var status_vars_defs[];
+  static std::string fillHelper(drizzled::system_status_var *status_var, char *value, 
+                                SHOW_TYPE show_type);
+
+  static drizzled::drizzle_show_var status_vars_defs[];
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_STATUS_HELPER_H */
