@@ -32,19 +32,22 @@
 #include <drizzled/message/table.pb.h>
 #include <google/protobuf/io/zero_copy_stream.h>
 #include <google/protobuf/io/zero_copy_stream_impl.h>
+#include <google/protobuf/message.h>
 
 #include <drizzled/table_proto.h>
+#include <drizzled/charset.h>
+
+#include "drizzled/function/time/typecast.h"
 
 using namespace std;
 
 namespace drizzled {
 
 static int fill_table_proto(message::Table &table_proto,
-                            const char *table_name,
                             List<CreateField> &create_fields,
                             HA_CREATE_INFO *create_info,
                             uint32_t keys,
-                            KEY *key_info)
+                            KeyInfo *key_info)
 {
   CreateField *field_arg;
   List_iterator<CreateField> it(create_fields);
@@ -59,8 +62,6 @@ static int fill_table_proto(message::Table &table_proto,
   assert(strcmp(table_proto.engine().name().c_str(),
 		create_info->db_type->getName().c_str())==0);
 
-  assert(strcmp(table_proto.name().c_str(),table_name)==0);
-
   int field_number= 0;
   bool use_existing_fields= table_proto.field_size() > 0;
   while ((field_arg= it++))
@@ -78,7 +79,7 @@ static int fill_table_proto(message::Table &table_proto,
       /* Other code paths still have to fill out the proto */
       attribute= table_proto.add_field();
 
-      if(field_arg->flags & NOT_NULL_FLAG)
+      if (field_arg->flags & NOT_NULL_FLAG)
       {
         message::Table::Field::FieldConstraints *constraints;
 
@@ -148,21 +149,20 @@ static int fill_table_proto(message::Table &table_proto,
       }
     case message::Table::Field::ENUM:
       {
-        message::Table::Field::SetFieldOptions *set_field_options;
+        message::Table::Field::EnumerationValues *enumeration_options;
 
         assert(field_arg->interval);
 
-        set_field_options= attribute->mutable_set_options();
+        enumeration_options= attribute->mutable_enumeration_values();
 
         for (uint32_t pos= 0; pos < field_arg->interval->count; pos++)
         {
           const char *src= field_arg->interval->type_names[pos];
 
-          set_field_options->add_field_value(src);
+          enumeration_options->add_field_value(src);
         }
-        set_field_options->set_count_elements(set_field_options->field_value_size());
-	set_field_options->set_collation_id(field_arg->charset->number);
-        set_field_options->set_collation(field_arg->charset->name);
+	enumeration_options->set_collation_id(field_arg->charset->number);
+        enumeration_options->set_collation(field_arg->charset->name);
         break;
       }
     case message::Table::Field::BLOB:
@@ -184,23 +184,6 @@ static int fill_table_proto(message::Table &table_proto,
     constraints->set_is_nullable(field_arg->def->null_value);
 #endif
 
-    switch(field_arg->column_format())
-    {
-    case COLUMN_FORMAT_TYPE_NOT_USED:
-      break;
-    case COLUMN_FORMAT_TYPE_DEFAULT:
-      attribute->set_format(message::Table::Field::DefaultFormat);
-      break;
-    case COLUMN_FORMAT_TYPE_FIXED:
-      attribute->set_format(message::Table::Field::FixedFormat);
-      break;
-    case COLUMN_FORMAT_TYPE_DYNAMIC:
-      attribute->set_format(message::Table::Field::DynamicFormat);
-      break;
-    default:
-      assert(0); /* Tell us, since this shouldn't happend */
-    }
-
     if (field_arg->comment.length)
     {
       uint32_t tmp_len;
@@ -224,27 +207,27 @@ static int fill_table_proto(message::Table &table_proto,
       assert(strcmp(attribute->comment().c_str(), field_arg->comment.str)==0);
     }
 
-    if(field_arg->unireg_check == Field::NEXT_NUMBER)
+    if (field_arg->unireg_check == Field::NEXT_NUMBER)
     {
       message::Table::Field::NumericFieldOptions *field_options;
       field_options= attribute->mutable_numeric_options();
       field_options->set_is_autoincrement(true);
     }
 
-    if(field_arg->unireg_check == Field::TIMESTAMP_DN_FIELD
+    if (field_arg->unireg_check == Field::TIMESTAMP_DN_FIELD
        || field_arg->unireg_check == Field::TIMESTAMP_DNUN_FIELD)
     {
       message::Table::Field::FieldOptions *field_options;
       field_options= attribute->mutable_options();
-      field_options->set_default_value("NOW()");
+      field_options->set_default_expression("CURRENT_TIMESTAMP");
     }
 
-    if(field_arg->unireg_check == Field::TIMESTAMP_UN_FIELD
+    if (field_arg->unireg_check == Field::TIMESTAMP_UN_FIELD
        || field_arg->unireg_check == Field::TIMESTAMP_DNUN_FIELD)
     {
       message::Table::Field::FieldOptions *field_options;
       field_options= attribute->mutable_options();
-      field_options->set_update_value("NOW()");
+      field_options->set_update_expression("CURRENT_TIMESTAMP");
     }
 
     if (field_arg->def == NULL  && attribute->constraints().is_nullable())
@@ -254,12 +237,12 @@ static int fill_table_proto(message::Table &table_proto,
 
       field_options->set_default_null(true);
     }
-    if(field_arg->def)
+    if (field_arg->def)
     {
       message::Table::Field::FieldOptions *field_options;
       field_options= attribute->mutable_options();
  
-      if(field_arg->def->is_null())
+      if (field_arg->def->is_null())
       {
 	field_options->set_default_null(true);
       }
@@ -270,7 +253,7 @@ static int fill_table_proto(message::Table &table_proto,
 
 	assert(default_value);
 
-	if((field_arg->sql_type==DRIZZLE_TYPE_VARCHAR
+	if ((field_arg->sql_type==DRIZZLE_TYPE_VARCHAR
 	   || field_arg->sql_type==DRIZZLE_TYPE_BLOB)
 	   && ((field_arg->length / field_arg->charset->mbmaxlen)
 	   < default_value->length()))
@@ -279,7 +262,44 @@ static int fill_table_proto(message::Table &table_proto,
 	  return 1;
 	}
 
-	if((field_arg->sql_type==DRIZZLE_TYPE_VARCHAR
+        if (field_arg->sql_type == DRIZZLE_TYPE_DATE
+            || field_arg->sql_type == DRIZZLE_TYPE_DATETIME
+            || field_arg->sql_type == DRIZZLE_TYPE_TIMESTAMP)
+        {
+          DRIZZLE_TIME ltime;
+
+          if (field_arg->def->get_date(&ltime, TIME_FUZZY_DATE))
+          {
+            my_error(ER_INVALID_DATETIME_VALUE, MYF(ME_FATALERROR),
+                     default_value->c_str());
+            return 1;
+          }
+
+          /* We now do the casting down to the appropriate type.
+
+             Yes, this implicit casting is balls.
+             It was previously done on reading the proto back in,
+             but we really shouldn't store the bogus things in the proto,
+             and instead do the casting behaviour here.
+
+             the timestamp errors are taken care of elsewhere.
+          */
+
+          if (field_arg->sql_type == DRIZZLE_TYPE_DATETIME)
+          {
+            Item *typecast= new Item_datetime_typecast(field_arg->def);
+            typecast->quick_fix_field();
+            typecast->val_str(default_value);
+          }
+          else if (field_arg->sql_type == DRIZZLE_TYPE_DATE)
+          {
+            Item *typecast= new Item_date_typecast(field_arg->def);
+            typecast->quick_fix_field();
+            typecast->val_str(default_value);
+          }
+        }
+
+	if ((field_arg->sql_type==DRIZZLE_TYPE_VARCHAR
 	    && field_arg->charset==&my_charset_bin)
 	   || (field_arg->sql_type==DRIZZLE_TYPE_BLOB
 	    && field_arg->charset==&my_charset_bin))
@@ -306,36 +326,12 @@ static int fill_table_proto(message::Table &table_proto,
 
   assert(! use_existing_fields || (field_number == table_proto.field_size()));
 
-  switch(create_info->row_type)
-  {
-  case ROW_TYPE_DEFAULT:
-    /* No use setting a default row type... just adds redundant info to message */
-    break;
-  case ROW_TYPE_FIXED:
-    table_options->set_row_type(message::Table::TableOptions::ROW_TYPE_FIXED);
-    break;
-  case ROW_TYPE_DYNAMIC:
-    table_options->set_row_type(message::Table::TableOptions::ROW_TYPE_DYNAMIC);
-    break;
-  case ROW_TYPE_COMPRESSED:
-    table_options->set_row_type(message::Table::TableOptions::ROW_TYPE_COMPRESSED);
-    break;
-  case ROW_TYPE_REDUNDANT:
-    table_options->set_row_type(message::Table::TableOptions::ROW_TYPE_REDUNDANT);
-    break;
-  case ROW_TYPE_COMPACT:
-    table_options->set_row_type(message::Table::TableOptions::ROW_TYPE_COMPACT);
-    break;
-  case ROW_TYPE_PAGE:
-    table_options->set_row_type(message::Table::TableOptions::ROW_TYPE_PAGE);
-    break;
-  default:
-    abort();
-  }
-
   if (create_info->table_options & HA_OPTION_PACK_RECORD)
     table_options->set_pack_record(true);
 
+  if (table_options->has_comment() && table_options->comment().length() == 0)
+    table_options->clear_comment();
+
   if (table_options->has_comment())
   {
     uint32_t tmp_len;
@@ -361,6 +357,11 @@ static int fill_table_proto(message::Table &table_proto,
     table_options->set_collation(create_info->default_table_charset->name);
   }
 
+  if (create_info->used_fields & HA_CREATE_USED_AUTO)
+    table_options->set_has_user_set_auto_increment_value(true);
+  else
+    table_options->set_has_user_set_auto_increment_value(false);
+
   if (create_info->auto_increment_value)
     table_options->set_auto_increment_value(create_info->auto_increment_value);
 
@@ -377,7 +378,7 @@ static int fill_table_proto(message::Table &table_proto,
 
     idx->set_key_length(key_info[i].key_length);
 
-    if(is_primary_key_name(key_info[i].name))
+    if (is_primary_key_name(key_info[i].name))
       idx->set_is_primary(true);
     else
       idx->set_is_primary(false);
@@ -405,27 +406,27 @@ static int fill_table_proto(message::Table &table_proto,
     else
       idx->set_is_unique(false);
 
-    message::Table::Index::IndexOptions *index_options= idx->mutable_options();
+    message::Table::Index::Options *index_options= idx->mutable_options();
 
-    if(key_info[i].flags & HA_USES_BLOCK_SIZE)
+    if (key_info[i].flags & HA_USES_BLOCK_SIZE)
       index_options->set_key_block_size(key_info[i].block_size);
 
-    if(key_info[i].flags & HA_PACK_KEY)
+    if (key_info[i].flags & HA_PACK_KEY)
       index_options->set_pack_key(true);
 
-    if(key_info[i].flags & HA_BINARY_PACK_KEY)
+    if (key_info[i].flags & HA_BINARY_PACK_KEY)
       index_options->set_binary_pack_key(true);
 
-    if(key_info[i].flags & HA_VAR_LENGTH_PART)
+    if (key_info[i].flags & HA_VAR_LENGTH_PART)
       index_options->set_var_length_key(true);
 
-    if(key_info[i].flags & HA_NULL_PART_KEY)
+    if (key_info[i].flags & HA_NULL_PART_KEY)
       index_options->set_null_part_key(true);
 
-    if(key_info[i].flags & HA_KEY_HAS_PART_KEY_SEG)
+    if (key_info[i].flags & HA_KEY_HAS_PART_KEY_SEG)
       index_options->set_has_partial_segments(true);
 
-    if(key_info[i].flags & HA_GENERATED_KEY)
+    if (key_info[i].flags & HA_GENERATED_KEY)
       index_options->set_auto_generated_key(true);
 
     if (key_info[i].flags & HA_USES_COMMENT)
@@ -456,42 +457,61 @@ static int fill_table_proto(message::Table &table_proto,
     for(unsigned int j=0; j< key_info[i].key_parts; j++)
     {
       message::Table::Index::IndexPart *idxpart;
+      const int fieldnr= key_info[i].key_part[j].fieldnr;
+      int mbmaxlen= 1;
 
       idxpart= idx->add_index_part();
 
-      idxpart->set_fieldnr(key_info[i].key_part[j].fieldnr);
+      idxpart->set_fieldnr(fieldnr);
 
-      idxpart->set_compare_length(key_info[i].key_part[j].length);
+      if (table_proto.field(fieldnr).type() == message::Table::Field::VARCHAR
+          || table_proto.field(fieldnr).type() == message::Table::Field::BLOB)
+      {
+        uint32_t collation_id;
 
-      idxpart->set_key_type(key_info[i].key_part[j].key_type);
+        if (table_proto.field(fieldnr).string_options().has_collation_id())
+          collation_id= table_proto.field(fieldnr).string_options().collation_id();
+        else
+          collation_id= table_proto.options().collation_id();
 
+        const CHARSET_INFO *cs= get_charset(collation_id);
+
+        mbmaxlen= cs->mbmaxlen;
+      }
+
+      idxpart->set_compare_length(key_info[i].key_part[j].length / mbmaxlen);
+    }
+  }
+
+  if (not table_proto.IsInitialized())
+  {
+    my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0), table_proto.InitializationErrorString().c_str());
+    return 1;
+  }
+
+  /*
+    Here we test to see if we can validate the Table Message before we continue. 
+    We do this by serializing the protobuffer.
+  */
+  {
+    string tmp_string;
+
+    try {
+      table_proto.SerializeToString(&tmp_string);
+    }
+
+    catch (...)
+    {
+      my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0),
+               table_proto.InitializationErrorString().empty() ? "": table_proto.InitializationErrorString().c_str());
+
+      return 1;
     }
   }
 
   return 0;
 }
 
-int rename_table_proto_file(const char *from, const char* to)
-{
-  string from_path(from);
-  string to_path(to);
-  string file_ext = ".dfe";
-
-  from_path.append(file_ext);
-  to_path.append(file_ext);
-
-  return internal::my_rename(from_path.c_str(),to_path.c_str(),MYF(MY_WME));
-}
-
-int delete_table_proto_file(const char *file_name)
-{
-  string new_path(file_name);
-  string file_ext = ".dfe";
-
-  new_path.append(file_ext);
-  return internal::my_delete(new_path.c_str(), MYF(0));
-}
-
 /*
   Create a table definition proto file and the tables
 
@@ -511,25 +531,28 @@ int delete_table_proto_file(const char *file_name)
     1  error
 */
 
-int rea_create_table(Session *session,
-                     TableIdentifier &identifier,
-		     message::Table &table_proto,
-                     HA_CREATE_INFO *create_info,
-                     List<CreateField> &create_fields,
-                     uint32_t keys, KEY *key_info)
+bool rea_create_table(Session *session,
+                      TableIdentifier &identifier,
+                      message::Table &table_proto,
+                      HA_CREATE_INFO *create_info,
+                      List<CreateField> &create_fields,
+                      uint32_t keys, KeyInfo *key_info)
 {
-  if (fill_table_proto(table_proto, identifier.getTableName(), create_fields, create_info,
-		      keys, key_info))
-    return 1;
+  assert(table_proto.has_name());
+  if (fill_table_proto(table_proto, create_fields, create_info,
+                       keys, key_info))
+    return false;
+
+  assert(table_proto.name() == identifier.getTableName());
 
   if (plugin::StorageEngine::createTable(*session,
                                          identifier,
-                                         false, table_proto))
+                                         table_proto))
   {
-    return 1;
+    return false;
   }
 
-  return 0;
+  return true;
 
 } /* rea_create_table */
 
