@@ -37,18 +37,16 @@
 #include "config.h"
 
 #include "drizzled/charset_info.h"
-#include "drizzled/my_decimal.h"
+#include "drizzled/decimal.h"
 #include "drizzled/calendar.h"
 #include "drizzled/temporal.h"
-#ifdef NOTYETIMPLEMENTED
-#include "drizzled/temporal_interval.h"
-#endif
 #include "drizzled/temporal_format.h"
 #include "drizzled/time_functions.h"
 #include "time.h"
 
 #include <time.h>
 
+#include <cstdio>
 #include <ostream>
 #include <iomanip>
 #include <vector>
@@ -83,6 +81,44 @@ uint64_t Temporal::_cumulative_seconds_in_time() const
       + _seconds);
 }
 
+#if defined(TARGET_OS_SOLARIS)
+/* @TODO: Replace this with Boost.DateTime */
+static time_t timegm(struct tm *my_time)
+{
+	time_t local_secs, gm_secs;
+	struct tm gm__rec, *gm_time;
+
+	// Interpret 't' as the local time and convert it to seconds since the Epoch
+	local_secs = mktime(my_time);
+	if (local_secs == -1)
+  {
+		my_time->tm_hour--;
+		local_secs = mktime (my_time);
+		if (local_secs == -1)
+			return -1; 
+		local_secs += 3600;
+	}
+	
+	// Get the gmtime based on the local seconds since the Epoch
+	gm_time = gmtime_r(&local_secs, &gm__rec);
+	gm_time->tm_isdst = 0;
+	
+	// Interpret gmtime as the local time and convert it to seconds since the Epoch
+	gm_secs = mktime (gm_time);
+	if (gm_secs == -1)
+  {
+		gm_time->tm_hour--;
+		gm_secs = mktime (gm_time);
+		if (gm_secs == -1)
+			return -1; 
+		gm_secs += 3600;
+	}
+	
+	// Return the local time adjusted by the difference from GM time.
+	return (local_secs - (gm_secs - local_secs));
+}
+#endif
+
 void Temporal::set_epoch_seconds()
 {
   /* 
@@ -795,199 +831,7 @@ Date& Date::operator+=(const DateTime &rhs)
 
   return *this;
 }
-#ifdef NOTYETIMPLEMENTED
-Date& Date::operator+=(const TemporalIntervalYear &rhs)
-{
-  /* Simple one...add the years and adjust for any leaps */
-  int64_t new_years= _years;
-  new_years+= rhs._years;
-  if (new_years > DRIZZLE_MAX_YEARS_SQL)
-  {
-    /* 
-     * Set everything to zero. We got an overflow.
-     * @TODO Exceptions would be great here...
-     */
-    _reset();
-    _overflow= true;
-    return *this;
-  }
-  _years= (uint32_t) new_years;
-  if (_months == 2 && _days == 29 && days_in_gregorian_year_month(_years, _months) != 366)
-    _days= 28;
-  return *this;
-} 
 
-Date& Date::operator-=(const TemporalIntervalYear &rhs)
-{
-  /* Simple one...subtract the years and adjust for any leaps */
-  int64_t new_years= _years;
-  new_years-= rhs._years;
-  if (new_years < 0)
-  {
-    /* 
-     * Set everything to zero. We got an overflow.
-     * @TODO Exceptions would be great here...
-     */
-    _reset();
-    _overflow= true;
-    return *this;
-  }
-  _years= (uint32_t) new_years;
-  if (_months == 2 && _days == 29 && days_in_gregorian_year_month(_years, _months) != 366)
-    _days= 28;
-  return *this;
-} 
-
-Date& Date::operator+=(const TemporalIntervalDayOrWeek &rhs)
-{
-  /* Simple one...add the days */
-  int64_t julian_day= julian_day_number_from_gregorian_date(_years, _months, _days) + rhs._days;
-  gregorian_date_from_julian_day_number(julian_day, &_years, &_months, &_days);
-  return *this;
-} 
-
-Date& Date::operator-=(const TemporalIntervalDayOrWeek &rhs)
-{
-  /* Simple one...subtract the days */
-  int64_t julian_day= julian_day_number_from_gregorian_date(_years, _months, _days) - rhs._days;
-  gregorian_date_from_julian_day_number(julian_day, &_years, &_months, &_days);
-  return *this;
-} 
-
-Date& Date::operator+=(const TemporalIntervalYearMonth &rhs)
-{
-  /* Simple one...add the months in the period adjust */
-  int64_t period= (_years * 12) + (rhs._years * 12) + (_months - 1) + rhs._months;
-  int64_t new_years= (period / 12);
-  if (new_years > DRIZZLE_MAX_YEARS_SQL)
-  {
-    /* 
-     * Set everything to zero. We got an overflow.
-     * @TODO Exceptions would be great here...
-     */
-    _reset();
-    _overflow= true;
-    return *this;
-  }
-  _years= (uint32_t) new_years;
-  _months= (uint32_t) (period % 12) + 1;
-  
-  /* Adjust day if the new month doesn't have enough days */
-  uint32_t days_in_new_month= days_in_gregorian_year_month(_years, _months);
-  if (_days > days_in_new_month)
-    _days= days_in_new_month;
-  return *this;
-} 
-
-Date& Date::operator-=(const TemporalIntervalYearMonth &rhs)
-{
-  /* Simple one...subtract the months in the period and adjust */
-  int64_t period= (_years * 12) - (rhs._years * 12) + (_months - 1) - rhs._months;
-  int64_t new_years= (period / 12);
-  if (new_years < 0)
-  {
-    /* 
-     * Set everything to zero. We got an overflow.
-     * @TODO Exceptions would be great here...
-     */
-    _reset();
-    _overflow= true;
-    return *this;
-  }
-  _years= (uint32_t) (period / 12);
-  _months= (uint32_t) (period % 12) + 1;
-  
-  /* Adjust day if the new month doesn't have enough days */
-  uint32_t days_in_new_month= days_in_gregorian_year_month(_years, _months);
-  if (_days > days_in_new_month)
-    _days= days_in_new_month;
-  return *this;
-} 
-
-Date& Date::operator+=(const TemporalIntervalDayOrLess &rhs)
-{
-  /* 
-   * Convert the temporal and the interval into a number of 
-   * microseconds, then add them together and convert the
-   * resulting microseconds back into a broken-down temporal
-   * component.
-   */
-  int64_t new_seconds;
-  int64_t new_microseconds;
-  int64_t extra_sec;
-  int64_t new_days;
-  new_microseconds= _useconds + rhs._useconds;
-  extra_sec= new_microseconds / INT64_C(1000000);
-  new_microseconds= new_microseconds % INT64_C(1000000);
-
-  new_seconds= ((_days - 1) * 3600 * 24) + (_hours * 3600) + (_minutes * 60) + _seconds;
-  new_seconds+= (rhs._days * 3600 * 24) + (rhs._hours * 3600) + (rhs._minutes * 60) + rhs._seconds;
-  new_seconds+= extra_sec;
-
-  if (new_microseconds < 0)
-  {
-    new_microseconds+= INT64_C(1000000);
-    new_seconds--;
-  }
-  
-  new_days= new_seconds / (3600 * 24L);
-  new_seconds-= new_days * 3600 * 24L;
-  if (new_seconds < 0)
-  {
-    new_days--;
-    new_seconds+= 3600 * 24L;
-  }
-  _useconds= (uint32_t) new_microseconds;
-  _seconds= (uint32_t) (new_seconds % 60);
-  _minutes= (uint32_t) ((new_seconds / 60) % 60);
-  _hours= (uint32_t) (new_seconds / 3600);
-  int64_t julian_day= julian_day_number_from_gregorian_date(_years, _months, 1) + new_days;
-  gregorian_date_from_julian_day_number(julian_day, &_years, &_months, &_days);
-  return *this;
-}
-
-Date& Date::operator-=(const TemporalIntervalDayOrLess &rhs)
-{
-  /* 
-   * Convert the temporal and the interval into a number of 
-   * microseconds, then subtract them from each other and convert 
-   * the resulting microseconds back into a broken-down temporal
-   * component.
-   */
-  int64_t new_seconds;
-  int64_t new_microseconds;
-  int64_t extra_sec;
-  int64_t new_days;
-  new_microseconds= _useconds - rhs._useconds;
-  extra_sec= new_microseconds / INT64_C(1000000);
-  new_microseconds= new_microseconds % INT64_C(1000000);
-
-  new_seconds= ((_days - 1) * 3600 * 24) + (_hours * 3600) + (_minutes * 60) + _seconds;
-  new_seconds-= (rhs._days * 3600 * 24) + (rhs._hours * 3600) + (rhs._minutes * 60) + rhs._seconds;
-  new_seconds+= extra_sec;
-
-  if (new_microseconds < 0)
-  {
-    new_microseconds+= INT64_C(1000000);
-    new_seconds--;
-  }
-  
-  new_days= new_seconds / (3600 * 24L);
-  new_seconds-= new_days * 3600 * 24L;
-  if (new_seconds < 0)
-  {
-    new_days--;
-    new_seconds+= 3600 * 24L;
-  }
-  _useconds= (uint32_t) new_microseconds;
-  _seconds= (uint32_t) (new_seconds % 60);
-  _minutes= (uint32_t) ((new_seconds / 60) % 60);
-  _hours= (uint32_t) (new_seconds / 3600);
-  int64_t julian_day= julian_day_number_from_gregorian_date(_years, _months, 1) + new_days;
-  gregorian_date_from_julian_day_number(julian_day, &_years, &_months, &_days);
-  return *this;
-}
-#endif /* NOTYETIMPLEMENTED */
 /*
  * Comparison operators between a Date and a Timestamp
  */
@@ -1019,11 +863,11 @@ bool Date::operator<=(const Timestamp& rhs)
 }
 bool Date::operator>(const Timestamp& rhs)
 {
-  return ! (*this < rhs);
+  return ! (*this <= rhs);
 }
 bool Date::operator>=(const Timestamp& rhs)
 {
-  return ! (*this <= rhs);
+  return ! (*this < rhs);
 }
 /*
  * Comparison operators between a Timestamp and a Date
@@ -1056,11 +900,11 @@ bool Timestamp::operator<=(const Date& rhs)
 }
 bool Timestamp::operator>(const Date& rhs)
 {
-  return ! (*this < rhs);
+  return ! (*this <= rhs);
 }
 bool Timestamp::operator>=(const Date& rhs)
 {
-  return ! (*this <= rhs);
+  return ! (*this < rhs);
 }
 /*
  * Comparison operators between a Timestamp and a DateTime
@@ -1109,11 +953,11 @@ bool Timestamp::operator<=(const DateTime& rhs)
 }
 bool Timestamp::operator>(const DateTime& rhs)
 {
-  return ! (*this < rhs);
+  return ! (*this <= rhs);
 }
 bool Timestamp::operator>=(const DateTime& rhs)
 {
-  return ! (*this <= rhs);
+  return ! (*this < rhs);
 }
 /*
  * Comparison operators between two Timestamps
@@ -1136,11 +980,11 @@ bool Timestamp::operator<=(const Timestamp& rhs)
 }
 bool Timestamp::operator>(const Timestamp& rhs)
 {
-  return ! (*this < rhs);
+  return ! (*this <= rhs);
 }
 bool Timestamp::operator>=(const Timestamp& rhs)
 {
-  return ! (*this <= rhs);
+  return ! (*this < rhs);
 }
 
 /**
@@ -1341,9 +1185,9 @@ bool Date::from_int32_t(const int32_t from)
 bool Time::from_int32_t(const int32_t from)
 {
   uint32_t copy_from= (uint32_t) from;
-  _hours= copy_from % INT32_C(10000);
-  _minutes= copy_from % INT32_C(100);
-  _seconds= copy_from & 3; /* Masks off all but last 2 digits */
+  _hours= copy_from / INT32_C(10000);
+  _minutes= (copy_from % INT32_C(10000)) / INT32_C(100);
+  _seconds= copy_from % INT32_C(100); /* Masks off all but last 2 digits */
   return is_valid();
 }
 
@@ -1382,7 +1226,7 @@ bool DateTime::from_int64_t(const int64_t from, bool convert)
     else if (copy_from <  DRIZZLE_YY_PART_YEAR * 10000000000LL + 101000000LL)
       return false;
     else if (copy_from <= 991231235959LL)
-      copy_from= copy_from + 19000000000000LL;		/* YYMMDDHHMMSS, 1970-1999 */
+      copy_from= copy_from + 19000000000000LL;    /* YYMMDDHHMMSS, 1970-1999 */
   }
 
   part1= (int64_t) (copy_from / 1000000LL);
@@ -1561,7 +1405,8 @@ bool DateTime::is_valid() const
 bool Timestamp::is_valid() const
 {
   return DateTime::is_valid() 
-      && in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds);
+      && in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds)
+      && (_seconds <= 59);
 }
 
 bool MicroTimestamp::is_valid() const
