@@ -18,22 +18,15 @@
 #ifndef DRIZZLED_THR_LOCK_H
 #define DRIZZLED_THR_LOCK_H
 
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/condition_variable.hpp>
 #include <pthread.h>
 
 namespace drizzled
 {
 
-struct st_thr_lock;
-extern uint32_t locks_immediate,locks_waited ;
-
-namespace internal
-{
-extern pthread_mutex_t THR_LOCK_lock;
-}
-
 extern uint64_t max_write_lock_count;
 extern uint64_t table_lock_wait_timeout;
-extern bool thr_lock_inited;
 
 
 enum thr_lock_type { TL_IGNORE=-1,
@@ -78,12 +71,21 @@ enum enum_thr_lock_result { THR_LOCK_SUCCESS= 0, THR_LOCK_ABORTED= 1,
   of an instance of this structure is used to uniquely identify the thread.
 */
 
-typedef struct st_thr_lock_info
+struct THR_LOCK_INFO
 {
   pthread_t thread;
   uint64_t thread_id;
   uint32_t n_cursors;
-} THR_LOCK_INFO;
+
+  THR_LOCK_INFO() : 
+    thread(0),
+    thread_id(0),
+    n_cursors(0)
+  { }
+
+  void init();
+
+};
 
 /*
   Lock owner identifier. Globally identifies the lock owner within the
@@ -91,27 +93,54 @@ typedef struct st_thr_lock_info
   structure is used as id.
 */
 
-typedef struct st_thr_lock_owner
+struct THR_LOCK_OWNER
 {
   THR_LOCK_INFO *info;
-} THR_LOCK_OWNER;
 
+  THR_LOCK_OWNER() :
+    info(0)
+  { }
 
-typedef struct st_thr_lock_data {
+};
+
+struct THR_LOCK;
+struct THR_LOCK_DATA;
+
+struct THR_LOCK_DATA {
   THR_LOCK_OWNER *owner;
-  struct st_thr_lock_data *next,**prev;
-  struct st_thr_lock *lock;
-  pthread_cond_t *cond;
+  struct THR_LOCK_DATA *next,**prev;
+  struct THR_LOCK *lock;
+  boost::condition_variable *cond;
   enum thr_lock_type type;
   void *status_param;			/* Param to status functions */
-} THR_LOCK_DATA;
+
+  THR_LOCK_DATA() :
+    owner(0),
+    next(0),
+    prev(0),
+    lock(0),
+    cond(0),
+    type(TL_UNLOCK),
+    status_param(0)
+  { }
+
+  void init(THR_LOCK *lock,
+            void *status_param= NULL);
+};
 
 struct st_lock_list {
   THR_LOCK_DATA *data,**last;
+
+  st_lock_list() :
+    data(0),
+    last(0)
+  { }
 };
 
-typedef struct st_thr_lock {
-  pthread_mutex_t mutex;
+struct THR_LOCK {
+private:
+  boost::mutex mutex;
+public:
   struct st_lock_list read_wait;
   struct st_lock_list read;
   struct st_lock_list write_wait;
@@ -119,26 +148,49 @@ typedef struct st_thr_lock {
   /* write_lock_count is incremented for write locks and reset on read locks */
   uint32_t write_lock_count;
   uint32_t read_no_write_count;
-  void (*get_status)(void*, int);	/* When one gets a lock */
-  void (*copy_status)(void*,void*);
-  void (*update_status)(void*);		/* Before release of write */
-  void (*restore_status)(void*);         /* Before release of read */
-  bool (*check_status)(void *);
-} THR_LOCK;
+
+  THR_LOCK() :
+    write_lock_count(0),
+    read_no_write_count(0)
+  { }
+
+  ~THR_LOCK()
+  { }
+
+  void abort_locks();
+  bool abort_locks_for_thread(uint64_t thread);
+
+  void lock()
+  {
+    mutex.lock();
+  }
+
+  void unlock()
+  {
+    mutex.unlock();
+  }
+
+  void init()
+  {
+  }
+
+  void deinit()
+  {
+  }
+
+  boost::mutex *native_handle()
+  {
+    return &mutex;
+  }
+};
 
 
-bool init_thr_lock(void);		/* Must be called once/thread */
 #define thr_lock_owner_init(owner, info_arg) (owner)->info= (info_arg)
 void thr_lock_info_init(THR_LOCK_INFO *info);
 void thr_lock_init(THR_LOCK *lock);
-void thr_lock_delete(THR_LOCK *lock);
-void thr_lock_data_init(THR_LOCK *lock,THR_LOCK_DATA *data,
-			void *status_param);
 enum enum_thr_lock_result thr_multi_lock(THR_LOCK_DATA **data,
                                          uint32_t count, THR_LOCK_OWNER *owner);
 void thr_multi_unlock(THR_LOCK_DATA **data,uint32_t count);
-void thr_abort_locks(THR_LOCK *lock);
-bool thr_abort_locks_for_thread(THR_LOCK *lock, uint64_t thread);
 
 } /* namespace drizzled */
 
