@@ -18,17 +18,22 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_UTIL_STORABLE_H
+#define DRIZZLED_UTIL_STORABLE_H
 
-namespace drizzled {
-namespace util {
-
-class Storable 
+namespace drizzled
 {
+
+namespace util
+{
+
+class Storable {
 public:
-  virtual ~Storable() { }
+  virtual ~Storable()
+  { }
 };
 
 } /* namespace util */
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_UTIL_STORABLE_H */
