@@ -1,7 +1,7 @@
 /* - mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,21 +18,31 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef DRIZZLED_UTIL_TEST_H
+#define DRIZZLED_UTIL_TEST_H
 
-namespace drizzled {
+#if defined(__cplusplus)
+
+namespace drizzled
+{
 
 template <class T>
-bool test(const T a)
+inline bool test(const T a)
 {
-  return bool(a);
+  return a ? true : false;
 }
 
 template <class T, class U>
-bool test_all_bits(const T a, const U b)
+inline bool test_all_bits(const T a, const U b)
 {
-  return (a & b) == b;
+  return ((a & b) == b);
 }
 
 } /* namespace drizzled */
 
+#else
+# define test(a)    ((a) ? 1 : 0)
+# define test_all_bits(a,b) (((a) & (b)) == (b))
+#endif
+
+#endif /* DRIZZLED_UTIL_TEST_H */
