@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,21 +18,24 @@
  */
 
 
-#pragma once
+#ifndef DRIZZLED_VAR_H
+#define DRIZZLED_VAR_H
 
-namespace drizzled {
-
-class var : public memory::SqlAlloc  
+namespace drizzled
 {
+
+class var : public memory::SqlAlloc  {
 public:
-  str_ref s;
+  LEX_STRING s;
   bool local;
   uint32_t offset;
   enum_field_types type;
-  var(str_ref j, bool i, uint32_t o, enum_field_types t) :
-    s(j), local(i), offset(o), type(t)
+  var (LEX_STRING& j, bool i, uint32_t o, enum_field_types t)
+    :s(j), local(i), offset(o), type(t)
   {}
+  ~var() {}
 };
 
 } /* namespace drizzled */
 
+#endif /* DRIZZLED_VAR_H */
