@@ -1,23 +1,19 @@
-#pragma once
-
 /**
  * @file
  * @brief SHA1 Declarations
  */
 
+#ifndef _SHA1_H
+#define _SHA1_H
+
 #ifdef  __cplusplus
 extern "C" {
 #endif
 
 /**
  * @addtogroup sha1 SHA-1 in C
- *
- * Copyright (C) 2010 nobody (this is public domain)
- *
- * This file is based on public domain code.
- * Initial source code is in the public domain, 
- * so clarified by Steve Reid <steve@edmweb.com>
- *
+ * By Steve Reid <steve@edmweb.com>
+ * 100% Public Domain
  * @{
  */
 
@@ -42,3 +38,5 @@ void SHA1Final(uint8_t [SHA1_DIGEST_LENGTH], SHA1_CTX *);
 #ifdef  __cplusplus
 }
 #endif
+
+#endif /* _SHA1_H */
