@@ -91,9 +91,6 @@ using namespace drizzled;
     -Brian
 */
 
-/* Variables for archive share methods */
-pthread_mutex_t archive_mutex= PTHREAD_MUTEX_INITIALIZER;
-
 /* When the engine starts up set the first version */
 static uint64_t global_version= 1;
 
@@ -132,43 +129,9 @@ void ArchiveEngine::deleteOpenTable(const string &table_name)
 }
 
 
-void ArchiveEngine::doGetTableNames(drizzled::CachedDirectory &directory, 
-                                    string&, 
-                                    set<string>& set_of_names)
+int ArchiveEngine::doDropTable(Session&, const TableIdentifier &identifier)
 {
-  drizzled::CachedDirectory::Entries entries= directory.getEntries();
-
-  for (drizzled::CachedDirectory::Entries::iterator entry_iter= entries.begin(); 
-       entry_iter != entries.end(); ++entry_iter)
-  {
-    drizzled::CachedDirectory::Entry *entry= *entry_iter;
-    const string *filename= &entry->filename;
-
-    assert(filename->size());
-
-    const char *ext= strchr(filename->c_str(), '.');
-
-    if (ext == NULL || my_strcasecmp(system_charset_info, ext, ARZ) ||
-        (filename->compare(0, strlen(TMP_FILE_PREFIX), TMP_FILE_PREFIX) == 0))
-    {  }
-    else
-    {
-      char uname[NAME_LEN + 1];
-      uint32_t file_name_len;
-
-      file_name_len= filename_to_tablename(filename->c_str(), uname, sizeof(uname));
-      // TODO: Remove need for memory copy here
-      uname[file_name_len - sizeof(ARZ) + 1]= '\0'; // Subtract ending, place NULL 
-      set_of_names.insert(uname);
-    }
-  }
-}
-
-
-int ArchiveEngine::doDropTable(Session&,
-                               const string &table_path)
-{
-  string new_path(table_path);
+  string new_path(identifier.getPath());
 
   new_path+= ARZ;
 
@@ -183,18 +146,15 @@ int ArchiveEngine::doDropTable(Session&,
 }
 
 int ArchiveEngine::doGetTableDefinition(Session&,
-                                        const char* path,
-                                        const char *,
-                                        const char *,
-                                        const bool,
-                                        drizzled::message::Table *table_proto)
+                                        const TableIdentifier &identifier,
+                                        drizzled::message::Table &table_proto)
 {
   struct stat stat_info;
   int error= ENOENT;
   string proto_path;
 
   proto_path.reserve(FN_REFLEN);
-  proto_path.assign(path);
+  proto_path.assign(identifier.getPath());
 
   proto_path.append(ARZ);
 
@@ -203,7 +163,6 @@ int ArchiveEngine::doGetTableDefinition(Session&,
   else
     error= EEXIST;
 
-  if (table_proto)
   {
     azio_stream proto_stream;
     char* proto_string;
@@ -219,13 +178,19 @@ int ArchiveEngine::doGetTableDefinition(Session&,
 
     azread_frm(&proto_stream, proto_string);
 
-    if (table_proto->ParseFromArray(proto_string, proto_stream.frm_length) == false)
+    if (table_proto.ParseFromArray(proto_string, proto_stream.frm_length) == false)
       error= HA_ERR_CRASHED_ON_USAGE;
 
     azclose(&proto_stream);
     free(proto_string);
   }
 
+  /* We set the name from what we've asked for as in RENAME TABLE for ARCHIVE
+     we do not rewrite the table proto (as it's wedged in the file header)
+  */
+  table_proto.set_schema(identifier.getSchemaName());
+  table_proto.set_name(identifier.getTableName());
+
   return error;
 }
 
@@ -269,18 +234,18 @@ ArchiveShare::ArchiveShare(const char *name):
 {
   memset(&archive_write, 0, sizeof(azio_stream));     /* Archive file we are working with */
   table_name.append(name);
-  internal::fn_format(data_file_name, table_name.c_str(), "",
-            ARZ, MY_REPLACE_EXT | MY_UNPACK_FILENAME);
+  data_file_name.assign(table_name);
+  data_file_name.append(ARZ);
   /*
     We will use this lock for rows.
   */
-  pthread_mutex_init(&mutex,MY_MUTEX_INIT_FAST);
+  pthread_mutex_init(&_mutex,MY_MUTEX_INIT_FAST);
 }
 
 ArchiveShare::~ArchiveShare()
 {
-  thr_lock_delete(&lock);
-  pthread_mutex_destroy(&mutex);
+  _lock.deinit();
+  pthread_mutex_destroy(&_mutex);
   /*
     We need to make sure we don't reset the crashed state.
     If we open a crashed file, wee need to close it as crashed unless
@@ -302,7 +267,7 @@ bool ArchiveShare::prime(uint64_t *auto_increment)
     anything but reading... open it for write and we will generate null
     compression writes).
   */
-  if (!(azopen(&archive_tmp, data_file_name, O_RDONLY,
+  if (!(azopen(&archive_tmp, data_file_name.c_str(), O_RDONLY,
                AZ_METHOD_BLOCK)))
     return false;
 
@@ -329,9 +294,10 @@ bool ArchiveShare::prime(uint64_t *auto_increment)
 */
 ArchiveShare *ha_archive::get_share(const char *table_name, int *rc)
 {
-  pthread_mutex_lock(&archive_mutex);
-
   ArchiveEngine *a_engine= static_cast<ArchiveEngine *>(engine);
+
+  pthread_mutex_lock(&a_engine->mutex());
+
   share= a_engine->findOpenTable(table_name);
 
   if (!share)
@@ -340,14 +306,14 @@ ArchiveShare *ha_archive::get_share(const char *table_name, int *rc)
 
     if (share == NULL)
     {
-      pthread_mutex_unlock(&archive_mutex);
+      pthread_mutex_unlock(&a_engine->mutex());
       *rc= HA_ERR_OUT_OF_MEM;
       return(NULL);
     }
 
     if (share->prime(&stats.auto_increment_value) == false)
     {
-      pthread_mutex_unlock(&archive_mutex);
+      pthread_mutex_unlock(&a_engine->mutex());
       *rc= HA_ERR_CRASHED_ON_REPAIR;
       delete share;
 
@@ -355,13 +321,13 @@ ArchiveShare *ha_archive::get_share(const char *table_name, int *rc)
     }
 
     a_engine->addOpenTable(share->table_name, share);
-    thr_lock_init(&share->lock);
+    thr_lock_init(&share->_lock);
   }
   share->use_count++;
 
   if (share->crashed)
     *rc= HA_ERR_CRASHED_ON_USAGE;
-  pthread_mutex_unlock(&archive_mutex);
+  pthread_mutex_unlock(&a_engine->mutex());
 
   return(share);
 }
@@ -373,14 +339,15 @@ ArchiveShare *ha_archive::get_share(const char *table_name, int *rc)
 */
 int ha_archive::free_share()
 {
-  pthread_mutex_lock(&archive_mutex);
+  ArchiveEngine *a_engine= static_cast<ArchiveEngine *>(engine);
+
+  pthread_mutex_lock(&a_engine->mutex());
   if (!--share->use_count)
   {
-    ArchiveEngine *a_engine= static_cast<ArchiveEngine *>(engine);
     a_engine->deleteOpenTable(share->table_name);
     delete share;
   }
-  pthread_mutex_unlock(&archive_mutex);
+  pthread_mutex_unlock(&a_engine->mutex());
 
   return 0;
 }
@@ -392,7 +359,7 @@ int ha_archive::init_archive_writer()
     a gzip file that can be both read and written we keep a writer open
     that is shared amoung all open tables.
   */
-  if (!(azopen(&(share->archive_write), share->data_file_name,
+  if (!(azopen(&(share->archive_write), share->data_file_name.c_str(),
                O_RDWR, AZ_METHOD_BLOCK)))
   {
     share->crashed= true;
@@ -429,7 +396,7 @@ int ha_archive::init_archive_reader()
     default:
       method= AZ_METHOD_BLOCK;
     }
-    if (!(azopen(&archive, share->data_file_name, O_RDONLY,
+    if (!(azopen(&archive, share->data_file_name.c_str(), O_RDONLY,
                  method)))
     {
       share->crashed= true;
@@ -447,10 +414,10 @@ int ha_archive::init_archive_reader()
   Init out lock.
   We open the file we will read from.
 */
-int ha_archive::open(const char *name, int, uint32_t)
+int ha_archive::doOpen(const TableIdentifier &identifier, int , uint32_t )
 {
   int rc= 0;
-  share= get_share(name, &rc);
+  share= get_share(identifier.getPath().c_str(), &rc);
 
   /** 
     We either fix it ourselves, or we just take it offline 
@@ -471,20 +438,20 @@ int ha_archive::open(const char *name, int, uint32_t)
 
   assert(share);
 
-  record_buffer= create_record_buffer(table->s->reclength +
-                                      ARCHIVE_ROW_HEADER_SIZE);
+  record_buffer.resize(table->getShare()->getRecordLength() + ARCHIVE_ROW_HEADER_SIZE);
 
-  if (!record_buffer)
-  {
-    free_share();
-    return(HA_ERR_OUT_OF_MEM);
-  }
-
-  thr_lock_data_init(&share->lock, &lock, NULL);
+  lock.init(&share->_lock);
 
   return(rc);
 }
 
+// Should never be called
+int ha_archive::open(const char *, int, uint32_t)
+{
+  assert(0);
+  return -1;
+}
+
 
 /*
   Closes the file.
@@ -507,7 +474,7 @@ int ha_archive::close(void)
 {
   int rc= 0;
 
-  destroy_record_buffer(record_buffer);
+  record_buffer.clear();
 
   /* First close stream */
   if (archive_reader_open == true)
@@ -531,12 +498,11 @@ int ha_archive::close(void)
   of creation.
 */
 
-int ArchiveEngine::doCreateTable(Session *,
-                                 const char *table_name,
+int ArchiveEngine::doCreateTable(Session &,
                                  Table& table_arg,
+                                 const drizzled::TableIdentifier &identifier,
                                  drizzled::message::Table& proto)
 {
-  char name_buff[FN_REFLEN];
   int error= 0;
   azio_stream create_stream;            /* Archive file we are working with */
   uint64_t auto_increment_value;
@@ -546,9 +512,9 @@ int ArchiveEngine::doCreateTable(Session *,
 
   for (uint32_t key= 0; key < table_arg.sizeKeys(); key++)
   {
-    KEY *pos= table_arg.key_info+key;
-    KEY_PART_INFO *key_part=     pos->key_part;
-    KEY_PART_INFO *key_part_end= key_part + pos->key_parts;
+    KeyInfo *pos= &table_arg.key_info[key];
+    KeyPartInfo *key_part=     pos->key_part;
+    KeyPartInfo *key_part_end= key_part + pos->key_parts;
 
     for (; key_part != key_part_end; key_part++)
     {
@@ -556,31 +522,41 @@ int ArchiveEngine::doCreateTable(Session *,
 
       if (!(field->flags & AUTO_INCREMENT_FLAG))
       {
-        error= -1;
-        goto error;
+        return -1;
       }
     }
   }
 
-  /*
-    We reuse name_buff since it is available.
-  */
-  internal::fn_format(name_buff, table_name, "", ARZ,
-            MY_REPLACE_EXT | MY_UNPACK_FILENAME);
+  std::string named_file= identifier.getPath();
+  named_file.append(ARZ);
 
   errno= 0;
-  if (azopen(&create_stream, name_buff, O_CREAT|O_RDWR,
+  if (azopen(&create_stream, named_file.c_str(), O_CREAT|O_RDWR,
              AZ_METHOD_BLOCK) == 0)
   {
     error= errno;
-    goto error2;
+    unlink(named_file.c_str());
+
+    return(error ? error : -1);
   }
 
-  proto.SerializeToString(&serialized_proto);
+  try {
+    proto.SerializeToString(&serialized_proto);
+  }
+  catch (...)
+  {
+    unlink(named_file.c_str());
+
+    return(error ? error : -1);
+  }
 
   if (azwrite_frm(&create_stream, serialized_proto.c_str(),
                   serialized_proto.length()))
-    goto error2;
+  {
+    unlink(named_file.c_str());
+
+    return(error ? error : -1);
+  }
 
   if (proto.options().has_comment())
   {
@@ -593,7 +569,9 @@ int ArchiveEngine::doCreateTable(Session *,
     if (write_length < 0)
     {
       error= errno;
-      goto error2;
+      unlink(named_file.c_str());
+
+      return(error ? error : -1);
     }
   }
 
@@ -607,17 +585,12 @@ int ArchiveEngine::doCreateTable(Session *,
   if (azclose(&create_stream))
   {
     error= errno;
-    goto error2;
+    unlink(named_file.c_str());
+
+    return(error ? error : -1);
   }
 
   return(0);
-
-error2:
-  unlink(name_buff);
-
-error:
-  /* Return error number, if we got one */
-  return(error ? error : -1);
 }
 
 /*
@@ -631,7 +604,7 @@ int ha_archive::real_write_row(unsigned char *buf, azio_stream *writer)
   /* We pack the row for writing */
   r_pack_length= pack_row(buf);
 
-  written= azwrite_row(writer, record_buffer->buffer, r_pack_length);
+  written= azwrite_row(writer, &record_buffer[0], r_pack_length);
   if (written != r_pack_length)
   {
     return(-1);
@@ -659,7 +632,7 @@ uint32_t ha_archive::max_row_length(const unsigned char *)
        ptr != end ;
        ptr++)
   {
-      length += 2 + ((Field_blob*)table->field[*ptr])->get_length();
+      length += 2 + ((Field_blob*)table->getField(*ptr))->get_length();
   }
 
   return length;
@@ -674,16 +647,16 @@ unsigned int ha_archive::pack_row(unsigned char *record)
     return(HA_ERR_OUT_OF_MEM);
 
   /* Copy null bits */
-  memcpy(record_buffer->buffer, record, table->s->null_bytes);
-  ptr= record_buffer->buffer + table->s->null_bytes;
+  memcpy(&record_buffer[0], record, table->getShare()->null_bytes);
+  ptr= &record_buffer[0] + table->getShare()->null_bytes;
 
-  for (Field **field=table->field ; *field ; field++)
+  for (Field **field=table->getFields() ; *field ; field++)
   {
     if (!((*field)->is_null()))
       ptr= (*field)->pack(ptr, record + (*field)->offset(record));
   }
 
-  return((unsigned int) (ptr - record_buffer->buffer));
+  return((unsigned int) (ptr - &record_buffer[0]));
 }
 
 
@@ -696,27 +669,25 @@ unsigned int ha_archive::pack_row(unsigned char *record)
   for implementing start_bulk_insert() is that we could skip
   setting dirty to true each time.
 */
-int ha_archive::write_row(unsigned char *buf)
+int ha_archive::doInsertRecord(unsigned char *buf)
 {
   int rc;
   unsigned char *read_buf= NULL;
   uint64_t temp_auto;
-  unsigned char *record=  table->record[0];
+  unsigned char *record=  table->getInsertRecord();
 
   if (share->crashed)
     return(HA_ERR_CRASHED_ON_USAGE);
 
-  ha_statistic_increment(&system_status_var::ha_write_count);
-  pthread_mutex_lock(&share->mutex);
+  pthread_mutex_lock(&share->mutex());
 
   if (share->archive_write_open == false)
     if (init_archive_writer())
       return(HA_ERR_CRASHED_ON_USAGE);
 
 
-  if (table->next_number_field && record == table->record[0])
+  if (table->next_number_field && record == table->getInsertRecord())
   {
-    KEY *mkey= &table->s->key_info[0]; // We only support one key right now
     update_auto_increment();
     temp_auto= table->next_number_field->val_int();
 
@@ -725,7 +696,7 @@ int ha_archive::write_row(unsigned char *buf)
       just cry.
     */
     if (temp_auto <= share->archive_write.auto_increment &&
-        mkey->flags & HA_NOSAME)
+        table->getShare()->getKeyInfo(0).flags & HA_NOSAME)
     {
       rc= HA_ERR_FOUND_DUPP_KEY;
       goto error;
@@ -745,7 +716,7 @@ int ha_archive::write_row(unsigned char *buf)
   share->rows_recorded++;
   rc= real_write_row(buf,  &(share->archive_write));
 error:
-  pthread_mutex_unlock(&share->mutex);
+  pthread_mutex_unlock(&share->mutex());
   if (read_buf)
     free((unsigned char*) read_buf);
 
@@ -760,8 +731,8 @@ void ha_archive::get_auto_increment(uint64_t, uint64_t, uint64_t,
   *first_value= share->archive_write.auto_increment + 1;
 }
 
-/* Initialized at each key walk (called multiple times unlike rnd_init()) */
-int ha_archive::index_init(uint32_t keynr, bool)
+/* Initialized at each key walk (called multiple times unlike doStartTableScan()) */
+int ha_archive::doStartIndexScan(uint32_t keynr, bool)
 {
   active_index= keynr;
   return(0);
@@ -773,25 +744,15 @@ int ha_archive::index_init(uint32_t keynr, bool)
   the optimizer that we have unique indexes, we scan
 */
 int ha_archive::index_read(unsigned char *buf, const unsigned char *key,
-                             uint32_t key_len, enum ha_rkey_function find_flag)
-{
-  int rc;
-  rc= index_read_idx(buf, active_index, key, key_len, find_flag);
-  return(rc);
-}
-
-
-int ha_archive::index_read_idx(unsigned char *buf, uint32_t index, const unsigned char *key,
-                               uint32_t key_len, enum ha_rkey_function)
+                             uint32_t key_len, enum ha_rkey_function)
 {
   int rc;
   bool found= 0;
-  KEY *mkey= &table->s->key_info[index];
-  current_k_offset= mkey->key_part->offset;
+  current_k_offset= table->getShare()->getKeyInfo(0).key_part->offset;
   current_key= key;
   current_key_len= key_len;
 
-  rc= rnd_init(true);
+  rc= doStartTableScan(true);
 
   if (rc)
     goto error;
@@ -835,7 +796,7 @@ int ha_archive::index_next(unsigned char * buf)
   we assume the position will be set.
 */
 
-int ha_archive::rnd_init(bool scan)
+int ha_archive::doStartTableScan(bool scan)
 {
   if (share->crashed)
       return(HA_ERR_CRASHED_ON_USAGE);
@@ -872,20 +833,9 @@ int ha_archive::get_row(azio_stream *file_to_read, unsigned char *buf)
 /* Reallocate buffer if needed */
 bool ha_archive::fix_rec_buff(unsigned int length)
 {
-  assert(record_buffer->buffer);
+  record_buffer.resize(length);
 
-  if (length > record_buffer->length)
-  {
-    unsigned char *newptr;
-    if (!(newptr= (unsigned char *)realloc(record_buffer->buffer, length)))
-      return(1);
-    record_buffer->buffer= newptr;
-    record_buffer->length= length;
-  }
-
-  assert(length <= record_buffer->length);
-
-  return(0);
+  return false;
 }
 
 int ha_archive::unpack_row(azio_stream *file_to_read, unsigned char *record)
@@ -905,11 +855,11 @@ int ha_archive::unpack_row(azio_stream *file_to_read, unsigned char *record)
   /* Copy null bits */
   memcpy(record, ptr, table->getNullBytes());
   ptr+= table->getNullBytes();
-  for (Field **field=table->field ; *field ; field++)
+  for (Field **field= table->getFields() ; *field ; field++)
   {
     if (!((*field)->is_null()))
     {
-      ptr= (*field)->unpack(record + (*field)->offset(table->record[0]), ptr);
+      ptr= (*field)->unpack(record + (*field)->offset(table->getInsertRecord()), ptr);
     }
   }
   return(0);
@@ -1003,7 +953,6 @@ int ha_archive::optimize()
 {
   int rc= 0;
   azio_stream writer;
-  char writer_filename[FN_REFLEN];
 
   init_archive_reader();
 
@@ -1023,10 +972,10 @@ int ha_archive::optimize()
   azread_frm(&archive, proto_string);
 
   /* Lets create a file to contain the new data */
-  internal::fn_format(writer_filename, share->table_name.c_str(), "", ARN,
-            MY_REPLACE_EXT | MY_UNPACK_FILENAME);
+  std::string writer_filename= share->table_name;
+  writer_filename.append(ARN);
 
-  if (!(azopen(&writer, writer_filename, O_CREAT|O_RDWR, AZ_METHOD_BLOCK)))
+  if (!(azopen(&writer, writer_filename.c_str(), O_CREAT|O_RDWR, AZ_METHOD_BLOCK)))
   {
     free(proto_string);
     return(HA_ERR_CRASHED_ON_USAGE);
@@ -1056,7 +1005,6 @@ int ha_archive::optimize()
     */
     if (!rc)
     {
-      uint64_t x;
       uint64_t rows_restored;
       share->rows_recorded= 0;
       stats.auto_increment_value= 1;
@@ -1064,14 +1012,14 @@ int ha_archive::optimize()
 
       rows_restored= archive.rows;
 
-      for (x= 0; x < rows_restored ; x++)
+      for (uint64_t x= 0; x < rows_restored ; x++)
       {
-        rc= get_row(&archive, table->record[0]);
+        rc= get_row(&archive, table->getInsertRecord());
 
         if (rc != 0)
           break;
 
-        real_write_row(table->record[0], &writer);
+        real_write_row(table->getInsertRecord(), &writer);
         /*
           Long term it should be possible to optimize this so that
           it is not called on each row.
@@ -1084,8 +1032,8 @@ int ha_archive::optimize()
           field->setReadSet();
 
           uint64_t auto_value=
-            (uint64_t) field->val_int(table->record[0] +
-                                       field->offset(table->record[0]));
+            (uint64_t) field->val_int(table->getInsertRecord() +
+                                       field->offset(table->getInsertRecord()));
           if (share->archive_write.auto_increment < auto_value)
             stats.auto_increment_value=
               (share->archive_write.auto_increment= auto_value) + 1;
@@ -1106,7 +1054,7 @@ int ha_archive::optimize()
   azclose(&archive);
 
   // make the file we just wrote be our data file
-  rc = internal::my_rename(writer_filename,share->data_file_name,MYF(0));
+  rc = internal::my_rename(writer_filename.c_str(), share->data_file_name.c_str(), MYF(0));
 
   free(proto_string);
   return(rc);
@@ -1168,7 +1116,7 @@ int ha_archive::info(uint32_t flag)
     If dirty, we lock, and then reset/flush the data.
     I found that just calling azflush() doesn't always work.
   */
-  pthread_mutex_lock(&share->mutex);
+  pthread_mutex_lock(&share->mutex());
   if (share->dirty == true)
   {
     azflush(&(share->archive_write), Z_SYNC_FLUSH);
@@ -1187,7 +1135,7 @@ int ha_archive::info(uint32_t flag)
     cause the number to be inaccurate.
   */
   stats.records= share->rows_recorded;
-  pthread_mutex_unlock(&share->mutex);
+  pthread_mutex_unlock(&share->mutex());
 
   scan_rows= stats.records;
   stats.deleted= 0;
@@ -1197,7 +1145,7 @@ int ha_archive::info(uint32_t flag)
   {
     struct stat file_stat;  // Stat information for the data file
 
-    stat(share->data_file_name, &file_stat);
+    stat(share->data_file_name.c_str(), &file_stat);
 
     stats.mean_rec_length= table->getRecordLength()+ buffer.alloced_length();
     stats.data_file_length= file_stat.st_size;
@@ -1211,9 +1159,9 @@ int ha_archive::info(uint32_t flag)
   if (flag & HA_STATUS_AUTO)
   {
     init_archive_reader();
-    pthread_mutex_lock(&share->mutex);
+    pthread_mutex_lock(&share->mutex());
     azflush(&archive, Z_SYNC_FLUSH);
-    pthread_mutex_unlock(&share->mutex);
+    pthread_mutex_unlock(&share->mutex());
     stats.auto_increment_value= archive.auto_increment + 1;
   }
 
@@ -1223,7 +1171,7 @@ int ha_archive::info(uint32_t flag)
 
 /*
   This method tells us that a bulk insert operation is about to occur. We set
-  a flag which will keep write_row from saying that its data is dirty. This in
+  a flag which will keep doInsertRecord from saying that its data is dirty. This in
   turn will keep selects from causing a sync to occur.
   Basically, yet another optimizations to keep compression working well.
 */
@@ -1264,14 +1212,13 @@ int ha_archive::check(Session* session)
 {
   int rc= 0;
   const char *old_proc_info;
-  uint64_t x;
 
   old_proc_info= get_session_proc_info(session);
   set_session_proc_info(session, "Checking table");
   /* Flush any waiting data */
-  pthread_mutex_lock(&share->mutex);
+  pthread_mutex_lock(&share->mutex());
   azflush(&(share->archive_write), Z_SYNC_FLUSH);
-  pthread_mutex_unlock(&share->mutex);
+  pthread_mutex_unlock(&share->mutex());
 
   /*
     Now we will rewind the archive file so that we are positioned at the
@@ -1280,9 +1227,9 @@ int ha_archive::check(Session* session)
   init_archive_reader();
   azflush(&archive, Z_SYNC_FLUSH);
   read_data_header(&archive);
-  for (x= 0; x < share->archive_write.rows; x++)
+  for (uint64_t x= 0; x < share->archive_write.rows; x++)
   {
-    rc= get_row(&archive, table->record[0]);
+    rc= get_row(&archive, table->getInsertRecord());
 
     if (rc != 0)
       break;
@@ -1301,27 +1248,66 @@ int ha_archive::check(Session* session)
   }
 }
 
-archive_record_buffer *ha_archive::create_record_buffer(unsigned int length)
+int ArchiveEngine::doRenameTable(Session&, const TableIdentifier &from, const TableIdentifier &to)
 {
-  archive_record_buffer *r;
-  if (!(r= (archive_record_buffer*) malloc(sizeof(archive_record_buffer))))
-  {
-    return(NULL);
-  }
-  r->length= (int)length;
+  int error= 0;
 
-  if (!(r->buffer= (unsigned char*) malloc(r->length)))
+  for (const char **ext= bas_ext(); *ext ; ext++)
   {
-    free((char*) r);
-    return(NULL);
+    if (rename_file_ext(from.getPath().c_str(), to.getPath().c_str(), *ext))
+    {
+      if ((error=errno) != ENOENT)
+        break;
+      error= 0;
+    }
   }
 
-  return(r);
+  return error;
 }
 
-void ha_archive::destroy_record_buffer(archive_record_buffer *r)
+bool ArchiveEngine::doDoesTableExist(Session&,
+                                     const TableIdentifier &identifier)
 {
-  free((char*) r->buffer);
-  free((char*) r);
-  return;
+  string proto_path(identifier.getPath());
+  proto_path.append(ARZ);
+
+  if (access(proto_path.c_str(), F_OK))
+  {
+    return false;
+  }
+
+  return true;
+}
+
+void ArchiveEngine::doGetTableIdentifiers(drizzled::CachedDirectory &directory,
+                                          const drizzled::SchemaIdentifier &schema_identifier,
+                                          drizzled::TableIdentifiers &set_of_identifiers)
+{
+  drizzled::CachedDirectory::Entries entries= directory.getEntries();
+
+  for (drizzled::CachedDirectory::Entries::iterator entry_iter= entries.begin(); 
+       entry_iter != entries.end(); ++entry_iter)
+  {
+    drizzled::CachedDirectory::Entry *entry= *entry_iter;
+    const string *filename= &entry->filename;
+
+    assert(filename->size());
+
+    const char *ext= strchr(filename->c_str(), '.');
+
+    if (ext == NULL || my_strcasecmp(system_charset_info, ext, ARZ) ||
+        (filename->compare(0, strlen(TMP_FILE_PREFIX), TMP_FILE_PREFIX) == 0))
+    {  }
+    else
+    {
+      char uname[NAME_LEN + 1];
+      uint32_t file_name_len;
+
+      file_name_len= TableIdentifier::filename_to_tablename(filename->c_str(), uname, sizeof(uname));
+      // TODO: Remove need for memory copy here
+      uname[file_name_len - sizeof(ARZ) + 1]= '\0'; // Subtract ending, place NULL 
+
+      set_of_identifiers.push_back(TableIdentifier(schema_identifier, uname));
+    }
+  }
 }
