@@ -23,15 +23,16 @@
 
 #include <string>
 #include <cassert>
-
+#include <boost/program_options.hpp>
+#include <drizzled/module/option_map.h>
 #include "drizzled/security_context.h"
 #include "drizzled/plugin/authentication.h"
 #include "drizzled/gettext.h"
-
+namespace po= boost::program_options;
 using namespace drizzled;
 using namespace std;
 
-static bool sysvar_auth_http_enable= false;
+static bool sysvar_auth_http_enable;
 static char* sysvar_auth_http_url= NULL;
 
 static size_t curl_cb_read(void *ptr, size_t size, size_t nmemb, void *stream)
@@ -70,6 +71,7 @@ public:
   ~Auth_http()
   {
     curl_easy_cleanup(curl_handle);
+    curl_global_cleanup();
   }
 
   virtual bool authenticate(const SecurityContext &sctx, const string &password)
@@ -119,7 +121,7 @@ public:
 
 Auth_http* auth= NULL;
 
-static int initialize(drizzled::plugin::Registry &registry)
+static int initialize(drizzled::module::Context &context)
 {
   /* 
    * Per libcurl manual, in multi-threaded applications, curl_global_init() should
@@ -130,23 +132,16 @@ static int initialize(drizzled::plugin::Registry &registry)
     return 1;
 
   auth= new Auth_http("auth_http");
-  registry.add(auth);
+  context.add(auth);
 
   return 0;
 }
 
-static int finalize(drizzled::plugin::Registry &registry)
+static void init_options(drizzled::module::option_context &context)
 {
-  if (auth)
-  {
-    registry.remove(auth);
-    delete auth;
-
-    curl_global_cleanup();
-  }
-
-  return 0;
-}
+   context("enable", po::value<bool>(&sysvar_auth_http_enable)->default_value(false)->zero_tokens(),
+           N_("Enable HTTP Auth check"));
+} 
 
 static DRIZZLE_SYSVAR_BOOL(
   enable,
@@ -177,14 +172,13 @@ static drizzle_sys_var* auth_http_system_variables[]= {
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
-  "auth_http",
+  "auth-http",
   "0.1",
   "Mark Atwood",
   "HTTP based authenication.",
   PLUGIN_LICENSE_GPL,
   initialize, /* Plugin Init */
-  finalize, /* Plugin Deinit */
   auth_http_system_variables,
-  NULL    /* config options */
+  init_options    /* config options */
 }
 DRIZZLE_DECLARE_PLUGIN_END;
