@@ -66,24 +66,15 @@ int64_t CharLengthFunction::val_int()
 plugin::Create_function<CharLengthFunction> *charlengthudf= NULL;
 plugin::Create_function<CharLengthFunction> *characterlengthudf= NULL;
 
-static int initialize(drizzled::plugin::Registry &registry)
+static int initialize(drizzled::module::Context &context)
 {
   charlengthudf= new plugin::Create_function<CharLengthFunction>("char_length");
   characterlengthudf= new plugin::Create_function<CharLengthFunction>("character_length");
-  registry.add(charlengthudf);
-  registry.add(characterlengthudf);
+  context.add(charlengthudf);
+  context.add(characterlengthudf);
   return 0;
 }
 
-static int finalize(drizzled::plugin::Registry &registry)
-{
-   registry.remove(charlengthudf);
-   registry.remove(characterlengthudf);
-   delete charlengthudf;
-   delete characterlengthudf;
-   return 0;
-}
-
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
@@ -93,7 +84,6 @@ DRIZZLE_DECLARE_PLUGIN
   "Return the number of characters in a string",
   PLUGIN_LICENSE_GPL,
   initialize, /* Plugin Init */
-  finalize,   /* Plugin Deinit */
   NULL,   /* system variables */
   NULL    /* config options */
 }
