@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,15 +17,19 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_COMPRESSION_UNCOMPRESS_H
+#define PLUGIN_COMPRESSION_UNCOMPRESS_H
 
+#include <drizzled/session.h>
 #include <drizzled/error.h>
 #include <drizzled/function/str/strfunc.h>
 
-class Item_func_uncompress : public drizzled::Item_str_func
+class Item_func_uncompress: public drizzled::Item_str_func
 {
   drizzled::String buffer;
 public:
+  Item_func_uncompress(): Item_str_func(){}
+  virtual ~Item_func_uncompress() {}
   void fix_length_and_dec(){ maybe_null= 1; max_length= MAX_BLOB_WIDTH; }
   bool check_argument_count(int n) { return (n==1); }
   const char *func_name() const{return "uncompress";}
@@ -33,3 +37,4 @@ public:
 };
 
 
+#endif /* PLUGIN_COMPRESSION_UNCOMPRESS_H */
