@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_COMPRESSION_UNCOMPRESSED_LENGTH_H
+#define PLUGIN_COMPRESSION_UNCOMPRESSED_LENGTH_H
 
 #include <drizzled/plugin/function.h>
 #include <drizzled/function/math/int.h>
@@ -35,3 +36,4 @@ public:
 };
 
 
+#endif /* PLUGIN_COMPRESSION_UNCOMPRESSED_LENGTH_H */
