@@ -68,21 +68,14 @@ public:
 
 plugin::Create_function<ConnectionIdFunction> *connection_idudf= NULL;
 
-static int initialize(plugin::Registry &registry)
+static int initialize(module::Context &context)
 {
   connection_idudf=
     new plugin::Create_function<ConnectionIdFunction>("connection_id");
-  registry.add(connection_idudf);
+  context.add(connection_idudf);
   return 0;
 }
 
-static int finalize(plugin::Registry &registry)
-{
-   registry.remove(connection_idudf);
-   delete connection_idudf;
-   return 0;
-}
-
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
@@ -92,7 +85,6 @@ DRIZZLE_DECLARE_PLUGIN
   "Return the current connection_id",
   PLUGIN_LICENSE_GPL,
   initialize, /* Plugin Init */
-  finalize,   /* Plugin Deinit */
   NULL,   /* system variables */
   NULL    /* config options */
 }
