@@ -18,17 +18,22 @@
 #include <drizzled/plugin/listen_tcp.h>
 #include <drizzled/plugin/client.h>
 #include <drizzled/session.h>
+#include <drizzled/module/option_map.h>
 
 #include <iostream>
 
+#include <boost/program_options.hpp>
+
 using namespace std;
 using namespace drizzled;
 
+namespace po= boost::program_options;
+
 static bool enabled= false;
 static bool debug_enabled= false;
-static char* user = (char*)"";
-static char* password = (char*)"";
-static char* db = NULL;
+static char* username= NULL;
+static char* password= NULL;
+static char* db= NULL;
 
 
 class ClientConsole: public plugin::Client
@@ -89,7 +94,7 @@ public:
   virtual bool authenticate(void)
   {
     printDebug("authenticate");
-    session->getSecurityContext().setUser(user);
+    session->getSecurityContext().setUser(username);
     return session->checkUser(password, strlen(password), db);
   }
 
@@ -266,8 +271,8 @@ class ListenConsole: public plugin::Listen
   int pipe_fds[2];
 
 public:
-  ListenConsole(std::string name_arg)
-    : plugin::Listen(name_arg)
+  ListenConsole(const std::string &name_arg) :
+    plugin::Listen(name_arg)
   {
     pipe_fds[0]= -1;
   }
@@ -279,6 +284,11 @@ public:
       close(pipe_fds[0]);
       close(pipe_fds[1]);
     }
+
+    /* Cleanup from the module strdup'ing these below */
+    free(username);
+    free(password);
+    free(db);
   }
 
   virtual bool getFileDescriptors(std::vector<int> &fds)
@@ -308,38 +318,65 @@ public:
   }
 };
 
-static ListenConsole *listen_obj= NULL;
-
-static int init(drizzled::plugin::Registry &registry)
+static int init(drizzled::module::Context &context)
 {
-  listen_obj= new ListenConsole("console");
-  registry.add(listen_obj);
-  return 0;
-}
+  const module::option_map &vm= context.getOptions();
+  /* duplicating these here means they need to be freed. They're global, so
+     we'll just have the ListenConsole object do it in its destructor */
+  if (vm.count("username"))
+    username= strdup(vm["username"].as<string>().c_str());
+  else
+    username= strdup("");
 
-static int deinit(drizzled::plugin::Registry &registry)
-{
-  registry.remove(listen_obj);
-  delete listen_obj;
+  if (vm.count("password"))
+    password= strdup(vm["password"].as<string>().c_str());
+  else
+    password= strdup("");
+
+  if (vm.count("db"))
+    db= strdup(vm["db"].as<string>().c_str());
+  else
+    db= strdup("");
+
+  context.add(new ListenConsole("console"));
   return 0;
 }
 
 static DRIZZLE_SYSVAR_BOOL(enable, enabled, PLUGIN_VAR_NOCMDARG,
                            N_("Enable the console."), NULL, NULL, false);
-
 static DRIZZLE_SYSVAR_BOOL(debug, debug_enabled, PLUGIN_VAR_NOCMDARG,
                            N_("Turn on extra debugging."), NULL, NULL, false);
-static DRIZZLE_SYSVAR_STR(user, user, PLUGIN_VAR_READONLY,
+
+static DRIZZLE_SYSVAR_STR(username, username, PLUGIN_VAR_READONLY,
                           N_("User to use for auth."), NULL, NULL, NULL);
 static DRIZZLE_SYSVAR_STR(password, password, PLUGIN_VAR_READONLY,
                           N_("Password to use for auth."), NULL, NULL, NULL);
 static DRIZZLE_SYSVAR_STR(db, db, PLUGIN_VAR_READONLY,
                           N_("Default database to use."), NULL, NULL, NULL);
 
+static void init_options(drizzled::module::option_context &context)
+{
+  context("enable",
+          po::value<bool>(&enabled)->default_value(false)->zero_tokens(),
+          N_("Enable the console."));
+  context("debug",
+          po::value<bool>(&debug_enabled)->default_value(false)->zero_tokens(),
+          N_("Turn on extra debugging."));
+  context("username",
+          po::value<string>(),
+          N_("User to use for auth."));
+  context("password",
+          po::value<string>(),
+          N_("Password to use for auth."));
+  context("db",
+          po::value<string>(),
+          N_("Default database to use."));
+}
+
 static drizzle_sys_var* vars[]= {
   DRIZZLE_SYSVAR(enable),
   DRIZZLE_SYSVAR(debug),
-  DRIZZLE_SYSVAR(user),
+  DRIZZLE_SYSVAR(username),
   DRIZZLE_SYSVAR(password),
   DRIZZLE_SYSVAR(db),
   NULL
@@ -354,8 +391,7 @@ DRIZZLE_DECLARE_PLUGIN
   "Console Client",
   PLUGIN_LICENSE_BSD,
   init,   /* Plugin Init */
-  deinit, /* Plugin Deinit */
   vars,   /* system variables */
-  NULL    /* config options */
+  init_options    /* config options */
 }
 DRIZZLE_DECLARE_PLUGIN_END;
