@@ -45,14 +45,6 @@ public:
   /** Destructor */
   ~DefaultReplicator() {}
 
-  /**
-   * Returns whether the replicator is active
-   */
-  virtual bool isEnabled() const;
-
-  virtual void enable();
-  virtual void disable();
-
   /**
    * Replicate a Transaction message to an Applier.
    *
@@ -66,9 +58,14 @@ public:
    * the supplied message to their own controlled memory storage
    * area.
    *
+   * @param Applier to replicate to
+   * @param Session descriptor
    * @param Transaction message to be replicated
    */
-  void replicate(drizzled::plugin::TransactionApplier *in_applier, drizzled::message::Transaction &to_replicate);
+  drizzled::plugin::ReplicationReturnCode
+  replicate(drizzled::plugin::TransactionApplier *in_applier,
+            drizzled::Session &in_session,
+            drizzled::message::Transaction &to_replicate);
   
 };
 
