@@ -26,19 +26,18 @@
 #include "drizzled/internal/my_sys.h"
 #include "drizzled/internal/m_string.h"
 #include <algorithm>
-
+#include <iostream>
+#include <boost/program_options.hpp>
+#include <drizzled/module/option_map.h>
 #include "pack.h"
 #include "errmsg.h"
 #include "drizzle_protocol.h"
 #include "options.h"
+#include "table_function.h"
 
 #define PROTOCOL_VERSION 10
 
-namespace drizzled
-{
-extern uint32_t global_thread_id;
-}
-
+namespace po= boost::program_options;
 using namespace drizzled;
 using namespace std;
 
@@ -54,7 +53,15 @@ static uint32_t read_timeout;
 static uint32_t write_timeout;
 static uint32_t retry_count;
 static uint32_t buffer_length;
-static char* bind_address;
+static char* bind_address= NULL;
+
+static plugin::TableFunction* drizzle_status_table_function_ptr= NULL;
+
+ListenDrizzleProtocol::~ListenDrizzleProtocol()
+{
+  /* This is strdup'd from the options */
+  free(bind_address);
+}
 
 const char* ListenDrizzleProtocol::getHost(void) const
 {
@@ -88,6 +95,10 @@ plugin::Client *ListenDrizzleProtocol::getClient(int fd)
   return new (nothrow) ClientDrizzleProtocol(new_fd, using_mysql41_protocol);
 }
 
+drizzled::atomic<uint64_t> ClientDrizzleProtocol::connectionCount;
+drizzled::atomic<uint64_t> ClientDrizzleProtocol::failedConnections;
+drizzled::atomic<uint64_t> ClientDrizzleProtocol::connected;
+
 ClientDrizzleProtocol::ClientDrizzleProtocol(int fd, bool using_mysql41_protocol_arg):
   using_mysql41_protocol(using_mysql41_protocol_arg)
 {
@@ -146,6 +157,7 @@ void ClientDrizzleProtocol::close(void)
   { 
     drizzleclient_net_close(&net);
     drizzleclient_net_end(&net);
+    connected.decrement();
   }
 }
 
@@ -153,6 +165,9 @@ bool ClientDrizzleProtocol::authenticate()
 {
   bool connection_is_valid;
 
+  connectionCount.increment();
+  connected.increment();
+
   /* Use "connect_timeout" value during connection phase */
   drizzleclient_net_set_read_timeout(&net, connect_timeout);
   drizzleclient_net_set_write_timeout(&net, connect_timeout);
@@ -164,6 +179,7 @@ bool ClientDrizzleProtocol::authenticate()
   else
   {
     sendError(session->main_da.sql_errno(), session->main_da.message());
+    failedConnections.increment();
     return false;
   }
 
@@ -663,14 +679,14 @@ bool ClientDrizzleProtocol::checkConnection(void)
     server_capabilites|= CLIENT_COMPRESS;
 #endif /* HAVE_COMPRESS */
 
-    end= buff + strlen(VERSION);
+    end= buff + strlen(PANDORA_RELEASE_VERSION);
     if ((end - buff) >= SERVER_VERSION_LENGTH)
       end= buff + (SERVER_VERSION_LENGTH - 1);
-    memcpy(buff, VERSION, end - buff);
+    memcpy(buff, PANDORA_RELEASE_VERSION, end - buff);
     *end= 0;
     end++;
 
-    int4store((unsigned char*) end, global_thread_id);
+    int4store((unsigned char*) end, session->variables.pseudo_thread_id);
     end+= 4;
 
     /* We don't use scramble anymore. */
@@ -812,19 +828,78 @@ void ClientDrizzleProtocol::writeEOFPacket(uint32_t server_status,
   drizzleclient_net_write(&net, buff, 5);
 }
 
-static ListenDrizzleProtocol *listen_obj= NULL;
-
-static int init(plugin::Registry &registry)
+static int init(module::Context &context)
 {
-  listen_obj= new ListenDrizzleProtocol("drizzle_protocol", false);
-  registry.add(listen_obj); 
-  return 0;
-}
+  drizzle_status_table_function_ptr= new DrizzleProtocolStatus;
 
-static int deinit(plugin::Registry &registry)
-{
-  registry.remove(listen_obj);
-  delete listen_obj;
+  context.add(drizzle_status_table_function_ptr);
+
+  const module::option_map &vm= context.getOptions();
+  if (vm.count("port"))
+  { 
+    if (port > 65535)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Invalid value of port\n"));
+      exit(-1);
+    }
+  }
+
+  if (vm.count("connect-timeout"))
+  {
+    if (connect_timeout < 1 || connect_timeout > 300)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Invalid value for connect_timeout\n"));
+      exit(-1);
+    }
+  }
+
+  if (vm.count("read-timeout"))
+  {
+    if (read_timeout < 1 || read_timeout > 300)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Invalid value for read_timeout\n"));
+      exit(-1);
+    }
+  }
+
+  if (vm.count("write-timeout"))
+  {
+    if (write_timeout < 1 || write_timeout > 300)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Invalid value for write_timeout\n"));
+      exit(-1);
+    }
+  }
+
+  if (vm.count("retry-count"))
+  {
+    if (retry_count < 1 || retry_count > 100)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Invalid value for retry_count\n"));
+      exit(-1);
+    }
+  }
+
+  if (vm.count("buffer-length"))
+  {
+    if (buffer_length < 1024 || buffer_length > 1024*1024)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Invalid value for buffer_length\n"));
+      exit(-1);
+    }
+  }
+
+  if (vm.count("bind-address"))
+  {
+    bind_address= strdup(vm["bind-address"].as<string>().c_str());
+  }
+
+  else
+  {
+    bind_address= NULL;
+  }
+  
+  context.add(new ListenDrizzleProtocol("drizzle_protocol", false)); 
   return 0;
 }
 
@@ -849,6 +924,34 @@ static DRIZZLE_SYSVAR_UINT(buffer_length, buffer_length, PLUGIN_VAR_RQCMDARG,
 static DRIZZLE_SYSVAR_STR(bind_address, bind_address, PLUGIN_VAR_READONLY,
                           N_("Address to bind to."), NULL, NULL, NULL);
 
+static void init_options(drizzled::module::option_context &context)
+{
+  context("port",
+          po::value<uint32_t>(&port)->default_value(0),
+          N_("Port number to use for connection or 0 for "
+                              "default to, in order of "
+                              "preference, drizzle.cnf, $DRIZZLE_TCP_PORT, "
+                              "built-in default (4427)."));
+  context("connect-timeout",
+          po::value<uint32_t>(&connect_timeout)->default_value(10),
+          N_("Connect Timeout."));
+  context("read-timeout",
+          po::value<uint32_t>(&read_timeout)->default_value(30),
+          N_("Read Timeout."));
+  context("write-timeout",
+          po::value<uint32_t>(&write_timeout)->default_value(60),
+          N_("Write Timeout."));
+  context("retry-count",
+          po::value<uint32_t>(&retry_count)->default_value(10),
+          N_("Retry Count."));
+  context("buffer-length",
+          po::value<uint32_t>(&buffer_length)->default_value(16384),
+          N_("Buffer length."));
+  context("bind-address",
+          po::value<string>(),
+          N_("Address to bind to."));
+}
+
 static drizzle_sys_var* sys_variables[]= {
   DRIZZLE_SYSVAR(port),
   DRIZZLE_SYSVAR(connect_timeout),
@@ -860,6 +963,103 @@ static drizzle_sys_var* sys_variables[]= {
   NULL
 };
 
+static int drizzle_protocol_connection_count_func(drizzle_show_var *var, char *buff)
+{
+  var->type= SHOW_LONGLONG;
+  var->value= buff;
+  *((uint64_t *)buff)= ClientDrizzleProtocol::connectionCount;
+  return 0;
+}
+
+static int drizzle_protocol_connected_count_func(drizzle_show_var *var, char *buff)
+{
+  var->type= SHOW_LONGLONG;
+  var->value= buff;
+  *((uint64_t *)buff)= ClientDrizzleProtocol::connected;
+  return 0;
+}
+
+static int drizzle_protocol_failed_count_func(drizzle_show_var *var, char *buff)
+{
+  var->type= SHOW_LONGLONG;
+  var->value= buff;
+  *((uint64_t *)buff)= ClientDrizzleProtocol::failedConnections;
+  return 0;
+}
+
+static st_show_var_func_container drizzle_protocol_connection_count=
+  { &drizzle_protocol_connection_count_func };
+
+static st_show_var_func_container drizzle_protocol_connected_count=
+  { &drizzle_protocol_connected_count_func };
+
+static st_show_var_func_container drizzle_protocol_failed_count=
+  { &drizzle_protocol_failed_count_func };
+
+static drizzle_show_var drizzle_protocol_status_variables[]= {
+  {"Connections",
+  (char*) &drizzle_protocol_connection_count, SHOW_FUNC},
+  {"Connected",
+  (char*) &drizzle_protocol_connected_count, SHOW_FUNC},
+  {"Failed_connections",
+  (char*) &drizzle_protocol_failed_count, SHOW_FUNC},
+  {NULL, NULL, SHOW_LONGLONG}
+};
+
+DrizzleProtocolStatus::Generator::Generator(drizzled::Field **fields) :
+  plugin::TableFunction::Generator(fields)
+{
+  status_var_ptr= drizzle_protocol_status_variables;
+}
+
+bool DrizzleProtocolStatus::Generator::populate()
+{
+  MY_ALIGNED_BYTE_ARRAY(buff_data, SHOW_VAR_FUNC_BUFF_SIZE, int64_t);
+  char * const buff= (char *) &buff_data;
+  drizzle_show_var tmp;
+
+  if (status_var_ptr->name)
+  {
+    std::ostringstream oss;
+    string return_value;
+    const char *value;
+    int type;
+
+    push(status_var_ptr->name);
+
+    if (status_var_ptr->type == SHOW_FUNC)
+    {
+      ((mysql_show_var_func)((st_show_var_func_container *)status_var_ptr->value)->func)(&tmp, buff);
+      value= buff;
+      type= tmp.type;
+    }
+    else
+    {
+      value= status_var_ptr->value;
+      type= status_var_ptr->type;
+    }
+
+    switch(type)
+    {
+    case SHOW_LONGLONG:
+      oss << *(uint64_t*) value;
+      return_value= oss.str();
+      break;
+    default:
+      assert(0);
+    }
+    if (return_value.length())
+      push(return_value);
+    else
+      push(" ");
+
+    status_var_ptr++;
+
+    return true;
+  }
+  return false;
+}
+
 } /* namespace drizzle_protocol */
 
-DRIZZLE_PLUGIN(drizzle_protocol::init, drizzle_protocol::deinit, drizzle_protocol::sys_variables);
+DRIZZLE_PLUGIN(drizzle_protocol::init, drizzle_protocol::sys_variables, drizzle_protocol::init_options);
