@@ -22,6 +22,8 @@
 
 #include <drizzled/plugin/listen_tcp.h>
 #include <drizzled/plugin/client.h>
+#include <drizzled/atomics.h>
+#include "drizzled/plugin/table_function.h"
 
 #include "net_serv.h"
 
@@ -38,6 +40,7 @@ public:
    drizzled::plugin::ListenTcp(name_arg),
    using_mysql41_protocol(using_mysql41_protocol_arg)
   { }
+  virtual ~ListenDrizzleProtocol();
   virtual const char* getHost(void) const;
   virtual in_port_t getPort(void) const;
   virtual drizzled::plugin::Client *getClient(int fd);
@@ -59,6 +62,10 @@ public:
   ClientDrizzleProtocol(int fd, bool using_mysql41_protocol_arg);
   virtual ~ClientDrizzleProtocol();
 
+  static drizzled::atomic<uint64_t> connectionCount;
+  static drizzled::atomic<uint64_t> failedConnections;
+  static drizzled::atomic<uint64_t> connected;
+
   virtual int getFileDescriptor(void);
   virtual bool isConnected();
   virtual bool isReading(void);
