@@ -21,7 +21,6 @@
 #include <drizzled/plugin/error_message.h>
 #include <drizzled/gettext.h>
 #include <drizzled/plugin.h>
-#include <drizzled/plugin/registry.h>
 
 #include <stdio.h>  /* for vsnprintf */
 #include <stdarg.h>  /* for va_list */
@@ -56,25 +55,14 @@ public:
 };
 
 static Error_message_stderr *handler= NULL;
-static int errmsg_stderr_plugin_init(plugin::Registry &registry)
+static int errmsg_stderr_plugin_init(module::Context &context)
 {
   handler= new Error_message_stderr();
-  registry.add(handler);
+  context.add(handler);
 
   return 0;
 }
 
-static int errmsg_stderr_plugin_deinit(plugin::Registry &registry)
-{
-
-  if (handler)
-  {
-    registry.remove(handler);
-    delete handler;
-  }
-  return 0;
-}
-
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
@@ -84,7 +72,6 @@ DRIZZLE_DECLARE_PLUGIN
   N_("Error Messages to stderr"),
   PLUGIN_LICENSE_GPL,
   errmsg_stderr_plugin_init,
-  errmsg_stderr_plugin_deinit,
   NULL, /* system variables */
   NULL
 }
