@@ -18,8 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <plugin/function_dictionary/dictionary.h>
+#include "config.h"
+#include "plugin/function_dictionary/dictionary.h"
 
 using namespace drizzled;
 
@@ -34,10 +34,10 @@ static int init(drizzled::module::Context &context)
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
-  "function_dictionary",
+  "Function Dictionary",
   "1.0",
   "Brian Aker",
-  N_("Dictionary of functions"),
+  "Data Dictionary for functions.",
   PLUGIN_LICENSE_GPL,
   init,
   NULL,
