@@ -25,6 +25,7 @@
 #include <drizzled/session.h>
 #include <drizzled/plugin/storage_engine.h>
 #include <drizzled/plugin/table_function.h>
+#include <drizzled/identifier/schema.h>
 
 extern const drizzled::CHARSET_INFO *default_charset_info;
 
@@ -34,7 +35,6 @@ static const char *function_exts[] = {
 
 class Function : public drizzled::plugin::StorageEngine
 {
-
 public:
   Function(const std::string &name_arg);
 
@@ -43,21 +43,20 @@ public:
 
   drizzled::plugin::TableFunction *getTool(const char *name_arg);
 
-  int doCreateTable(drizzled::Session *,
-                    const char *,
+  int doCreateTable(drizzled::Session&,
                     drizzled::Table&,
+                    const drizzled::TableIdentifier &,
                     drizzled::message::Table&)
   {
     return EPERM;
   }
 
-  int doDropTable(drizzled::Session&, const std::string&) 
+  int doDropTable(drizzled::Session&, const drizzled::TableIdentifier&)
   { 
     return EPERM; 
   }
 
-  virtual drizzled::Cursor *create(drizzled::TableShare &table,
-                                   drizzled::memory::Root *mem_root);
+  virtual drizzled::Cursor *create(drizzled::TableShare &table);
 
   const char **bas_ext() const 
   {
@@ -72,20 +71,24 @@ public:
   bool doCanCreateTable(const drizzled::TableIdentifier &identifier);
 
 
-  void doGetTableNames(drizzled::CachedDirectory&, 
-                       std::string &db, 
-                       std::set<std::string> &set_of_names);
-
   int doGetTableDefinition(drizzled::Session &session,
-                           const char *path,
-                           const char *db,
-                           const char *table_name,
-                           const bool is_tmp,
-                           drizzled::message::Table *table_proto);
+                           const drizzled::TableIdentifier &identifier,
+                           drizzled::message::Table &table_message);
 
-  void doGetSchemaNames(std::set<std::string>& set_of_names);
+  void doGetSchemaIdentifiers(drizzled::SchemaIdentifiers&);
 
-  bool doGetSchemaDefinition(const std::string &schema_name, drizzled::message::Schema &schema_message);
+  bool doDoesTableExist(drizzled::Session& session, const drizzled::TableIdentifier &identifier);
+
+  bool doGetSchemaDefinition(const drizzled::SchemaIdentifier &schema, drizzled::message::Schema &schema_message);
+
+  int doRenameTable(drizzled::Session&, const drizzled::TableIdentifier &, const drizzled::TableIdentifier &)
+  {
+    return EPERM;
+  }
+
+  void doGetTableIdentifiers(drizzled::CachedDirectory &directory,
+                             const drizzled::SchemaIdentifier &schema_identifier,
+                             drizzled::TableIdentifiers &set_of_identifiers);
 };
 
 #endif /* PLUGIN_FUNCTION_ENGINE_FUNCTION_H */
