@@ -1,4 +1,4 @@
-/* Copyright (C) 2009 Sun Microsystems, Inc.
+/* Copyright (C) 2009 Sun Microsystems
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,8 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
-#pragma once
+#ifndef PLUGIN_GEARMAN_UDF_FUNCTION_MAP_H
+#define PLUGIN_GEARMAN_UDF_FUNCTION_MAP_H
 
 #include <map>
 #include <string>
@@ -35,3 +36,4 @@ public:
 /* This returns a reference to the global function map class. */
 GearmanFunctionMap& GetFunctionMap(void);
 
+#endif /* PLUGIN_GEARMAN_UDF_FUNCTION_MAP_H */
