@@ -1,4 +1,4 @@
-/* Copyright (C) 2009 Sun Microsystems, Inc.
+/* Copyright (C) 2009 Sun Microsystems
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
-#include <config.h>
-#include "plugin/gearman_udf/gman_servers_set.h"
-#include "plugin/gearman_udf/function_map.h"
+#include "config.h"
+#include "gman_servers_set.h"
+#include "function_map.h"
 
 using namespace std;
 using namespace drizzled;
