@@ -1,4 +1,4 @@
-/* Copyright (C) 2009 Sun Microsystems, Inc.
+/* Copyright (C) 2009 Sun Microsystems
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,8 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
-#pragma once
+#ifndef PLUGIN_GEARMAN_UDF_GMAN_SERVERS_SET_H
+#define PLUGIN_GEARMAN_UDF_GMAN_SERVERS_SET_H
 
 #include <drizzled/item/func.h>
 #include <drizzled/function/str/strfunc.h>
@@ -29,3 +30,4 @@ public:
   drizzled::String *val_str(drizzled::String *);
 };
 
+#endif /* PLUGIN_GEARMAN_UDF_GMAN_SERVERS_SET_H */
