@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INFORMATION_SCHEMA_DICTIONARY_CHECK_CONSTRAINTS_H
+#define PLUGIN_INFORMATION_SCHEMA_DICTIONARY_CHECK_CONSTRAINTS_H
 
 /*
   @note if implemented this should only provide information based on the current user.
@@ -50,3 +51,4 @@ public:
 
 };
 
+#endif /* PLUGIN_INFORMATION_SCHEMA_DICTIONARY_CHECK_CONSTRAINTS_H */
