@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INFORMATION_SCHEMA_DICTIONARY_COLUMNS_H
+#define PLUGIN_INFORMATION_SCHEMA_DICTIONARY_COLUMNS_H
 
 class Columns : public InformationSchema
 {
@@ -28,7 +29,9 @@ public:
 
   class Generator : public InformationSchema::Generator 
   {
-    drizzled::generator::AllFields field_generator;
+    const drizzled::message::Table *table_message;
+    drizzled::generator::AllTables generator;
+    int32_t field_iterator;
 
   public:
     Generator(drizzled::Field **arg);
@@ -43,3 +46,4 @@ public:
 
 };
 
+#endif /* PLUGIN_INFORMATION_SCHEMA_DICTIONARY_COLUMNS_H */
