@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INFORMATION_SCHEMA_DICTIONARY_CONSTRAINT_COLUMN_USAGE_H
+#define PLUGIN_INFORMATION_SCHEMA_DICTIONARY_CONSTRAINT_COLUMN_USAGE_H
 
 /*
   @note if implemented, this should only return the tables that were owned by the user.
@@ -50,3 +51,4 @@ public:
 
 };
 
+#endif /* PLUGIN_INFORMATION_SCHEMA_DICTIONARY_CONSTRAINT_COLUMN_USAGE_H */
