@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INFORMATION_SCHEMA_DICTIONARY_INFORMATION_SCHEMA_H
+#define PLUGIN_INFORMATION_SCHEMA_DICTIONARY_INFORMATION_SCHEMA_H
 
 class InformationSchema : public drizzled::plugin::TableFunction
 {
@@ -34,6 +35,8 @@ public:
     Generator(drizzled::Field **arg):
       drizzled::plugin::TableFunction::Generator(arg)
     { }
+
+    void pushType(drizzled::message::Table::Field::FieldType type);
   };
 
   Generator *generator(drizzled::Field **arg)
@@ -43,3 +46,4 @@ public:
 
 };
 
+#endif /* PLUGIN_INFORMATION_SCHEMA_DICTIONARY_INFORMATION_SCHEMA_H */
