@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INFORMATION_SCHEMA_DICTIONARY_KEY_COLUMN_USAGE_H
+#define PLUGIN_INFORMATION_SCHEMA_DICTIONARY_KEY_COLUMN_USAGE_H
 
 class KeyColumnUsage : public InformationSchema
 {
@@ -28,14 +29,11 @@ public:
 
   class Generator : public InformationSchema::Generator 
   {
-    drizzled::message::table::shared_ptr table_message;
+    const drizzled::message::Table *table_message;
     drizzled::generator::AllTables generator;
     int32_t index_iterator;
     int32_t index_part_iterator;
 
-    int32_t fk_constraint_iterator;
-    int32_t fk_constraint_column_name_iterator;
-
   public:
     Generator(drizzled::Field **arg);
 
@@ -49,3 +47,4 @@ public:
 
 };
 
+#endif /* PLUGIN_INFORMATION_SCHEMA_DICTIONARY_KEY_COLUMN_USAGE_H */
