@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INFORMATION_SCHEMA_DICTIONARY_REFERENTIAL_CONSTRAINTS_H
+#define PLUGIN_INFORMATION_SCHEMA_DICTIONARY_REFERENTIAL_CONSTRAINTS_H
 
 class ReferentialConstraints : public InformationSchema
 {
@@ -28,7 +29,10 @@ public:
 
   class Generator : public InformationSchema::Generator 
   {
-    drizzled::generator::AllForeignKeys foreign_key_generator;
+    bool is_primed;
+    void fill();
+    bool nextCore();
+    bool next();
 
   public:
     Generator(drizzled::Field **arg);
@@ -43,3 +47,4 @@ public:
 
 };
 
+#endif /* PLUGIN_INFORMATION_SCHEMA_DICTIONARY_REFERENTIAL_CONSTRAINTS_H */
