@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INFORMATION_SCHEMA_DICTIONARY_ROUTINE_COLUMNS_H
+#define PLUGIN_INFORMATION_SCHEMA_DICTIONARY_ROUTINE_COLUMNS_H
 
 class RoutineColumns : public InformationSchema
 {
@@ -46,3 +47,4 @@ public:
 
 };
 
+#endif /* PLUGIN_INFORMATION_SCHEMA_DICTIONARY_ROUTINE_COLUMNS_H */
