@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_INFORMATION_SCHEMA_DICTIONARY_TABLE_CONSTRAINTS_H
+#define PLUGIN_INFORMATION_SCHEMA_DICTIONARY_TABLE_CONSTRAINTS_H
 
 class TableConstraints : public InformationSchema
 {
@@ -28,7 +29,7 @@ public:
 
   class Generator : public InformationSchema::Generator 
   {
-    drizzled::message::table::shared_ptr table_message;
+    const drizzled::message::Table *table_message;
     drizzled::generator::AllTables generator;
     int32_t index_iterator;
 
@@ -45,3 +46,4 @@ public:
 
 };
 
+#endif /* PLUGIN_INFORMATION_SCHEMA_DICTIONARY_TABLE_CONSTRAINTS_H */
