@@ -35,11 +35,18 @@ using namespace drizzled;
 typedef struct st_innobase_share {
 	THR_LOCK	lock;		/*!< MySQL lock protecting
 					this structure */
-	const char*	table_name;	/*!< InnoDB table name */
+	char	table_name[FN_REFLEN];	/*!< InnoDB table name */
 	uint		use_count;	/*!< reference count,
 					incremented in get_share()
 					and decremented in free_share() */
 	void*		table_name_hash;/*!< hash table chain node */
+
+        st_innobase_share(const char *arg) :
+          use_count(0)
+        {
+          strncpy(table_name, arg, FN_REFLEN);
+        }
+
 } INNOBASE_SHARE;
 
 
@@ -66,10 +73,10 @@ class ha_innobase: public Cursor
 	INNOBASE_SHARE*	share;		/*!< information for MySQL
 					table locking */
 
-	unsigned char*	upd_buff;	/*!< buffer used in updates */
-	unsigned char*	key_val_buff;	/*!< buffer used in converting
-					search key values from MySQL format
-					to Innodb format */
+        std::vector<unsigned char> upd_buff; /*!< buffer used in updates */
+        std::vector<unsigned char> key_val_buff; /*!< buffer used in converting
+                                                     search key values from MySQL format
+                                                     to Innodb format */
 	ulong		upd_and_key_val_buff_len;
 					/* the length of each of the previous
 					two buffers */
@@ -80,12 +87,11 @@ class ha_innobase: public Cursor
 	uint		last_match_mode;/* match mode of the latest search:
 					ROW_SEL_EXACT, ROW_SEL_EXACT_PREFIX,
 					or undefined */
-	uint		num_write_row;	/*!< number of write_row() calls */
+	uint		num_write_row;	/*!< number of doInsertRecord() calls */
 
 	UNIV_INTERN uint store_key_val_for_row(uint keynr, char* buff, 
                                    uint buff_len, const unsigned char* record);
 	UNIV_INTERN void update_session(Session* session);
-	UNIV_INTERN void update_session();
 	UNIV_INTERN int change_active_index(uint32_t keynr);
 	UNIV_INTERN int general_fetch(unsigned char* buf, uint32_t direction, uint32_t match_mode);
 	UNIV_INTERN ulint innobase_lock_autoinc();
@@ -116,29 +122,23 @@ class ha_innobase: public Cursor
     return static_cast<plugin::TransactionalStorageEngine *>(engine);
   }
 
-	/*
-	  Get the row type from the storage engine.  If this method returns
-	  ROW_TYPE_NOT_USED, the information in HA_CREATE_INFO should be used.
-	*/
-	UNIV_INTERN enum row_type get_row_type() const;
-
 	UNIV_INTERN const char* index_type(uint key_number);
 	UNIV_INTERN const key_map* keys_to_use_for_scanning();
 
-	UNIV_INTERN int open(const char *name, int mode, uint test_if_locked);
+	UNIV_INTERN int doOpen(const drizzled::TableIdentifier &identifier, int mode, uint test_if_locked);
 	UNIV_INTERN int close(void);
 	UNIV_INTERN double scan_time();
 	UNIV_INTERN double read_time(uint index, uint ranges, ha_rows rows);
 
-	UNIV_INTERN int write_row(unsigned char * buf);
-	UNIV_INTERN int update_row(const unsigned char * old_data, unsigned char * new_data);
-	UNIV_INTERN int delete_row(const unsigned char * buf);
+	UNIV_INTERN int doInsertRecord(unsigned char * buf);
+	UNIV_INTERN int doUpdateRecord(const unsigned char * old_data, unsigned char * new_data);
+	UNIV_INTERN int doDeleteRecord(const unsigned char * buf);
 	UNIV_INTERN bool was_semi_consistent_read();
 	UNIV_INTERN void try_semi_consistent_read(bool yes);
 	UNIV_INTERN void unlock_row();
 
-	UNIV_INTERN int index_init(uint index, bool sorted);
-	UNIV_INTERN int index_end();
+	UNIV_INTERN int doStartIndexScan(uint index, bool sorted);
+	UNIV_INTERN int doEndIndexScan();
 	UNIV_INTERN int index_read(unsigned char * buf, const unsigned char * key,
 		uint key_len, enum ha_rkey_function find_flag);
 	UNIV_INTERN int index_read_idx(unsigned char * buf, uint index, const unsigned char * key,
@@ -150,8 +150,8 @@ class ha_innobase: public Cursor
 	UNIV_INTERN int index_first(unsigned char * buf);
 	UNIV_INTERN int index_last(unsigned char * buf);
 
-	UNIV_INTERN int rnd_init(bool scan);
-	UNIV_INTERN int rnd_end();
+	UNIV_INTERN int doStartTableScan(bool scan);
+	UNIV_INTERN int doEndTableScan();
 	UNIV_INTERN int rnd_next(unsigned char *buf);
 	UNIV_INTERN int rnd_pos(unsigned char * buf, unsigned char *pos);
 
@@ -171,7 +171,7 @@ class ha_innobase: public Cursor
 	UNIV_INTERN int check(Session* session);
 	UNIV_INTERN char* update_table_comment(const char* comment);
 	UNIV_INTERN char* get_foreign_key_create_info();
-	UNIV_INTERN int get_foreign_key_list(Session *session, List<FOREIGN_KEY_INFO> *f_key_list);
+	UNIV_INTERN int get_foreign_key_list(Session *session, List<ForeignKeyInfo> *f_key_list);
 	UNIV_INTERN bool can_switch_engines();
 	UNIV_INTERN uint referenced_by_foreign_key();
 	UNIV_INTERN void free_foreign_key_create_info(char* str);
@@ -187,10 +187,16 @@ class ha_innobase: public Cursor
 	UNIV_INTERN bool primary_key_is_clustered();
 	UNIV_INTERN int cmp_ref(const unsigned char *ref1, const unsigned char *ref2);
 	/** Fast index creation (smart ALTER TABLE) @see handler0alter.cc @{ */
-	UNIV_INTERN int add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
-	UNIV_INTERN int prepare_drop_index(TABLE *table_arg, uint *key_num,
+        // Don't use these, I have just left them in here as reference for
+        // the future. -Brian 
+#if 0
+	UNIV_INTERN int add_index(Session *session, TABLE *table_arg, KeyInfo *key_info, uint num_of_keys);
+	UNIV_INTERN int prepare_drop_index(Session *session,
+                                           TABLE *table_arg,
+                                           uint *key_num,
                                            uint num_of_keys);
-        UNIV_INTERN int final_drop_index(TABLE *table_arg);
+        UNIV_INTERN int final_drop_index(Session *session, TABLE *table_arg);
+#endif
 	/** @} */
 public:
   int read_range_first(const key_range *start_key, const key_range *end_key,
@@ -219,20 +225,6 @@ uint64_t drizzle_bin_log_file_pos(void);
 */
 int session_slave_thread(const Session *session);
 
-/**
-  Check if a user thread is running a non-transactional update
-  @param session  user thread
-  @retval 0 the user thread is not running a non-transactional update
-  @retval 1 the user thread is running a non-transactional update
-*/
-int session_non_transactional_update(const Session *session);
-
-/**
-  Mark transaction to rollback and mark error as fatal to a sub-statement.
-  @param  session   Thread handle
-  @param  all   TRUE <=> rollback main transaction.
-*/
-void session_mark_transaction_to_rollback(Session *session, bool all);
 }
 
 typedef struct trx_struct trx_t;
