@@ -65,24 +65,15 @@ int64_t LengthFunction::val_int()
 plugin::Create_function<LengthFunction> *lengthudf= NULL;
 plugin::Create_function<LengthFunction> *octet_lengthudf= NULL;
 
-static int initialize(drizzled::plugin::Registry &registry)
+static int initialize(drizzled::module::Context &context)
 {
   lengthudf= new plugin::Create_function<LengthFunction>("length");
   octet_lengthudf= new plugin::Create_function<LengthFunction>("octet_length");
-  registry.add(lengthudf);
-  registry.add(octet_lengthudf);
+  context.add(lengthudf);
+  context.add(octet_lengthudf);
   return 0;
 }
 
-static int finalize(drizzled::plugin::Registry &registry)
-{
-   registry.remove(lengthudf);
-   registry.remove(octet_lengthudf);
-   delete lengthudf;
-   delete octet_lengthudf;
-   return 0;
-}
-
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
@@ -92,7 +83,6 @@ DRIZZLE_DECLARE_PLUGIN
   "Return the byte length of a string",
   PLUGIN_LICENSE_GPL,
   initialize, /* Plugin Init */
-  finalize,   /* Plugin Deinit */
   NULL,   /* system variables */
   NULL    /* config options */
 }
