@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_MATH_FUNCTIONS_ABS_H
+#define PLUGIN_MATH_FUNCTIONS_ABS_H
 
 #include <drizzled/function/func.h>
 #include <drizzled/function/num1.h>
@@ -31,7 +32,7 @@ public:
   Item_func_abs() :Item_func_num1() {}
   double real_op();
   int64_t int_op();
-  type::Decimal *decimal_op(type::Decimal *);
+  my_decimal *decimal_op(my_decimal *);
   const char *func_name() const { return "abs"; }
   void fix_length_and_dec();
   bool check_argument_count(int n) { return n == 1; }
@@ -39,3 +40,4 @@ public:
 
 } /* namespace drizzled */
 
+#endif /* PLUGIN_MATH_FUNCTIONS_ABS_H */
