@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_MATH_FUNCTIONS_CEILING_H
+#define PLUGIN_MATH_FUNCTIONS_CEILING_H
 
 #include <drizzled/function/func.h>
 #include <drizzled/function/math/int_val.h>
@@ -32,9 +33,10 @@ public:
   const char *func_name() const { return "ceiling"; }
   int64_t int_op();
   double real_op();
-  type::Decimal *decimal_op(type::Decimal *);
+  my_decimal *decimal_op(my_decimal *);
   bool check_argument_count(int n) { return n == 1; }
 };
 
 } /* namespace drizzled */
 
+#endif /* PLUGIN_MATH_FUNCTIONS_CEILING_H */
