@@ -18,7 +18,9 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_MATH_FUNCTIONS_FUNCTIONS_H
+#define PLUGIN_MATH_FUNCTIONS_FUNCTIONS_H
 
-#include <drizzled/plugin/table_function.h>
+#include "drizzled/plugin/table_function.h"
 
+#endif /* PLUGIN_MATH_FUNCTIONS_FUNCTIONS_H */
