@@ -30,8 +30,8 @@
 #include "config.h"
 #include "drizzled/show.h"
 #include "drizzled/gettext.h"
-#include "drizzled/plugin/info_schema_table.h"
-
+#include <boost/program_options.hpp>
+#include <drizzled/module/option_map.h>
 #include "stats_table.h"
 #include "analysis_table.h"
 #include "sysvar_holder.h"
@@ -39,6 +39,7 @@
 #include <string>
 #include <map>
 
+namespace po=boost::program_options;
 using namespace std;
 using namespace drizzled;
 
@@ -60,37 +61,34 @@ static char *sysvar_memcached_servers= NULL;
  * @param[in] registry the drizzled::plugin::Registry singleton
  * @return false on success; true on failure.
  */
-static int init(plugin::Registry &registry)
+static int init(module::Context &context)
 {
+  const module::option_map &vm= context.getOptions();
+
+  if(vm.count("servers"))
+  {
+    sysvar_memcached_servers= strdup(vm["servers"].as<string>().c_str());
+  }
+
+  else
+  {
+    sysvar_memcached_servers= strdup("");
+  }
 
   SysvarHolder &sysvar_holder= SysvarHolder::singleton();
   sysvar_holder.setServersString(sysvar_memcached_servers);
+  sysvar_holder.setMemoryPtr(sysvar_memcached_servers);
 
   /* we are good to go */
   stats_table_tool= new(std::nothrow)StatsTableTool; 
-  registry.add(stats_table_tool);
+  context.add(stats_table_tool);
 
   analysis_table_tool= new(std::nothrow)AnalysisTableTool;
-  registry.add(analysis_table_tool);
+  context.add(analysis_table_tool);
 
   return 0;
 }
 
-/**
- * Clean up the memcached stats plugin.
- *
- * @param[in] registry the drizzled::plugin::Registry singleton
- * @return false on success; true on failure
- */
-static int deinit(plugin::Registry &registry)
-{
-  registry.remove(stats_table_tool);
-  delete stats_table_tool;
-  registry.remove(analysis_table_tool);
-  delete analysis_table_tool;
-  return 0;
-}
-
 static int check_memc_servers(Session *,
                               drizzle_sys_var *,
                               void *save,
@@ -132,6 +130,13 @@ static DRIZZLE_SYSVAR_STR(servers,
                           set_memc_servers, /* update func */
                           ""); /* default value */
 
+static void init_options(drizzled::module::option_context &context)
+{
+  context("servers",
+          po::value<string>(),
+          N_("List of memcached servers."));
+}
+
 static drizzle_sys_var *system_variables[]=
 {
   DRIZZLE_SYSVAR(servers),
@@ -147,8 +152,7 @@ DRIZZLE_DECLARE_PLUGIN
   N_("Memcached Stats as I_S tables"),
   PLUGIN_LICENSE_BSD,
   init,   /* Plugin Init      */
-  deinit, /* Plugin Deinit    */
   system_variables, /* system variables */
-  NULL    /* config options   */
+  init_options    /* config options   */
 }
 DRIZZLE_DECLARE_PLUGIN_END;
