@@ -11,19 +11,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include "heap_priv.h"
 #include <drizzled/error.h>
 #include <drizzled/table.h>
 #include <drizzled/session.h>
+#include <drizzled/field/timestamp.h>
 #include <drizzled/field/varstring.h>
-#include <drizzled/plugin/daemon.h>
-#include <drizzled/plugin/storage_engine.h>
-#include <drizzled/util/test.h>
-#include <drizzled/session/table_messages.h>
-#include <drizzled/statistics_variables.h>
-#include <drizzled/system_variables.h>
+#include "drizzled/plugin/daemon.h"
 
 #include <boost/thread/mutex.hpp>
 
@@ -32,6 +28,7 @@
 
 #include <string>
 
+
 using namespace drizzled;
 using namespace std;
 
@@ -63,25 +60,19 @@ public:
     hp_panic(HA_PANIC_CLOSE);
   }
 
-  virtual Cursor *create(Table &table)
+  virtual Cursor *create(TableShare &table)
   {
     return new ha_heap(*this, table);
   }
 
-  const char **bas_ext() const 
-  {
+  const char **bas_ext() const {
     return ha_heap_exts;
   }
 
-  drizzled::message::Table::Index::IndexType default_index_type() const
-  {
-    return drizzled::message::Table::Index::HASH;
-  }
-
   int doCreateTable(Session &session,
                     Table &table_arg,
-                    const identifier::Table &identifier,
-                    const message::Table &create_proto);
+                    const TableIdentifier &identifier,
+                    message::Table &create_proto);
 
   /* For whatever reason, internal tables can be created by Cursor::open()
      for MEMORY.
@@ -91,15 +82,15 @@ public:
   int heap_create_table(Session *session, const char *table_name,
                         Table *table_arg,
                         bool internal_table,
-                        const message::Table &create_proto,
+                        message::Table &create_proto,
                         HP_SHARE **internal_share);
 
-  int doRenameTable(Session&, const identifier::Table &from, const identifier::Table &to);
+  int doRenameTable(Session&, const TableIdentifier &from, const TableIdentifier &to);
 
-  int doDropTable(Session&, const identifier::Table &identifier);
+  int doDropTable(Session&, const TableIdentifier &identifier);
 
   int doGetTableDefinition(Session& session,
-                           const identifier::Table &identifier,
+                           const TableIdentifier &identifier,
                            message::Table &table_message);
 
   uint32_t max_supported_keys()          const { return MAX_KEY; }
@@ -110,28 +101,28 @@ public:
     return ( HA_ONLY_WHOLE_INDEX | HA_KEY_SCAN_NOT_ROR);
   }
 
-  bool doDoesTableExist(Session& session, const identifier::Table &identifier);
+  bool doDoesTableExist(Session& session, const TableIdentifier &identifier);
   void doGetTableIdentifiers(CachedDirectory &directory,
-                             const identifier::Schema &schema_identifier,
-                             identifier::table::vector &set_of_identifiers);
+                             const SchemaIdentifier &schema_identifier,
+                             TableIdentifiers &set_of_identifiers);
 };
 
 void HeapEngine::doGetTableIdentifiers(CachedDirectory&,
-                                       const identifier::Schema&,
-                                       identifier::table::vector&)
+                                       const SchemaIdentifier&,
+                                       TableIdentifiers&)
 {
 }
 
-bool HeapEngine::doDoesTableExist(Session& session, const identifier::Table &identifier)
+bool HeapEngine::doDoesTableExist(Session& session, const TableIdentifier &identifier)
 {
-  return session.getMessageCache().doesTableMessageExist(identifier);
+  return session.doesTableMessageExist(identifier);
 }
 
 int HeapEngine::doGetTableDefinition(Session &session,
-                                     const identifier::Table &identifier,
+                                     const TableIdentifier &identifier,
                                      message::Table &table_proto)
 {
-  if (session.getMessageCache().getTableMessage(identifier, table_proto))
+  if (session.getTableMessage(identifier, table_proto))
     return EEXIST;
 
   return ENOENT;
@@ -140,9 +131,9 @@ int HeapEngine::doGetTableDefinition(Session &session,
   We have to ignore ENOENT entries as the MEMORY table is created on open and
   not when doing a CREATE on the table.
 */
-int HeapEngine::doDropTable(Session &session, const identifier::Table &identifier)
+int HeapEngine::doDropTable(Session &session, const TableIdentifier &identifier)
 {
-  session.getMessageCache().removeTableMessage(identifier);
+  session.removeTableMessage(identifier);
 
   int error= heap_delete_table(identifier.getPath().c_str());
 
@@ -167,7 +158,7 @@ static int heap_init(module::Context &context)
 *****************************************************************************/
 
 ha_heap::ha_heap(plugin::StorageEngine &engine_arg,
-                 Table &table_arg)
+                 TableShare &table_arg)
   :Cursor(engine_arg, table_arg), file(0), records_changed(0), key_stat_version(0),
   internal_table(0)
 {}
@@ -185,7 +176,7 @@ ha_heap::ha_heap(plugin::StorageEngine &engine_arg,
 */
 #define MEMORY_STATS_UPDATE_THRESHOLD 10
 
-int ha_heap::doOpen(const drizzled::identifier::Table &identifier, int mode, uint32_t test_if_locked)
+int ha_heap::doOpen(const drizzled::TableIdentifier &identifier, int mode, uint32_t test_if_locked)
 {
   if ((test_if_locked & HA_OPEN_INTERNAL_TABLE) || (!(file= heap_open(identifier.getPath().c_str(), mode)) && errno == ENOENT))
   {
@@ -194,9 +185,9 @@ int ha_heap::doOpen(const drizzled::identifier::Table &identifier, int mode, uin
     HP_SHARE *internal_share= NULL;
     message::Table create_proto;
 
-    if (not heap_storage_engine->heap_create_table(getTable()->in_use,
+    if (not heap_storage_engine->heap_create_table(table->in_use,
                                                    identifier.getPath().c_str(),
-                                                   getTable(),
+                                                   table,
                                                    internal_table,
                                                    create_proto,
                                                    &internal_share))
@@ -228,7 +219,7 @@ int ha_heap::doOpen(const drizzled::identifier::Table &identifier, int mode, uin
     */
     key_stat_version= file->getShare()->key_stat_version - 1;
   }
-  return file ? 0 : 1;
+  return (file ? 0 : 1);
 }
 
 int ha_heap::close(void)
@@ -249,10 +240,12 @@ int ha_heap::close(void)
 
 Cursor *ha_heap::clone(memory::Root *)
 {
-  Cursor *new_handler= getTable()->getMutableShare()->db_type()->getCursor(*getTable());
-  identifier::Table identifier(*getTable());
+  Cursor *new_handler= table->getMutableShare()->db_type()->getCursor(*(table->getMutableShare()));
+  TableIdentifier identifier(table->getShare()->getSchemaName(),
+                             table->getShare()->getTableName(),
+                             table->getShare()->getPath());
 
-  if (new_handler && !new_handler->ha_open(identifier, getTable()->db_stat,
+  if (new_handler && !new_handler->ha_open(identifier, table, table->db_stat,
                                            HA_OPEN_IGNORE_IF_LOCKED))
     return new_handler;
   return NULL;
@@ -288,9 +281,9 @@ void ha_heap::set_keys_for_scanning(void)
 
 void ha_heap::update_key_stats()
 {
-  for (uint32_t i= 0; i < getTable()->getShare()->sizeKeys(); i++)
+  for (uint32_t i= 0; i < table->getShare()->sizeKeys(); i++)
   {
-    KeyInfo *key= &getTable()->key_info[i];
+    KeyInfo *key= &table->key_info[i];
 
     if (!key->rec_per_key)
       continue;
@@ -317,7 +310,7 @@ void ha_heap::update_key_stats()
 int ha_heap::doInsertRecord(unsigned char * buf)
 {
   int res;
-  if (getTable()->next_number_field && buf == getTable()->getInsertRecord())
+  if (table->next_number_field && buf == table->getInsertRecord())
   {
     if ((res= update_auto_increment()))
       return res;
@@ -357,7 +350,7 @@ int ha_heap::doDeleteRecord(const unsigned char * buf)
   int res;
 
   res= heap_delete(file,buf);
-  if (!res && getTable()->getShare()->getType() == message::Table::STANDARD &&
+  if (!res && table->getShare()->getType() == message::Table::STANDARD &&
       ++records_changed*MEMORY_STATS_UPDATE_THRESHOLD > file->getShare()->records)
   {
     /*
@@ -376,7 +369,7 @@ int ha_heap::index_read_map(unsigned char *buf, const unsigned char *key,
   assert(inited==INDEX);
   ha_statistic_increment(&system_status_var::ha_read_key_count);
   int error = heap_rkey(file,buf,active_index, key, keypart_map, find_flag);
-  getTable()->status = error ? STATUS_NOT_FOUND : 0;
+  table->status = error ? STATUS_NOT_FOUND : 0;
   return error;
 }
 
@@ -387,7 +380,7 @@ int ha_heap::index_read_last_map(unsigned char *buf, const unsigned char *key,
   ha_statistic_increment(&system_status_var::ha_read_key_count);
   int error= heap_rkey(file, buf, active_index, key, keypart_map,
 		       HA_READ_PREFIX_LAST);
-  getTable()->status= error ? STATUS_NOT_FOUND : 0;
+  table->status= error ? STATUS_NOT_FOUND : 0;
   return error;
 }
 
@@ -397,7 +390,7 @@ int ha_heap::index_read_idx_map(unsigned char *buf, uint32_t index, const unsign
 {
   ha_statistic_increment(&system_status_var::ha_read_key_count);
   int error = heap_rkey(file, buf, index, key, keypart_map, find_flag);
-  getTable()->status = error ? STATUS_NOT_FOUND : 0;
+  table->status = error ? STATUS_NOT_FOUND : 0;
   return error;
 }
 
@@ -406,7 +399,7 @@ int ha_heap::index_next(unsigned char * buf)
   assert(inited==INDEX);
   ha_statistic_increment(&system_status_var::ha_read_next_count);
   int error=heap_rnext(file,buf);
-  getTable()->status=error ? STATUS_NOT_FOUND: 0;
+  table->status=error ? STATUS_NOT_FOUND: 0;
   return error;
 }
 
@@ -415,7 +408,7 @@ int ha_heap::index_prev(unsigned char * buf)
   assert(inited==INDEX);
   ha_statistic_increment(&system_status_var::ha_read_prev_count);
   int error=heap_rprev(file,buf);
-  getTable()->status=error ? STATUS_NOT_FOUND: 0;
+  table->status=error ? STATUS_NOT_FOUND: 0;
   return error;
 }
 
@@ -424,7 +417,7 @@ int ha_heap::index_first(unsigned char * buf)
   assert(inited==INDEX);
   ha_statistic_increment(&system_status_var::ha_read_first_count);
   int error=heap_rfirst(file, buf, active_index);
-  getTable()->status=error ? STATUS_NOT_FOUND: 0;
+  table->status=error ? STATUS_NOT_FOUND: 0;
   return error;
 }
 
@@ -433,7 +426,7 @@ int ha_heap::index_last(unsigned char * buf)
   assert(inited==INDEX);
   ha_statistic_increment(&system_status_var::ha_read_last_count);
   int error=heap_rlast(file, buf, active_index);
-  getTable()->status=error ? STATUS_NOT_FOUND: 0;
+  table->status=error ? STATUS_NOT_FOUND: 0;
   return error;
 }
 
@@ -446,7 +439,7 @@ int ha_heap::rnd_next(unsigned char *buf)
 {
   ha_statistic_increment(&system_status_var::ha_read_rnd_next_count);
   int error=heap_scan(file, buf);
-  getTable()->status=error ? STATUS_NOT_FOUND: 0;
+  table->status=error ? STATUS_NOT_FOUND: 0;
   return error;
 }
 
@@ -457,7 +450,7 @@ int ha_heap::rnd_pos(unsigned char * buf, unsigned char *pos)
   ha_statistic_increment(&system_status_var::ha_read_rnd_count);
   memcpy(&heap_position, pos, sizeof(HEAP_PTR));
   error=heap_rrnd(file, buf, heap_position);
-  getTable()->status=error ? STATUS_NOT_FOUND: 0;
+  table->status=error ? STATUS_NOT_FOUND: 0;
   return error;
 }
 
@@ -506,7 +499,7 @@ int ha_heap::reset()
 int ha_heap::delete_all_rows()
 {
   heap_clear(file);
-  if (getTable()->getShare()->getType() == message::Table::STANDARD)
+  if (table->getShare()->getType() == message::Table::STANDARD)
   {
     /*
        We can perform this safely since only one writer at the time is
@@ -619,21 +612,21 @@ int ha_heap::enable_indexes(uint32_t mode)
    [2  non-unique indexes are disabled - NOT YET IMPLEMENTED]
 */
 
-int ha_heap::indexes_are_disabled()
+int ha_heap::indexes_are_disabled(void)
 {
   return heap_indexes_are_disabled(file);
 }
 
-void ha_heap::drop_table()
+void ha_heap::drop_table(const char *)
 {
   file->getShare()->delete_on_close= 1;
   close();
 }
 
 
-int HeapEngine::doRenameTable(Session &session, const identifier::Table &from, const identifier::Table &to)
+int HeapEngine::doRenameTable(Session &session, const TableIdentifier &from, const TableIdentifier &to)
 {
-  session.getMessageCache().renameTableMessage(from, to);
+  session.renameTableMessage(from, to);
   return heap_rename(from.getPath().c_str(), to.getPath().c_str());
 }
 
@@ -641,7 +634,7 @@ int HeapEngine::doRenameTable(Session &session, const identifier::Table &from, c
 ha_rows ha_heap::records_in_range(uint32_t inx, key_range *min_key,
                                   key_range *max_key)
 {
-  KeyInfo *key= &getTable()->key_info[inx];
+  KeyInfo *key= &table->key_info[inx];
 
   if (!min_key || !max_key ||
       min_key->length != max_key->length ||
@@ -660,8 +653,8 @@ ha_rows ha_heap::records_in_range(uint32_t inx, key_range *min_key,
 
 int HeapEngine::doCreateTable(Session &session,
                               Table &table_arg,
-                              const identifier::Table &identifier,
-                              const message::Table& create_proto)
+                              const TableIdentifier &identifier,
+                              message::Table& create_proto)
 {
   int error;
   HP_SHARE *internal_share;
@@ -674,7 +667,7 @@ int HeapEngine::doCreateTable(Session &session,
 
   if (error == 0)
   {
-    session.getMessageCache().storeTableMessage(identifier, create_proto);
+    session.storeTableMessage(identifier, create_proto);
   }
 
   return error;
@@ -684,7 +677,7 @@ int HeapEngine::doCreateTable(Session &session,
 int HeapEngine::heap_create_table(Session *session, const char *table_name,
                                   Table *table_arg,
                                   bool internal_table, 
-                                  const message::Table &create_proto,
+                                  message::Table &create_proto,
                                   HP_SHARE **internal_share)
 {
   uint32_t key, parts, mem_per_row_keys= 0;
@@ -777,10 +770,10 @@ int HeapEngine::heap_create_table(Session *session, const char *table_name,
         auto_key= key+ 1;
 	auto_key_type= field->key_type();
       }
-      if ((uint)field->position() + 1 > max_key_fieldnr)
+      if ((uint)field->field_index + 1 > max_key_fieldnr)
       {
         /* Do not use seg->fieldnr as it's not reliable in case of temp tables */
-        max_key_fieldnr= field->position() + 1;
+        max_key_fieldnr= field->field_index + 1;
       }
     }
   }
@@ -841,13 +834,13 @@ int ha_heap::cmp_ref(const unsigned char *ref1, const unsigned char *ref2)
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
-  "memory",
+  "MEMORY",
   "1.0",
   "MySQL AB",
-  N_("MEMORY storage engine"),
+  "Hash based, stored in memory, useful for temporary tables",
   PLUGIN_LICENSE_GPL,
   heap_init,
-  NULL,
-  NULL
+  NULL,                       /* system variables                */
+  NULL                        /* config options                  */
 }
 DRIZZLE_DECLARE_PLUGIN_END;
