@@ -11,19 +11,18 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* Test if a record has changed since last read */
 
 #include "heap_priv.h"
-#include <drizzled/error_t.h>
 
 int hp_rectest(register HP_INFO *info, register const unsigned char *old_record)
 {
 
   if (hp_compare_record_data_to_chunkset(info->getShare(), old_record, info->current_ptr))
   {
-    return((errno= drizzled::HA_ERR_RECORD_CHANGED)); /* Record have changed */
+    return((errno=HA_ERR_RECORD_CHANGED)); /* Record have changed */
   }
 
   return(0);
