@@ -11,15 +11,14 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
   Static variables for heap library. All definied here for easy making of
   a shared library
 */
 
-#include <config.h>
-#include "heap.h"
+#include "heap_priv.h"
 #include <list>
 
 using namespace std;
