@@ -29,24 +29,12 @@ class MultiThreadScheduler: public drizzled::plugin::Scheduler
 {
 private:
   drizzled::atomic<uint32_t> thread_count;
-  pthread_attr_t attr;
 
 public:
   MultiThreadScheduler(const char *name_arg): 
     Scheduler(name_arg)
   {
-    struct sched_param tmp_sched_param;
-
-    memset(&tmp_sched_param, 0, sizeof(struct sched_param));
-
-    /* Setup attribute parameter for session threads. */
-    (void) pthread_attr_init(&attr);
-    (void) pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-    pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
-
-    tmp_sched_param.sched_priority= WAIT_PRIOR;
-    (void) pthread_attr_setschedparam(&attr, &tmp_sched_param);
-
+    setStackSize();
     thread_count= 0;
   }
 
@@ -54,19 +42,9 @@ public:
   bool addSession(drizzled::Session *session);
   void killSessionNow(drizzled::Session *session);
   
-  void runSession(drizzled::Session *session)
-  {
-    if (drizzled::internal::my_thread_init())
-    {
-      session->disconnect(drizzled::ER_OUT_OF_RESOURCES, true);
-      statistic_increment(drizzled::aborted_connects, &LOCK_status);
-      killSessionNow(session);
-    }
-
-    session->thread_stack= (char*) &session;
-    session->run();
-    killSessionNow(session);
-  }
+  void runSession(drizzled::Session *session);
+private:
+  void setStackSize();
 };
 
 #endif /* PLUGIN_MULTI_THREAD_MULTI_THREAD_H */
