@@ -11,12 +11,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* Remove a row from a MyISAM table */
 
 #include "myisam_priv.h"
-#include <drizzled/internal/m_string.h>
+#include "drizzled/internal/m_string.h"
 #include <drizzled/util/test.h>
 
 using namespace drizzled;
