@@ -36,13 +36,11 @@ int mi_lock_database(MI_INFO *info, int lock_type)
   MYISAM_SHARE *share=info->s;
   uint32_t flag;
 
-  pthread_mutex_lock(&share->intern_lock);
   if (!info->s->in_use)
     info->s->in_use= new list<Session *>;
 
   if (lock_type == F_EXTRA_LCK)                 /* Used by TMP tables */
   {
-    pthread_mutex_unlock(&share->intern_lock);
     ++share->w_locks;
     ++share->tot_locks;
     info->lock_type= lock_type;
@@ -61,7 +59,7 @@ int mi_lock_database(MI_INFO *info, int lock_type)
 	count= --share->w_locks;
       --share->tot_locks;
       if (info->lock_type == F_WRLCK && !share->w_locks &&
-	  !share->delay_key_write && flush_key_blocks(share->key_cache,
+	  !share->delay_key_write && flush_key_blocks(share->getKeyCache(),
 						      share->kfile,FLUSH_KEEP))
       {
 	error=errno;
@@ -84,12 +82,8 @@ int mi_lock_database(MI_INFO *info, int lock_type)
     if ((info->s->mmaped_length != info->s->state.state.data_file_length) &&
         (info->s->nonmmaped_inserts > MAX_NONMAPPED_INSERTS))
     {
-      if (info->s->concurrent_insert)
-        pthread_rwlock_wrlock(&info->s->mmap_lock);
       mi_remap_file(info, info->s->state.state.data_file_length);
       info->s->nonmmaped_inserts= 0;
-      if (info->s->concurrent_insert)
-        pthread_rwlock_unlock(&info->s->mmap_lock);
     }
 	  share->state.process= share->last_process=share->this_process;
 	  share->state.unique=   info->last_unique=  info->this_unique;
@@ -213,7 +207,6 @@ int mi_lock_database(MI_INFO *info, int lock_type)
     }
   }
 #endif
-  pthread_mutex_unlock(&share->intern_lock);
 #if defined(FULL_LOG) || defined(_lint)
   lock_type|=(int) (flag << 8);		/* Set bit to set if real lock */
   myisam_log_command(MI_LOG_LOCK,info,(unsigned char*) &lock_type,sizeof(lock_type),
@@ -223,112 +216,6 @@ int mi_lock_database(MI_INFO *info, int lock_type)
 } /* mi_lock_database */
 
 
-/****************************************************************************
-  The following functions are called by thr_lock() in threaded applications
-****************************************************************************/
-
-/*
-  Create a copy of the current status for the table
-
-  SYNOPSIS
-    mi_get_status()
-    param		Pointer to Myisam handler
-    concurrent_insert	Set to 1 if we are going to do concurrent inserts
-			(THR_WRITE_CONCURRENT_INSERT was used)
-*/
-
-void mi_get_status(void* param, int concurrent_insert)
-{
-  MI_INFO *info=(MI_INFO*) param;
-
-  info->save_state=info->s->state.state;
-  info->state= &info->save_state;
-  info->append_insert_at_end= concurrent_insert;
-  return;
-}
-
-
-void mi_update_status(void* param)
-{
-  MI_INFO *info=(MI_INFO*) param;
-  /*
-    Because someone may have closed the table we point at, we only
-    update the state if its our own state.  This isn't a problem as
-    we are always pointing at our own lock or at a read lock.
-    (This is enforced by thr_multi_lock.c)
-  */
-  if (info->state == &info->save_state)
-  {
-    info->s->state.state= *info->state;
-    info->state= &info->s->state.state;
-  }
-  info->append_insert_at_end= 0;
-
-  /*
-    We have to flush the write cache here as other threads may start
-    reading the table before mi_lock_database() is called
-  */
-  if (info->opt_flag & WRITE_CACHE_USED)
-  {
-    if (end_io_cache(&info->rec_cache))
-    {
-      mi_print_error(info->s, HA_ERR_CRASHED);
-      mi_mark_crashed(info);
-    }
-    info->opt_flag&= ~WRITE_CACHE_USED;
-  }
-}
-
-
-void mi_restore_status(void *param)
-{
-  MI_INFO *info= (MI_INFO*) param;
-  info->state= &info->s->state.state;
-  info->append_insert_at_end= 0;
-}
-
-
-void mi_copy_status(void* to,void *from)
-{
-  ((MI_INFO*) to)->state= &((MI_INFO*) from)->save_state;
-}
-
-
-/*
-  Check if should allow concurrent inserts
-
-  IMPLEMENTATION
-    Allow concurrent inserts if we don't have a hole in the table or
-    if there is no active write lock and there is active read locks and
-    myisam_concurrent_insert == 2. In this last case the new
-    row('s) are inserted at end of file instead of filling up the hole.
-
-    The last case is to allow one to inserts into a heavily read-used table
-    even if there is holes.
-
-  NOTES
-    If there is a an rtree indexes in the table, concurrent inserts are
-    disabled in mi_open()
-
-  RETURN
-    0  ok to use concurrent inserts
-    1  not ok
-*/
-
-bool mi_check_status(void *param)
-{
-  MI_INFO *info=(MI_INFO*) param;
-  /*
-    The test for w_locks == 1 is here because this thread has already done an
-    external lock (in other words: w_locks == 1 means no other threads has
-    a write lock)
-  */
-  return (bool) !(info->s->state.dellink == HA_OFFSET_ERROR ||
-                     (myisam_concurrent_insert == 2 && info->s->r_locks &&
-                      info->s->w_locks == 1));
-}
-
-
 /****************************************************************************
  ** functions to read / write the state
 ****************************************************************************/
@@ -400,7 +287,7 @@ int _mi_test_if_changed(register MI_INFO *info)
       share->state.update_count != info->last_loop)
   {						/* Keyfile has changed */
     if (share->state.process != share->this_process)
-      flush_key_blocks(share->key_cache, share->kfile, FLUSH_RELEASE);
+      flush_key_blocks(share->getKeyCache(), share->kfile, FLUSH_RELEASE);
     share->last_process=share->state.process;
     info->last_unique=	share->state.unique;
     info->last_loop=	share->state.update_count;
