@@ -76,7 +76,7 @@ MI_INFO *test_if_reopen(char *filename)
   have an open count of 0.
 ******************************************************************************/
 
-MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
+MI_INFO *mi_open(const drizzled::TableIdentifier &identifier, int mode, uint32_t open_flags)
 {
   int lock_error,kfile,open_mode,save_errno,have_rtree=0;
   uint32_t i,j,len,errpos,head_length,base_pos,offset,info_length,keys,
@@ -97,12 +97,16 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
   head_length=sizeof(share_buff.state.header);
   memset(&info, 0, sizeof(info));
 
-  (void)internal::fn_format(org_name,name,"",MI_NAME_IEXT, MY_UNPACK_FILENAME);
+  (void)internal::fn_format(org_name,
+                            identifier.getPath().c_str(), 
+                            "",
+                            MI_NAME_IEXT,
+                            MY_UNPACK_FILENAME);
   if (!realpath(org_name,rp_buff))
     internal::my_load_path(rp_buff,org_name, NULL);
   rp_buff[FN_REFLEN-1]= '\0';
   strcpy(name_buff,rp_buff);
-  pthread_mutex_lock(&THR_LOCK_myisam);
+  THR_LOCK_myisam.lock();
   if (!(old_info=test_if_reopen(name_buff)))
   {
     share= &share_buff;
@@ -110,7 +114,7 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
     share_buff.state.rec_per_key_part=rec_per_key_part;
     share_buff.state.key_root=key_root;
     share_buff.state.key_del=key_del;
-    share_buff.key_cache= dflt_key_cache;
+    share_buff.setKeyCache();
 
     if ((kfile=internal::my_open(name_buff,(open_mode=O_RDWR),MYF(0))) < 0)
     {
@@ -247,8 +251,6 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
            &share->state.key_root,keys*sizeof(uint64_t),
            &share->state.key_del,
            (share->state.header.max_block_size_index*sizeof(uint64_t)),
-           &share->key_root_lock,sizeof(pthread_rwlock_t)*keys,
-           &share->mmap_lock,sizeof(pthread_rwlock_t),
            NULL))
       goto err;
     errpos=4;
@@ -389,17 +391,7 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
     disk_cache= NULL;
     mi_setup_functions(share);
     share->is_log_table= false;
-    thr_lock_init(&share->lock);
-    pthread_mutex_init(&share->intern_lock,MY_MUTEX_INIT_FAST);
-    for (i=0; i<keys; i++)
-      pthread_rwlock_init(&share->key_root_lock[i], NULL);
-    pthread_rwlock_init(&share->mmap_lock, NULL);
-    if (!thr_lock_inited)
-    {
-      /* Probably a single threaded program; Don't use concurrent inserts */
-      myisam_concurrent_insert=0;
-    }
-    else if (myisam_concurrent_insert)
+    if (myisam_concurrent_insert)
     {
       share->concurrent_insert=
 	((share->options & (HA_OPTION_READ_ONLY_DATA | HA_OPTION_TMP_TABLE |
@@ -408,11 +400,7 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
 	 (open_flags & HA_OPEN_TMP_TABLE) || have_rtree) ? 0 : 1;
       if (share->concurrent_insert)
       {
-	share->lock.get_status= mi_get_status;
-	share->lock.copy_status= mi_copy_status;
-	share->lock.update_status= mi_update_status;
-        share->lock.restore_status= mi_restore_status;
-	share->lock.check_status= mi_check_status;
+        assert(0);
       }
     }
   }
@@ -438,7 +426,7 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
                      share->base.max_key_length),
          &info.lastkey,share->base.max_key_length*3+1,
          &info.first_mbr_key, share->base.max_key_length,
-         &info.filename,strlen(name)+1,
+         &info.filename, identifier.getPath().length()+1,
          &info.rtree_recursion_state,have_rtree ? 1024 : 0,
          NULL))
     goto err;
@@ -447,7 +435,7 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
   if (!have_rtree)
     info.rtree_recursion_state= NULL;
 
-  strcpy(info.filename,name);
+  strcpy(info.filename, identifier.getPath().c_str());
   memcpy(info.blobs,share->blobs,sizeof(MI_BLOB)*share->base.blobs);
   info.lastkey2=info.lastkey+share->base.max_key_length;
 
@@ -468,7 +456,6 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
   info.bulk_insert=0;
   info.errkey= -1;
   info.page_changed=1;
-  pthread_mutex_lock(&share->intern_lock);
   info.read_record=share->read_record;
   share->reopen++;
   share->write_flag=MYF(MY_NABP | MY_WAIT_IF_FULL);
@@ -498,7 +485,6 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
 
   share->delay_key_write= 1;
   info.state= &share->state.state;	/* Change global values by default */
-  pthread_mutex_unlock(&share->intern_lock);
 
   /* Allocate buffer for one record */
 
@@ -508,10 +494,9 @@ MI_INFO *mi_open(const char *name, int mode, uint32_t open_flags)
   memset(info.rec_buff, 0, mi_get_rec_buff_len(&info, info.rec_buff));
 
   *m_info=info;
-  thr_lock_data_init(&share->lock,&m_info->lock,(void*) m_info);
   myisam_open_list.push_front(m_info);
 
-  pthread_mutex_unlock(&THR_LOCK_myisam);
+  THR_LOCK_myisam.unlock();
   return(m_info);
 
 err:
@@ -521,7 +506,7 @@ err:
   if ((save_errno == HA_ERR_CRASHED) ||
       (save_errno == HA_ERR_CRASHED_ON_USAGE) ||
       (save_errno == HA_ERR_CRASHED_ON_REPAIR))
-    mi_report_error(save_errno, name);
+    mi_report_error(save_errno, identifier.getPath().c_str());
   switch (errpos) {
   case 6:
     free((unsigned char*) m_info);
@@ -543,7 +528,7 @@ err:
   default:
     break;
   }
-  pthread_mutex_unlock(&THR_LOCK_myisam);
+  THR_LOCK_myisam.unlock();
   errno=save_errno;
   return (NULL);
 } /* mi_open */
