@@ -68,9 +68,6 @@ int mi_rkey(MI_INFO *info, unsigned char *buf, int inx, const unsigned char *key
   if (fast_mi_readinfo(info))
     goto err;
 
-  if (share->concurrent_insert)
-    pthread_rwlock_rdlock(&share->key_root_lock[inx]);
-
   nextflag=myisam_read_vec[search_flag];
   use_key_length=pack_key_length;
   if (!(nextflag & (SEARCH_FIND | SEARCH_NO_FIND | SEARCH_LAST)))
@@ -135,8 +132,6 @@ int mi_rkey(MI_INFO *info, unsigned char *buf, int inx, const unsigned char *key
       if (res == 2)
       {
         info->lastpos= HA_OFFSET_ERROR;
-        if (share->concurrent_insert)
-          pthread_rwlock_unlock(&share->key_root_lock[inx]);
         return((errno= HA_ERR_KEY_NOT_FOUND));
       }
       /*
@@ -151,8 +146,6 @@ int mi_rkey(MI_INFO *info, unsigned char *buf, int inx, const unsigned char *key
       }
     }
   }
-  if (share->concurrent_insert)
-    pthread_rwlock_unlock(&share->key_root_lock[inx]);
 
   /* Calculate length of the found key;  Used by mi_rnext_same */
   if ((keyinfo->flag & HA_VAR_LENGTH_KEY) && last_used_keyseg &&
