@@ -37,9 +37,6 @@ int mi_rnext_same(MI_INFO *info, unsigned char *buf)
   if (fast_mi_readinfo(info))
     return(errno);
 
-  if (info->s->concurrent_insert)
-    pthread_rwlock_rdlock(&info->s->key_root_lock[inx]);
-
   switch (keyinfo->key_alg)
   {
     case HA_KEY_ALG_BTREE:
@@ -69,9 +66,7 @@ int mi_rnext_same(MI_INFO *info, unsigned char *buf)
           break;
       }
   }
-  if (info->s->concurrent_insert)
-    pthread_rwlock_unlock(&info->s->key_root_lock[inx]);
-	/* Don't clear if database-changed */
+  /* Don't clear if database-changed */
   info->update&= (HA_STATE_CHANGED | HA_STATE_ROW_CHANGED);
   info->update|= HA_STATE_NEXT_FOUND | HA_STATE_RNEXT_SAME;
 
