@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* Read through all rows sequntially */
 
@@ -21,7 +21,7 @@ int mi_scan_init(register MI_INFO *info)
 {
   info->nextpos=info->s->pack.header_length;	/* Read first record */
   info->lastinx= -1;				/* Can't forward or backward */
-  if (info->opt_flag & WRITE_CACHE_USED && info->rec_cache.flush())
+  if (info->opt_flag & WRITE_CACHE_USED && flush_io_cache(&info->rec_cache))
     return(errno);
   return(0);
 }
