@@ -100,24 +100,18 @@ int mi_write(MI_INFO *info, unsigned char *record)
                                   is_tree_inited(&info->bulk_insert[i])));
       if (local_lock_tree)
       {
-	pthread_rwlock_wrlock(&share->key_root_lock[i]);
 	share->keyinfo[i].version++;
       }
       {
         if (share->keyinfo[i].ck_insert(info,i,buff,
 			_mi_make_key(info,i,buff,record,filepos)))
         {
-          if (local_lock_tree)
-            pthread_rwlock_unlock(&share->key_root_lock[i]);
           goto err;
         }
       }
 
       /* The above changed info->lastkey2. Inform mi_rnext_same(). */
       info->update&= ~HA_STATE_RNEXT_SAME;
-
-      if (local_lock_tree)
-        pthread_rwlock_unlock(&share->key_root_lock[i]);
     }
   }
   if (share->calc_checksum)
@@ -145,8 +139,8 @@ int mi_write(MI_INFO *info, unsigned char *record)
     not critical to use outdated share->is_log_table value (2) locking
     mutex here for every write is too expensive.
   */
-  if (share->is_log_table)
-    mi_update_status((void*) info);
+  if (share->is_log_table) // Log table do not exist in Drizzle
+    assert(0);
 
   return(0);
 
@@ -166,22 +160,13 @@ err:
     {
       if (mi_is_key_active(share->state.key_map, i))
       {
-	bool local_lock_tree= (lock_tree &&
-                                  !(info->bulk_insert &&
-                                    is_tree_inited(&info->bulk_insert[i])));
-	if (local_lock_tree)
-	  pthread_rwlock_wrlock(&share->key_root_lock[i]);
 	{
 	  uint32_t key_length=_mi_make_key(info,i,buff,record,filepos);
 	  if (_mi_ck_delete(info,i,buff,key_length))
 	  {
-	    if (local_lock_tree)
-	      pthread_rwlock_unlock(&share->key_root_lock[i]);
 	    break;
 	  }
 	}
-	if (local_lock_tree)
-	  pthread_rwlock_unlock(&share->key_root_lock[i]);
       }
     }
   }
@@ -764,7 +749,6 @@ static int keys_free(unsigned char *key, TREE_FREE mode, bulk_insert_param *para
   case free_init:
     if (param->info->s->concurrent_insert)
     {
-      pthread_rwlock_wrlock(&param->info->s->key_root_lock[param->keynr]);
       param->info->s->keyinfo[param->keynr].version++;
     }
     return 0;
@@ -775,8 +759,6 @@ static int keys_free(unsigned char *key, TREE_FREE mode, bulk_insert_param *para
     return _mi_ck_write_btree(param->info,param->keynr,lastkey,
 			      keylen - param->info->s->rec_reflength);
   case free_end:
-    if (param->info->s->concurrent_insert)
-      pthread_rwlock_unlock(&param->info->s->key_root_lock[param->keynr]);
     return 0;
   }
   return -1;
