@@ -18,6 +18,8 @@
 #ifndef PLUGIN_MYISAM_MYISAM_H
 #define PLUGIN_MYISAM_MYISAM_H
 
+#include "drizzled/identifier/table.h"
+
 #include <drizzled/key_map.h>
 
 #include <drizzled/base.h>
@@ -162,6 +164,20 @@ typedef struct st_mi_create_info
   uint32_t old_options;
   uint8_t language;
   bool with_auto_increment;
+
+  st_mi_create_info():
+    index_file_name(0),
+    data_file_name(0),
+    max_rows(0),
+    reloc_rows(0),
+    auto_increment(0),
+    data_file_length(0),
+    key_file_length(0),
+    old_options(0),
+    language(0),
+    with_auto_increment(0)
+  { }
+
 } MI_CREATE_INFO;
 
 struct st_myisam_info;			/* For referense */
@@ -261,7 +277,8 @@ extern uint32_t data_pointer_size;
 
 extern int mi_close(struct st_myisam_info *file);
 extern int mi_delete(struct st_myisam_info *file,const unsigned char *buff);
-extern struct st_myisam_info *mi_open(const char *name,int mode,
+extern struct st_myisam_info *mi_open(const drizzled::TableIdentifier &identifier,
+                                      int mode,
 				      uint32_t wait_if_locked);
 extern int mi_panic(enum drizzled::ha_panic_function function);
 extern int mi_rfirst(struct st_myisam_info *file,unsigned char *buf,int inx);
@@ -448,8 +465,6 @@ int mi_repair(MI_CHECK *param, register MI_INFO *info,
 int mi_sort_index(MI_CHECK *param, register MI_INFO *info, char * name);
 int mi_repair_by_sort(MI_CHECK *param, register MI_INFO *info,
 		      const char * name, int rep_quick);
-int mi_repair_parallel(MI_CHECK *param, register MI_INFO *info,
-		      const char * name, int rep_quick);
 int change_to_newfile(const char * filename, const char * old_ext,
 		      const char * new_ext, uint32_t raid_chunks,
 		      drizzled::myf myflags);
@@ -466,7 +481,6 @@ int movepoint(MI_INFO *info,unsigned char *record,drizzled::internal::my_off_t o
 	      drizzled::internal::my_off_t newpos, uint32_t prot_key);
 int write_data_suffix(SORT_INFO *sort_info, bool fix_datafile);
 int test_if_almost_full(MI_INFO *info);
-int recreate_table(MI_CHECK *param, MI_INFO **org_info, char *filename);
 bool mi_test_if_sort_rep(MI_INFO *info, drizzled::ha_rows rows, uint64_t key_map,
 			    bool force);
 
