@@ -21,7 +21,7 @@
 #include "config.h"
 #include "myisam.h"			/* Structs & some defines */
 #include "myisampack.h"			/* packing of keys */
-#include "drizzled/my_tree.h"
+#include "drizzled/tree.h"
 #include "drizzled/internal/my_pthread.h"
 #include <drizzled/thr_lock.h>
 #include <drizzled/common.h>
@@ -31,10 +31,18 @@
 #include <string.h>
 #include <list>
 
+#include <boost/thread/mutex.hpp>
+
 #if defined(my_write)
 #undef my_write				/* undef map from my_nosys; We need test-if-disk full */
 #endif
 
+/* Typical key cash */
+static const uint32_t KEY_CACHE_SIZE= 8*1024*1024;
+
+/* Default size of a key cache block  */
+static const uint32_t KEY_CACHE_BLOCK_SIZE= 1024;
+
 typedef struct st_mi_status_info
 {
   drizzled::ha_rows records;			/* Rows in table */
@@ -177,7 +185,16 @@ typedef struct st_mi_isam_share {	/* Shared between opens */
   char  *data_file_name,		/* Resolved path names from symlinks */
         *index_file_name;
   unsigned char *file_map;			/* mem-map of file if possible */
-  drizzled::KEY_CACHE *key_cache;			/* ref to the current key cache */
+private:
+  drizzled::KEY_CACHE key_cache;			/* ref to the current key cache */
+public:
+  drizzled::KEY_CACHE *getKeyCache()
+  {
+    return &key_cache;
+  }
+
+  void setKeyCache();
+
   MI_DECODE_TREE *decode_trees;
   uint16_t *decode_tables;
   int (*read_record)(struct st_myisam_info*, drizzled::internal::my_off_t, unsigned char*);
@@ -217,13 +234,9 @@ typedef struct st_mi_isam_share {	/* Shared between opens */
     not_flushed,
     temporary,delay_key_write,
     concurrent_insert;
-  drizzled::THR_LOCK lock;
-  pthread_mutex_t intern_lock;		/* Locking for use with _locking */
-  pthread_rwlock_t *key_root_lock;
   drizzled::internal::my_off_t mmaped_length;
   uint32_t     nonmmaped_inserts;           /* counter of writing in non-mmaped
                                            area */
-  pthread_rwlock_t mmap_lock;
 } MYISAM_SHARE;
 
 
@@ -459,7 +472,7 @@ typedef struct st_mi_sort_param
 #define MI_UNIQUE_HASH_TYPE	HA_KEYTYPE_ULONG_INT
 #define mi_unique_store(A,B)    mi_int4store((A),(B))
 
-extern pthread_mutex_t THR_LOCK_myisam;
+extern boost::mutex THR_LOCK_myisam;
 
 	/* Some extern variables */
 
@@ -704,11 +717,6 @@ int _mi_cmp_dynamic_unique(MI_INFO *info, MI_UNIQUEDEF *def,
 			   const unsigned char *record, drizzled::internal::my_off_t pos);
 int mi_unique_comp(MI_UNIQUEDEF *def, const unsigned char *a, const unsigned char *b,
 		   bool null_are_equal);
-void mi_get_status(void* param, int concurrent_insert);
-void mi_update_status(void* param);
-void mi_restore_status(void* param);
-void mi_copy_status(void* to,void *from);
-bool mi_check_status(void* param);
 
 extern MI_INFO *test_if_reopen(char *filename);
 bool check_table_is_closed(const char *name, const char *where);
@@ -727,8 +735,6 @@ void mi_check_print_warning(MI_CHECK *param, const char *fmt,...);
 void mi_check_print_info(MI_CHECK *param, const char *fmt,...);
 int flush_pending_blocks(MI_SORT_PARAM *param);
 int thr_write_keys(MI_SORT_PARAM *sort_param);
-/* needs extern "C" because we pass it to pthread_create */
-extern "C" pthread_handler_t thr_find_all_keys(void *arg);
 int flush_blocks(MI_CHECK *param, drizzled::KEY_CACHE *key_cache, int file);
 
 int sort_write_record(MI_SORT_PARAM *sort_param);
