@@ -302,177 +302,6 @@ static ha_rows  find_all_keys(MI_SORT_PARAM *info, uint32_t keys,
 } /* find_all_keys */
 
 
-/* Search after all keys and place them in a temp. file */
-
-pthread_handler_t thr_find_all_keys(void *arg)
-{
-  MI_SORT_PARAM *sort_param= (MI_SORT_PARAM*) arg;
-  int error;
-  uint32_t memavl,old_memavl,keys,sort_length;
-  uint32_t idx;
-  size_t maxbuffer;
-  unsigned char **sort_keys=0;
-
-  error=1;
-
-  if (internal::my_thread_init())
-    goto err;
-
-  {
-    if (sort_param->sort_info->got_error)
-      goto err;
-
-    if (sort_param->keyinfo->flag & HA_VAR_LENGTH_KEY)
-    {
-      sort_param->write_keys=     write_keys_varlen;
-      sort_param->read_to_buffer= read_to_buffer_varlen;
-      sort_param->write_key=      write_merge_key_varlen;
-    }
-    else
-    {
-      sort_param->write_keys=     write_keys;
-      sort_param->read_to_buffer= read_to_buffer;
-      sort_param->write_key=      write_merge_key;
-    }
-
-    my_b_clear(&sort_param->tempfile);
-    my_b_clear(&sort_param->tempfile_for_exceptions);
-    memset(&sort_param->buffpek, 0, sizeof(sort_param->buffpek));
-    memset(&sort_param->unique, 0,  sizeof(sort_param->unique));
-    sort_keys= (unsigned char **) NULL;
-
-    memavl=       max(sort_param->sortbuff_size, (uint32_t)MIN_SORT_MEMORY);
-    idx=          (uint)sort_param->sort_info->max_records;
-    sort_length=  sort_param->key_length;
-    maxbuffer=    1;
-
-    while (memavl >= MIN_SORT_MEMORY)
-    {
-      if ((internal::my_off_t) (idx+1)*(sort_length+sizeof(char*)) <=
-          (internal::my_off_t) memavl)
-        keys= idx+1;
-      else
-      {
-        uint32_t skr;
-        do
-        {
-          skr= maxbuffer;
-          if (memavl < sizeof(BUFFPEK)*maxbuffer ||
-              (keys=(memavl-sizeof(BUFFPEK)*maxbuffer)/
-               (sort_length+sizeof(char*))) <= 1 ||
-              keys < maxbuffer)
-          {
-            mi_check_print_error(sort_param->sort_info->param,
-                                 "myisam_sort_buffer_size is too small");
-            goto err;
-          }
-        }
-        while ((maxbuffer= (idx/(keys-1)+1)) != skr);
-      }
-      if ((sort_keys= (unsigned char**)
-           malloc(keys*(sort_length+sizeof(char*)))))
-      {
-        if (my_init_dynamic_array(&sort_param->buffpek, sizeof(BUFFPEK),
-                                  maxbuffer, maxbuffer/2))
-        {
-          free((unsigned char*) sort_keys);
-          sort_keys= (unsigned char **) NULL; /* for err: label */
-        }
-        else
-          break;
-      }
-      old_memavl= memavl;
-      if ((memavl= memavl/4*3) < MIN_SORT_MEMORY &&
-          old_memavl > MIN_SORT_MEMORY)
-        memavl= MIN_SORT_MEMORY;
-    }
-    if (memavl < MIN_SORT_MEMORY)
-    {
-      mi_check_print_error(sort_param->sort_info->param,
-                           "MyISAM sort buffer too small");
-      goto err;
-    }
-
-    if (sort_param->sort_info->param->testflag & T_VERBOSE)
-      printf("Key %d - Allocating buffer for %d keys\n",
-             sort_param->key + 1, keys);
-    sort_param->sort_keys= sort_keys;
-
-    idx= error= 0;
-    sort_keys[0]= (unsigned char*) (sort_keys+keys);
-
-    while (!(error= sort_param->sort_info->got_error) &&
-           !(error= (*sort_param->key_read)(sort_param, sort_keys[idx])))
-    {
-      if (sort_param->real_key_length > sort_param->key_length)
-      {
-        if (write_key(sort_param, sort_keys[idx],
-                      &sort_param->tempfile_for_exceptions))
-          goto err;
-        continue;
-      }
-
-      if (++idx == keys)
-      {
-        if (sort_param->write_keys(sort_param, sort_keys, idx - 1,
-                                   (BUFFPEK*) alloc_dynamic(&sort_param->buffpek),
-                                   &sort_param->tempfile))
-          goto err;
-        sort_keys[0]= (unsigned char*) (sort_keys+keys);
-        memcpy(sort_keys[0], sort_keys[idx - 1], (size_t) sort_param->key_length);
-        idx= 1;
-      }
-      sort_keys[idx]= sort_keys[idx - 1] + sort_param->key_length;
-    }
-    if (error > 0)
-      goto err;
-    if (sort_param->buffpek.elements)
-    {
-      if (sort_param->write_keys(sort_param, sort_keys, idx,
-                                 (BUFFPEK*) alloc_dynamic(&sort_param->buffpek),
-                                 &sort_param->tempfile))
-        goto err;
-      sort_param->keys= (sort_param->buffpek.elements - 1) * (keys - 1) + idx;
-    }
-    else
-      sort_param->keys= idx;
-
-    sort_param->sort_keys_length= keys;
-    goto ok;
-
-err:
-    sort_param->sort_info->got_error= 1; /* no need to protect with a mutex */
-    if (sort_keys)
-      free((unsigned char*) sort_keys);
-    sort_param->sort_keys= 0;
-    delete_dynamic(& sort_param->buffpek);
-    close_cached_file(&sort_param->tempfile);
-    close_cached_file(&sort_param->tempfile_for_exceptions);
-
-ok:
-    free_root(&sort_param->wordroot, MYF(0));
-    /*
-      Detach from the share if the writer is involved. Avoid others to
-      be blocked. This includes a flush of the write buffer. This will
-      also indicate EOF to the readers.
-    */
-    if (sort_param->sort_info->info->rec_cache.share)
-      remove_io_thread(&sort_param->sort_info->info->rec_cache);
-
-    /* Readers detach from the share if any. Avoid others to be blocked. */
-    if (sort_param->read_cache.share)
-      remove_io_thread(&sort_param->read_cache);
-
-    pthread_mutex_lock(&sort_param->sort_info->mutex);
-    if (!--sort_param->sort_info->threads_running)
-      pthread_cond_signal(&sort_param->sort_info->cond);
-    pthread_mutex_unlock(&sort_param->sort_info->mutex);
-  }
-  internal::my_thread_end();
-  return NULL;
-}
-
-
 int thr_write_keys(MI_SORT_PARAM *sort_param)
 {
   SORT_INFO *sort_info= sort_param->sort_info;
