@@ -13,23 +13,27 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
-#pragma once
+#ifndef PLUGIN_MYSQL_PROTOCOL_MYSQL_PASSWORD_H
+#define PLUGIN_MYSQL_PROTOCOL_MYSQL_PASSWORD_H
 
 #include <drizzled/item/func.h>
 #include <drizzled/function/str/strfunc.h>
 
-namespace drizzle_plugin {
+namespace drizzled
+{
 
-extern const char* MySQLPasswordName;
+extern "C" const char* MySQLPasswordName;
 
 class MySQLPassword: public drizzled::Item_str_func
 {
 public:
-  const char *func_name() const;
-  void fix_length_and_dec();
+  MySQLPassword(void);
+  const char *func_name(void) const;
+  void fix_length_and_dec(void);
   bool check_argument_count(int n);
   drizzled::String *val_str(drizzled::String *);
 };
 
-} /* namespace drizzle_plugin */
+} /* namespace drizzled */
 
+#endif /* PLUGIN_MYSQL_PROTOCOL_MYSQL_PASSWORD_H */
