@@ -18,8 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <plugin/performance_dictionary/dictionary.h>
+#include "config.h"
+#include "plugin/performance_dictionary/dictionary.h"
 
 using namespace drizzled;
 
@@ -37,7 +37,7 @@ DRIZZLE_DECLARE_PLUGIN
   "Performance Dictionary",
   "1.0",
   "Brian Aker",
-  N_("Dictionary of session usage and performance"),
+  "Data Dictionary for performance related table functions.",
   PLUGIN_LICENSE_GPL,
   init,
   NULL,
