@@ -18,13 +18,15 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_PERFORMANCE_DICTIONARY_DICTIONARY_H
+#define PLUGIN_PERFORMANCE_DICTIONARY_DICTIONARY_H
 
 #include <drizzled/plugin/table_function.h>
 #include <drizzled/plugin/logging.h>
 
 // Prototypes
-#include <plugin/performance_dictionary/query_usage.h>
-#include <plugin/performance_dictionary/session_usage_logger.h>
-#include <plugin/performance_dictionary/session_usage.h>
+#include "plugin/performance_dictionary/query_usage.h"
+#include "plugin/performance_dictionary/session_usage_logger.h"
+#include "plugin/performance_dictionary/session_usage.h"
 
+#endif /* PLUGIN_PERFORMANCE_DICTIONARY_DICTIONARY_H */
