@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_H
+#define PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_H
 
 namespace performance_dictionary {
 
@@ -49,3 +50,4 @@ public:
 
 } /* namespace performance_dictionary */
 
+#endif /* PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_H */
