@@ -23,12 +23,17 @@
 #include <plugin/pool_of_threads/pool_of_threads.h>
 #include "drizzled/pthread_globals.h"
 #include "drizzled/internal/my_pthread.h"
+#include <boost/program_options.hpp>
+#include <drizzled/module/option_map.h>
 
+#include <boost/thread/thread.hpp>
+#include <boost/bind.hpp>
+
+namespace po= boost::program_options;
 using namespace std;
 using namespace drizzled;
 
 /* Global's (TBR) */
-static PoolOfThreadsScheduler *scheduler= NULL;
 
 /**
  * Set this to true to trigger killing of all threads in the pool
@@ -36,7 +41,6 @@ static PoolOfThreadsScheduler *scheduler= NULL;
 static volatile bool kill_pool_threads= false;
 
 static volatile uint32_t created_threads= 0;
-static int deinit(drizzled::plugin::Registry &registry);
 
 static struct event session_add_event;
 static struct event session_kill_event;
@@ -50,14 +54,17 @@ static bool libevent_needs_immediate_processing(Session *session);
 static void libevent_connection_close(Session *session);
 void libevent_session_add(Session* session);
 bool libevent_should_close_connection(Session* session);
+void libevent_thread_proc(PoolOfThreadsScheduler *pot_scheduler);
 extern "C" {
+#if 0
   void *libevent_thread_proc(void *arg);
+#endif
   void libevent_io_callback(int Fd, short Operation, void *ctx);
   void libevent_add_session_callback(int Fd, short Operation, void *ctx);
   void libevent_kill_session_callback(int Fd, short Operation, void *ctx);
 }
 
-static uint32_t size= 0;
+static uint32_t pool_size;
 
 /**
  * @brief 
@@ -130,7 +137,7 @@ void PoolOfThreadsScheduler::killSession(int Fd)
   char c;
   int count= 0;
 
-  pthread_mutex_lock(&LOCK_session_kill);
+  LOCK_session_kill.lock();
   while (! sessions_to_be_killed.empty())
   {
 
@@ -138,7 +145,7 @@ void PoolOfThreadsScheduler::killSession(int Fd)
      Fetch a session from the queue
     */
     Session* session= sessions_to_be_killed.front();
-    pthread_mutex_unlock(&LOCK_session_kill);
+    LOCK_session_kill.unlock();
 
     session_scheduler *sched= static_cast<session_scheduler *>(session->scheduler_arg);
     assert(sched);
@@ -157,7 +164,7 @@ void PoolOfThreadsScheduler::killSession(int Fd)
     */
     sessions_need_processing.push(sched->session);
 
-    pthread_mutex_lock(&LOCK_session_kill);
+    LOCK_session_kill.lock();
     /*
      Pop until this session is already processed
     */
@@ -173,7 +180,7 @@ void PoolOfThreadsScheduler::killSession(int Fd)
     count++;
   }
   assert(count == 1);
-  pthread_mutex_unlock(&LOCK_session_kill);
+  LOCK_session_kill.unlock();
 }
 
 
@@ -202,14 +209,14 @@ void PoolOfThreadsScheduler::addSession(int Fd)
   char c;
   int count= 0;
 
-  pthread_mutex_lock(&LOCK_session_add);
+  LOCK_session_add.lock();
   while (! sessions_need_adding.empty())
   {
     /*
      Pop the first session off the queue 
     */
     Session* session= sessions_need_adding.front();
-    pthread_mutex_unlock(&LOCK_session_add);
+    LOCK_session_add.unlock();
 
     session_scheduler *sched= static_cast<session_scheduler *>(session->scheduler_arg);
     assert(sched);
@@ -237,7 +244,7 @@ void PoolOfThreadsScheduler::addSession(int Fd)
       }
     }
 
-    pthread_mutex_lock(&LOCK_session_add);
+    LOCK_session_add.lock();
     /*
      Pop until this session is already processed
     */
@@ -253,7 +260,7 @@ void PoolOfThreadsScheduler::addSession(int Fd)
     count++;
   }
   assert(count == 1);
-  pthread_mutex_unlock(&LOCK_session_add);
+  LOCK_session_add.unlock();
 }
 
 /**
@@ -301,7 +308,7 @@ bool libevent_should_close_connection(Session* session)
  *  These procs only return/terminate on shutdown (kill_pool_threads ==
  *  true).
  */
-void *libevent_thread_proc(void *ctx)
+void libevent_thread_proc(PoolOfThreadsScheduler *pot_scheduler)
 {
   if (internal::my_thread_init())
   {
@@ -310,9 +317,7 @@ void *libevent_thread_proc(void *ctx)
     exit(1);
   }
 
-  PoolOfThreadsScheduler *pot_scheduler=
-    reinterpret_cast<PoolOfThreadsScheduler *>(ctx);
-  return pot_scheduler->mainLoop();
+  (void)pot_scheduler->mainLoop();
 }
 
 void *PoolOfThreadsScheduler::mainLoop()
@@ -321,16 +326,17 @@ void *PoolOfThreadsScheduler::mainLoop()
    Signal libevent_init() when all threads has been created and are ready
    to receive events.
   */
-  (void) pthread_mutex_lock(&LOCK_thread_count);
+  (void) LOCK_thread_count.lock();
   created_threads++;
-  if (created_threads == size)
-    (void) pthread_cond_signal(&COND_thread_count);
-  (void) pthread_mutex_unlock(&LOCK_thread_count);
+  if (created_threads == pool_size)
+    COND_thread_count.notify_one();
+
+  (void) LOCK_thread_count.unlock();
 
   for (;;)
   {
     Session *session= NULL;
-    (void) pthread_mutex_lock(&LOCK_event_loop);
+    LOCK_event_loop.lock();
 
     /* get session(s) to process */
     while (sessions_need_processing.empty())
@@ -338,7 +344,7 @@ void *PoolOfThreadsScheduler::mainLoop()
       if (kill_pool_threads)
       {
         /* the flag that we should die has been set */
-        (void) pthread_mutex_unlock(&LOCK_event_loop);
+        LOCK_event_loop.unlock();
         goto thread_exit;
       }
       event_loop(EVLOOP_ONCE);
@@ -349,7 +355,7 @@ void *PoolOfThreadsScheduler::mainLoop()
     sessions_need_processing.pop();
     session_scheduler *sched= (session_scheduler *)session->scheduler_arg;
 
-    (void) pthread_mutex_unlock(&LOCK_event_loop);
+    LOCK_event_loop.lock();
 
     /* now we process the connection (session) */
 
@@ -396,10 +402,10 @@ void *PoolOfThreadsScheduler::mainLoop()
   }
 
 thread_exit:
-  (void) pthread_mutex_lock(&LOCK_thread_count);
+  (void) LOCK_thread_count.lock();
   created_threads--;
-  pthread_cond_broadcast(&COND_thread_count);
-  (void) pthread_mutex_unlock(&LOCK_thread_count);
+  COND_thread_count.notify_all();
+  (void) LOCK_thread_count.unlock();
   internal::my_thread_end();
   pthread_exit(0);
 
@@ -465,7 +471,7 @@ void libevent_session_add(Session* session)
 void PoolOfThreadsScheduler::sessionAddToQueue(session_scheduler *sched)
 {
   char c= 0;
-  pthread_mutex_lock(&LOCK_session_add);
+  boost::mutex::scoped_lock scopedLock(LOCK_session_add);
   if (sessions_need_adding.empty())
   {
     /* notify libevent */
@@ -474,7 +480,6 @@ void PoolOfThreadsScheduler::sessionAddToQueue(session_scheduler *sched)
   }
   /* queue for libevent */
   sessions_need_adding.push(sched->session);
-  pthread_mutex_unlock(&LOCK_session_add);
 }
 
 
@@ -482,27 +487,16 @@ PoolOfThreadsScheduler::PoolOfThreadsScheduler(const char *name_arg)
   : Scheduler(name_arg), sessions_need_adding(), sessions_to_be_killed(),
     sessions_need_processing(), sessions_waiting_for_io()
 {
-  struct sched_param tmp_sched_param;
-
-  memset(&tmp_sched_param, 0, sizeof(struct sched_param));
   /* Setup attribute parameter for session threads. */
   (void) pthread_attr_init(&attr);
   (void) pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
   pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
-
-  tmp_sched_param.sched_priority= WAIT_PRIOR;
-  (void) pthread_attr_setschedparam(&attr, &tmp_sched_param);
-
-  pthread_mutex_init(&LOCK_session_add, NULL);
-  pthread_mutex_init(&LOCK_session_kill, NULL);
-  pthread_mutex_init(&LOCK_event_loop, NULL);
-
 }
 
 
 PoolOfThreadsScheduler::~PoolOfThreadsScheduler()
 {
-  (void) pthread_mutex_lock(&LOCK_thread_count);
+  (void) LOCK_thread_count.lock();
 
   kill_pool_threads= true;
   while (created_threads)
@@ -514,9 +508,9 @@ PoolOfThreadsScheduler::~PoolOfThreadsScheduler()
     size_t written= write(session_add_pipe[1], &c, sizeof(c));
     assert(written == sizeof(c));
 
-    pthread_cond_wait(&COND_thread_count, &LOCK_thread_count);
+    pthread_cond_wait(COND_thread_count.native_handle(), LOCK_thread_count.native_handle());
   }
-  (void) pthread_mutex_unlock(&LOCK_thread_count);
+  (void) LOCK_thread_count.unlock();
 
   event_del(&session_add_event);
   close(session_add_pipe[0]);
@@ -525,9 +519,6 @@ PoolOfThreadsScheduler::~PoolOfThreadsScheduler()
   close(session_kill_pipe[0]);
   close(session_kill_pipe[1]);
 
-  (void) pthread_mutex_destroy(&LOCK_event_loop);
-  (void) pthread_mutex_destroy(&LOCK_session_add);
-  (void) pthread_mutex_destroy(&LOCK_session_kill);
   (void) pthread_attr_destroy(&attr);
 }
 
@@ -552,7 +543,7 @@ void PoolOfThreadsScheduler::killSession(Session *session)
 {
   char c= 0;
 
-  pthread_mutex_lock(&LOCK_session_kill);
+  boost::mutex::scoped_lock scopedLock(LOCK_session_kill);
 
   if (sessions_to_be_killed.empty())
   {
@@ -568,14 +559,11 @@ void PoolOfThreadsScheduler::killSession(Session *session)
     Push into the sessions_to_be_killed queue
   */
   sessions_to_be_killed.push(session);
-  pthread_mutex_unlock(&LOCK_session_kill);
 }
 
 
 bool PoolOfThreadsScheduler::libevent_init(void)
 {
-  uint32_t x;
-
   event_init();
 
 
@@ -607,25 +595,22 @@ bool PoolOfThreadsScheduler::libevent_init(void)
 
   }
   /* Set up the thread pool */
-  pthread_mutex_lock(&LOCK_thread_count);
+  boost::mutex::scoped_lock scopedLock(LOCK_thread_count);
 
-  for (x= 0; x < size; x++)
+  for (uint32_t x= 0; x < pool_size; x++)
   {
-    pthread_t thread;
-    int error;
-    if ((error= pthread_create(&thread, &attr, libevent_thread_proc, this)))
+    if (not new boost::thread(boost::bind(libevent_thread_proc, this)))
     {
-      errmsg_printf(ERRMSG_LVL_ERROR, _("Can't create completion port thread (error %d)"),
-                    error);
-      pthread_mutex_unlock(&LOCK_thread_count);
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Can't create completion port thread (error %d)"), 1);
       return true;
     }
   }
 
   /* Wait until all threads are created */
-  while (created_threads != size)
-    pthread_cond_wait(&COND_thread_count,&LOCK_thread_count);
-  pthread_mutex_unlock(&LOCK_thread_count);
+  while (created_threads != pool_size)
+  {
+    COND_thread_count.wait(scopedLock);
+  }
 
   return false;
 }
@@ -637,24 +622,22 @@ bool PoolOfThreadsScheduler::libevent_init(void)
  * 
  * @param[in] registry holding the record of the plugins
  */
-static int init(drizzled::plugin::Registry &registry)
+static int init(drizzled::module::Context &context)
 {
-  assert(size != 0);
+  const module::option_map &vm= context.getOptions();
+ 
+  if (vm.count("size"))
+  {
+    if (pool_size > 1024 || pool_size < 1)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR, _("Invalid value for size\n"));
+      exit(-1);
+    }
+  }
 
-  scheduler= new PoolOfThreadsScheduler("pool_of_threads");
-  registry.add(scheduler);
+  assert(pool_size != 0);
 
-  return 0;
-}
-
-/**
- * @brief
- *  Waits until all pool threads have been deleted for clean shutdown
- */
-static int deinit(drizzled::plugin::Registry &registry)
-{
-  registry.remove(scheduler);
-  delete scheduler;
+  context.add(new PoolOfThreadsScheduler("pool_of_threads"));
 
   return 0;
 }
@@ -663,11 +646,18 @@ static int deinit(drizzled::plugin::Registry &registry)
  The defaults here were picked based on what I see (aka Brian). They should
  be vetted across a larger audience.
 */
-static DRIZZLE_SYSVAR_UINT(size, size,
+static DRIZZLE_SYSVAR_UINT(size, pool_size,
                            PLUGIN_VAR_RQCMDARG,
                            N_("Size of Pool."),
                            NULL, NULL, 8, 1, 1024, 0);
 
+static void init_options(drizzled::module::option_context &context)
+{
+  context("size",
+          po::value<uint32_t>(&pool_size)->default_value(8),
+          N_("Size of Pool."));
+}
+
 static drizzle_sys_var* sys_variables[]= {
   DRIZZLE_SYSVAR(size),
   NULL,
@@ -682,8 +672,7 @@ DRIZZLE_DECLARE_PLUGIN
   "Pool of Threads Scheduler",
   PLUGIN_LICENSE_GPL,
   init, /* Plugin Init */
-  deinit, /* Plugin Deinit */
   sys_variables,   /* system variables */
-  NULL    /* config options */
+  init_options    /* config options */
 }
 DRIZZLE_DECLARE_PLUGIN_END;
