@@ -33,17 +33,18 @@ ColumnsTool::ColumnsTool() :
 
   add_field("COLUMN_NAME");
   add_field("COLUMN_TYPE");
-  add_field("ORDINAL_POSITION", plugin::TableFunction::NUMBER);
-  add_field("COLUMN_DEFAULT");
-  add_field("COLUMN_DEFAULT_IS_NULL", plugin::TableFunction::BOOLEAN);
+  add_field("ORDINAL_POSITION", plugin::TableFunction::NUMBER, 0, false);
+  add_field("COLUMN_DEFAULT", plugin::TableFunction::VARBINARY, 65535, true);
+  add_field("COLUMN_DEFAULT_IS_NULL", plugin::TableFunction::BOOLEAN, 0, false);
   add_field("COLUMN_DEFAULT_UPDATE");
-  add_field("IS_NULLABLE", plugin::TableFunction::BOOLEAN);
-  add_field("IS_INDEXED", plugin::TableFunction::BOOLEAN);
-  add_field("IS_USED_IN_PRIMARY", plugin::TableFunction::BOOLEAN);
-  add_field("IS_UNIQUE", plugin::TableFunction::BOOLEAN);
-  add_field("IS_MULTI", plugin::TableFunction::BOOLEAN);
-  add_field("IS_FIRST_IN_MULTI", plugin::TableFunction::BOOLEAN);
-  add_field("INDEXES_FOUND_IN", plugin::TableFunction::NUMBER);
+  add_field("IS_AUTO_INCREMENT", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("IS_NULLABLE", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("IS_INDEXED", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("IS_USED_IN_PRIMARY", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("IS_UNIQUE", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("IS_MULTI", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("IS_FIRST_IN_MULTI", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("INDEXES_FOUND_IN", plugin::TableFunction::NUMBER, 0, false);
   add_field("DATA_TYPE");
 
   add_field("CHARACTER_MAXIMUM_LENGTH", plugin::TableFunction::NUMBER);
@@ -51,9 +52,11 @@ ColumnsTool::ColumnsTool() :
   add_field("NUMERIC_PRECISION", plugin::TableFunction::NUMBER);
   add_field("NUMERIC_SCALE", plugin::TableFunction::NUMBER);
 
+  add_field("ENUM_VALUES", plugin::TableFunction::STRING, 1024, true);
+
   add_field("COLLATION_NAME");
 
-  add_field("COLUMN_COMMENT", 1024);
+  add_field("COLUMN_COMMENT", plugin::TableFunction::STRING, 1024, true);
 }
 
 
@@ -62,11 +65,6 @@ ColumnsTool::Generator::Generator(Field **arg) :
   column_iterator(0),
   is_columns_primed(false)
 {
-  Session *session= current_session;
-  drizzled::statement::Select *select= static_cast<statement::Select *>(session->lex->statement);
-
-  setSchemaPredicate(select->getShowSchema());
-  setTablePredicate(select->getShowTable());
 }
 
 
@@ -120,10 +118,12 @@ bool ColumnsTool::Generator::populate()
 void ColumnsTool::Generator::fill()
 {
   /* TABLE_SCHEMA */
-  push(schema_name());
+  assert(getTableProto().schema().length());
+  assert(getTableProto().schema().c_str());
+  push(getTableProto().schema());
 
   /* TABLE_NAME */
-  push(table_name());
+  push(getTableProto().name());
 
   /* COLUMN_NAME */
   push(column.name());
@@ -135,13 +135,31 @@ void ColumnsTool::Generator::fill()
   push(static_cast<int64_t>(column_iterator));
 
   /* COLUMN_DEFAULT */
-  push(column.options().default_value());
+  if (column.options().has_default_value())
+  {
+    push(column.options().default_value());
+  }
+  else if (column.options().has_default_bin_value())
+  {
+    push(column.options().default_bin_value().c_str(), column.options().default_bin_value().length());
+  }
+  else if (column.options().has_default_expression())
+  {
+    push(column.options().default_expression());
+  }
+  else
+  {
+    push();
+  }
 
   /* COLUMN_DEFAULT_IS_NULL */
   push(column.options().default_null());
 
   /* COLUMN_DEFAULT_UPDATE */
-  push(column.options().update_value());
+  push(column.options().update_expression());
+
+  /* IS_AUTO_INCREMENT */
+  push(column.numeric_options().is_autoincrement());
 
   /* IS_NULLABLE */
   push(column.constraints().is_nullable());
@@ -207,9 +225,37 @@ void ColumnsTool::Generator::fill()
  /* "NUMERIC_SCALE" */
   push(static_cast<int64_t>(column.numeric_options().scale()));
 
+ /* "ENUM_VALUES" */
+  if (column.type() == drizzled::message::Table::Field::ENUM)
+  {
+    string destination;
+    size_t num_field_values= column.enumeration_values().field_value_size();
+    for (size_t x= 0; x < num_field_values; ++x)
+    {
+      const string &type= column.enumeration_values().field_value(x);
+
+      if (x != 0)
+        destination.push_back(',');
+
+      destination.push_back('\'');
+      destination.append(type);
+      destination.push_back('\'');
+    }
+    push(destination);
+  }
+  else
+    push();
+
  /* "COLLATION_NAME" */
   push(column.string_options().collation());
 
  /* "COLUMN_COMMENT" */
-  push(column.comment());
+  if (column.has_comment())
+  {
+    push(column.comment());
+  }
+  else
+  {
+    push();
+  }
 }
