@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2010 Sun Microsystems, Inc.
+ *  Copyright (C) 2010 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,22 +18,34 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_SCHEMA_DICTIONARY_COLUMNS_H
+#define PLUGIN_SCHEMA_DICTIONARY_COLUMNS_H
 
-class ColumnsTool : public DataDictionary
+class ColumnsTool : public TablesTool
 {
 public:
 
   ColumnsTool();
 
-  class Generator : public DataDictionary::Generator
+  class Generator : public TablesTool::Generator 
   {
-    drizzled::generator::AllFields field_generator;
+    int32_t column_iterator;
+    bool is_columns_primed;
+    drizzled::message::Table::Field column;
+
+    bool nextColumnCore();
+    bool nextColumn();
 
   public:
     Generator(drizzled::Field **arg);
 
     bool populate();
+    void fill();
+
+    const drizzled::message::Table::Field& getColumn()
+    {
+      return column;
+    }
   };
 
   Generator *generator(drizzled::Field **arg)
@@ -42,3 +54,4 @@ public:
   }
 };
 
+#endif /* PLUGIN_SCHEMA_DICTIONARY_COLUMNS_H */
