@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2010 Sun Microsystems, Inc.
+ *  Copyright (C) 2010 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_SCHEMA_DICTIONARY_INDEX_PARTS_H
+#define PLUGIN_SCHEMA_DICTIONARY_INDEX_PARTS_H
 
 
 class IndexPartsTool : public IndexesTool
@@ -50,3 +51,4 @@ public:
   }
 };
 
+#endif /* PLUGIN_SCHEMA_DICTIONARY_INDEX_PARTS_H */
