@@ -30,12 +30,12 @@ IndexesTool::IndexesTool() :
   add_field("TABLE_SCHEMA");
   add_field("TABLE_NAME");
   add_field("INDEX_NAME");
-  add_field("IS_USED_IN_PRIMARY", plugin::TableFunction::BOOLEAN);
-  add_field("IS_UNIQUE", plugin::TableFunction::BOOLEAN);
-  add_field("IS_NULLABLE", plugin::TableFunction::BOOLEAN);
-  add_field("KEY_LENGTH", plugin::TableFunction::NUMBER);
+  add_field("IS_USED_IN_PRIMARY", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("IS_UNIQUE", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("IS_NULLABLE", plugin::TableFunction::BOOLEAN, 0, false);
+  add_field("KEY_LENGTH", plugin::TableFunction::NUMBER, 0, false);
   add_field("INDEX_TYPE");
-  add_field("INDEX_COMMENT", 1024);
+  add_field("INDEX_COMMENT", plugin::TableFunction::STRING, 1024, true);
 }
 
 IndexesTool::Generator::Generator(Field **arg) :
@@ -93,10 +93,10 @@ bool IndexesTool::Generator::populate()
 void IndexesTool::Generator::fill()
 {
   /* TABLE_SCHEMA */
-  push(schema_name());
+  push(getTableProto().schema());
 
   /* TABLE_NAME */
-  push(table_name());
+  push(getTableProto().name());
 
   /* INDEX_NAME */
   push(index.name());
@@ -142,9 +142,17 @@ void IndexesTool::Generator::fill()
       length= sizeof("FULLTEXT");
       break;
     }
-    push(str, length);
+    /* Subtract 1 here, because sizeof gives us the wrong amount */
+    push(str, length - 1);
   }
 
  /* "INDEX_COMMENT" */
-  push(index.comment());
+  if (index.has_comment())
+  {
+    push(index.comment());
+  }
+  else
+  {
+    push();
+  }
 }
