@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2010 Sun Microsystems, Inc.
+ *  Copyright (C) 2010 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <plugin/schema_dictionary/dictionary.h>
+#include "config.h"
+#include "plugin/schema_dictionary/dictionary.h"
 
 using namespace std;
 using namespace drizzled;
@@ -29,7 +29,7 @@ TableConstraintsTool::TableConstraintsTool() :
 {
   add_field("CONSTRAINT_SCHEMA");
   add_field("CONSTRAINT_NAME");
-  add_field("TABLE_SCHEMA", plugin::TableFunction::STRING, MAXIMUM_IDENTIFIER_LENGTH, false);
-  add_field("TABLE_NAME", plugin::TableFunction::STRING, MAXIMUM_IDENTIFIER_LENGTH, false);
+  add_field("TABLE_SCHEMA");
+  add_field("TABLE_NAME");
   add_field("CONSTRAINT_TYPE");
 }
