@@ -20,7 +20,7 @@
 
 #include "config.h"
 #include "plugin/schema_dictionary/dictionary.h"
-#include "drizzled/table_identifier.h"
+#include "drizzled/identifier.h"
 
 using namespace std;
 using namespace drizzled;
@@ -30,13 +30,6 @@ static const string TEMPORARY("TEMPORARY");
 static const string INTERNAL("INTERNAL");
 static const string FUNCTION("FUNCTION");
 
-static const string DEFAULT("DEFAULT");
-static const string FIXED("FIXED");
-static const string DYNAMIC("DYNAMIC");
-static const string COMPRESSED("COMPRESSED");
-static const string REDUNDANT("REDUNDANT");
-static const string COMPACT("COMPACT");
-static const string PAGE("PAGE");
 
 static const string VARCHAR("VARCHAR");
 static const string DOUBLE("DOUBLE");
@@ -50,7 +43,7 @@ static const string TIMESTAMP("TIMESTAMP");
 static const string DATETIME("DATETIME");
 
 TablesTool::TablesTool() :
-  SchemasTool("TABLES")
+  plugin::TableFunction("DATA_DICTIONARY", "TABLES")
 {
   add_field("TABLE_SCHEMA");
   add_field("TABLE_NAME");
@@ -58,118 +51,38 @@ TablesTool::TablesTool() :
   add_field("ENGINE");
   add_field("ROW_FORMAT", 10);
   add_field("TABLE_COLLATION");
-  add_field("TABLE_COMMENT", 2048);
+  add_field("TABLE_CREATION_TIME");
+  add_field("TABLE_UPDATE_TIME");
+  add_field("TABLE_COMMENT", plugin::TableFunction::STRING, 2048, true);
 }
 
 TablesTool::Generator::Generator(Field **arg) :
-  SchemasTool::Generator(arg),
-  is_tables_primed(false)
+  plugin::TableFunction::Generator(arg),
+  all_tables_generator(getSession())
 {
 }
 
-bool TablesTool::Generator::nextTableCore()
-{
-  if (is_tables_primed)
-  {
-    table_iterator++;
-  }
-  else
-  {
-    if (not isSchemaPrimed())
-     return false;
-
-    table_names.clear();
-    plugin::StorageEngine::getTableNames(schema_name(), table_names);
-    table_iterator= table_names.begin();
-    is_tables_primed= true;
-  }
-
-  if (table_iterator == table_names.end())
-    return false;
-
-  table_proto.Clear();
-  {
-    Session *session= current_session;
-    char path[FN_REFLEN];
-    build_table_filename(path, sizeof(path), schema_name().c_str(), table_name().c_str(), false);
-    plugin::StorageEngine::getTableDefinition(*session,
-                                             path,
-                                             schema_name().c_str(),
-                                             table_name().c_str(),
-                                             false,
-                                             &table_proto);
-  }
-
-  if (checkTableName())
-    return false;
-
-  return true;
-}
-
 bool TablesTool::Generator::nextTable()
 {
-  while (not nextTableCore())
+  const drizzled::message::Table *table_ptr;
+  while ((table_ptr= all_tables_generator))
   {
-
-    if (is_tables_primed && table_iterator != table_names.end())
-      continue;
-
-    if (not nextSchema())
-      return false;
-    is_tables_primed= false;
+    table_message.CopyFrom(*table_ptr);
+    return true;
   }
 
-  return true;
-}
-
-bool TablesTool::Generator::checkTableName()
-{
-  if (isWild(table_name()))
-    return true;
-
-  if (not table_predicate.empty() && table_predicate.compare(table_name()))
-    return true;
-
   return false;
 }
 
 bool TablesTool::Generator::populate()
 {
-  if (not nextTable())
-    return false;
-
-  fill();
-
-  return true;
-}
-
-void TablesTool::Generator::pushRow(message::Table::TableOptions::RowType type)
-{
-  switch (type)
+  if (nextTable())
   {
-  default:
-  case message::Table::TableOptions::ROW_TYPE_DEFAULT:
-    push(DEFAULT);
-    break;
-  case message::Table::TableOptions::ROW_TYPE_FIXED:
-    push(FIXED);
-    break;
-  case message::Table::TableOptions::ROW_TYPE_DYNAMIC:
-    push(DYNAMIC);
-    break;
-  case message::Table::TableOptions::ROW_TYPE_COMPRESSED:
-    push(COMPRESSED);
-    break;
-  case message::Table::TableOptions::ROW_TYPE_REDUNDANT:
-    push(REDUNDANT);
-    break;
-  case message::Table::TableOptions::ROW_TYPE_COMPACT:
-    push(COMPACT);
-    break;
-  case message::Table::TableOptions::ROW_TYPE_PAGE:
-    push(PAGE);
-    break;
+    fill();
+    return true;
   }
+
+  return false;
 }
 
 void TablesTool::Generator::pushType(message::Table::Field::FieldType type)
@@ -213,15 +126,19 @@ void TablesTool::Generator::pushType(message::Table::Field::FieldType type)
 void TablesTool::Generator::fill()
 {
 
+  /**
+    @note use --replace-column
+  */
+
   /* TABLE_SCHEMA */
-  push(schema_name());
+  push(getTableMessage().schema());
 
   /* TABLE_NAME */
-  push(table_name());
+  push(getTableMessage().name());
 
   /* TABLE_TYPE */
   {
-    switch (table_proto.type())
+    switch (getTableMessage().type())
     {
     default:
     case message::Table::STANDARD:
@@ -240,25 +157,36 @@ void TablesTool::Generator::fill()
   }
 
   /* ENGINE */
-  push(table_proto.engine().name());
+  push(getTableMessage().engine().name());
 
   /* ROW_FORMAT */
-  pushRow(table_proto.options().row_type());
+  push("DEFAULT");
 
   /* TABLE_COLLATION */
-  push(table_proto.options().collation());
+  push(getTableMessage().options().collation());
+
+  /* TABLE_CREATION_TIME */
+  time_t time_arg= getTableMessage().creation_timestamp();
+  char buffer[40];
+  struct tm tm_buffer;
+
+  localtime_r(&time_arg, &tm_buffer);
+  strftime(buffer, sizeof(buffer), "%a %b %d %H:%M:%S %Y", &tm_buffer);
+  push(buffer);
+
+  /* TABLE_UPDATE_TIME */
+  time_arg= getTableMessage().update_timestamp();
+  localtime_r(&time_arg, &tm_buffer);
+  strftime(buffer, sizeof(buffer), "%a %b %d %H:%M:%S %Y", &tm_buffer);
+  push(buffer);
 
   /* TABLE_COMMENT */
-  push(table_proto.options().comment());
-}
-
-bool ShowTables::Generator::checkSchema()
-{
-  Session *session= current_session;
-
-  if (session->lex->select_lex.db)
+  if (getTableMessage().options().has_comment())
   {
-    return schema_name().compare(session->lex->select_lex.db);
+    push(getTableMessage().options().comment());
+  }
+  else
+  {
+    push();
   }
-  return session->db.compare(schema_name());
 }
