@@ -23,10 +23,8 @@
 
 #include <assert.h>
 #include <drizzled/plugin/storage_engine.h>
-#include <drizzled/data_home.h>
-#include <drizzled/hash.h>
-
-#include <pthread.h>
+#include <boost/unordered_map.hpp>
+#include <boost/thread/shared_mutex.hpp>
 
 extern const drizzled::CHARSET_INFO *default_charset_info;
 
@@ -36,65 +34,84 @@ static const char *schema_exts[] = {
 
 class Schema : public drizzled::plugin::StorageEngine
 {
-  bool writeSchemaFile(const char *path, const drizzled::message::Schema &db);
-  bool readTableFile(const std::string &path, drizzled::message::Table &table_message);
-  bool readSchemaFile(const std::string &path, drizzled::message::Schema &schema);
+  bool writeSchemaFile(const drizzled::SchemaIdentifier &schema_identifier, const drizzled::message::Schema &db);
+  bool readSchemaFile(const drizzled::SchemaIdentifier &schema_identifier, drizzled::message::Schema &schema);
 
   void prime();
 
-  typedef drizzled::hash_map<std::string, drizzled::message::Schema> SchemaCache;
+  typedef boost::unordered_map<std::string, drizzled::message::Schema> SchemaCache;
   SchemaCache schema_cache;
   bool schema_cache_filled;
 
-  pthread_rwlock_t schema_lock;
+  boost::shared_mutex mutex;
 
 public:
   Schema();
 
   ~Schema();
 
-  int doCreateTable(drizzled::Session *,
-                    const char *,
-                    drizzled::Table&,
-                    drizzled::message::Table&)
-  {
-    return EPERM;
-  }
-
-  int doDropTable(drizzled::Session&, const std::string &table_path);
 
   bool doCanCreateTable(const drizzled::TableIdentifier &identifier);
 
-  drizzled::Cursor *create(drizzled::TableShare &,
-                           drizzled::memory::Root *)
+  drizzled::Cursor *create(drizzled::TableShare &)
   {
     return NULL;
   }
 
-  void doGetSchemaNames(std::set<std::string>& set_of_names);
-  bool doGetSchemaDefinition(const std::string &schema_name, drizzled::message::Schema &proto);
+  void doGetSchemaIdentifiers(drizzled::SchemaIdentifiers &set_of_names);
+  bool doGetSchemaDefinition(const drizzled::SchemaIdentifier&, drizzled::message::Schema &proto);
 
   bool doCreateSchema(const drizzled::message::Schema &schema_message);
 
   bool doAlterSchema(const drizzled::message::Schema &schema_message);
 
-  bool doDropSchema(const std::string &schema_name);
+  bool doDropSchema(const drizzled::SchemaIdentifier&);
 
-  int doGetTableDefinition(drizzled::Session& session,
-                           const char *path,
-                           const char *db,
-                           const char *table_name,
-                           const bool is_tmp,
-                           drizzled::message::Table *table_proto);
+  // Below are table methods that we don't implement (and don't need)
 
-  void doGetTableNames(drizzled::CachedDirectory &directory,
-                       std::string &db_name,
-                       std::set<std::string> &set_of_names);
+  int doGetTableDefinition(drizzled::Session&,
+                           const drizzled::TableIdentifier&,
+                           drizzled::message::Table&)
+  {
+    return ENOENT;
+  }
+
+  bool doDoesTableExist(drizzled::Session&, const drizzled::TableIdentifier&)
+  {
+    return false;
+  }
+
+  int doRenameTable(drizzled::Session&, const drizzled::TableIdentifier&, const drizzled::TableIdentifier&)
+  {
+    return EPERM;
+  }
+
+  int doCreateTable(drizzled::Session&,
+                    drizzled::Table&,
+                    const drizzled::TableIdentifier&,
+                    drizzled::message::Table&)
+  {
+    return EPERM;
+  }
+
+  int doDropTable(drizzled::Session&, const drizzled::TableIdentifier&)
+  {
+    return 0;
+  }
 
   const char **bas_ext() const 
   {
     return schema_exts;
   }
+
+  void get_auto_increment(uint64_t, uint64_t,
+                          uint64_t,
+                          uint64_t *,
+                          uint64_t *)
+  {}
+  void doGetTableIdentifiers(drizzled::CachedDirectory &directory,
+                             const drizzled::SchemaIdentifier &schema_identifier,
+                             drizzled::TableIdentifiers &set_of_identifiers);
 };
 
 #endif /* PLUGIN_SCHEMA_ENGINE_SCHEMA_H */
