@@ -18,12 +18,10 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_SESSION_DICTIONARY_DICTIONARY_H
+#define PLUGIN_SESSION_DICTIONARY_DICTIONARY_H
 
-#include <drizzled/plugin/table_function.h>
-#include <drizzled/sql_parse.h>
-#include <plugin/session_dictionary/processlist.h>
-#include <plugin/session_dictionary/savepoints.h>
-#include <plugin/session_dictionary/sessions.h>
-#include <plugin/session_dictionary/variables.h>
+#include "drizzled/plugin/table_function.h"
+#include "plugin/session_dictionary/processlist.h"
 
+#endif /* PLUGIN_SESSION_DICTIONARY_DICTIONARY_H */
