@@ -28,6 +28,7 @@
 #include "drizzled/plugin/client.h"
 #include "drizzled/plugin/authorization.h"
 #include "drizzled/internal/my_sys.h"
+#include "drizzled/internal/thread_var.h"
 
 #include <set>
 
@@ -37,14 +38,14 @@ using namespace drizzled;
 ProcesslistTool::ProcesslistTool() :
   plugin::TableFunction("DATA_DICTIONARY", "PROCESSLIST")
 {
-  add_field("ID", plugin::TableFunction::NUMBER);
+  add_field("ID", plugin::TableFunction::NUMBER, 0, false);
   add_field("USER", 16);
   add_field("HOST", NI_MAXHOST);
   add_field("DB");
   add_field("COMMAND", 16);
-  add_field("TIME", plugin::TableFunction::NUMBER);
-  add_field("STATE");
-  add_field("INFO", PROCESS_LIST_WIDTH);
+  add_field("TIME", plugin::TableFunction::NUMBER, 0, false);
+  add_field("STATE", plugin::TableFunction::STRING, 256, true);
+  add_field("INFO", plugin::TableFunction::STRING, PROCESS_LIST_WIDTH, true);
 }
 
 ProcesslistTool::Generator::Generator(Field **arg) :
@@ -52,13 +53,13 @@ ProcesslistTool::Generator::Generator(Field **arg) :
 {
   now= time(NULL);
 
-  pthread_mutex_lock(&LOCK_thread_count);
+  LOCK_thread_count.lock();
   it= getSessionList().begin();
 }
 
 ProcesslistTool::Generator::~Generator()
 {
-  pthread_mutex_unlock(&LOCK_thread_count);
+  LOCK_thread_count.unlock();
 }
 
 bool ProcesslistTool::Generator::populate()
@@ -90,7 +91,7 @@ bool ProcesslistTool::Generator::populate()
   if (not tmp_sctx->getUser().empty())
     push(tmp_sctx->getUser());
   else 
-    push("unauthenticated user");
+    push(_("no user"));
 
   /* HOST */
   push(tmp_sctx->getIp());
@@ -125,8 +126,8 @@ bool ProcesslistTool::Generator::populate()
                 (tmp->command == COM_SLEEP ?
                  NULL : "Reading from net") :
                 tmp->get_proc_info() ? tmp->get_proc_info() :
-                tmp->mysys_var &&
-                tmp->mysys_var->current_cond ?
+                tmp->getThreadVar() &&
+                tmp->getThreadVar()->current_cond ?
                 "Waiting on cond" : NULL);
   val ? push(val) : push();
 
