@@ -1,7 +1,7 @@
 /* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2010 Sun Microsystems, Inc.
+ *  Copyright (C) 2010 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,10 +18,11 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <drizzled/session.h>
-#include <drizzled/generator/session.h>
+#include "drizzled/session.h"
+#include "drizzled/session_list.h"
 
-#pragma once
+#ifndef PLUGIN_SESSION_DICTIONARY_PROCESSLIST_H
+#define PLUGIN_SESSION_DICTIONARY_PROCESSLIST_H
 
 class ProcesslistTool : public drizzled::plugin::TableFunction
 {
@@ -29,12 +30,14 @@ public:
 
   ProcesslistTool();
 
-  class Generator : public drizzled::plugin::TableFunction::Generator
+  class Generator : public drizzled::plugin::TableFunction::Generator 
   {
-    drizzled::generator::Session session_generator;
+    time_t now;
+    drizzled::SessionList::iterator it;
 
   public:
     Generator(drizzled::Field **arg);
+    ~Generator();
 
     bool populate();
   };
@@ -45,3 +48,4 @@ public:
   }
 };
 
+#endif /* PLUGIN_SESSION_DICTIONARY_PROCESSLIST_H */
