@@ -18,23 +18,39 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_SHOW_DICTIONARY_SHOW_SCHEMAS_H
+#define PLUGIN_SHOW_DICTIONARY_SHOW_SCHEMAS_H
 
-class ShowSchemas : public show_dictionary::Show
+class ShowSchemas : public drizzled::plugin::TableFunction
 {
 public:
 
   ShowSchemas();
 
-  class Generator : public show_dictionary::Show::Generator 
+  ShowSchemas(const char *schema_arg, const char *table_arg) :
+    drizzled::plugin::TableFunction(schema_arg, table_arg)
+  { }
+
+  class Generator : public drizzled::plugin::TableFunction::Generator 
   {
+    drizzled::SchemaIdentifiers schema_names;
+    drizzled::SchemaIdentifiers::const_iterator schema_iterator;
+
+    bool is_schema_primed;
+
+    virtual void fill();
+    virtual bool checkSchema();
+
   public:
     Generator(drizzled::Field **arg);
 
     bool populate();
-
-  private:
-    drizzled::generator::Schema schema_generator;
+    bool nextSchemaCore();
+    bool nextSchema();
+    bool isSchemaPrimed()
+    {
+      return is_schema_primed;
+    }
   };
 
   Generator *generator(drizzled::Field **arg)
@@ -44,3 +60,4 @@ public:
 };
 
 
+#endif /* PLUGIN_SHOW_DICTIONARY_SHOW_SCHEMAS_H */
