@@ -18,18 +18,19 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_SHOW_DICTIONARY_SHOW_TABLES_H
+#define PLUGIN_SHOW_DICTIONARY_SHOW_TABLES_H
 
-class ShowTables : public show_dictionary::Show
+class ShowTables : public drizzled::plugin::TableFunction
 {
 public:
   ShowTables();
 
-  class Generator : public show_dictionary::Show::Generator
+  class Generator : public drizzled::plugin::TableFunction::Generator
   {
     bool is_primed;
-    drizzled::identifier::table::vector set_of_identifiers;
-    drizzled::identifier::table::vector::iterator table_iterator;
+    drizzled::TableIdentifiers set_of_identifiers;
+    drizzled::TableIdentifiers::iterator table_iterator;
     std::string schema_name;
 
     void fill();
@@ -48,3 +49,4 @@ public:
   }
 };
 
+#endif /* PLUGIN_SHOW_DICTIONARY_SHOW_TABLES_H */
