@@ -1,7 +1,7 @@
 /* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
  *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
  *
- *  Copyright (C) 2008 Sun Microsystems, Inc.
+ *  Copyright (C) 2008 Sun Microsystems
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_STRING_FUNCTIONS_FORMAT_H
+#define PLUGIN_STRING_FUNCTIONS_FORMAT_H
 
 #include <drizzled/function/str/strfunc.h>
 
@@ -33,9 +34,10 @@ public:
   String *val_str(String *);
   void fix_length_and_dec();
   const char *func_name() const { return "format"; }
-  virtual void print(String *str);
+  virtual void print(String *str, enum_query_type query_type);
   bool check_argument_count(int n) { return n == 2; }
 };
 
 } /* namespace drizzled */
 
+#endif /* PLUGIN_STRING_FUNCTIONS_FORMAT_H */
