@@ -258,24 +258,15 @@ String *SubstrIndexFunction::val_str(String *str)
 plugin::Create_function<SubstrFunction> *substr_function= NULL;
 plugin::Create_function<SubstrIndexFunction> *substr_index_function= NULL;
 
-static int initialize(drizzled::plugin::Registry &registry)
+static int initialize(drizzled::module::Context &context)
 {
   substr_function= new plugin::Create_function<SubstrFunction>("substr");
   substr_index_function= new plugin::Create_function<SubstrIndexFunction>("substring_index");
-  registry.add(substr_function);
-  registry.add(substr_index_function);
+  context.add(substr_function);
+  context.add(substr_index_function);
   return 0;
 }
 
-static int finalize(drizzled::plugin::Registry &registry)
-{
-   registry.remove(substr_function);
-   registry.remove(substr_index_function);
-   delete substr_function;
-   delete substr_index_function;
-   return 0;
-}
-
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
@@ -285,7 +276,6 @@ DRIZZLE_DECLARE_PLUGIN
   "SUBSTR and SUBSTR",
   PLUGIN_LICENSE_GPL,
   initialize, /* Plugin Init */
-  finalize,   /* Plugin Deinit */
   NULL,   /* system variables */
   NULL    /* config options */
 }
