@@ -17,27 +17,26 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_SYSLOG_ERRMSG_H
+#define PLUGIN_SYSLOG_ERRMSG_H
 
+#include "module.h"
 #include <stdarg.h>
 #include <drizzled/plugin/error_message.h>
 
-namespace drizzle_plugin
-{
-namespace error_message
+class ErrorMessage_syslog : public drizzled::plugin::ErrorMessage
 {
+ private:
+  int syslog_facility;
+  int syslog_priority;
 
-class Syslog : public drizzled::plugin::ErrorMessage
-{
-private:
-  int _facility;
+  ErrorMessage_syslog(const ErrorMessage_syslog&);
+  ErrorMessage_syslog& operator=(const ErrorMessage_syslog&);
 
-public:
-  explicit Syslog(const std::string& facility);
+ public:
+  ErrorMessage_syslog();
 
-  virtual bool errmsg(drizzled::error::priority_t, const char *format, va_list ap);
+  virtual bool errmsg(drizzled::Session *, int, const char *format, va_list ap);
 };
 
-} /* namespace error_message */
-} /* namespace drizzle_plugin */
-
+#endif /* PLUGIN_SYSLOG_ERRMSG_H */
