@@ -17,20 +17,23 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_SYSLOG_FUNCTION_H
+#define PLUGIN_SYSLOG_FUNCTION_H
+
+#include "module.h"
 
 #include <drizzled/plugin/function.h>
 #include <drizzled/item/func.h>
 #include <drizzled/function/str/strfunc.h>
 
-namespace drizzle_plugin {
-namespace udf {
-
-class Syslog : public drizzled::Item_str_func
+class Function_syslog : public drizzled::Item_str_func
 {
 private:
+  Function_syslog(const Function_syslog&);
+  Function_syslog& operator=(const Function_syslog&);
+
 public:
-  Syslog();
+  Function_syslog();
 
   const char *func_name() const { return "syslog"; }
 
@@ -39,7 +42,4 @@ public:
   bool check_argument_count(int n);
 };
 
-} /* namespace udf */
-} /* namepsace drizzle_plugin */
-
-
+#endif /* PLUGIN_SYSLOG_FUNCTION_H */
