@@ -17,39 +17,32 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_SYSLOG_WRAP_H
+#define PLUGIN_SYSLOG_WRAP_H
 
-#include <cstdarg>
-
-#include <string>
-
-#include <drizzled/error/priority_t.h>
-
-namespace drizzle_plugin {
+#include <stdarg.h>
 
 class WrapSyslog
 {
-private:
-  bool _check;
+ private:
+  WrapSyslog(const WrapSyslog&);
+  WrapSyslog& operator=(const WrapSyslog&);
 
   WrapSyslog();
-public:
+
+  bool openlog_check;
+  char openlog_ident[32];
+
+ public:
   ~WrapSyslog();
   static WrapSyslog& singleton();
 
   static int getFacilityByName(const char *);
   static int getPriorityByName(const char *);
 
-  void openlog(const std::string &ident);
-  void vlog(int facility, const drizzled::error::priority_t priority, const char *format, va_list ap);
-  void log(int facility, const drizzled::error::priority_t priority, const char *format, ...);
+  void openlog(char *ident);
+  void vlog(int facility, int priority, const char *format, va_list ap);
+  void log(int facility, int priority, const char *format, ...);
 };
 
-inline WrapSyslog& WrapSyslog::singleton()
-{
-  static WrapSyslog handle;
-  return handle;
-}
-
-} /* namespsace drizzle_plugin */
-
+#endif /* PLUGIN_SYSLOG_WRAP_H */
