@@ -52,43 +52,30 @@ public:
                                      HTON_NULL_IN_KEY |
                                      HTON_CAN_INDEX_BLOBS |
                                      HTON_SKIP_STORE_LOCK |
-                                     HTON_AUTO_PART_KEY |
-                                     HTON_HAS_DATA_DICTIONARY)
+                                     HTON_AUTO_PART_KEY)
   {
     table_definition_ext= TABLEPROTOTESTER_EXT;
   }
 
-  virtual Cursor *create(TableShare &table,
-                         drizzled::memory::Root *mem_root)
+  virtual Cursor *create(TableShare &table)
   {
-    return new (mem_root) TableProtoTesterCursor(*this, table);
+    return new TableProtoTesterCursor(*this, table);
   }
 
   const char **bas_ext() const {
     return TableProtoTesterCursor_exts;
   }
 
-  int doCreateTable(Session*,
-                    const char *,
+  int doCreateTable(Session&,
                     Table&,
+                    const drizzled::TableIdentifier &identifier,
                     drizzled::message::Table&);
 
-  int doDropTable(Session&, const string &table_name);
+  int doDropTable(Session&, const drizzled::TableIdentifier &identifier);
 
-  int doGetTableDefinition(Session& session,
-                           const char* path,
-                           const char *db,
-                           const char *table_name,
-                           const bool is_tmp,
-                           drizzled::message::Table *table_proto);
-
-  void doGetTableNames(drizzled::CachedDirectory &directory,
-                       string&, set<string>& set_of_names)
-  {
-    (void)directory;
-    set_of_names.insert("t1");
-
-  }
+  int doGetTableDefinition(Session &session,
+                           const drizzled::TableIdentifier &identifier,
+                           drizzled::message::Table &table_proto);
 
   /* The following defines can be increased if necessary */
   uint32_t max_supported_keys()          const { return 64; }
@@ -104,16 +91,50 @@ public:
             HA_KEYREAD_ONLY);
   }
 
+  bool doDoesTableExist(Session &session, const drizzled::TableIdentifier &identifier);
+
+  int doRenameTable(Session&, const drizzled::TableIdentifier&, const drizzled::TableIdentifier&)
+  {
+    return EPERM;
+  }
+
+  void doGetTableIdentifiers(drizzled::CachedDirectory &directory,
+                             const drizzled::SchemaIdentifier &schema_identifier,
+                             drizzled::TableIdentifiers &set_of_identifiers);
 };
 
+void TableProtoTesterEngine::doGetTableIdentifiers(drizzled::CachedDirectory&,
+                                                   const drizzled::SchemaIdentifier &schema_identifier,
+                                                   drizzled::TableIdentifiers &set_of_identifiers)
+{
+  if (schema_identifier.compare("test"))
+  {
+    set_of_identifiers.push_back(TableIdentifier(schema_identifier, "t1"));
+    set_of_identifiers.push_back(TableIdentifier(schema_identifier, "too_many_enum_values"));
+    set_of_identifiers.push_back(TableIdentifier(schema_identifier, "invalid_table_collation"));
+  }
+}
+
+bool TableProtoTesterEngine::doDoesTableExist(Session&, const drizzled::TableIdentifier &identifier)
+{
+  if (not identifier.getPath().compare("test/t1"))
+    return true;
+  if (not identifier.getPath().compare("test/too_many_enum_values"))
+    return true;
+  if (not identifier.getPath().compare("test/invalid_table_collation"))
+    return true;
+
+  return false;
+}
+
 TableProtoTesterCursor::TableProtoTesterCursor(drizzled::plugin::StorageEngine &engine_arg,
-                           TableShare &table_arg)
-  :Cursor(engine_arg, table_arg)
+                           TableShare &table_arg) :
+  Cursor(engine_arg, table_arg)
 { }
 
 int TableProtoTesterCursor::open(const char *, int, uint32_t)
 {
-  return(0);
+  return 0;
 }
 
 int TableProtoTesterCursor::close(void)
@@ -121,48 +142,115 @@ int TableProtoTesterCursor::close(void)
   return 0;
 }
 
-int TableProtoTesterEngine::doCreateTable(Session*, const char *,
-                                   Table&,
-                                   drizzled::message::Table&)
+int TableProtoTesterEngine::doCreateTable(Session&,
+                                          Table&,
+                                          const drizzled::TableIdentifier&,
+                                          drizzled::message::Table&)
 {
   return EEXIST;
 }
 
 
-int TableProtoTesterEngine::doDropTable(Session&, const string&)
+int TableProtoTesterEngine::doDropTable(Session&, const drizzled::TableIdentifier&)
 {
   return EPERM;
 }
 
-static void fill_table1(message::Table *table)
+static void fill_table1(message::Table &table)
 {
   message::Table::Field *field;
   message::Table::TableOptions *tableopts;
 
-  table->set_name("t1");
-  table->set_type(message::Table::INTERNAL);
+  table.set_name("t1");
+  table.set_type(message::Table::INTERNAL);
 
-  tableopts= table->mutable_options();
+  tableopts= table.mutable_options();
   tableopts->set_comment("Table without a StorageEngine message");
 
   {
-    field= table->add_field();
+    field= table.add_field();
     field->set_name("number");
     field->set_type(message::Table::Field::INTEGER);
   }
 
 }
-int TableProtoTesterEngine::doGetTableDefinition(Session&,
-                                          const char* path,
-                                          const char *,
-                                          const char *,
-                                          const bool,
-                                          drizzled::message::Table *table_proto)
+
+static void fill_table_too_many_enum_values(message::Table &table)
 {
-  if (strcmp(path, "./test/t1") == 0)
+  message::Table::Field *field;
+  message::Table::TableOptions *tableopts;
+
+  table.set_schema("test");
+  table.set_name("too_many_enum_values");
+  table.set_type(message::Table::STANDARD);
+  table.mutable_engine()->set_name("tableprototester");
+  table.set_creation_timestamp(0);
+  table.set_update_timestamp(0);
+
+  tableopts= table.mutable_options();
+  tableopts->set_comment("Table with too many enum options");
+  tableopts->set_collation("utf8_general_ci");
+  tableopts->set_collation_id(45);
+
   {
-    if (table_proto)
-      fill_table1(table_proto);
+    field= table.add_field();
+    field->set_name("many_values");
+    field->set_type(message::Table::Field::ENUM);
+
+    message::Table::Field::EnumerationValues *field_options= field->mutable_enumeration_values();
+    for(int i=0; i<70000; i++)
+    {
+      char enum_value[100];
+      snprintf(enum_value, sizeof(enum_value), "a%d", i);
+      field_options->add_field_value(enum_value);
+    }
+  }
+
+}
+
+static void fill_table_invalid_table_collation(message::Table &table)
+{
+  message::Table::Field *field;
+  message::Table::TableOptions *tableopts;
+
+  table.set_name("invalid_table_collation");
+  table.set_type(message::Table::STANDARD);
+  table.set_schema("test");
+  table.set_creation_timestamp(0);
+  table.set_update_timestamp(0);
+  table.mutable_engine()->set_name("tableprototester");
+
+  tableopts= table.mutable_options();
+  tableopts->set_comment("Invalid table collation ");
+
+  {
+    field= table.add_field();
+    field->set_name("number");
+    field->set_type(message::Table::Field::INTEGER);
+  }
+
+  tableopts->set_collation("pi_pi_pi");
+  tableopts->set_collation_id(123456);
+
+}
+
+int TableProtoTesterEngine::doGetTableDefinition(Session&,
+                                                 const drizzled::TableIdentifier &identifier,
+                                                 drizzled::message::Table &table_proto)
+{
+  if (not identifier.getPath().compare("test/t1"))
+  {
+    fill_table1(table_proto);
+    return EEXIST;
+  }
+  else if (not identifier.getPath().compare("test/too_many_enum_values"))
+  {
+    fill_table_too_many_enum_values(table_proto);
+    return EEXIST;
+  }
+  else if (not identifier.getPath().compare("test/invalid_table_collation"))
+  {
+    fill_table_invalid_table_collation(table_proto);
     return EEXIST;
   }
   return ENOENT;
@@ -173,12 +261,12 @@ const char *TableProtoTesterCursor::index_type(uint32_t)
   return("BTREE");
 }
 
-int TableProtoTesterCursor::write_row(unsigned char *)
+int TableProtoTesterCursor::doInsertRecord(unsigned char *)
 {
   return(table->next_number_field ? update_auto_increment() : 0);
 }
 
-int TableProtoTesterCursor::rnd_init(bool)
+int TableProtoTesterCursor::doStartTableScan(bool)
 {
   return(0);
 }
@@ -258,19 +346,11 @@ int TableProtoTesterCursor::index_last(unsigned char *)
 
 static drizzled::plugin::StorageEngine *tableprototester_engine= NULL;
 
-static int tableprototester_init(drizzled::plugin::Registry &registry)
+static int tableprototester_init(drizzled::module::Context &context)
 {
 
   tableprototester_engine= new TableProtoTesterEngine("TABLEPROTOTESTER");
-  registry.add(tableprototester_engine);
-
-  return 0;
-}
-
-static int tableprototester_fini(drizzled::plugin::Registry &registry)
-{
-  registry.remove(tableprototester_engine);
-  delete tableprototester_engine;
+  context.add(tableprototester_engine);
 
   return 0;
 }
@@ -284,7 +364,6 @@ DRIZZLE_DECLARE_PLUGIN
   "Used to test rest of server with various table proto messages",
   PLUGIN_LICENSE_GPL,
   tableprototester_init,     /* Plugin Init */
-  tableprototester_fini,     /* Plugin Deinit */
   NULL,               /* system variables */
   NULL                /* config options   */
 }
