@@ -36,8 +36,8 @@ public:
   uint32_t index_flags(uint32_t inx) const;
   int open(const char *name, int mode, uint32_t test_if_locked);
   int close(void);
-  int write_row(unsigned char * buf);
-  int rnd_init(bool scan);
+  int doInsertRecord(unsigned char * buf);
+  int doStartTableScan(bool scan);
   int rnd_next(unsigned char *buf);
   int rnd_pos(unsigned char * buf, unsigned char *pos);
 
@@ -53,6 +53,12 @@ public:
   int index_last(unsigned char * buf);
   void position(const unsigned char *record);
   int info(uint32_t flag);
+
+  void get_auto_increment(uint64_t, uint64_t,
+                          uint64_t,
+                          uint64_t *,
+                          uint64_t *)
+  {}
 };
 
 #endif /* PLUGIN_TABLEPROTOTESTER_TABLEPROTOTESTER_H */
