@@ -18,7 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_UTILITY_DICTIONARY_COUNTER_H
+#define PLUGIN_UTILITY_DICTIONARY_COUNTER_H
 
 namespace utility_dictionary {
 
@@ -49,3 +50,4 @@ public:
 
 } /* namespace utility_dictionary */
 
+#endif /* PLUGIN_UTILITY_DICTIONARY_COUNTER_H */
