@@ -18,15 +18,14 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-#include <plugin/utility_dictionary/dictionary.h>
+#include "config.h"
+#include "plugin/utility_dictionary/dictionary.h"
 
 using namespace drizzled;
 
 static int init(drizzled::module::Context &context)
 {
   context.add(new utility_dictionary::Counter);
-  context.add(new utility_dictionary::Environmental);
   context.add(new utility_dictionary::RandomNumber);
   context.add(new utility_dictionary::RandomString);
   
@@ -36,10 +35,10 @@ static int init(drizzled::module::Context &context)
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
-  "utility_dictionary",
+  "Utility Dictionary",
   "1.0",
   "Brian Aker",
-  N_("Utility dictionaries"),
+  "Data Dictionary for utility tables",
   PLUGIN_LICENSE_GPL,
   init,
   NULL,
