@@ -18,11 +18,12 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#pragma once
+#ifndef PLUGIN_UTILITY_DICTIONARY_DICTIONARY_H
+#define PLUGIN_UTILITY_DICTIONARY_DICTIONARY_H
 
-#include <drizzled/plugin/table_function.h>
-#include <plugin/utility_dictionary/counter.h>
-#include <plugin/utility_dictionary/environmental.h>
-#include <plugin/utility_dictionary/random_number.h>
-#include <plugin/utility_dictionary/random_string.h>
+#include "drizzled/plugin/table_function.h"
+#include "plugin/utility_dictionary/counter.h"
+#include "plugin/utility_dictionary/random_number.h"
+#include "plugin/utility_dictionary/random_string.h"
 
+#endif /* PLUGIN_UTILITY_DICTIONARY_DICTIONARY_H */
