@@ -55,7 +55,7 @@ String *UuidFunction::val_str(String *str)
   str->length(UUID_LENGTH);
   str->set_charset(system_charset_info);
   uuid_string= (char *) str->ptr();
-  uuid_generate_time(uu);
+  uuid_generate_random(uu);
   uuid_unparse(uu, uuid_string);
 
   return str;
@@ -63,20 +63,13 @@ String *UuidFunction::val_str(String *str)
 
 plugin::Create_function<UuidFunction> *uuid_function= NULL;
 
-static int initialize(drizzled::plugin::Registry &registry)
+static int initialize(drizzled::module::Context &context)
 {
   uuid_function= new plugin::Create_function<UuidFunction>("uuid");
-  registry.add(uuid_function);
+  context.add(uuid_function);
   return 0;
 }
 
-static int finalize(drizzled::plugin::Registry &registry)
-{
-   registry.remove(uuid_function);
-   delete uuid_function;
-   return 0;
-}
-
 DRIZZLE_DECLARE_PLUGIN
 {
   DRIZZLE_VERSION_ID,
@@ -86,7 +79,6 @@ DRIZZLE_DECLARE_PLUGIN
   "UUID() function using libuuid",
   PLUGIN_LICENSE_GPL,
   initialize, /* Plugin Init */
-  finalize,   /* Plugin Deinit */
   NULL,   /* system variables */
   NULL    /* config options */
 }
